/*
 *  astar.c
 *
 *  Created by frederic Vernier on 1/22/07.
 *  Copyright 2007 frederic Vernier. All rights reserved.
 *
 */

#include "astar.h"

// this pointer has no guarentee to point on something. It is there for debug purpose;
list_t            *foundPath           = NULL;
list_t            *openList            = NULL;
list_t            *closedList          = NULL;
int                destX               = 0;
int                destY               = 0;
etile             *found               = NULL;
unit              *movingUnit          = NULL;
const unsigned int MAXTESTS            = 4000;
float              initialHeuristic    = 0;
SDL_mutex         *astarmut            = NULL;
char              *errorMessage        = NULL;


/**
 * comparizon function which pointer is given to the search algorithm of the list data structure
 */
int etileCompare(const void *a, const void *b) {
  tile *ea = ((etile*)a)->location;
  return (ea != b);
}// etileCompare() --------------------------------------------------------------------------------


/**
 * heuristic function to guess how far the hypothesis is from the destination
 */
float heuristic (const int hypothesisX, const int hypothesisY) {
  float heurist = 0;
  int w = hypothesisX-destX;
  int h = hypothesisY-destY;
  if (w<0)
    w = -1*w;
  if (h<0)
    h = -1*h;
  // WE MUST BE PESSIMISTIC OR OPTIMISTIC ? AND do not hope for a road or a river

  if (isNaval(movingUnit) && w-h>0)
    heurist = (int)(4.0f*h+(w-h)*3.0f);
  else if (isNaval(movingUnit) && w-h<=0)
    heurist = (int)(4.0f*w+(h-w)*3.0f);
  else if (w-h>0)
    heurist = (int)(14.0f*h+(w-h)*10.0f);
  else
    heurist = (int)(14.0f*w+(h-w)*10.0f);
  return heurist;
}// heuristic() --------------------------------------------------------------------------------


/**
 * initialize a* engine
 */
void initAStar(){
  errorMessage = (char*)malloc(1024);
  sprintf(errorMessage, "ERROR A*: No error\n");
}// initAStar()


/**
 * perform the a* algorithm on the grid to find the best path to the given destination
 * return NULL if destination is un-reachable
 */
list_t *astarSearch (const int startX, const int startY, const int _destX, const int _destY, unit *u) {
  //printf("astarSearch(%i, %i, %i, %i) for unit %i %i\n", startX, startY, _destX, _destY, u->type, u->horses);fflush(stdout);
  if (errorMessage==NULL)
    initAStar();

  if (astarmut==NULL)
    astarmut=SDL_CreateMutex();

  if(SDL_mutexP(astarmut)==-1){
    fprintf(stderr, "Couldn't lock astar mutex\n");
    exit(-1);
  }

  destX = _destX;
  destY = _destY;
  movingUnit = u;
  found = NULL;
  // boarding special case
  if (u->horses!=-101 && u->maxLoad==0 && (map[destX+MapWidth*(destY)].elevation<SEALEVEL) && (map[destX+MapWidth*(destY)].lastExplored>=0 || gamemode!=PLAY) && isFreeBoarding(&map[destX+MapWidth*(destY)], u)) {

  } else if (u->horses!=-101 && !isNaval(u) && (map[destX+MapWidth*(destY)].elevation<SEALEVEL || map[destX+MapWidth*(destY)].elevation>=HIGHMOUNTAIN) && (map[destX+MapWidth*(destY)].lastExplored>=0 || gamemode!=PLAY)) {
    clean();
    sprintf(errorMessage, "  ERROR A*: %s(%i horses) cannot climb hight mountains or swim at level %i(%i,%i)\n", getUnitText(u), u->horses, map[destX+MapWidth*(destY)].elevation, destX, destY);
    return NULL;
  } else if (u->horses!=-101 && isNaval(u)==1 && map[destX+MapWidth*(destY)].elevation>=SEALEVEL && map[destX+MapWidth*(destY)].construction==NULL && map[destX+MapWidth*(destY)].irrigation<BIG_RIVER &&
             (map[destX+MapWidth*(destY)].lastExplored>=0|| gamemode!=PLAY)) {
    clean();
    sprintf(errorMessage, "  ERROR A*: %s(small ship) cannot climb on land where there ins't a city or a large river %i(%i,%i)\n", getUnitText(u), map[destX+MapWidth*(destY)].elevation, destX, destY);
    return NULL;
  } else if (u->horses!=-101 && isNaval(u)==2 && map[destX+MapWidth*(destY)].elevation>=SEALEVEL && map[destX+MapWidth*(destY)].construction==NULL &&
             (map[destX+MapWidth*(destY)].lastExplored>=0 || gamemode!=PLAY)) {
    clean();
    sprintf(errorMessage, "  ERROR A*: %s(big ship) cannot climb on land where there ins't a city %i(%i,%i)\n", getUnitText(u), map[destX+MapWidth*(destY)].elevation, destX, destY);
    return NULL;
  }
  if (openList!=NULL && list_count(openList)>0) {
    lnode_t *etileelem = list_first(openList);
    while (etileelem!=NULL && etileelem != &openList->list_nilnode) {
      etile *et = (etile *)etileelem->list_data;
      //printf("freeing etile in open list %p = %p = %d, %d  => %p\n", etileelem, et, et->location->posX, et->location->posY, etileelem);fflush(stdout);
      free(et);
      etileelem->list_data = NULL;
      etileelem = etileelem->list_next;
    }
    ///printf("openList %i \n", list_count(openList));fflush(stdout);
    list_destroy_nodes(openList);
    //printf("openList %i \n", list_count(openList));fflush(stdout);
    //list_destroy(openList);
    //openList = NULL;
  }
  if (closedList!=NULL && list_count(closedList)>0) {
    lnode_t *etileelem = list_first(closedList);
    while (etileelem != &closedList->list_nilnode) {
      etile *et = (etile *)etileelem->list_data;
      free(et);
      etileelem->list_data = NULL;
      etileelem = etileelem->list_next;
    }
    list_destroy_nodes(closedList);
    //list_destroy(closedList);
    //closedList = NULL;
  }
  if (openList==NULL)
    openList             = list_create(LISTCOUNT_T_MAX);

  if (closedList==NULL)
    closedList           = list_create(LISTCOUNT_T_MAX);

  etile *startETile    = (etile *)malloc (sizeof(etile));
  startETile->location = &map[startX+MapWidth*(startY)];
  startETile->f        = 0;
  startETile->g        = 0;
  startETile->h        = 0;
  int stop = 0;
  initialHeuristic = heuristic(startX, startY);
  list_append(openList, lnode_create(startETile));
  while (!stop && !list_isempty(openList) && list_count(closedList)+list_count(openList)<MAXTESTS) {
    lnode_t *currentelem = getLowestOpenETile();

    etile *current = (etile *)currentelem->list_data;
    list_delete(openList, currentelem);
    list_prepend(closedList, currentelem);
    //list_t *nextSearches = list_create(LISTCOUNT_T_MAX);

    // we want to test neightbour tiles in the right order (according to which one is toward the destination .. or the closest to this direction)
    double angle = atan2(((float)current->location->posY-_destY), _destX-current->location->posX);
    //if (_destX==179 && _destY==66)
    //  printf("angle =%i,%i->%i,%i   angle=%f\n", current->location->posX, current->location->posY, _destX, _destY, angle);
    if (angle>=0*M_PI/8 && angle < 1*M_PI/8) {
      testNext(current,  1,  0);
      testNext(current,  1, -1);
      testNext(current,  1,  1);
      testNext(current,  0, -1);
      testNext(current,  0,  1);
      testNext(current, -1, -1);
      testNext(current, -1,  1);
      testNext(current, -1,  0);
    } else if (angle>=1*M_PI/8 && angle < 2*M_PI/8) {
      testNext(current,  1, -1);
      testNext(current,  1,  0);
      testNext(current,  0, -1);
      testNext(current,  1,  1);
      testNext(current, -1, -1);
      testNext(current,  0,  1);
      testNext(current, -1,  0);
      testNext(current, -1,  1);
    } else if (angle>=2*M_PI/8 && angle < 3*M_PI/8) {
      testNext(current,  1, -1);
      testNext(current,  0, -1);
      testNext(current,  1,  0);
      testNext(current, -1, -1);
      testNext(current,  1,  1);
      testNext(current, -1,  0);
      testNext(current,  0,  1);
      testNext(current, -1,  1);
    } else if (angle>=3*M_PI/8 && angle < 4*M_PI/8) {
      testNext(current,  0, -1);
      testNext(current,  1, -1);
      testNext(current, -1, -1);
      testNext(current,  1,  0);
      testNext(current, -1,  0);
      testNext(current,  1,  1);
      testNext(current, -1,  1);
      testNext(current,  0,  1);
    } else if (angle>=4*M_PI/8 && angle < 5*M_PI/8) {
      testNext(current,  0, -1);
      testNext(current, -1, -1);
      testNext(current,  1, -1);
      testNext(current, -1,  0);
      testNext(current,  1,  0);
      testNext(current, -1,  1);
      testNext(current,  1,  1);
      testNext(current,  0,  1);
    } else if (angle>=5*M_PI/8 && angle < 6*M_PI/8) {
      testNext(current, -1, -1);
      testNext(current,  0, -1);
      testNext(current, -1,  0);
      testNext(current,  1, -1);
      testNext(current, -1,  1);
      testNext(current,  1,  0);
      testNext(current,  0,  1);
      testNext(current,  1,  1);
    } else if (angle>=6*M_PI/8 && angle < 7*M_PI/8) {
      testNext(current, -1, -1);
      testNext(current, -1,  0);
      testNext(current,  0, -1);
      testNext(current, -1,  1);
      testNext(current,  1, -1);
      testNext(current,  0,  1);
      testNext(current,  1,  0);
      testNext(current,  1,  1);
    } else if (angle>=7*M_PI/8 && angle <= 8*M_PI/8) {
      testNext(current, -1,  0);
      testNext(current, -1, -1);
      testNext(current, -1,  1);
      testNext(current,  0, -1);
      testNext(current,  0,  1);
      testNext(current,  1, -1);
      testNext(current,  1,  1);
      testNext(current,  1,  0);
    } else     if (angle<=0*M_PI/8 && angle > -1*M_PI/8) {
      testNext(current,  1,  0);
      testNext(current,  1,  1);
      testNext(current,  1, -1);
      testNext(current,  0,  1);
      testNext(current,  0, -1);
      testNext(current, -1,  1);
      testNext(current, -1, -1);
      testNext(current, -1,  0);
    } else if (angle<=-1*M_PI/8 && angle > -2*M_PI/8) {
      testNext(current,  1,  1);
      testNext(current,  1,  0);
      testNext(current,  0,  1);
      testNext(current,  1, -1);
      testNext(current, -1,  1);
      testNext(current,  0, -1);
      testNext(current, -1,  0);
      testNext(current, -1, -1);
    } else if (angle<=-2*M_PI/8 && angle > -3*M_PI/8) {
      testNext(current,  1,  1);
      testNext(current,  0,  1);
      testNext(current,  1,  0);
      testNext(current, -1,  1);
      testNext(current,  1, -1);
      testNext(current, -1,  0);
      testNext(current,  0, -1);
      testNext(current, -1, -1);
    } else if (angle<=-3*M_PI/8 && angle > -4*M_PI/8) {
      testNext(current,  0,  1);
      testNext(current,  1,  1);
      testNext(current, -1,  1);
      testNext(current,  1,  0);
      testNext(current, -1,  0);
      testNext(current,  1, -1);
      testNext(current, -1, -1);
      testNext(current,  0, -1);
    } else if (angle<=-4*M_PI/8 && angle > -5*M_PI/8) {
      testNext(current,  0,  1);
      testNext(current, -1,  1);
      testNext(current,  1,  1);
      testNext(current, -1,  0);
      testNext(current,  1,  0);
      testNext(current, -1, -1);
      testNext(current,  1, -1);
      testNext(current,  0, -1);
    } else if (angle<=-5*M_PI/8 && angle > -6*M_PI/8) {
      testNext(current, -1,  1);
      testNext(current,  0,  1);
      testNext(current, -1,  0);
      testNext(current,  1,  1);
      testNext(current, -1, -1);
      testNext(current,  1,  0);
      testNext(current,  0, -1);
      testNext(current,  1, -1);
    } else if (angle<=-6*M_PI/8 && angle > -7*M_PI/8) {
      testNext(current, -1,  1);
      testNext(current, -1,  0);
      testNext(current,  0,  1);
      testNext(current, -1, -1);
      testNext(current,  1,  1);
      testNext(current,  0, -1);
      testNext(current,  1,  0);
      testNext(current,  1, -1);
    } else {
      testNext(current, -1,  0);
      testNext(current, -1,  1);
      testNext(current, -1, -1);
      testNext(current,  0,  1);
      testNext(current,  0, -1);
      testNext(current,  1,  1);
      testNext(current,  1, -1);
      testNext(current,  1,  0);
    }
    if (current->h>2*initialHeuristic+200)
      stop =true;
    if (found != NULL && current->h>found->h*1.05)
      stop =true;
  }
  //printf("almost astarSearchdone %p %p %p\n", openList, closedList, foundPath);fflush(stdout);
  //if (openList!=NULL) printf("  openList %d\n", list_count(openList));fflush(stdout);
  //if (closedList!=NULL) printf("  closedList %d\n", list_count(closedList));fflush(stdout);
  //if (foundPath!=NULL) printf("  foundPath %d\n", list_count(foundPath));fflush(stdout);
  if (found!=NULL) {
    if (foundPath!=NULL) {
      list_destroy_nodes(foundPath);
      list_destroy(foundPath);
    }
    foundPath          = list_create(LISTCOUNT_T_MAX);
    list_t *foundPath2 = list_create(LISTCOUNT_T_MAX);

    // get back along the found path to build the list describing the solution
    while(found !=startETile) {
      tile *t = found->location;
      list_prepend(foundPath, lnode_create(t));
      list_prepend(foundPath2, lnode_create(t));
      found = found->parent;
    }
    clean();
    sprintf(errorMessage, "  A*: No error ! found a path with %i tiles\n", list_count(foundPath2));
    return foundPath2;
  } else {
    clean();
    sprintf(errorMessage, "  A*: A* didn't found a path\n");
    return NULL;
  }
}// astarSearch() --------------------------------------------------------------------------------


/**
 * perform memory cleaning before finishing
 */
void clean() {
  /*list_destroy_nodes(openList);
  list_destroy(openList);
  list_destroy_nodes(closedList);
  list_destroy(closedList);*/
  destX = 0;
  destY = 0;
  found = NULL;

  if(SDL_mutexV(astarmut)==-1){
    fprintf(stderr, "Couldn't unlock astar mutex\n");
    exit(-1);
  }
}// clean() --------------------------------------------------------------------------------


/**
 * test the next tile according to the movement specified by dx, dy
 */
void testNext(etile *current, const int dx, const int dy) {
  //if (movingUnit!=NULL && current->location->posX+dx==164 && current->location->posY+dy==64)
  //  printf("\ntestNext(%i    %i, %i -> %i, %i) \n", movingUnit->type, current->location->posX, current->location->posY, current->location->posX+dx, current->location->posY+dy);

  tile *tileToExplore = current->location;
  if (tileToExplore->posX+dx>=0 && tileToExplore->posX+dx<MapWidth && tileToExplore->posY+dy>=0 && tileToExplore->posY+dy<MapHeight) {
    tile *neighbour = &map[tileToExplore->posX+dx+MapWidth*(tileToExplore->posY+dy)];

    etile *newETile = NULL;
    lnode_t *nodeetile = NULL;

    // we already get there from a faster path
    if (list_find(closedList, neighbour, *etileCompare)) {
      //printf(" -> dead end\n");
      return;

      // we already planned to get there but maybe the new path is faster?
    } else if ((nodeetile = list_find(openList, neighbour, *etileCompare))) {
      newETile = (etile *) nodeetile->list_data;

      //printf(" -> already in my wish list\n");
      int difficulty = computeDifficulty(tileToExplore, newETile->location, movingUnit);

      // it will never happen !
      if (difficulty>movingUnit->maxMovement)
        return;

      // we found a shortest path to this known place
      if (current->f+difficulty<newETile->f) {
        newETile->f = current->f+difficulty;
        newETile->h = newETile->f+newETile->g;
        newETile->parent = current;
      }

      // we found an equal path to this known place
      if (current->f+difficulty == newETile->f) {
        int dice = 1+rand()%6;
        if (dice>3) {
          newETile->f = current->f+difficulty;
          newETile->h = newETile->f+newETile->g;
          newETile->parent = current;
        }
      }

      return;
    } else if (movingUnit->horses!=-101 && neighbour->unitList!=NULL && list_count(neighbour->unitList)>0 &&
               isEnnemyIn(neighbour->unitList) && neighbour!=&map[destX+MapWidth*(destY)] &&
               (neighbour->lastExplored>=0 || gamemode!=PLAY)) {
      return;
    } else if (movingUnit->horses!=-101 && !isNaval(movingUnit) &&
               !isFreeBoarding(neighbour, movingUnit) &&
               (neighbour->elevation<SEALEVEL) &&
               (neighbour->lastExplored>=0 || gamemode!=PLAY)) {
      return;
    } else if (movingUnit->horses!=-101 && !isNaval(movingUnit) &&
              (neighbour->elevation>=HIGHMOUNTAIN) &&
              (neighbour->lastExplored>=0 || gamemode!=PLAY)) {
      return;
    } else if (movingUnit->horses!=-101 && isNaval(movingUnit)==1 &&
               neighbour->elevation>=SEALEVEL && neighbour->irrigation<BIG_RIVER &&
               (neighbour->lastExplored>=0  || gamemode!=PLAY) && neighbour->construction==NULL) {
      return;
    } else if (movingUnit->horses!=-101 && isNaval(movingUnit)==2 && neighbour->elevation>=SEALEVEL &&
               (neighbour->lastExplored>=0 || gamemode!=PLAY) && neighbour->construction==NULL) {
      return;
    } else {
      int difficulty = computeDifficulty(tileToExplore, neighbour, movingUnit);

      //if (movingUnit!=NULL && current->location->posX+dx==164 && current->location->posY+dy==64)
      //  printf("  diff=%i / %i \n", difficulty, movingUnit->maxMovement);

      // it will never happen !
      if (difficulty>movingUnit->maxMovement)
        return;

      newETile            = (etile *)malloc(sizeof(etile));
      newETile->location  = neighbour;
      newETile->f         = current->f+difficulty;
      newETile->g         = (int)heuristic(tileToExplore->posX+dx, tileToExplore->posY+dy);
      newETile->h         = newETile->f+newETile->g;
      newETile->parent    = current;
      list_append(openList, lnode_create(newETile));
      //printf(" -> added diff=%i with fgh[%i, %i, %i]  dx=%i dy=%i\n", difficulty, newETile->f, newETile->g, newETile->h, destX, destY);

      if (neighbour->posX==destX && neighbour->posY==destY) {
        if (found == NULL || found->h > newETile->h)
          found = newETile;
      }

      return;
    }
  }
  return;
}// testNext() --------------------------------------------------------------------------------


/**
 * return the next tile to test in the a* algorithm
 */
lnode_t *getLowestOpenETile() {
  //printf("getLowestOpenETile()");

  lnode_t *etileelem = list_first(openList);
  lnode_t *lowest = etileelem;
  etile *firstetile = (etile *)lowest->list_data;
  int min = firstetile->h;
  while (etileelem != &openList->list_nilnode) {
    etile *nextetile = (etile *)etileelem->list_data;
    if (nextetile->h<min) {
      min = nextetile->h;
      lowest = etileelem;
    }
    etileelem = etileelem->list_next;
  }
  //etile *lowestetile = (etile *)lowest->list_data;
  //printf(" -> %i,%i  (val=%i)\n", lowestetile->location->posX, lowestetile->location->posY, min);
  return lowest;
}// getLowestOpenETile() --------------------------------------------------------------------------------
