/*
 *  binarytree.cpp
 *  VisBoardGame
 *
 *  Created by fredericVernier on 2/6/07.
 *  Copyright 2007 Frederic Vernier. All rights reserved.
 *
 */

#include "binarytree.h"
#pragma warning ( disable : 4996 )

////////////////////////////
// node related functions //
////////////////////////////

/**
 * create a node for binary tree
 */
btnode_t *btnode_create(void * data) {
  btnode_t *node = (btnode_t *)malloc(sizeof(*node));
  node->elem     = data;
  node->left     = NULL;
  node->right    = NULL;
  node->father   = NULL;
  node->metadata = NULL;
  return node;
}// btnode_create()


/**
 * create a node for binary tree and its meta info
 */
btnode_t *btnode_createWithMeta(void * data, void* meta) {
  btnode_t *node = (btnode_t *)malloc(sizeof(*node));
  node->elem     = data;
  node->left     = NULL;
  node->right    = NULL;
  node->father   = NULL;
  node->metadata = meta;
  return node;
}// btnode_createWithMeta()


/**
 * re-init an existing node
 */
btnode_t *btnode_init(btnode_t *node, void *data, void*meta) {
  btnode_t *initnode = node;
  if (initnode==NULL) {
    initnode = (btnode_t *)malloc(sizeof(*node));
  }
  initnode->elem = data;
  initnode->metadata = meta;

  return node;
}// btnode_init()


/**
 * destroy an existing node
 */
void btnode_destroy(btnode_t* node ) {
  if (node!=NULL) {
    if (node->left!=NULL)
      btnode_destroy(node->left);
    if (node->right!=NULL)
      btnode_destroy(node->right);
    free(node);
  }
}// btnode_destroy()


/**
 * set the given data to a node
 */
void  btnode_put(btnode_t* node , void *data) {
  node->elem =data;
}// btnode_put()


/**
 * retrieve the attached data to a node
 */
void *btnode_get(btnode_t* node) {
  return node->elem;
}// btnode_get()


/**
 * retrieve the meta info of a node
 */

void  btnode_putMeta(btnode_t* node , void *meta) {
  node->metadata = meta;
}// btnode_putMeta()


/**
 * set the given meta info to a node
 */
void *btnode_getMeta(btnode_t* node) {
  return node->metadata;
}// btnode_getMeta()


int btnode_is_in_a_binarytree(btnode_t* node) {
  return node->father!=NULL;
}// btnode_is_in_a_binarytree()


///////////////////////////////////////
//  data structure related functions //
///////////////////////////////////////
/**
 * init a new or old data structure with no nodes
 */
binarytree_t *binarytree_init(binarytree_t* tree, unsigned long maxsize) {
  binarytree_t *tree_init = tree;
  if (tree_init==NULL)
    tree_init = (binarytree_t *)malloc(sizeof(*tree));
  tree_init->nodecount = 0;
  tree_init->maxcount = maxsize;
  tree_init->root = NULL;
  return tree_init;
}// binarytree_init()


/**
 * init a new data structure with no nodes
 */
binarytree_t *binarytree_create(unsigned long maxsize) {
  binarytree_t *tree_init = (binarytree_t *)malloc(sizeof(*tree_init));
  tree_init->nodecount = 0;
  tree_init->maxcount = maxsize;
  tree_init->root = NULL;
  return tree_init;
}// binarytree_create()


/**
 * destroy a structure
 */
void   binarytree_destroy(binarytree_t *tree) {
  free(tree);
}


void   binarytree_destroy_nodes(binarytree_t *) {
}


unsigned long binarytree_count(binarytree_t *tree) {
  return tree->nodecount;
}


/**
 * return if the data structure is empty (no nodes !)
 */
int binarytree_isempty(binarytree_t *tree) {
  return tree->nodecount==0;
}// binarytree_isempty()


/**
 * return if the data structure is full
 */
int binarytree_isfull(binarytree_t *tree) {
  return tree->nodecount==tree->maxcount;
}// binarytree_isfull


/**
 * return the root of the data structure
 */
btnode_t * binarytree_root(binarytree_t *tree) {
  return tree->root;
}// binarytree_root()


/**
 * recursively search a given node starting from the given root inside the structure
 */
int  binarytree_contains_r(btnode_t *root, btnode_t *node) {
  if (node==root->left || node==root->right)
    return true;
  else {
    int isleft  = binarytree_contains_r(root->left, node);
    if (isleft)
      return true;
    int isright = binarytree_contains_r(root->right, node);
    return isright;
  }
}// binarytree_contains_r()


/**
 * return true if the given data structure contains a given node
 */
int  binarytree_contains(binarytree_t *tree, btnode_t *node) {
  return binarytree_contains_r(tree->root, node);
}// binarytree_contains()


/**
 * insert a new node to the left of an existing node
 */
btnode_t *binarytree_ins_left(binarytree_t *tree, btnode_t *node_loc, btnode_t *node_new) {
  if (tree==NULL)
    return NULL;
  if (node_loc==NULL && tree->root==NULL)
    tree->root=node_new;
  else if (node_loc==NULL && tree->selected!=NULL) {
    tree->selected->left = node_new;
    node_new->father = tree->selected;
  } else if (node_loc->left==NULL) {
    node_loc->left = node_new;
    node_new->father = node_loc;
  } else {
    printf("ERROR binarytree_ins_left \n");
    return NULL;
  }
  tree->selected = node_new;
  node_new->tree = tree;
  return tree->selected;
}// binarytree_ins_left()


/**
 * insert a new node to the right of an existing node
 */
btnode_t *binarytree_ins_right(binarytree_t *tree, btnode_t *node_loc, btnode_t *node_new) {
  if (tree==NULL)
    return NULL;
  if (node_loc==NULL && tree->root==NULL)
    tree->root=node_new;
  else if (node_loc==NULL && tree->selected!=NULL) {
    tree->selected->left = node_new;
    node_new->father = tree->selected;
  } else if (node_loc->right==NULL) {
    node_loc->right = node_new;
    node_new->father = node_loc;
  } else {
    printf("ERROR binarytree_ins_right \n");
    return NULL;
  }
  tree->selected = node_new;
  node_new->tree = tree;
  return tree->selected;
}// binarytree_ins_right()
