/*
 *  binarytree.h
 *  VisBoardGame
 *
 *  Created by fredericVernier on 2/6/07.
 *  Copyright 2007 Frederic Vernier. All rights reserved.
 *
 */

#ifndef BINARYTREE_H
#define BINARYTREE_H

#ifdef __cplusplus
extern "C" {
#endif

#include <stdio.h>
#include <stdlib.h>

typedef struct btnode_t     btnode_t;
typedef struct binarytree_t binarytree_t;

struct btnode_t {
   void                *elem;
   void                *metadata;
   struct btnode_t     *left;
   struct btnode_t     *right;
   struct btnode_t     *father;
   struct binarytree_t *tree;
};

struct binarytree_t {
    btnode_t *root;
    btnode_t *selected;
    unsigned long nodecount;
    unsigned long maxcount;
};

// node related functions
btnode_t     *btnode_createWithMeta(void *, void*);
btnode_t     *btnode_create(void *);
btnode_t     *btnode_init(btnode_t *, void *, void*);
void          btnode_destroy(btnode_t *);
void          btnode_put(btnode_t *, void *);
void          btnode_setMeta(btnode_t *, void *);
void         *btnode_getMeta(btnode_t *);
int           btnode_is_in_a_binarytree(btnode_t *);

//  data structure related functions
binarytree_t *binarytree_init(binarytree_t *, unsigned long);
binarytree_t *binarytree_create(unsigned long);
void          binarytree_destroy(binarytree_t *);
void          binarytree_destroy_nodes(binarytree_t *);
btnode_t     *binarytree_root(binarytree_t *tree);

unsigned long binarytree_count(binarytree_t *);
int           binarytree_isempty(binarytree_t *);
int           binarytree_isfull(binarytree_t *);
int           binarytree_contains(binarytree_t *, btnode_t *);

btnode_t *binarytree_ins_left(binarytree_t *, btnode_t *node_loc, btnode_t *node_new);
btnode_t *binarytree_ins_right(binarytree_t *, btnode_t *node_loc, btnode_t *node_new);

#ifdef __cplusplus
}
#endif

#endif
