/*
 *  cityview.h
 *  VisBoardGame
 *
 *  Created by fredericVernier on 2/15/07.
 *  Copyright 2007 Frederic Vernier. All rights reserved.
 *
 */

#ifndef CITYVIEW_H
#define CITYVIEW_H

#include <limits.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "unitview.h"
#include "typenewcol.h"
#include "extragraphics.h"
#include <kazlib/list.h>
#include "newcol.h"
#include "SDL.h"
#include "popup.h"

#ifdef __cplusplus
extern "C" {
#endif

// what can be specifically dragged in this view
#define NEWCOMER  20
#define OUTWORKER 21
#define INWORKER  22

#define BUILDING_BAR_HEIGHT     148
#define RAW_GOODS_BAR_HEIGHT     32
#define REFINED_GOOGD_BAR_HEIGHT 32

#define GRIDSPACE 32

extern       int         cvDrag;
extern       SDL_Rect   *cityViewRect;
extern       int         lastcvmouseX;
extern       int         lastcvmouseY;
extern       int         cvneedUpdate;
extern       int         cvOutX;
extern       int         cvOutY;
extern       city       *selectedCity;

// city view related functions
void         displayCityView(city*);
int          isInCityView(int x, int y);
void         openCityView(city *c);
void         mouseDropCV(city *c, unitview *uv, int x, int y);
void         mouseDragCV(city *c, unitview *uv, int x, int y);
void         mouseDraggedCV (city *, int , int );
void         mousePressedCV (city *, int , int , int, int);
void         mouseReleasedCV(city *, int , int , int, int);
void         mouseMovedCV(city *c, int cvmouseX, int cvmouseY);
  
// city related functions
//void         setOutsideJob(city *c, unit *u, int dx, int dy);
void         updateCityNumbers(city *c);
int          computeOutsideProduction(city *c, unit *u, int g, int x, int y);
int          computeValuableProduction(city *c, unit *u, int g, int x, int y);
int          computeInsideProduction(city *c, unit *u, building *b);
float        getBaseProduction(tile *t, int g);
building    *isBuildingExist(city*, int);
void         updateCityProduction(city *c);
int          findBestJob(unit *u, city *c);
city        *findCity(char *name);
int          findBestNextConstruction(city *c);
void         updateCityConstruction(city *c);
void         updateCityStock(city *c);
void         updateCityNumbers(city *c);
int          getSpecialistForGood(int goodtype);
int          getGoodForBonus(int bt);
int          getStatusForGood(int goodtype);
int          getRangeForSize(int);
char        *getBuildingText(building*);
char        *getBuildingTextFromType(int type);
int          getMaxQuantity(city *c, int goodType);
city        *createCity (int, char*, tile*, player*);
city        *createLostCityRumor (tile* location);
building    *createBuilding(int type, city *c);
int          isGoodAvailable(int good, city *c);
void         changeCityOwner(city *c, player *newOwner);
int          getCityLocationScore(int x, int y);

// callbacks
void doWorkOutside(list_t *ul, char *order);
void doEquipMuskets(list_t *ul, char *order);
void doEquipHorses(list_t *ul, char *order);
void doEquipTools(list_t *ul, char *order);
void doChangeConstruction(list_t *ul, char *order);
void doUnequipTools(list_t *ul, char *order);
void doUnequipHorses(list_t *ul, char *order);
void doUnequipMuskets(list_t *ul, char *order);
void doFindBestJob (list_t *ul, char *order);



int compareBuildingType(const void *a, const void *b);
int compareCityName(const void *a, const void *b);

#ifdef __cplusplus
}
#endif
#endif
