/*
 *  extragraphics.c
 *  SDLApp
 *
 *  Created by fredericVernier on 1/25/07.
 *  Copyright 2007 Frederic Vernier. All rights reserved.
 *
 */

#include "extragraphics.h"
#pragma warning ( disable : 4996 )
#include "IMG_savepng.h"
#include "png.h"

SDL_Surface *textures[LAST_GROUND-FIRST_GROUND+1];
SDL_Surface *lastLoadedImage     = NULL;
char        *lastImageNameLoaded = NULL;

/**
 * display info about the surface in stdout
 */
void SDL_SurfaceInfo(const char * name, const SDL_Surface *thing) {
  printf(" surface %s: w:%d h:%d bpp:%d %x %x %x %x \n", name, thing->w, thing->h, thing->format->BitsPerPixel, thing->format->Rmask, thing->format->Gmask, thing->format->Bmask, thing->format->Amask);
}// SDL_SurfaceInfo()--------------------------------------------------------------------


/**
 * return if the given position in the given surface is an opaque pixel (alpha>0) or not
 */
int isOpaquePixel (const SDL_Surface *src, const int x, const int y){
  if (x<0 || x>=src->w || y<0||y>=src->h)
    return 0;
  Uint32 *pixelsrc = (Uint32 *) src->pixels + (y) * src->w + (x);
  Uint32 Ashiftsrc = src->format->Ashift;
  Uint32 Amasksrc  = src->format->Amask;
  unsigned char a1 = (*pixelsrc & Amasksrc) >> Ashiftsrc;
  //printf("isOpaquePixel(%i, %i) return %i because alpha=%i \n", x, y, (a1!=0), a1);
  return a1!=0;
}// isOpaquePixel()--------------------------------------------------------------------


/**
 * create a surface bigger than the given one and make the shadow of it in the bigger image
 */
SDL_Surface *createShadow(SDL_Surface *src, const unsigned char maxTrans, const Uint32 color){
  SDL_Surface *dst = SDL_CreateRGBSurface(SDL_SWSURFACE|SDL_SRCALPHA, src->w+SHADOW_SIZE*2, src->h+SHADOW_SIZE*2,
                                          32, 0xFF000000, 0x00FF0000, 0x0000FF00, 0x000000FF);
  return updateShadow(src, dst, maxTrans, color);
}// createShadow()


/**
 * some picture are bigger than they need to be, a(some) full line of pixel is/are transparent on the border side
 */
SDL_Surface *removeTransBorder(SDL_Surface *src){
  //printf("remove transparent colors for %i*%i\n", src->w, src->h);
  if (src->format->Amask==0)
    return src;

  int i, j;
  int mini = src->w/2, maxi = src->w/2;
  int minj = src->h/2, maxj = src->h/2;

  for (j=0; j<src->h; j++) {
    for (i=0; i<src->w; i++) {
      Uint32 *pixelsrc = (Uint32 *) src->pixels + (j) * src->w + (i);
      if ((*pixelsrc&src->format->Amask)>0) {
        if (i<mini)
          mini=i;
        if (i>maxi)
          maxi=i;
        if (j<minj)
          minj=j;
        if (j>maxj)
          maxj=j;
        //printf("1");
      }
      //else printf("0");
    }
    //printf("\n");
  }
  //printf("  found  %i--%i   %i--%i\n", mini, maxi, minj, maxj);

  SDL_Surface *dst = SDL_CreateRGBSurface(SDL_SWSURFACE ,maxi-mini+1, maxj-minj+1, 32, src->format->Rmask, src->format->Gmask , src->format->Bmask, src->format->Amask);

  if (src->w==(maxi-mini+1) && src->h==(maxj-minj+1))
    return src;
  else {
    //printf("  removed %i*%i\n\n", src->w-(maxi-mini+1), src->h-(maxj-minj+1));
    SDL_LockSurface(dst);
    for (j=minj; j<=maxj; j++) {
      for (i=mini; i<=maxi; i++) {
        Uint32 *pixelsrc = (Uint32 *) src->pixels + (j) * src->w + (i);
        Uint32 *pixeldst = (Uint32 *) dst->pixels + (j-minj) * dst->w + (i-mini);
        *pixeldst = *pixelsrc;
        }
      }

    SDL_UnlockSurface(dst);

    SDL_FreeSurface(src);
    return dst;
  }
}// removeTransBorder()


/**
 * render text in black first then is white shifted to make a shadow effect
 */
SDL_Surface *renderTextShadowed_Blended(TTF_Font *font, const char *text, SDL_Color fg, SDL_Color bg){
  SDL_Rect *rect = (SDL_Rect *)malloc(sizeof(SDL_Rect));
  int w, h;
  TTF_SizeText(font, text, &w, &h);
  SDL_Surface *tc = SDL_CreateRGBSurface(SDL_SWSURFACE|SDL_SRCALPHA, w+1, h+1, 32, 0x000000FF, 0x0000FF00, 0x00FF0000, 0xFF000000);
  SDL_FillRect(tc, NULL, SDL_MapRGBA(tc->format, 0, 0, 0, 0));
  SDL_Surface *imgText = TTF_RenderText_Blended(font, text, bg);
  rect->x = 1;
  rect->y = 1;
  SDL_BlitSurfaceRGBAMin(imgText, NULL, tc, rect);
  SDL_FreeSurface(imgText);
  imgText = TTF_RenderText_Blended(font, text, fg);
  rect->x = 0;
  rect->y = 0;
  SDL_BlitSurfaceRGBAMul(imgText, NULL, tc, rect);
  SDL_FreeSurface(imgText);
  free(rect);
  return tc;
}// renderTextShadowed_Blended()


/**
 * load and convert a loaded bitmap to a format compatible with the screen
 */
SDL_Surface *IMG_LoadAndFormatAlpha(const char* imageName){
  //printf("loading %s \n", imageName);
  initcpt++;
  SDL_Surface *tmp  = (SDL_Surface *)IMG_Load(imageName);
  //printf("loaded %p \n", tmp);
  if (tmp!=NULL) {
    //int size = tmp->w*tmp->h;
    //if (strcmp(imageName, "images/unitsmix/14/gun6.png")==0) {
    if (strstr(imageName, "unitsmix")!=0 || strstr(imageName, "Building")!=0) {
      tmp = removeTransBorder(tmp);
    if (strstr(imageName, ".tga")!=NULL) {
      //if(tmp->w*tmp->h<size)
      char *imageName2 = (char *)malloc(256);
      sprintf(imageName2, "%s", imageName);
      char *pos = strstr(imageName2, ".tga");
      strncpy (pos, ".png",4);
      //printf("   writing %s \n", imageName2);
      //IMG_SavePNG(imageName2, tmp, Z_NO_COMPRESSION);
      free(imageName2);
    }

    //} else {
    //  printf("  no need to remove transparent colors for %s\n", imageName);
    }
    SDL_Surface *tmp2 = SDL_DisplayFormatAlpha(tmp);
    SDL_FreeSurface(tmp);
    lastLoadedImage = tmp2;
    if (lastLoadedImage->format==0 || lastLoadedImage->w>12000 || lastLoadedImage->h>12000){
      printf("PROBLEM");fflush(stdout);
    }

    if (lastImageNameLoaded!=NULL)
      free(lastImageNameLoaded);
    lastImageNameLoaded = (char *)malloc(255);
    sprintf(lastImageNameLoaded, "%s", imageName);

   // printf("loaded %p = %s = %i*%i\n", lastLoadedImage, imageName, lastLoadedImage->w, lastLoadedImage->h);fflush(stdout);
    return tmp2;
  } else
    return NULL;
}// IMG_LoadAndFormatAlpha()


/**
 * load and convert a loaded bitmap to a format compatible with the screen
 */
SDL_Surface *IMG_LoadAndFormatNoAlpha(const char* imageName){
  //printf("loading %s ", imageName);
  initcpt++;
  SDL_Surface *tmp  = (SDL_Surface *)IMG_Load(imageName);
  if (tmp!=NULL) {
    SDL_Surface *tmp2 = SDL_DisplayFormat(tmp);
    SDL_FreeSurface(tmp);
    //tmp2->flags = tmp2->flags & ~SDL_SRCALPHA;
    //printf("loaded no alpha %8X mask=%8X   \n", tmp2->flags, SDL_SRCALPHA);
    lastLoadedImage = tmp2;
    if (lastLoadedImage->format==0 || lastLoadedImage->w>12000 || lastLoadedImage->h>12000){
      printf("PROBLEM");fflush(stdout);
    }
    //printf("loaded %p = %s = %i*%i\n", lastLoadedImage, imageName, lastLoadedImage->w, lastLoadedImage->h);fflush(stdout);
    return tmp2;
  } else
    return NULL;
}// IMG_LoadAndFormatNoAlpha()


/**
 * udate shadow of a surface
 */
SDL_Surface *updateShadow(SDL_Surface *src, SDL_Surface *dst, const unsigned char maxTrans, const Uint32 color){
  int i, j, k, l;

  /*Uint32 Rshiftdst = dst->format->Rshift;
  Uint32 Gshiftdst = dst->format->Gshift;
  Uint32 Bshiftdst = dst->format->Bshift;

  Uint32 Rmaskdst  = dst->format->Rmask;
  Uint32 Gmaskdst  = dst->format->Gmask;
  Uint32 Bmaskdst  = dst->format->Bmask;*/
  Uint32 Ashiftdst = dst->format->Ashift;
  //Uint32 Amaskdst  = dst->format->Amask;
  Uint32 *pdst = (Uint32 *)dst->pixels;
  //Uint32 *psrc = (Uint32 *)src->pixels;
  SDL_LockSurface(dst);

  for (j=0; j<dst->h; j++) {
    for (i=0; i<dst->w; i++) {
      pdst[(i)+(j)*dst->w] = 0;
    }
  }

  for (k=-SHADOW_SIZE; k<=SHADOW_SIZE; k++)
    for (l=-SHADOW_SIZE; l<=SHADOW_SIZE; l++)
      for (j=0; j<src->h; j++) {
        for (i=0; i<src->w; i++) {
          Uint32 *pixelsrc = (Uint32 *) src->pixels + (j) * src->w + (i);
          if (src->format->Ashift==0)
            pdst[(i+SHADOW_SIZE+k)+(j+SHADOW_SIZE+l)*dst->w] += 255;
          else {
            unsigned char l1 = ((*pixelsrc) & src->format->Amask) >> src->format->Ashift;
            for (k=-SHADOW_SIZE; k<=SHADOW_SIZE; k++)
              for (l=-SHADOW_SIZE; l<=SHADOW_SIZE; l++)
                pdst[(i+SHADOW_SIZE+k)+(j+SHADOW_SIZE+l)*dst->w] += l1;
          }
    }
  }

  /*for (j=0; j<21; j++) {
    for (i=0; i<21; i++) {
      Uint32 l0 = pdst[(i)+(j)*dst->w];
      printf ("%4x|%2x ", l0, l);
    }
    printf("\n");
  }   */

  for (j=0; j<dst->h; j++) {
    for (i=0; i<dst->w; i++) {
      Uint32 l0 = pdst[(i)+(j)*dst->w];
      unsigned char l = 0;
      if (l0>255*4*SHADOW_SIZE*SHADOW_SIZE)
        l =255;
      else
        l = (unsigned char)(l0/(4*SHADOW_SIZE*SHADOW_SIZE));
      //l = l*maxTrans/255;
      if(l>maxTrans)
        l=maxTrans;

      pdst[(i)+(j)*dst->w] = color&0xFFFFFF00<<16 | l<<Ashiftdst;
    }
  }

  /*for (j=0; j<21; j++) {
    for (i=0; i<21; i++) {
      Uint32 l0 = pdst[(i)+(j)*dst->w]&0x000000FF;
      printf ("%2x ", l0);
    }
    printf("\n");
  } */

  SDL_UnlockSurface(dst);
  return dst;
}// updateShadow()


/**
 * blit a transparent surface with its transparency (copy)
 */
void SDL_BlitSurfaceRGBAMin(SDL_Surface *src, SDL_Rect *rectsrc, SDL_Surface *dst, SDL_Rect *rectdst){
  SDL_LockSurface(dst);
  SDL_LockSurface(src);
  int toErase = 0;
  Uint32 Rshiftdst = dst->format->Rshift;
  Uint32 Gshiftdst = dst->format->Gshift;
  Uint32 Bshiftdst = dst->format->Bshift;
  Uint32 Ashiftdst = dst->format->Ashift;
  /*  Uint32 Rmaskdst  = dst->format->Rmask;
  Uint32 Gmaskdst  = dst->format->Gmask;
  Uint32 Bmaskdst  = dst->format->Bmask;
  Uint32 Amaskdst  = dst->format->Amask;*/
  Uint32 Rshiftsrc = src->format->Rshift;
  Uint32 Gshiftsrc = src->format->Gshift;
  Uint32 Bshiftsrc = src->format->Bshift;
  Uint32 Ashiftsrc = src->format->Ashift;
  Uint32 Rmasksrc  = src->format->Rmask;
  Uint32 Gmasksrc  = src->format->Gmask;
  Uint32 Bmasksrc  = src->format->Bmask;
  Uint32 Amasksrc  = src->format->Amask;

  SDL_Rect* rectsrc2;
  if (rectsrc==NULL) {
    rectsrc2 = (SDL_Rect*)malloc(sizeof(SDL_Rect));
    rectsrc2->x = 0;
    rectsrc2->y = 0;
    rectsrc2->w = src->w;
    rectsrc2->h = src->h;
    toErase = 1;
  } else
    rectsrc2 = rectsrc;

  if (rectdst==NULL)
    rectdst = rectsrc2;
  rectdst->w = rectsrc2->w;
  rectdst->h = rectsrc2->h;

  int i, j;
  for (j=0; j<rectsrc2->h; j++) {
    for (i=0; i<rectsrc2->w; i++) {
      Uint32 *pixelsrc = (Uint32 *) src->pixels + (j+rectsrc2->y) * src->w + ((i)+rectsrc2->x);
      Uint32 *pixeldst = (Uint32 *) dst->pixels + (j+rectdst->y)  * dst->w + (i+rectdst->x);
      if (j+rectdst->y<0 || j+rectdst->y>=dst->h || i+rectdst->x<0 || i+rectdst->x>=dst->w)
        continue;
      unsigned char r1 = (*pixelsrc & Rmasksrc) >> Rshiftsrc;
      unsigned char g1 = (*pixelsrc & Gmasksrc) >> Gshiftsrc;
      unsigned char b1 = (*pixelsrc & Bmasksrc) >> Bshiftsrc;
      unsigned char a1 = (*pixelsrc & Amasksrc) >> Ashiftsrc;
      if (a1>0)
        *pixeldst = r1<<Rshiftdst | g1<< Gshiftdst | b1<<Bshiftdst | 255<< Ashiftdst;
    }
  }
  if(toErase)
    free(rectsrc2);
  SDL_UnlockSurface(dst);
  SDL_UnlockSurface(src);
}// SDL_BlitSurfaceRGBAMin()--------------------------------------------------------------------


/**
 * blit a transparent surface with its transparency (copy)
 */
void SDL_BlitSurfaceRGBAMax(SDL_Surface *src, SDL_Rect *rectsrc, SDL_Surface *dst, SDL_Rect *rectdst){
  SDL_LockSurface(dst);
  SDL_LockSurface(src);
  int toErase = 0;
  Uint32 Rshiftdst = dst->format->Rshift;
  Uint32 Gshiftdst = dst->format->Gshift;
  Uint32 Bshiftdst = dst->format->Bshift;
  Uint32 Ashiftdst = dst->format->Ashift;
  /*  Uint32 Rmaskdst  = dst->format->Rmask;
  Uint32 Gmaskdst  = dst->format->Gmask;
  Uint32 Bmaskdst  = dst->format->Bmask;
  Uint32 Amaskdst  = dst->format->Amask;*/
  Uint32 Rshiftsrc = src->format->Rshift;
  Uint32 Gshiftsrc = src->format->Gshift;
  Uint32 Bshiftsrc = src->format->Bshift;
  Uint32 Ashiftsrc = src->format->Ashift;
  Uint32 Rmasksrc  = src->format->Rmask;
  Uint32 Gmasksrc  = src->format->Gmask;
  Uint32 Bmasksrc  = src->format->Bmask;
  Uint32 Amasksrc  = src->format->Amask;

  SDL_Rect* rectsrc2;
  if (rectsrc==NULL) {
    rectsrc2 = (SDL_Rect*)malloc(sizeof(SDL_Rect));
    rectsrc2->x = 0;
    rectsrc2->y = 0;
    rectsrc2->w = src->w;
    rectsrc2->h = src->h;
    toErase = 1;
  } else
    rectsrc2 = rectsrc;

  if (rectdst==NULL)
    rectdst = rectsrc2;
  rectdst->w = rectsrc2->w;
  rectdst->h = rectsrc2->h;

  int i, j;
  for (j=0; j<rectsrc2->h; j++) {
    for (i=0; i<rectsrc2->w; i++) {
      Uint32 *pixelsrc = (Uint32 *) src->pixels + (j+rectsrc2->y) * src->w + ((i)+rectsrc2->x);
      Uint32 *pixeldst = (Uint32 *) dst->pixels + (j+rectdst->y)  * dst->w + (i+rectdst->x);
      if (j+rectdst->y<0 || j+rectdst->y>=dst->h || i+rectdst->x<0 || i+rectdst->x>=dst->w)
        continue;
      unsigned char r1 = (*pixelsrc & Rmasksrc) >> Rshiftsrc;
      unsigned char g1 = (*pixelsrc & Gmasksrc) >> Gshiftsrc;
      unsigned char b1 = (*pixelsrc & Bmasksrc) >> Bshiftsrc;
      unsigned char a1 = (*pixelsrc & Amasksrc) >> Ashiftsrc;
      if (a1==255)
        *pixeldst = r1<<Rshiftdst | g1<< Gshiftdst | b1<<Bshiftdst | a1<< Ashiftdst;
    }
  }
  if(toErase)
    free(rectsrc2);
  SDL_UnlockSurface(dst);
  SDL_UnlockSurface(src);
}// SDL_BlitSurfaceRGBAMax()--------------------------------------------------------------------


/**
 * blit a transparent surface with its transparency (copy)
 */
void SDL_BlitSurfaceRGBAMul(SDL_Surface *src, SDL_Rect *rectsrc, SDL_Surface *dst, SDL_Rect *rectdst){
  SDL_LockSurface(dst);
  SDL_LockSurface(src);
  int toErase = 0;
  Uint32 Rshiftdst = dst->format->Rshift;
  Uint32 Gshiftdst = dst->format->Gshift;
  Uint32 Bshiftdst = dst->format->Bshift;
  Uint32 Ashiftdst = dst->format->Ashift;
    Uint32 Rmaskdst  = dst->format->Rmask;
  Uint32 Gmaskdst  = dst->format->Gmask;
  Uint32 Bmaskdst  = dst->format->Bmask;
  Uint32 Amaskdst  = dst->format->Amask;
  Uint32 Rshiftsrc = src->format->Rshift;
  Uint32 Gshiftsrc = src->format->Gshift;
  Uint32 Bshiftsrc = src->format->Bshift;
  Uint32 Ashiftsrc = src->format->Ashift;
  Uint32 Rmasksrc  = src->format->Rmask;
  Uint32 Gmasksrc  = src->format->Gmask;
  Uint32 Bmasksrc  = src->format->Bmask;
  Uint32 Amasksrc  = src->format->Amask;

  SDL_Rect* rectsrc2;
  if (rectsrc==NULL) {
    rectsrc2 = (SDL_Rect*)malloc(sizeof(SDL_Rect));
    rectsrc2->x = 0;
    rectsrc2->y = 0;
    rectsrc2->w = src->w;
    rectsrc2->h = src->h;
    toErase = 1;
  } else
    rectsrc2 = rectsrc;

  if (rectdst==NULL)
    rectdst = rectsrc2;
  rectdst->w = rectsrc2->w;
  rectdst->h = rectsrc2->h;

  int i, j;
  for (j=0; j<rectsrc2->h; j++) {
    for (i=0; i<rectsrc2->w; i++) {
      Uint32 *pixelsrc = (Uint32 *) src->pixels + (j+rectsrc2->y) * src->w + ((i)+rectsrc2->x);
      Uint32 *pixeldst = (Uint32 *) dst->pixels + (j+rectdst->y)  * dst->w + (i+rectdst->x);
      if (j+rectdst->y<0 || j+rectdst->y>=dst->h || i+rectdst->x<0 || i+rectdst->x>=dst->w)
        continue;
      unsigned char r1 = (*pixelsrc & Rmasksrc) >> Rshiftsrc;
      unsigned char g1 = (*pixelsrc & Gmasksrc) >> Gshiftsrc;
      unsigned char b1 = (*pixelsrc & Bmasksrc) >> Bshiftsrc;
      unsigned char a1 = (*pixelsrc & Amasksrc) >> Ashiftsrc;
      unsigned char r2 = (*pixeldst & Rmaskdst) >> Rshiftdst;
      unsigned char g2 = (*pixeldst & Gmaskdst) >> Gshiftdst;
      unsigned char b2 = (*pixeldst & Bmaskdst) >> Bshiftdst;
      unsigned char a2 = (*pixeldst & Amaskdst) >> Ashiftdst;

      if (a1!=0)
        *pixeldst = ((r1*a1/256)+(r2*(255-a1)/256))<<Rshiftdst | ((g1*a1/256)+(g2*(255-a1)/256))<< Gshiftdst | ((b1*a1/256)+(b2*(255-a1)/256))<<Bshiftdst | 255<< Ashiftdst;
      //*pixeldst = (r1*a1/256)<<Rshiftdst | (g1*a1/256)<< Gshiftdst | (b1*a1/256)<<Bshiftdst | 255<< Ashiftdst;
    }
  }
  if(toErase)
    free(rectsrc2);
  SDL_UnlockSurface(dst);
  SDL_UnlockSurface(src);
}// SDL_BlitSurfaceRGBAMul()--------------------------------------------------------------------


/**
 * blit a transparent surface with its transparency (copy)
 */
void SDL_BlitSurfaceRGBA(SDL_Surface *src, SDL_Rect *rectsrc, SDL_Surface *dst, SDL_Rect *rectdst){
  SDL_LockSurface(dst);
  SDL_LockSurface(src);
  int toErase = 0;
  Uint32 Rshiftdst = dst->format->Rshift;
  Uint32 Gshiftdst = dst->format->Gshift;
  Uint32 Bshiftdst = dst->format->Bshift;
  Uint32 Ashiftdst = dst->format->Ashift;
  /*  Uint32 Rmaskdst  = dst->format->Rmask;
  Uint32 Gmaskdst  = dst->format->Gmask;
  Uint32 Bmaskdst  = dst->format->Bmask;
  Uint32 Amaskdst  = dst->format->Amask;*/
  Uint32 Rshiftsrc = src->format->Rshift;
  Uint32 Gshiftsrc = src->format->Gshift;
  Uint32 Bshiftsrc = src->format->Bshift;
  Uint32 Ashiftsrc = src->format->Ashift;
  Uint32 Rmasksrc  = src->format->Rmask;
  Uint32 Gmasksrc  = src->format->Gmask;
  Uint32 Bmasksrc  = src->format->Bmask;
  Uint32 Amasksrc  = src->format->Amask;

  SDL_Rect* rectsrc2;
  if (rectsrc==NULL) {
    rectsrc2 = (SDL_Rect*)malloc(sizeof(SDL_Rect));
    rectsrc2->x = 0;
    rectsrc2->y = 0;
    rectsrc2->w = src->w;
    rectsrc2->h = src->h;
    toErase = 1;
  } else
    rectsrc2 = rectsrc;

  if (rectdst==NULL)
    rectdst = rectsrc2;
  rectdst->w = rectsrc2->w;
  rectdst->h = rectsrc2->h;

  int i, j;
  for (j=0; j<rectsrc2->h; j++) {
    for (i=0; i<rectsrc2->w; i++) {
      Uint32 *pixelsrc = (Uint32 *) src->pixels + (j+rectsrc2->y) * src->w + ((i)+rectsrc2->x);
      Uint32 *pixeldst = (Uint32 *) dst->pixels + (j+rectdst->y)  * dst->w + (i+rectdst->x);
      if (j+rectdst->y<0 || j+rectdst->y>=dst->h || i+rectdst->x<0 || i+rectdst->x>=dst->w)
        continue;
      *pixeldst = *pixelsrc;
    }
  }
  if(toErase)
    free(rectsrc2);
  SDL_UnlockSurface(dst);
  SDL_UnlockSurface(src);
}// SDL_BlitSurfaceRGBA()--------------------------------------------------------------------


/**
 * blit a opaque/transparent surface in black and white
 */
void SDL_BlitSurfaceBW(SDL_Surface *src, SDL_Rect *rectsrc, SDL_Surface *dst, SDL_Rect *rectdst, const unsigned char aglob, Uint8 VertMirror){
  SDL_LockSurface(dst);
  SDL_LockSurface(src);
  int toErase = 0;
  Uint32 Rshiftdst = dst->format->Rshift;
  Uint32 Gshiftdst = dst->format->Gshift;
  Uint32 Bshiftdst = dst->format->Bshift;
  Uint32 Ashiftdst = dst->format->Ashift;
/*  Uint32 Rmaskdst  = dst->format->Rmask;
  Uint32 Gmaskdst  = dst->format->Gmask;
  Uint32 Bmaskdst  = dst->format->Bmask;
  Uint32 Amaskdst  = dst->format->Amask;*/
  Uint32 Rshiftsrc = src->format->Rshift;
  Uint32 Gshiftsrc = src->format->Gshift;
  Uint32 Bshiftsrc = src->format->Bshift;
  Uint32 Ashiftsrc = src->format->Ashift;
  Uint32 Rmasksrc  = src->format->Rmask;
  Uint32 Gmasksrc  = src->format->Gmask;
  Uint32 Bmasksrc  = src->format->Bmask;
  Uint32 Amasksrc  = src->format->Amask;

  SDL_Rect* rectsrc2;
  if (rectsrc==NULL) {
    rectsrc2 = (SDL_Rect*)malloc(sizeof(SDL_Rect));
    rectsrc2->x = 0;
    rectsrc2->y = 0;
    rectsrc2->w = src->w;
    rectsrc2->h = src->h;
    toErase =1;
  } else
    rectsrc2 = rectsrc;

  rectdst->w = rectsrc2->w;
  rectdst->h = rectsrc2->h;

  int i, j;
  for (j=0; j<rectsrc2->h; j++) {
    for (i=0; i<rectsrc2->w; i++) {
      //int i2 = rectsrc2->w - i;
      Uint32 *pixelsrc;
      if (VertMirror)
        pixelsrc = (Uint32 *) src->pixels + (j+rectsrc2->y) * src->w + ((rectsrc2->w-i-1)+rectsrc2->x);
      else
        pixelsrc = (Uint32 *) src->pixels + (j+rectsrc2->y) * src->w + ((i)+rectsrc2->x);
      Uint32 *pixeldst = (Uint32 *) dst->pixels + (j+rectdst->y)  * dst->w + (i+rectdst->x);
      if (j+rectdst->y<0 || j+rectdst->y>=dst->h || i+rectdst->x<0 || i+rectdst->x>=dst->w)
        continue;
      unsigned char r1 = (*pixelsrc & Rmasksrc) >> Rshiftsrc;
      unsigned char g1 = (*pixelsrc & Gmasksrc) >> Gshiftsrc;
      unsigned char b1 = (*pixelsrc & Bmasksrc) >> Bshiftsrc;
      unsigned char a1 = (*pixelsrc & Amasksrc) >> Ashiftsrc;
      //unsigned char r2 = (*pixeldst & Rmaskdst) >> Rshiftdst;
      //unsigned char g2 = (*pixeldst & Gmaskdst) >> Gshiftdst;
      //unsigned char b2 = (*pixeldst & Bmaskdst) >> Bshiftdst;
      //unsigned char a2 = (*pixeldst & Amaskdst) >> Ashiftdst;

      /*r1 = (r1*aglob+r2*(255-aglob))/255;
      g1 = (g1*aglob+g2*(255-aglob))/255;
      b1 = (b1*aglob+b2*(255-aglob))/255;*/

      unsigned char  l = (r1+g1+b1)/3;

      if (dst->format->Amask==0 && a1>0)
        *pixeldst = a1<<Rshiftdst | a1<< Gshiftdst | a1<<Bshiftdst;
      else if (src->format->Amask==0)
        *pixeldst = l<<Rshiftdst | l<< Gshiftdst | l<<Bshiftdst | 255<< Ashiftdst;
      else if (a1>0) {
        //a1 = 127+(i*128)/rectsrc2->w;
        *pixeldst = a1<<Rshiftdst | a1<< Gshiftdst | a1<<Bshiftdst | aglob<< Ashiftdst;
      }
    }
  }
  if(toErase)
    free(rectsrc2);
  SDL_UnlockSurface(dst);
  SDL_UnlockSurface(src);
}// SDL_BlitSurfaceBW()--------------------------------------------------------------------


/**
* blit a opaque/transparent surface between black and white ans colored (0=BW, 255= fully colored)
 */
void SDL_BlitSurfacePastel(SDL_Surface *src, SDL_Rect *rectsrc, SDL_Surface *dst, SDL_Rect *rectdst, const unsigned char colprop){
  SDL_LockSurface(dst);
  SDL_LockSurface(src);
  int toErase = 0;
  Uint32 Rshiftdst = dst->format->Rshift;
  Uint32 Gshiftdst = dst->format->Gshift;
  Uint32 Bshiftdst = dst->format->Bshift;
  Uint32 Ashiftdst = dst->format->Ashift;
  /*  Uint32 Rmaskdst  = dst->format->Rmask;
  Uint32 Gmaskdst  = dst->format->Gmask;
  Uint32 Bmaskdst  = dst->format->Bmask;
  Uint32 Amaskdst  = dst->format->Amask;*/
  Uint32 Rshiftsrc = src->format->Rshift;
  Uint32 Gshiftsrc = src->format->Gshift;
  Uint32 Bshiftsrc = src->format->Bshift;
  Uint32 Ashiftsrc = src->format->Ashift;
  Uint32 Rmasksrc  = src->format->Rmask;
  Uint32 Gmasksrc  = src->format->Gmask;
  Uint32 Bmasksrc  = src->format->Bmask;
  Uint32 Amasksrc  = src->format->Amask;

  SDL_Rect* rectsrc2;
  if (rectsrc==NULL) {
    rectsrc2 = (SDL_Rect*)malloc(sizeof(SDL_Rect));
    rectsrc2->x = 0;
    rectsrc2->y = 0;
    rectsrc2->w = src->w;
    rectsrc2->h = src->h;
    toErase =1;
  } else
    rectsrc2 = rectsrc;

  rectdst->w = rectsrc2->w;
  rectdst->h = rectsrc2->h;

  int i, j;
  for (j=0; j<rectsrc2->h; j++) {
    for (i=0; i<rectsrc2->w; i++) {
      Uint32 *pixelsrc = (Uint32 *) src->pixels + (j+rectsrc2->y) * src->w + (i+rectsrc2->x);
      Uint32 *pixeldst = (Uint32 *) dst->pixels + (j+rectdst->y)  * dst->w + (i+rectdst->x);
      if (j+rectdst->y<0 || j+rectdst->y>=dst->h || i+rectdst->x<0 || i+rectdst->x>=dst->w)
        continue;
      unsigned char r1 = (*pixelsrc & Rmasksrc) >> Rshiftsrc;
      unsigned char g1 = (*pixelsrc & Gmasksrc) >> Gshiftsrc;
      unsigned char b1 = (*pixelsrc & Bmasksrc) >> Bshiftsrc;
      unsigned char a1 = (*pixelsrc & Amasksrc) >> Ashiftsrc;
      //unsigned char r2 = (*pixeldst & Rmaskdst) >> Rshiftdst;
      //unsigned char g2 = (*pixeldst & Gmaskdst) >> Gshiftdst;
      //unsigned char b2 = (*pixeldst & Bmaskdst) >> Bshiftdst;
      //unsigned char a2 = (*pixeldst & Amaskdst) >> Ashiftdst;

      unsigned char  l = (r1+g1+b1)/3;
      r1 = (r1*colprop+l*(255-colprop))/255;
      g1 = (g1*colprop+l*(255-colprop))/255;
      b1 = (b1*colprop+l*(255-colprop))/255;
      a1 = 255;
      if (dst->format->Amask!=0)
        *pixeldst = r1<<Rshiftdst | g1<< Gshiftdst | b1<<Bshiftdst | a1<< Ashiftdst;
      else
        *pixeldst = r1<<Rshiftdst | g1<< Gshiftdst | b1<<Bshiftdst;
    }
  }
  if(toErase)
    free(rectsrc2);
  SDL_UnlockSurface(dst);
  SDL_UnlockSurface(src);
}// SDL_BlitSurfacePastel()--------------------------------------------------------------------


/**
* blit a opaque/transparent surface in given color(transparent border) and white (opaque pixels)
 */
void SDL_BlitSurfaceColW(SDL_Surface *src, SDL_Rect *rectsrc, SDL_Surface *dst, SDL_Rect *rectdst,
                         const unsigned char aglob, Uint8 VertMirror, SDL_Color *col){
  SDL_LockSurface(dst);
  SDL_LockSurface(src);
  int toErase = 0;
  Uint32 Rshiftdst = dst->format->Rshift;
  Uint32 Gshiftdst = dst->format->Gshift;
  Uint32 Bshiftdst = dst->format->Bshift;
  Uint32 Ashiftdst = dst->format->Ashift;
/*  Uint32 Rmaskdst  = dst->format->Rmask;
  Uint32 Gmaskdst  = dst->format->Gmask;
  Uint32 Bmaskdst  = dst->format->Bmask;
  Uint32 Amaskdst  = dst->format->Amask;*/
  Uint32 Rshiftsrc = src->format->Rshift;
  Uint32 Gshiftsrc = src->format->Gshift;
  Uint32 Bshiftsrc = src->format->Bshift;
  Uint32 Ashiftsrc = src->format->Ashift;
  Uint32 Rmasksrc  = src->format->Rmask;
  Uint32 Gmasksrc  = src->format->Gmask;
  Uint32 Bmasksrc  = src->format->Bmask;
  Uint32 Amasksrc  = src->format->Amask;

  SDL_Rect* rectsrc2;
  if (rectsrc==NULL) {
    rectsrc2 = (SDL_Rect*)malloc(sizeof(SDL_Rect));
    rectsrc2->x = 0;
    rectsrc2->y = 0;
    rectsrc2->w = src->w;
    rectsrc2->h = src->h;
    toErase =1;
  } else
    rectsrc2 = rectsrc;

  rectdst->w = rectsrc2->w;
  rectdst->h = rectsrc2->h;

  int i, j;
  for (j=0; j<rectsrc2->h; j++) {
    for (i=0; i<rectsrc2->w; i++) {
      //int i2 = rectsrc2->w - i;
      Uint32 *pixelsrc;
      if (VertMirror)
        pixelsrc = (Uint32 *) src->pixels + (j+rectsrc2->y) * src->w + ((rectsrc2->w-i-1)+rectsrc2->x);
      else
        pixelsrc = (Uint32 *) src->pixels + (j+rectsrc2->y) * src->w + ((i)+rectsrc2->x);
      Uint32 *pixeldst = (Uint32 *) dst->pixels + (j+rectdst->y)  * dst->w + (i+rectdst->x);
      if (j+rectdst->y<0 || j+rectdst->y>=dst->h || i+rectdst->x<0 || i+rectdst->x>=dst->w)
        continue;
      unsigned char r1 = (*pixelsrc & Rmasksrc) >> Rshiftsrc;
      unsigned char g1 = (*pixelsrc & Gmasksrc) >> Gshiftsrc;
      unsigned char b1 = (*pixelsrc & Bmasksrc) >> Bshiftsrc;
      unsigned char a1 = (*pixelsrc & Amasksrc) >> Ashiftsrc;
      //unsigned char r2 = (*pixeldst & Rmaskdst) >> Rshiftdst;
      //unsigned char g2 = (*pixeldst & Gmaskdst) >> Gshiftdst;
      //unsigned char b2 = (*pixeldst & Bmaskdst) >> Bshiftdst;
      //unsigned char a2 = (*pixeldst & Amaskdst) >> Ashiftdst;

      /*r1 = (r1*aglob+r2*(255-aglob))/255;
      g1 = (g1*aglob+g2*(255-aglob))/255;
      b1 = (b1*aglob+b2*(255-aglob))/255;*/

      unsigned char  l = (r1+g1+b1)/3;

      if (dst->format->Amask==0 && a1>0)
        *pixeldst = a1<<Rshiftdst | a1<< Gshiftdst | a1<<Bshiftdst;
      else if (src->format->Amask==0)
        *pixeldst = l<<Rshiftdst | l<< Gshiftdst | l<<Bshiftdst | 255<< Ashiftdst;
      else if (a1==255) {
        *pixeldst= 255<<Rshiftdst | 255<< Gshiftdst | 255<<Bshiftdst | aglob<< Ashiftdst;
      } else if (a1>0) {
        //a1 = 127+(i*128)/rectsrc2->w;
        *pixeldst = col->r<<Rshiftdst | col->g<< Gshiftdst | col->b<<Bshiftdst | aglob<< Ashiftdst;
      }
    }
  }
  if(toErase)
    free(rectsrc2);
  SDL_UnlockSurface(dst);
  SDL_UnlockSurface(src);
}// SDL_BlitSurfaceColW()--------------------------------------------------------------------


/**
 * blit a opaque/transparent surface with a Vertical mirror effect
 */
void SDL_BlitSurfaceMirrorV(SDL_Surface *src, SDL_Rect *rectsrc, SDL_Surface *dst, SDL_Rect *rectdst, const int ddisp){
  SDL_LockSurface(dst);
  SDL_LockSurface(src);
  int toErase = 0;
  Uint32 Rshiftdst = dst->format->Rshift;
  Uint32 Gshiftdst = dst->format->Gshift;
  Uint32 Bshiftdst = dst->format->Bshift;
  Uint32 Ashiftdst = dst->format->Ashift;
  Uint32 Rmaskdst  = dst->format->Rmask;
  Uint32 Gmaskdst  = dst->format->Gmask;
  Uint32 Bmaskdst  = dst->format->Bmask;
  //Uint32 Amaskdst  = dst->format->Amask;
  Uint32 Rshiftsrc = src->format->Rshift;
  Uint32 Gshiftsrc = src->format->Gshift;
  Uint32 Bshiftsrc = src->format->Bshift;
  Uint32 Ashiftsrc = src->format->Ashift;
  Uint32 Rmasksrc  = src->format->Rmask;
  Uint32 Gmasksrc  = src->format->Gmask;
  Uint32 Bmasksrc  = src->format->Bmask;
  Uint32 Amasksrc  = src->format->Amask;

  SDL_Rect* rectsrc2;
  if (rectsrc==NULL) {
    rectsrc2 = (SDL_Rect*)malloc(sizeof(SDL_Rect));
    rectsrc2->x = 0;
    rectsrc2->y = 0;
    rectsrc2->w = src->w;
    rectsrc2->h = src->h;
    toErase =1;
  } else
    rectsrc2 = rectsrc;

  rectdst->w = rectsrc2->w;
  rectdst->h = rectsrc2->h;

  int i, j;
  for (j=0; j<rectsrc2->h; j++) {
    for (i=0; i<rectsrc2->w; i++) {
      //int i2 = rectsrc2->w - i;
      Uint32 *pixelsrc = (Uint32 *) src->pixels + ((rectsrc2->h-j-1)+rectsrc2->y) * src->w + (i+rectsrc2->x);
      Uint32 *pixeldst = (Uint32 *) dst->pixels + (j+rectdst->y)  * dst->w + (i+rectdst->x);
      if (j+rectdst->y<0 || j+rectdst->y>=dst->h || i+rectdst->x<0 || i+rectdst->x>=dst->w)
        continue;
      unsigned char r1 = (*pixelsrc & Rmasksrc) >> Rshiftsrc;
      unsigned char g1 = (*pixelsrc & Gmasksrc) >> Gshiftsrc;
      unsigned char b1 = (*pixelsrc & Bmasksrc) >> Bshiftsrc;
      unsigned char a1 = (*pixelsrc & Amasksrc) >> Ashiftsrc;
      unsigned char r2 = (*pixeldst & Rmaskdst) >> Rshiftdst;
      unsigned char g2 = (*pixeldst & Gmaskdst) >> Gshiftdst;
      unsigned char b2 = (*pixeldst & Bmaskdst) >> Bshiftdst;
      //unsigned char a2 = (*pixeldst & Amaskdst) >> Ashiftdst;
      unsigned char a;
      unsigned char  l = (r1+g1+b1)/3;
      if (ddisp>0) {
        a = (rectsrc2->h-j)*ddisp/rectsrc2->h;
        r1 = (r1*a+r2*(255-a))/255;
        g1 = (g1*a+g2*(255-a))/255;
        b1 = (b1*a+b2*(255-a))/255;
      } else {
        a = 255;
      }

      if (dst->format->Amask==0 && a1>0)
        *pixeldst = r1<<Rshiftdst | g1<< Gshiftdst | b1<<Bshiftdst ;
      else if (src->format->Amask==0)
        *pixeldst = l<<Rshiftdst | l<< Gshiftdst | l<<Bshiftdst | 255<< Ashiftdst;
      else if (a1>0) {
        //a1 = 127+(i*128)/rectsrc2->w;
        *pixeldst = r1<<Rshiftdst | g1<< Gshiftdst | b1<<Bshiftdst | 255<< Ashiftdst;
      }
    }
  }
  if(toErase)
    free(rectsrc2);
  SDL_UnlockSurface(dst);
  SDL_UnlockSurface(src);
}// SDL_BlitSurfaceMirrorV()--------------------------------------------------------------------


/**
 * blit a opaque/transparent surface with a Horizontal mirror effect
 */
void SDL_BlitSurfaceMirrorH(SDL_Surface *src, SDL_Rect *rectsrc, SDL_Surface *dst, SDL_Rect *rectdst, const int ddisp){
  SDL_LockSurface(dst);
  SDL_LockSurface(src);
  int toErase = 0;
  Uint32 Rshiftdst = dst->format->Rshift;
  Uint32 Gshiftdst = dst->format->Gshift;
  Uint32 Bshiftdst = dst->format->Bshift;
  Uint32 Ashiftdst = dst->format->Ashift;
  Uint32 Rmaskdst  = dst->format->Rmask;
  Uint32 Gmaskdst  = dst->format->Gmask;
  Uint32 Bmaskdst  = dst->format->Bmask;
  //Uint32 Amaskdst  = dst->format->Amask;
  Uint32 Rshiftsrc = src->format->Rshift;
  Uint32 Gshiftsrc = src->format->Gshift;
  Uint32 Bshiftsrc = src->format->Bshift;
  Uint32 Ashiftsrc = src->format->Ashift;
  Uint32 Rmasksrc  = src->format->Rmask;
  Uint32 Gmasksrc  = src->format->Gmask;
  Uint32 Bmasksrc  = src->format->Bmask;
  Uint32 Amasksrc  = src->format->Amask;

  SDL_Rect* rectsrc2;
  if (rectsrc==NULL) {
    rectsrc2 = (SDL_Rect*)malloc(sizeof(SDL_Rect));
    rectsrc2->x = 0;
    rectsrc2->y = 0;
    rectsrc2->w = src->w;
    rectsrc2->h = src->h;
    toErase =1;
  } else
    rectsrc2 = rectsrc;

  rectdst->w = rectsrc2->w;
  rectdst->h = rectsrc2->h;

  int i, j;
  for (j=0; j<rectsrc2->h; j++) {
    for (i=0; i<rectsrc2->w; i++) {
      //int i2 = rectsrc2->w - i;
      Uint32 *pixelsrc = (Uint32 *) src->pixels + ((j)+rectsrc2->y) * src->w + ((rectsrc2->w-i-1)+rectsrc2->x);
      Uint32 *pixeldst = (Uint32 *) dst->pixels + (j+rectdst->y)  * dst->w + (i+rectdst->x);
      if (j+rectdst->y<0 || j+rectdst->y>=dst->h || i+rectdst->x<0 || i+rectdst->x>=dst->w)
        continue;
      unsigned char r1 = (*pixelsrc & Rmasksrc) >> Rshiftsrc;
      unsigned char g1 = (*pixelsrc & Gmasksrc) >> Gshiftsrc;
      unsigned char b1 = (*pixelsrc & Bmasksrc) >> Bshiftsrc;
      unsigned char a1 = (*pixelsrc & Amasksrc) >> Ashiftsrc;
      unsigned char r2 = (*pixeldst & Rmaskdst) >> Rshiftdst;
      unsigned char g2 = (*pixeldst & Gmaskdst) >> Gshiftdst;
      unsigned char b2 = (*pixeldst & Bmaskdst) >> Bshiftdst;
      //unsigned char a2 = (*pixeldst & Amaskdst) >> Ashiftdst;
      unsigned char a;
      unsigned char  l = (r1+g1+b1)/3;

      if (ddisp>0) {
        r1 = (r1*a+r2*(255-a))/255;
        g1 = (g1*a+g2*(255-a))/255;
        b1 = (b1*a+b2*(255-a))/255;
        //a = (rectsrc2->h-j)*ddisp/rectsrc2->h;
      } else {
        a = 255;
      }

      if (dst->format->Amask==0 && a1>0)
        *pixeldst = r1<<Rshiftdst | g1<< Gshiftdst | b1<<Bshiftdst ;
      else if (src->format->Amask==0)
        *pixeldst = l<<Rshiftdst | l<< Gshiftdst | l<<Bshiftdst | 255<< Ashiftdst;
      else if (a1==255) {
        //a1 = 127+(i*128)/rectsrc2->w;
        *pixeldst = r1<<Rshiftdst | g1<< Gshiftdst | b1<<Bshiftdst | 255<< Ashiftdst;
      }
    }
  }
  if(toErase)
    free(rectsrc2);
  SDL_UnlockSurface(dst);
  SDL_UnlockSurface(src);
}// SDL_BlitSurfaceMirrorH()--------------------------------------------------------------------


/**
 * Display rounded box (filled rectangle) with rgba color
 */
void roundedBoxRGBA(SDL_Surface *dst, Sint16 x, Sint16 y, Sint16 w, Sint16 h,
                    Uint8 r, Uint8 g, Uint8 b, Uint8 a) {
  hlineRGBA(dst, x+5, x+w-6, y+1,    r, g, b, a);
  hlineRGBA(dst, x+3, x+w-4, y+2,    r, g, b, a);
  hlineRGBA(dst, x+2, x+w-3, y+3,    r, g, b, a);
  hlineRGBA(dst, x+2, x+w-3, y+4,    r, g, b, a);
  boxRGBA(  dst, x+1, y+5, x+w-1, y+h-5, r, g, b, a);
  hlineRGBA(dst, x+2, x+w-3, y+h-4,  r, g, b, a);
  hlineRGBA(dst, x+2, x+w-3, y+h-3,  r, g, b, a);
  hlineRGBA(dst, x+3, x+w-4, y+h-2,  r, g, b, a);
  hlineRGBA(dst, x+5, x+w-6, y+h-1 , r, g, b, a);
}// roundedBoxRGBA()--------------------------------------------------------------------


/**
 * Display an horizontal gradient colored box
 */
void boxRGBAAlpha(SDL_Surface *dst, Sint16 x, Sint16 y, Sint16 w, Sint16 h, Uint8 r1, Uint8 g1, Uint8 b1, Uint8 a1) {
  Uint32 Rshift = dst->format->Rshift;
  Uint32 Gshift = dst->format->Gshift;
  Uint32 Bshift = dst->format->Bshift;
  Uint32 Rmask  = dst->format->Rmask;
  Uint32 Gmask  = dst->format->Gmask;
  Uint32 Bmask  = dst->format->Bmask;


  // impossible to even start
  if (x>=dst->w || y>=dst->h)
    return;

  if ((x+w) >= dst->w)
    w = dst->w-x-1;
  if ((y+h) >= dst->h)
    h = dst->h-y-1;

  int i, j;
  SDL_LockSurface(dst);
  unsigned int r, g, b;
  for (j=0; j<h; j++) {
    Uint32 *pixel = (Uint32 *) dst->pixels + (j+y) * dst->pitch / 4 + (x);
    for (i=0; i<w; i++) {
      Uint8 r2 = (*pixel&Rmask)>>Rshift;
      Uint8 g2 = (*pixel&Gmask)>>Gshift;
      Uint8 b2 = (*pixel&Bmask)>>Bshift;
      r = (r1*a1+r2*(255-a1))/255;
      g = (g1*a1+g2*(255-a1))/255;
      b = (b1*a1+b2*(255-a1))/255;
      *pixel = SDL_MapRGB(dst->format, r, g, b);
      pixel++;
    }
    pixel+=dst->w-w;
  }
  SDL_UnlockSurface(dst);
}// hgradientboxRGBA()--------------------------------------------------------------------


/**
* blit a opaque/transparent surface with a Horizontal and vertical mirror effect
 */
void SDL_BlitSurfaceMirrorHV(SDL_Surface *src, SDL_Rect *rectsrc, SDL_Surface *dst, SDL_Rect *rectdst, const int ddisp){
  SDL_LockSurface(dst);
  SDL_LockSurface(src);
  int toErase = 0;
  Uint32 Rshiftdst = dst->format->Rshift;
  Uint32 Gshiftdst = dst->format->Gshift;
  Uint32 Bshiftdst = dst->format->Bshift;
  Uint32 Ashiftdst = dst->format->Ashift;
  Uint32 Rmaskdst  = dst->format->Rmask;
  Uint32 Gmaskdst  = dst->format->Gmask;
  Uint32 Bmaskdst  = dst->format->Bmask;
  //Uint32 Amaskdst  = dst->format->Amask;
  Uint32 Rshiftsrc = src->format->Rshift;
  Uint32 Gshiftsrc = src->format->Gshift;
  Uint32 Bshiftsrc = src->format->Bshift;
  Uint32 Ashiftsrc = src->format->Ashift;
  Uint32 Rmasksrc  = src->format->Rmask;
  Uint32 Gmasksrc  = src->format->Gmask;
  Uint32 Bmasksrc  = src->format->Bmask;
  Uint32 Amasksrc  = src->format->Amask;

  SDL_Rect* rectsrc2;
  if (rectsrc==NULL) {
    rectsrc2 = (SDL_Rect*)malloc(sizeof(SDL_Rect));
    rectsrc2->x = 0;
    rectsrc2->y = 0;
    rectsrc2->w = src->w;
    rectsrc2->h = src->h;
    toErase =1;
  } else
    rectsrc2 = rectsrc;

  rectdst->w = rectsrc2->w;
  rectdst->h = rectsrc2->h;

  int i, j;
  for (j=0; j<rectsrc2->h; j++) {
    for (i=0; i<rectsrc2->w; i++) {
      //int i2 = rectsrc2->w - i;
      Uint32 *pixelsrc = (Uint32 *) src->pixels + ((rectsrc2->h-j-1)+rectsrc2->y) * src->w + ((rectsrc2->w-i-1)+rectsrc2->x);
      Uint32 *pixeldst = (Uint32 *) dst->pixels + (j+rectdst->y)  * dst->w + (i+rectdst->x);
      if (j+rectdst->y<0 || j+rectdst->y>=dst->h || i+rectdst->x<0 || i+rectdst->x>=dst->w)
        continue;
      unsigned char r1 = (*pixelsrc & Rmasksrc) >> Rshiftsrc;
      unsigned char g1 = (*pixelsrc & Gmasksrc) >> Gshiftsrc;
      unsigned char b1 = (*pixelsrc & Bmasksrc) >> Bshiftsrc;
      unsigned char a1 = (*pixelsrc & Amasksrc) >> Ashiftsrc;
      unsigned char r2 = (*pixeldst & Rmaskdst) >> Rshiftdst;
      unsigned char g2 = (*pixeldst & Gmaskdst) >> Gshiftdst;
      unsigned char b2 = (*pixeldst & Bmaskdst) >> Bshiftdst;
      //unsigned char a2 = (*pixeldst & Amaskdst) >> Ashiftdst;

      unsigned char a = (rectsrc2->h-j)*ddisp/rectsrc2->h;

      unsigned char  l = (r1+g1+b1)/3;
      r1 = (r1*a+r2*(255-a))/255;
      g1 = (g1*a+g2*(255-a))/255;
      b1 = (b1*a+b2*(255-a))/255;

      if (dst->format->Amask==0 && a1>0)
        *pixeldst = r1<<Rshiftdst | g1<< Gshiftdst | b1<<Bshiftdst ;
      else if (src->format->Amask==0)
        *pixeldst = l<<Rshiftdst | l<< Gshiftdst | l<<Bshiftdst | 255<< Ashiftdst;
      else if (a1>0) {
        //a1 = 127+(i*128)/rectsrc2->w;
        *pixeldst = r1<<Rshiftdst | g1<< Gshiftdst | b1<<Bshiftdst | 255<< Ashiftdst;
      }
    }
  }
  if(toErase)
    free(rectsrc2);
  SDL_UnlockSurface(dst);
  SDL_UnlockSurface(src);
}// SDL_BlitSurfaceMirrorHV()--------------------------------------------------------------------


/**
 * Display an horizontal gradient colored box
 */
void hgradientboxRGBA(SDL_Surface *dst, Sint16 x, Sint16 y, Sint16 w, Sint16 h, Uint8 r1, Uint8 g1,
                      Uint8 b1, Uint8 a1, Uint8 r2, Uint8 g2, Uint8 b2, Uint8 a2) {
  /*Uint32 Rshift = dst->format->Rshift;
  Uint32 Gshift = dst->format->Gshift;
  Uint32 Bshift = dst->format->Bshift;
  Uint32 Ashift = dst->format->Ashift;*/

  // impossible to even start
  if (x>=dst->w || y>=dst->h)
    return;

  if ((x+w) >= dst->w)
    w = dst->w-x-1;
  if ((y+h) >= dst->h)
    h = dst->h-y-1;

  int i, j;
  SDL_LockSurface(dst);
  unsigned int r, g, b, a;
  for (j=0; j<=h; j++) {
    Uint32 *pixel = (Uint32 *) dst->pixels + (j+y) * dst->pitch / 4 + (x);
    for (i=0; i<w; i++) {
      r = (r2*i+r1*(w-i))/w;
      g = (g2*i+g1*(w-i))/w;
      b = (b2*i+b1*(w-i))/w;
      a = (a2*i+a1*(w-i))/w;
      *pixel = SDL_MapRGBA(dst->format, r, g, b, a);
      pixel++;
    }
    pixel+=dst->w-w;
  }
  SDL_UnlockSurface(dst);
}// hgradientboxRGBA()--------------------------------------------------------------------


/**
 * Display a vertical gradient colored box
 */
void vgradientboxRGBA(SDL_Surface *dst, Sint16 x, Sint16 y, Sint16 w, Sint16 h, Uint8 r1,
                      Uint8 g1, Uint8 b1, Uint8 a1, Uint8 r2, Uint8 g2, Uint8 b2, Uint8 a2) {
/*  Uint32 Rshift = dst->format->Rshift;
  Uint32 Gshift = dst->format->Gshift;
  Uint32 Bshift = dst->format->Bshift;
  Uint32 Ashift = dst->format->Ashift;*/

  // impossible to even start
  if (x>=dst->w || y>=dst->h)
    return;

  if ((x+w) >= dst->w)
    w = dst->w-x-1;
  if ((y+h) >= dst->h)
    h = dst->h-y-1;

  int i, j;
  SDL_LockSurface(dst);

  for (j=y; j<=y+h; j++) {
    Uint32 *pixel = (Uint32 *) dst->pixels + (j) * dst->pitch / 4 + (x);
    int r = (r2*(j-y)+r1*(y+h-j))/h;
    int g = (g2*(j-y)+g1*(y+h-j))/h;
    int b = (b2*(j-y)+b1*(y+h-j))/h;
    int a = (a2*(j-y)+a1*(y+h-j))/h;
    for (i=x; i<=x+w; i++) {
      *pixel = SDL_MapRGBA(dst->format, r, g, b, a);
      pixel++;
    }
    pixel+=dst->w-w;
  }
  SDL_UnlockSurface(dst);
}// vgradientboxRGBA()--------------------------------------------------------------------

/**
 * Display a vertical gradient colored box with noise
 */
void vgradientboxRGBANoise(SDL_Surface *dst, Sint16 x, Sint16 y, Sint16 w, Sint16 h, Uint8 r1,
                      Uint8 g1, Uint8 b1, Uint8 a1, Uint8 r2, Uint8 g2, Uint8 b2, Uint8 a2, Uint8 noise) {

  // impossible to even start
  if (x>=dst->w || y>=dst->h)
    return;

  if ((x+w) >= dst->w)
    w = dst->w-x-1;
  if ((y+h) >= dst->h)
    h = dst->h-y-1;

  int i, j;
  SDL_LockSurface(dst);
  srand(noise);
  for (j=y; j<=y+h; j++) {
    Uint32 *pixel = (Uint32 *) dst->pixels + (j) * dst->pitch / 4 + (x);
    int r = (r2*(j-y)+r1*(y+h-j))/h;
    int g = (g2*(j-y)+g1*(y+h-j))/h;
    int b = (b2*(j-y)+b1*(y+h-j))/h;
    int add = rand()%noise-noise/2;
    if (r+add<=255 && r+add>=0) r = r+add;
    if (g+add<=255 && g+add>=0) g = g+add;
    if (b+add<=255 && b+add>=0) b = b+add;
    int a = (a2*(j-y)+a1*(y+h-j))/h;
    for (i=x; i<=x+w; i++) {
      *pixel = SDL_MapRGBA(dst->format, r, g, b, a);
      pixel++;
    }
    pixel+=dst->w-w;
  }
  SDL_UnlockSurface(dst);
}// vgradientboxNoiseRGBA()--------------------------------------------------------------------


/**
 * Display a vertical gradient colored box
 */
void vgradientIsoTrapezoidRGBA(SDL_Surface *dst, Sint16 x, Sint16 y, Sint16 w1, Sint16 w2, Sint16 h, Uint8 r1,
                               Uint8 g1, Uint8 b1, Uint8 a1, Uint8 r2, Uint8 g2, Uint8 b2, Uint8 a2) {
  /*Uint32 Rshift = dst->format->Rshift;
  Uint32 Gshift = dst->format->Gshift;
  Uint32 Bshift = dst->format->Bshift;
  Uint32 Ashift = dst->format->Ashift;*/

  // impossible to even start
  if (x>=dst->w || y>=dst->h)
    return;
  int w = w1;
  if (w2>w1)
    w = w2;

  if ((x+w1) >= dst->w)
    w1 = dst->w-x-1;
  if ((x+w2) >= dst->w)
    w2 = dst->w-x-1;
  if ((y+h) >= dst->h)
    h = dst->h-y-1;

  int i, j;
  SDL_LockSurface(dst);
  int m = 0;
  for (j=y; j<y+h; j++) {
    Uint32 *pixel = (Uint32 *) dst->pixels + (j) * dst->pitch / 4 + (x);
    m = ((w2-w1)*(h-j+y))/(2*h);
    //m=j-y;
    pixel += m;
    int r = (r2*(j-y)+r1*(y+h-j))/h;
    int g = (g2*(j-y)+g1*(y+h-j))/h;
    int b = (b2*(j-y)+b1*(y+h-j))/h;
    int a = (a2*(j-y)+a1*(y+h-j))/h;
    for (i=0; i<=w2-2*m; i++) {
      *pixel = SDL_MapRGBA(dst->format, r, g, b, a);
      pixel++;
    }
    pixel+=m;
    pixel+=dst->w-w;
  }
  SDL_UnlockSurface(dst);
}// vgradientIsoTrapezoidRGBA()--------------------------------------------------------------------


/**
 * Display a vertical gradient colored box
 */
void vtextureIsoTrapezoidRGBA(SDL_Surface *dst, Sint16 x, Sint16 y, Sint16 w1, Sint16 w2, Sint16 h, SDL_Surface *texture) {
  /*Uint32 Rshift = dst->format->Rshift;
  Uint32 Gshift = dst->format->Gshift;
  Uint32 Bshift = dst->format->Bshift;*/
  Uint32 Ashift = dst->format->Ashift;

  // impossible to even start
  if (x>=dst->w || y>=dst->h)
    return;
  int w = w1;
  if (w2>w1)
    w = w2;

  if ((x+w1) >= dst->w)
    w1 = dst->w-x-1;
  if ((x+w2) >= dst->w)
    w2 = dst->w-x-1;
  if ((y+h) >= dst->h)
    h = dst->h-y-1;

  int i, j;
  SDL_LockSurface(dst);
  int m = 0;
  for (j=y; j<y+h; j++) {
    Uint32 *pixel = (Uint32 *) dst->pixels + (j) * dst->pitch / 4 + (x);
    m = ((w2-w1)*(h-j+y))/(2*h);
    pixel += m;
    for (i=0; i<w2-2*m; i++) {
      float texx = i*((float)texture->w)/(w2-2.0f*m);
      Uint32 *tex1 = (Uint32 *) texture->pixels + (j-y) * texture->pitch / 4 + (int)(texx);
      //int r = (r2*(j-y)+r1*(y+h-j))/h;

      *pixel = *tex1|(0xFF<<Ashift);
      pixel++;
    }
    pixel+=m;
    pixel+=dst->w-w;
  }
  SDL_UnlockSurface(dst);
}// vtextureIsoTrapezoidRGBA()--------------------------------------------------------------------


/**
 * Display a diagonal gradient colored box (from top left to bottom right)
 */
void d1gradientboxRGBA(SDL_Surface *dst, Sint16 x, Sint16 y, Sint16 w, Sint16 h,
                       Uint8 r1, Uint8 g1, Uint8 b1, Uint8 a1,
                       Uint8 r2, Uint8 g2, Uint8 b2, Uint8 a2) {
  // impossible to even start
  if (x>=dst->w || y>=dst->h)
    return;

  if ((x+w) >= dst->w)
    w = dst->w-x-1;
  if ((y+h) >= dst->h)
    h = dst->h-y-1;

  int i, j;
  SDL_LockSurface(dst);
  for (j=y; j<=y+h; j++) {
    for (i=x; i<=x+w; i++) {
      Uint32 *pixel = (Uint32 *) dst->pixels + (j) * dst->pitch / 4 + (i);
      int r = (r2*(j-y+i-x)+r1*(y+h-j+x+w-i))/(h+w);
      int g = (g2*(j-y+i-x)+g1*(y+h-j+x+w-i))/(h+w);
      int b = (b2*(j-y+i-x)+b1*(y+h-j+x+w-i))/(h+w);
      int a = (a2*(j-y+i-x)+a1*(y+h-j+x+w-i))/(h+w);
      *pixel = SDL_MapRGBA(dst->format, r, g, b, a);
    }
  }
  SDL_UnlockSurface(dst);
}// d1gradientboxRGBA()--------------------------------------------------------------------


/**
* Display a diagonal gradient colored box (from top right to bottom left)
 */
void d2gradientboxRGBA(SDL_Surface *dst, Sint16 x, Sint16 y, Sint16 w, Sint16 h,
                       Uint8 r1, Uint8 g1, Uint8 b1, Uint8 a1,
                       Uint8 r2, Uint8 g2, Uint8 b2, Uint8 a2) {
  // impossible to even start
  if (x>=dst->w || y>=dst->h)
    return;

  if ((x+w) >= dst->w)
    w = dst->w-x-1;
  if ((y+h) >= dst->h)
    h = dst->h-y-1;

  int i, j;
  SDL_LockSurface(dst);
  for (j=y; j<=y+h; j++) {
    for (i=x; i<=x+w; i++) {
      Uint32 *pixel = (Uint32 *) dst->pixels + (j) * dst->pitch / 4 + (i);
      int r = (r2*(j-y+w-(i-x))+r1*(y+h-j+x+w-i))/(h+w);
      int g = (g2*(j-y+w-(i-x))+g1*(y+h-j+x+w-i))/(h+w);
      int b = (b2*(j-y+w-(i-x))+b1*(y+h-j+x+w-i))/(h+w);
      int a = (a2*(j-y+w-(i-x))+a1*(y+h-j+x+w-i))/(h+w);
      *pixel = SDL_MapRGBA(dst->format, r, g, b, a);
    }
  }
  SDL_UnlockSurface(dst);
}// d2gradientboxRGBA()--------------------------------------------------------------------


/**
 * Display elongated diamond with rgba color
 */
void diamondRGBA(SDL_Surface *dst, Sint16 x, Sint16 y, Sint16 w, Sint16 h,
                 Uint8 r, Uint8 g, Uint8 b, Uint8 a) {
  int i;
  float ratio = (float)h/(float)w;
  for (i=1; i<w-1; i++)
    if (i<w/2)
      vlineRGBA(dst, x+i, y+h/2-(int)(ratio*i-0.5), y+h/2+(int)(ratio*i)-1, r, g, b, a);
    else
      vlineRGBA(dst, x+i, y+h/2-(int)(ratio*(w-i)-0.5), y+h/2+(int)(ratio*(w-i))-1, r, g, b, a);
}// diamondRGBA()--------------------------------------------------------------------


/**
 * Display rounded rectangle with rgba color
 */
void roundedRectangleRGBA(SDL_Surface *dst, Sint16 x, Sint16 y, Sint16 w, Sint16 h,
                          Uint8 r, Uint8 g, Uint8 b, Uint8 a) {
  hlineRGBA(dst, x+5, x+w-5, y+1   , r, g, b, a);
  hlineRGBA(dst, x+5, x+w-5, y+h-1 , r, g, b, a);
  vlineRGBA(dst, x+1, y+5, y+h-6   , r, g, b, a);
  vlineRGBA(dst, x+w-1, y+5, y+h-6 , r, g, b, a);
  pixelRGBA(dst, x+4, y+2          , r, g, b, a);
  pixelRGBA(dst, x+3, y+2          , r, g, b, a);
  pixelRGBA(dst, x+2, y+3          , r, g, b, a);
  pixelRGBA(dst, x+2, y+4          , r, g, b, a);
  pixelRGBA(dst, x+w-4, y+2        , r, g, b, a);
  pixelRGBA(dst, x+w-3, y+2        , r, g, b, a);
  pixelRGBA(dst, x+w-2, y+3        , r, g, b, a);
  pixelRGBA(dst, x+w-2, y+4        , r, g, b, a);
  pixelRGBA(dst, x+4, y+h-2        , r, g, b, a);
  pixelRGBA(dst, x+3, y+h-2        , r, g, b, a);
  pixelRGBA(dst, x+2, y+h-3        , r, g, b, a);
  pixelRGBA(dst, x+2, y+h-4        , r, g, b, a);
  pixelRGBA(dst, x+w-4, y+h-2      , r, g, b, a);
  pixelRGBA(dst, x+w-3, y+h-2      , r, g, b, a);
  pixelRGBA(dst, x+w-3, y+h-3      , r, g, b, a);
  pixelRGBA(dst, x+w-2, y+h-4      , r, g, b, a);
}// roundedRectangleRGBA()--------------------------------------------------------------------


/**
 * evaluate bezier segment by weighted sum of control points
 */
double evaluateBezier2(double *data, int ndata, double t) {
  double mu, result;
  int    n,k,kn,nn,nkn;
  double blend,muk,munk;

  /* Sanity check bounds */
  if (t<0.0) {
    return(data[0]);
  }
  if (t>=(double)ndata) {
    return(data[ndata-1]);
  }

  /* Adjust t to the range 0.0 to 1.0 */
  mu=t/(double)ndata;

  /* Calculate interpolate */
  n=ndata-1;
  result=0.0;
  muk = 1;
  munk = pow(1-mu,(double)n);
  for (k=0;k<=n;k++) {
    nn = n;
    kn = k;
    nkn = n - k;
    blend = muk * munk;
    muk *= mu;
    munk /= (1-mu);
    while (nn >= 1) {
      blend *= nn;
      nn--;
      if (kn > 1) {
        blend /= (double)kn;
        kn--;
      }
      if (nkn > 1) {
        blend /= (double)nkn;
        nkn--;
      }
    }
    result += data[k] * blend;
  }

  return(result);
}// evaluateBezier2()--------------------------------------------------------------------


/**
 * draw a bezier curve with 2 alternatives colors
 */
int bezier3Texture(SDL_Surface * dst, const Sint16 * vx, const Sint16 * vy,
                   float coeff1, float coeff2, int n, int s, int w, int dispBorder) {
  int result;
  int i;
  SDL_Surface *texsea   = textures[SEALANE-100];
  SDL_Rect *rect = (SDL_Rect *)malloc(sizeof(SDL_Rect));

  double *x, *y, t, stepsize;
  Sint16 x1, y1;

  /*
   * Sanity check
   */
  if (n < 3) {
    return (-1);
  }
  if (s < 2) {
    return (-1);
  }

  // Variable setup
  stepsize=(double)1.0/(double)s;

  /* Transfer vertices into float arrays */
  if ((x=(double *)malloc(sizeof(double)*(n+1)))==NULL) {
    return(-1);
  }
  if ((y=(double *)malloc(sizeof(double)*(n+1)))==NULL) {
    free(x);
    return(-1);
  }
  for (i=0; i<n; i++) {
    x[i]=vx[i];
    y[i]=vy[i];
  }

  float dx = x[n-1]-x[0];
  float dy = y[n-1]-y[0];

  float dev = sqrt(dx*dx+dy*dy)/n;
  float dx1 = (dy/dev*(coeff1-1))*-1;
  float dy1 = (dx/dev*(coeff1-1));
  float dx2 = (dy/dev*(coeff2-1))*-1;
  float dy2 = (dx/dev*(coeff2-1));
  //float f = 0;
  result = 0;
  t=0.0;
  x1=(Sint16)evaluateBezier2(x,n+1,t);
  y1=(Sint16)evaluateBezier2(y,n+1,t);
  for (i = 0; i <= (n*s)/2; i++) {
    t += stepsize;
    x1=(Sint16)(evaluateBezier2(x,n,t)+dx1*sin(M_PI*2.0*(float)i/((n*s)/2)));
    y1=(Sint16)(evaluateBezier2(y,n,t)+dy1*sin(M_PI*2.0*(float)i/((n*s)/2)));
    rect->x=x1-w/2;
    rect->y=y1-w/2;
    rect->w=w+1;
    rect->h=w+1;
    SDL_BlitSurface(texsea, rect, dst, rect);
  }
  for (; i <= (n*s); i++) {
    t += stepsize;
    x1=(Sint16)(evaluateBezier2(x,n,t)+dx2*sin(M_PI*2.0*(float)(i-(n*s)/2)/((n*s)/2)));
    y1=(Sint16)(evaluateBezier2(y,n,t)+dy2*sin(M_PI*2.0*(float)(i-(n*s)/2)/((n*s)/2)));
    rect->x=x1-w/2;
    rect->y=y1-w/2;
    rect->w=w+1;
    rect->h=w+1;
    SDL_BlitSurface(texsea, rect, dst, rect);
  }
  t=0.0;
  x1=(Sint16)evaluateBezier2(x,n+1,t);
  y1=(Sint16)evaluateBezier2(y,n+1,t);
  int ox1=0;
  int oy1 = 0;
  for (i = 0; i <= (n*s)/2; i++) {
    t += stepsize;
    x1=(Sint16)(evaluateBezier2(x,n,t)+dx1*sin(M_PI*2.0*(float)i/((n*s)/2)));
    y1=(Sint16)(evaluateBezier2(y,n,t)+dy1*sin(M_PI*2.0*(float)i/((n*s)/2)));
    dx= ox1-x1;
    dy = oy1-y1;
    if (dispBorder && dx!=0 && dy!=0 && fabs(dx)/dx!=fabs(dy)/dy) {
      pixelRGBA(dst, x1-w/2+w+1, y1-w/2+w+1, 128, 128, 128, 255);
      pixelRGBA(dst, x1-w/2,     y1-w/2,     64,  64,  64,  255);
    } else if (dispBorder && dx!=0 && dy!=0 && fabs(dx)/dx==fabs(dy)/dy) {
      pixelRGBA(dst, x1-(w/2),     y1-w/2+w+1, 128, 128, 128, 255);
      pixelRGBA(dst, x1-(w/2)+w+1, y1-w/2,     64,  64,  64,  255);
    }
    ox1 = x1;
    oy1 = y1;
  }
  for (; i <= (n*s); i++) {
    t += stepsize;
    x1=(Sint16)(evaluateBezier2(x,n,t)+dx2*sin(M_PI*2.0*(float)(i-(n*s)/2)/((n*s)/2)));
    y1=(Sint16)(evaluateBezier2(y,n,t)+dy2*sin(M_PI*2.0*(float)(i-(n*s)/2)/((n*s)/2)));
    dx= ox1-x1;
    dy = oy1-y1;
    if (dispBorder && dx!=0 && dy!=0 && fabs(dx)/dx!=fabs(dy)/dy) {
      pixelRGBA(dst, x1-w/2+w+1, y1-w/2+w+1, 128, 128, 128, 255);
      pixelRGBA(dst, x1-w/2, y1-w/2, 64, 64, 64, 255);
    } else if (dispBorder && dx!=0 && dy!=0 && fabs(dx)/dx==fabs(dy)/dy) {
      pixelRGBA(dst, x1-(w/2), y1-w/2+w+1, 128, 128, 128, 255);
      pixelRGBA(dst, x1-(w/2)+w+1, y1-w/2, 64, 64, 64, 255);
    }
    ox1 = x1;
    oy1 = y1;
  }

  /* Clean up temporary array */
  free(x);
  free(y);
  free(rect);
  return (result);
}// bezier3Texture()--------------------------------------------------------------------


/**
 * draw a bezier curve with 2 alternatives colors
 */
int bezier2Color(SDL_Surface * dst, const Sint16 * vx, const Sint16 * vy,
                 float coeff1, float coeff2, int n, int s,
                 Uint32 color1,  Uint32 color2, float percentage, int size) {
  int result;
  int i;
  double *x, *y, t, stepsize;
  Sint16 x1, y1, x2, y2;
  Uint32 color;
  // Sanity check
  if (n < 3) {
    return (-1);
  }
  if (s < 2) {
    return (-1);
  }

  // Variable setup
  stepsize=(double)1.0/(double)s;

  /* Transfer vertices into float arrays */
  if ((x=(double *)malloc(sizeof(double)*(n+1)))==NULL) {
    return(-1);
  }
  if ((y=(double *)malloc(sizeof(double)*(n+1)))==NULL) {
    free(x);
    return(-1);
  }
  for (i=0; i<n; i++) {
    x[i]=vx[i];
    y[i]=vy[i];
  }
  x[n]=vx[0];
  y[n]=vy[0];

  float dx = x[n-1]-x[0];
  float dy = y[n-1]-y[0];

  float dev = sqrt(dx*dx+dy*dy)/n;

  // get a small normal vector
  float dx1 = (dy/dev*(coeff1-1))*-1;
  float dy1 = (dx/dev*(coeff1-1));
  float dx2 = (dy/dev*(coeff2-1))*-1;
  float dy2 = (dx/dev*(coeff2-1));

  float percent1 = 1.0;
  if (percentage<0.5) {
    percent1 = percentage*2;
  }

  result = 0;
  t=0.0;
  x1=(int)evaluateBezier2(x, n+1, t);
  y1=(int)evaluateBezier2(y, n+1, t);
  for (i = 0; i <= percent1*(n*s)/2; i++) {
    if (i/2%2)
      color=color1;
    else
      color=color2;
    t += stepsize;
    x2=(Sint16)(evaluateBezier2(x,n,t)+dx1*cos((float)i));
    y2=(Sint16)(evaluateBezier2(y,n,t)+dy1*cos((float)i));
    if (size==1) {
      result |= lineColor(dst, x1, y1, x2, y2, color);
    } else if (size==2) {
      result |= lineColor(dst, x1, y1, x2, y2, color);
      result |= lineColor(dst, x1, y1+1, x2, y2+1, color);
      result |= lineColor(dst, x1+1, y1, x2+1, y2, color);
    } else if (size==3) {
      result |= lineColor(dst, x1, y1+1, x2, y2+1, color);
      result |= lineColor(dst, x1-1, y1-1, x2-1, y2-1, color);
      result |= lineColor(dst, x1, y1-1, x2, y2-1, color);
      result |= lineColor(dst, x1-1, y1, x2-1, y2, color);
      result |= lineColor(dst, x1+1, y1, x2+1, y2, color);
      result |= lineColor(dst, x1+1, y1+1, x2+1, y2+1, color);
      result |= lineColor(dst, x1-1, y1+1, x2-1, y2+1, color);
      result |= lineColor(dst, x1+1, y1-1, x2+1, y2-1, color);
      result |= lineColor(dst, x1, y1, x2, y2, 0xCCCCCCFF);
    } else if (size==4) {
      result |= lineColor(dst, (int)(x1+dy/dev), (int)(y1-dx/dev), x1, y1, color2);
      result |= lineColor(dst, (int)(x1-dy/dev), (int)(y1+dx/dev), x1, y1, color2);
      if (x1*x2==y1*y2 || y2-y1==0 || x2-x1==0) {
        result |= lineColor(dst, x1+1, y1+1, x2+1, y2+1, color1);
        result |= lineColor(dst, x1-1, y1+1, x2-1, y2+1, color1);
      }
      if (x1*x2!=y1*y2 || y2-y1==0 || x2-x1==0) {
        result |= lineColor(dst, x1+1, y1-1, x2+1, y2-1, color1);
        result |= lineColor(dst, x1-1, y1-1, x2-1, y2-1, color1);
      }
    }
    x1 = x2;
    y1 = y2;
  }
  for (; i <= percentage*(n*s); i++) {
    if (i/2%2)
      color=color1;
    else
      color=color2;
    t += stepsize;
    x2=(Sint16)(evaluateBezier2(x,n,t)+dx2*cos((float)i));
    y2=(Sint16)(evaluateBezier2(y,n,t)+dy2*cos((float)i));
    if (size==1) {
      result |= lineColor(dst, x1, y1, x2, y2, color);
    } else if (size==2) {
      result |= lineColor(dst, x1, y1, x2, y2, color);
      result |= lineColor(dst, x1, y1+1, x2, y2+1, color);
      result |= lineColor(dst, x1+1, y1, x2+1, y2, color);
    } else if (size==3) {
      result |= lineColor(dst, x1, y1+1, x2, y2+1, color);
      result |= lineColor(dst, x1-1, y1-1, x2-1, y2-1, color);
      result |= lineColor(dst, x1, y1-1, x2, y2-1, color);
      result |= lineColor(dst, x1-1, y1, x2-1, y2, color);
      result |= lineColor(dst, x1+1, y1, x2+1, y2, color);
      result |= lineColor(dst, x1+1, y1+1, x2+1, y2+1, color);
      result |= lineColor(dst, x1-1, y1+1, x2-1, y2+1, color);
      result |= lineColor(dst, x1+1, y1-1, x2+1, y2-1, color);
      result |= lineColor(dst, x1, y1, x2, y2, 0xCCCCCCFF);
    } else if (size==4) {
      result |= lineColor(dst, (int)(x1+dy/dev), (int)(y1-dx/dev), x1, y1, color2);
      result |= lineColor(dst, (int)(x1-dy/dev), (int)(y1+dx/dev), x1, y1, color2);
      if (x1*x2==y1*y2 || y2-y1==0 || x2-x1==0) {
        result |= lineColor(dst, x1+1, y1+1, x2+1, y2+1, color1);
        result |= lineColor(dst, x1-1, y1+1, x2-1, y2+1, color1);
      }
      if (x1*x2!=y1*y2 || y2-y1==0 || x2-x1==0) {
        result |= lineColor(dst, x1+1, y1-1, x2+1, y2-1, color1);
        result |= lineColor(dst, x1-1, y1-1, x2-1, y2-1, color1);
      }
    }
    x1 = x2;
    y1 = y2;
  }
  /* Clean up temporary array */
  free(x);
  free(y);

  return (result);
}// bezier2Color()--------------------------------------------------------------------


/**
 * draw a bezier curves with n points and s segments approximating the curve
 */
int bezier2RGBA(SDL_Surface * dst, const Sint16 * vx, const Sint16 * vy,
                float coeff1, float coeff2, int n, int s,
                Uint8 r1, Uint8 g1, Uint8 b1, Uint8 a1,
                Uint8 r2, Uint8 g2, Uint8 b2, Uint8 a2, float percentage, int size) {
  return bezier2Color(dst, vx, vy, coeff1, coeff2, n, s,
                      ((Uint32) r1 << 24) | ((Uint32) g1 << 16) | ((Uint32) b1 << 8) | (Uint32) a1,
                      ((Uint32) r2 << 24) | ((Uint32) g2 << 16) | ((Uint32) b2 << 8) | (Uint32) a2, percentage, size);
}// bezie2rRGBA()--------------------------------------------------------------------


/**
 * cartesian distance
*/
int dist(int x1, int y1, int x2, int y2) {
  int dx = x2-x1;
  int dy = y2-y1;
  if (dx<0)
    dx*= -1;
  if (dy<0)
    dy*= -1;
  return dx+dy;
}// dist() --------------------------------------------------------------------


/**
 * Initialize textures udes in extra graphics primitives
 */
void initTextures() {
  //int i;
  addMsg("Loading textures");
  updateLoading();
  textures[DESERT-FIRST_GROUND]      = (SDL_Surface *)IMG_LoadAndFormatNoAlpha("images/tiles/Desert.png");
  initcpt++;updateLoading();
  textures[GRASSLAND-FIRST_GROUND]   = (SDL_Surface *)IMG_LoadAndFormatNoAlpha("images/tiles/Grassland.png");
  initcpt++;updateLoading();
                      
  textures[PLAIN-FIRST_GROUND]       = (SDL_Surface *)IMG_LoadAndFormatNoAlpha("images/tiles/Plain.png");
  initcpt++;updateLoading();
  textures[PRAIRIE-FIRST_GROUND]     = (SDL_Surface *)IMG_LoadAndFormatNoAlpha("images/tiles/Prairie.png");
  initcpt++;updateLoading();
  textures[ROCKDESERT-FIRST_GROUND]  = (SDL_Surface *)IMG_LoadAndFormatNoAlpha("images/tiles/RockDesert.png");
  initcpt++;updateLoading();
  textures[OCEAN-FIRST_GROUND]       = (SDL_Surface *)IMG_LoadAndFormatNoAlpha("images/tiles/Ocean.png");
  initcpt++;updateLoading();
  textures[SEALANE-FIRST_GROUND]     = (SDL_Surface *)IMG_LoadAndFormatNoAlpha("images/tiles/SeaLane.png");
  initcpt++;updateLoading();
  textures[TUNDRA-FIRST_GROUND]      = (SDL_Surface *)IMG_LoadAndFormatNoAlpha("images/tiles/Desert.png");
  initcpt++;updateLoading();
  textures[SWAMP-FIRST_GROUND]       = (SDL_Surface *)IMG_LoadAndFormatNoAlpha("images/tiles/Swamp.png");
  initcpt++;updateLoading();
  textures[MARSH-FIRST_GROUND]       = (SDL_Surface *)IMG_LoadAndFormatNoAlpha("images/tiles/Desert.png");
  initcpt++;updateLoading();
  textures[SAVANNAH-FIRST_GROUND]    = (SDL_Surface *)IMG_LoadAndFormatNoAlpha("images/tiles/Savannah.png");
  initcpt++;updateLoading();
  textures[HILLROCK-FIRST_GROUND]    = (SDL_Surface *)IMG_LoadAndFormatNoAlpha("images/tiles/HillRock.png");
  initcpt++;updateLoading();
  textures[ROCK2-FIRST_GROUND]       = (SDL_Surface *)IMG_LoadAndFormatNoAlpha("images/tiles/HillRock2.png");
  initcpt++;updateLoading();
  textures[ROCK-FIRST_GROUND]        = (SDL_Surface *)IMG_LoadAndFormatNoAlpha("images/tiles/Mountain.png");
  initcpt++;updateLoading();
  textures[ARCTIC-FIRST_GROUND]      = (SDL_Surface *)IMG_LoadAndFormatNoAlpha("images/tiles/Arctic.png");
  initcpt++;updateLoading();
  textures[FORESTTREES-FIRST_GROUND] = (SDL_Surface *)IMG_LoadAndFormatNoAlpha("images/Forest.png");
  initcpt++;updateLoading();
  
  textures[GRASSLAND1-FIRST_GROUND]   = (SDL_Surface *)IMG_LoadAndFormatNoAlpha("images/tiles/Grassland01.png");
  initcpt++;updateLoading();
  textures[GRASSLAND2-FIRST_GROUND]   = (SDL_Surface *)IMG_LoadAndFormatNoAlpha("images/tiles/Grassland02.png");
  initcpt++;updateLoading();
  textures[GRASSLAND3-FIRST_GROUND]   = (SDL_Surface *)IMG_LoadAndFormatNoAlpha("images/tiles/Grassland03.png");
  initcpt++;updateLoading();
  textures[GRASSLAND4-FIRST_GROUND]   = (SDL_Surface *)IMG_LoadAndFormatNoAlpha("images/tiles/Grassland04.png");
  initcpt++;updateLoading();
  textures[GRASSLAND5-FIRST_GROUND]   = (SDL_Surface *)IMG_LoadAndFormatNoAlpha("images/tiles/Grassland05.png");
  initcpt++;updateLoading();
  textures[GRASSLAND6-FIRST_GROUND]   = (SDL_Surface *)IMG_LoadAndFormatNoAlpha("images/tiles/Grassland06.png");
  initcpt++;updateLoading();
  textures[GRASSLAND7-FIRST_GROUND]   = (SDL_Surface *)IMG_LoadAndFormatNoAlpha("images/tiles/Grassland07.png");
  initcpt++;updateLoading();
  textures[GRASSLAND8-FIRST_GROUND]   = (SDL_Surface *)IMG_LoadAndFormatNoAlpha("images/tiles/Grassland08.png");
  initcpt++;updateLoading();
  textures[GRASSLAND9-FIRST_GROUND]   = (SDL_Surface *)IMG_LoadAndFormatNoAlpha("images/tiles/Grassland09.png");
  initcpt++;updateLoading();
  textures[GRASSLAND10-FIRST_GROUND]   = (SDL_Surface *)IMG_LoadAndFormatNoAlpha("images/tiles/Grassland10.png");
  initcpt++;updateLoading();
  textures[GRASSLAND11-FIRST_GROUND]   = (SDL_Surface *)IMG_LoadAndFormatNoAlpha("images/tiles/Grassland11.png");
  initcpt++;updateLoading();
  textures[GRASSLAND12-FIRST_GROUND]   = (SDL_Surface *)IMG_LoadAndFormatNoAlpha("images/tiles/Grassland12.png");
  initcpt++;updateLoading(); 
  
  textures[PLAIN1-FIRST_GROUND]   = (SDL_Surface *)IMG_LoadAndFormatNoAlpha("images/tiles/Plain01.png");
  initcpt++;updateLoading();
  textures[PLAIN2-FIRST_GROUND]   = (SDL_Surface *)IMG_LoadAndFormatNoAlpha("images/tiles/Plain02.png");
  initcpt++;updateLoading();
  textures[PLAIN3-FIRST_GROUND]   = (SDL_Surface *)IMG_LoadAndFormatNoAlpha("images/tiles/Plain03.png");
  initcpt++;updateLoading();
  textures[PLAIN4-FIRST_GROUND]   = (SDL_Surface *)IMG_LoadAndFormatNoAlpha("images/tiles/Plain04.png");
  initcpt++;updateLoading();
  textures[PLAIN5-FIRST_GROUND]   = (SDL_Surface *)IMG_LoadAndFormatNoAlpha("images/tiles/Plain05.png");
  initcpt++;updateLoading();
  textures[PLAIN6-FIRST_GROUND]   = (SDL_Surface *)IMG_LoadAndFormatNoAlpha("images/tiles/Plain06.png");
  initcpt++;updateLoading();
  textures[PLAIN7-FIRST_GROUND]   = (SDL_Surface *)IMG_LoadAndFormatNoAlpha("images/tiles/Plain07.png");
  initcpt++;updateLoading();
  textures[PLAIN8-FIRST_GROUND]   = (SDL_Surface *)IMG_LoadAndFormatNoAlpha("images/tiles/Plain08.png");
  initcpt++;updateLoading();
  textures[PLAIN9-FIRST_GROUND]   = (SDL_Surface *)IMG_LoadAndFormatNoAlpha("images/tiles/Plain09.png");
  initcpt++;updateLoading();
  textures[PLAIN10-FIRST_GROUND]   = (SDL_Surface *)IMG_LoadAndFormatNoAlpha("images/tiles/Plain10.png");
  initcpt++;updateLoading();
  textures[PLAIN11-FIRST_GROUND]   = (SDL_Surface *)IMG_LoadAndFormatNoAlpha("images/tiles/Plain11.png");
  initcpt++;updateLoading();
  textures[PLAIN12-FIRST_GROUND]   = (SDL_Surface *)IMG_LoadAndFormatNoAlpha("images/tiles/Plain12.png");
  initcpt++;updateLoading(); 
  
  
}// initTextures() --------------------------------------------------------------------


/**
 * return the color that describe the best a given texture
 **/
SDL_Color *getColorforTexture(int tiletype) {
  int r1=0, g1=0, b1=0;
  int dx, dy;
  //Uint32 col = 0x00000000;
  SDL_Surface *tex = textures[tiletype-FIRST_GROUND];
  if (tex==NULL)
    return NULL;
  Uint32 Rshift = tex->format->Rshift;
  Uint32 Gshift = tex->format->Gshift;
  Uint32 Bshift = tex->format->Bshift;
  Uint32 Rmask  = tex->format->Rmask, Gmask =tex->format->Gmask, Bmask = tex->format->Bmask;

  for (dx=0; dx<4; dx++)
    for (dy=0; dy<4; dy++) {
      Uint32 *pixeltex = (Uint32 *) tex->pixels + (dy) * tex->pitch / 4 + (dx);
      r1 += (*pixeltex & Rmask) >> Rshift;
      g1 += (*pixeltex & Gmask) >> Gshift;
      b1 += (*pixeltex & Bmask) >> Bshift;
    }
  //col = ((Uint32) r1/16 << tex->format->Rshift) | ((Uint32) g1/16 << tex->format->Gshift) | ((Uint32) b1/16 << tex->format->Bshift) | 0xFF<< tex->format->Ashift;
  //return col;
  SDL_Color *col = (SDL_Color *)malloc(sizeof(SDL_Color));
  col->r = r1/16;
  col->g = g1/16;
  col->b = b1/16;
  return col;
}// getColorforTexture() --------------------------------------------------------------------


/**
 * display the quarter of a landscape tile
*/
void displaylandscapeQuarter(SDL_Surface *dst, SDL_Rect *rect, int borders[3][3],
                          int ax, int ay, int bx, int by, int centerWeight, const int tt[3][3],
                          SDL_Surface *fieldTile, const int month, const float lattitude) {
   //printf("displaylandscapeQuarter(month=%d)\n",month); 
  int i, j, z, oz, z2, a, ai, aj, c;
  int sw0=0, sw1=0, sh0=0, sh1=0;
  int a1, r1, g1, b1, r00, r01, r10, r11, g00, g01, g10, g11, b00, b01, b10, b11;
  int hh = rect->h/2;
  int ww = rect->w/2;
  int dx = ax*rect->w/2;
  int dy = ay*rect->h/2;

  int ddx = ww*ax;
  int ddy = hh*ay;

  int shadow = 1;

  int w00 = 2;
  int w01 = 2;
  int w10 = 2;
  int w11 = 2;

  if (borders[1][1]<SEALEVEL&&centerWeight<8) centerWeight = centerWeight+4;
  //BR
  if (ax==1 && ay==1) {
    //else
    w00=centerWeight;
    sw1=ww;
    sh1=hh;
  }
  if (ax==1 && by==1) {
    //if (borders[2][2]<SEALEVEL) w01 = centerWeight*4;
    //else
    w01=centerWeight;
    sw1=ww;
    sh0=hh;
  }//
  if (bx==1 && ay==1) {
    //if (borders[2][2]<SEALEVEL) w10 = centerWeight*4;
    //else
    w10=centerWeight;
    sw0=ww;
    sh1=hh;
  }
  if (bx==1 && by==1) {
    //if (borders[2][2]<SEALEVEL) w11 = centerWeight*4;
    //else
    w11=centerWeight;
    sw0=ww;
    sh0=hh;
  }

  SDL_Surface *tex00    = textures[tt[ax][ay]-FIRST_GROUND];
  SDL_Surface *tex01    = textures[tt[ax][by]-FIRST_GROUND];
  SDL_Surface *tex10    = textures[tt[bx][ay]-FIRST_GROUND];
  SDL_Surface *tex11    = textures[tt[bx][by]-FIRST_GROUND];
  
  if (tt[ax][ay]==GRASSLAND) tex00    = textures[GRASSLAND1+month-FIRST_GROUND];
  if (tt[ax][by]==GRASSLAND) tex01    = textures[GRASSLAND1+month-FIRST_GROUND];
  if (tt[bx][ay]==GRASSLAND) tex10    = textures[GRASSLAND1+month-FIRST_GROUND];
  if (tt[bx][by]==GRASSLAND) tex11    = textures[GRASSLAND1+month-FIRST_GROUND];

  if (tt[ax][ay]==PLAIN) tex00    = textures[PLAIN1+month-FIRST_GROUND];
  if (tt[ax][by]==PLAIN) tex01    = textures[PLAIN1+month-FIRST_GROUND];
  if (tt[bx][ay]==PLAIN) tex10    = textures[PLAIN1+month-FIRST_GROUND];
  if (tt[bx][by]==PLAIN) tex11    = textures[PLAIN1+month-FIRST_GROUND];

  SDL_Surface *texsand  = textures[DESERT    -FIRST_GROUND];
  SDL_Surface *texsea   = textures[SEALANE   -FIRST_GROUND];
  SDL_Surface *texocean = textures[OCEAN     -FIRST_GROUND];

  SDL_Surface *texhill1     = textures[HILLROCK-FIRST_GROUND];
  SDL_Surface *texhill2     = textures[ROCK2-FIRST_GROUND];
  SDL_Surface *texmountain1 = textures[ROCK-FIRST_GROUND];
  SDL_Surface *texmountain2 = textures[ARCTIC-FIRST_GROUND];

  Uint32 Rshift = tex00->format->Rshift;
  Uint32 Gshift = tex00->format->Gshift;
  Uint32 Bshift = tex00->format->Bshift;
  Uint32 Ashift = 24;
  Uint32 Rmask  = tex00->format->Rmask;
  Uint32 Gmask  = tex00->format->Gmask;
  Uint32 Bmask  = tex00->format->Bmask;
  Uint32 Amask  = 0xFF000000;

  SDL_LockSurface(dst);

  if (borders[ax][ay]==borders[ax][by] && borders[ax][by]==borders[bx][by] && borders[bx][by]==borders[bx][ay] &&
      tex00==tex01 && tex01==tex11 && tex11==tex10){
    z  = borders[ax][ay];
    z2 = (z/LC)*LC;
    SDL_Rect *rect2 = (SDL_Rect*)malloc(sizeof(SDL_Rect));
    SDL_Rect *rect3 = (SDL_Rect*)malloc(sizeof(SDL_Rect));
    rect2->w = rect->w/2;
    rect2->h = rect->h/2;
    rect2->x = rect->x+dx;
    rect2->y = rect->y+dy;
    rect3->w = rect->w/2;
    rect3->h = rect->w/2;
    rect3->x = dx;
    rect3->y = dy;
    //SDL_FillRect(dst, rect2, SDL_MapRGB( screen->format, dx, dy, 0));
    SDL_UnlockSurface(dst);
    if (z>=-50 && z<100)
      SDL_BlitSurface(texsand, rect3, dst, rect2);
    else if (z>=-700 && z<-80)
      SDL_BlitSurface(texsea, rect3, dst, rect2);
    else if (z<=-700 )
      SDL_BlitSurface(texocean, rect3, dst, rect2);
    else if (z2>=4*LC && z2<=6*LC) {
      SDL_BlitSurface(tex00, rect3, dst, rect2);
      a = ((z2-4*LC)*255)/(6*LC);
      SDL_SetAlpha(texhill1, SDL_SRCALPHA, a);
      SDL_BlitSurface(texhill1, rect3, dst, rect2);
      SDL_SetAlpha(texhill1, SDL_SRCALPHA, 255);
    } else if (z2>=7*LC && z2<=9*LC) {
      SDL_BlitSurface(texhill1, rect3, dst, rect2);
      a = ((z2-7*LC)*255)/(3*LC);
      SDL_SetAlpha(texhill2, SDL_SRCALPHA, a);
      SDL_BlitSurface(texhill2, rect3, dst, rect2);
      SDL_SetAlpha(texhill2, SDL_SRCALPHA, 255);
    } else if (z2>=10*LC && z2<=12*LC) {
      SDL_BlitSurface(texhill2, rect3, dst, rect2);
      a = ((z2-10*LC)*255)/(3*LC);
      SDL_SetAlpha(texmountain1, SDL_SRCALPHA, a);
      SDL_BlitSurface(texmountain1, rect3, dst, rect2);
      SDL_SetAlpha(texmountain1, SDL_SRCALPHA, 255);
    } else if (z2>=13*LC && z2<=18*LC) {
      SDL_BlitSurface(texmountain1, rect3, dst, rect2);
      a = ((z2-13*LC)*255)/(3*LC);
      if (a>255) a = 255;
      SDL_SetAlpha(texmountain2, SDL_SRCALPHA, a);
      SDL_BlitSurface(texmountain2, rect3, dst, rect2);
      SDL_SetAlpha(texmountain2, SDL_SRCALPHA, 255);
    } else
      SDL_BlitSurface(tex00, rect3, dst, rect2);

    if (fieldTile!=NULL){
      SDL_SetAlpha(fieldTile, SDL_SRCALPHA, 192);
      SDL_BlitSurface(fieldTile, rect3, dst, rect2);
      SDL_SetAlpha(texmountain2, SDL_SRCALPHA, 255);
    }

    /*char *text2 = (char*)malloc(12);
    sprintf(text2 , "%i", a);
    SDL_Surface *imgText = TTF_RenderText_Blended(fonts[8], text2, BLACK);
    SDL_BlitSurface(imgText, 0, dst, rect2);
    free(text2);
    SDL_FreeSurface(imgText); */

    free(rect2);
    free(rect3);

    return;
  }

  oz = borders[ax][ay];
  for (j=0; j<hh; j++) {
    for (i=0; i<ww; i++) {

      ai = ww-i;
      aj = hh-j;

      z =  (w00*borders[ax][ay]*(ai*aj) + w01*borders[ax][by]*(ai*j) +
            w10*borders[bx][ay]*(i*aj)  + w11*borders[bx][by]*(i*j)) / (w00*ai*aj+w01*ai*j+w10*i*aj+w11*i*j);

      Uint32 *pixel00 = (Uint32 *) tex00->pixels + (j+ddy) * tex00->w + (i+ddx);
      Uint32 *pixel01 = (Uint32 *) tex01->pixels + (j+ddy) * tex00->w + (i+ddx);
      Uint32 *pixel10 = (Uint32 *) tex10->pixels + (j+ddy) * tex00->w + (i+ddx);
      Uint32 *pixel11 = (Uint32 *) tex11->pixels + (j+ddy) * tex00->w + (i+ddx);
      r00 = (*pixel00 & Rmask) >> Rshift;
      g00 = (*pixel00 & Gmask) >> Gshift;
      b00 = (*pixel00 & Bmask) >> Bshift;
      r01 = (*pixel01 & Rmask) >> Rshift;
      g01 = (*pixel01 & Gmask) >> Gshift;
      b01 = (*pixel01 & Bmask) >> Bshift;
      r10 = (*pixel10 & Rmask) >> Rshift;
      g10 = (*pixel10 & Gmask) >> Gshift;
      b10 = (*pixel10 & Bmask) >> Bshift;
      r11 = (*pixel11 & Rmask) >> Rshift;
      g11 = (*pixel11 & Gmask) >> Gshift;
      b11 = (*pixel11 & Bmask) >> Bshift;

      r1 = (w00*r00*((ai+sw1)*(aj+sh1)) + w01*r01*((ai+sw1)*(j+sh0)) + w10*r10*((i+sw0)*(aj+sh1)) + w11*r11*((i+sw0)*(j+sh0)))/(w00*(ai+sw1)*(aj+sh1)+w01*(ai+sw1)*(j+sh0)+w10*(i+sw0)*(aj+sh1)+w11*(i+sw0)*(j+sh0));
      g1 = (w00*g00*((ai+sw1)*(aj+sh1)) + w01*g01*((ai+sw1)*(j+sh0)) + w10*g10*((i+sw0)*(aj+sh1)) + w11*g11*((i+sw0)*(j+sh0)))/(w00*(ai+sw1)*(aj+sh1)+w01*(ai+sw1)*(j+sh0)+w10*(i+sw0)*(aj+sh1)+w11*(i+sw0)*(j+sh0));
      b1 = (w00*b00*((ai+sw1)*(aj+sh1)) + w01*b01*((ai+sw1)*(j+sh0)) + w10*b10*((i+sw0)*(aj+sh1)) + w11*b11*((i+sw0)*(j+sh0)))/(w00*(ai+sw1)*(aj+sh1)+w01*(ai+sw1)*(j+sh0)+w10*(i+sw0)*(aj+sh1)+w11*(i+sw0)*(j+sh0));

      //printf("i=%i r1=%i \n", i, r1);
      //fflush(stdout);
      //if (i==ww-1) exit(0);
      //r1 = r00;
      //g1 = g00;
      //b1 = b00;
      // beach
      if (z>=-50 && z<100) {
        Uint32 *pixelSand = (Uint32 *) texsand->pixels + (j+ddy) * texsand->pitch / 4 + (i+ddx);
        r1 = (*pixelSand & Rmask) >> Rshift;
        g1 = (*pixelSand & Gmask) >> Gshift;
        b1 = (*pixelSand & Bmask) >> Bshift;

        // sand+sea
      } else if (z>=-80 && z<-50) {
        Uint32 *pixelSea  = (Uint32 *) texsea->pixels + (j+ddy) * texsea->pitch / 4 + (i+ddx);
        Uint32 *pixelSand = (Uint32 *) texsand->pixels + (j+ddy) * texsand->pitch / 4 + (i+ddx);
        r1 = (*pixelSand & Rmask) >> Rshift;
        g1 = (*pixelSand & Gmask) >> Gshift;
        b1 = (*pixelSand & Bmask) >> Bshift;
        int r2 = (*pixelSea & Rmask) >> Rshift;
        int g2 = (*pixelSea & Gmask) >> Gshift;
        int b2 = (*pixelSea & Bmask) >> Bshift;
        r1=((-50-z)*r2+(80+z)*r1)/30;
        g1=((-50-z)*g2+(80+z)*g1)/30;
        b1=((-50-z)*b2+(80+z)*b1)/30;
        // sea
      } else if (z<=-80 && z>-700) {
        Uint32 *pixelSea = (Uint32 *) texsea->pixels + (j+ddy) * texsea->pitch / 4 + (i+ddx);
        r1 = (*pixelSea & Rmask) >> Rshift;
        g1 = (*pixelSea & Gmask) >> Gshift;
        b1 = (*pixelSea & Bmask) >> Bshift;

        // high sea / ocean
      } else if (z<=-700 && z>-1000) {
        Uint32 *pixelSea = (Uint32 *) texsea->pixels + (j+ddy) * texsea->pitch / 4 + (i+ddx);
        Uint32 *pixelOcean = (Uint32 *) texocean->pixels + (j+ddy) * texocean->pitch / 4 + (i+ddx);
        r1 = (*pixelOcean & Rmask) >> Rshift;
        g1 = (*pixelOcean & Gmask) >> Gshift;
        b1 = (*pixelOcean & Bmask) >> Bshift;
        int r2 = (*pixelSea & Rmask) >> Rshift;
        int g2 = (*pixelSea & Gmask) >> Gshift;
        int b2 = (*pixelSea & Bmask) >> Bshift;
        r1=((1000+z)*r2+(-z-700)*r1)/300;
        g1=((1000+z)*g2+(-z-700)*g1)/300;
        b1=((1000+z)*b2+(-z-700)*b1)/300;
      } else if (z<=-1000) {
        Uint32 *pixelOcean = (Uint32 *) texocean->pixels + (j+ddy) * texocean->pitch / 4 + (i+ddx);
        r1 = (*pixelOcean & Rmask) >> Rshift;
        g1 = (*pixelOcean & Gmask) >> Gshift;
        b1 = (*pixelOcean & Bmask) >> Bshift;
      }

      z2= (z/LC)*LC;
      c = z2/12;
      if (c>192)
        c=255;
      // light brown rock
      if (z2>=4*LC && z2<=6*LC) {
        Uint32 *pixelHill1 = (Uint32 *) texhill1->pixels + (j+ddy) * texhill1->pitch / 4 + (i+ddx);
        int r2 = (int)(((*pixelHill1 & Rmask) >> Rshift)/**(c/158.0)*/);
        int g2 = (int)(((*pixelHill1 & Gmask) >> Gshift)/**(c/158.0)*/);
        int b2 = (int)(((*pixelHill1 & Bmask) >> Bshift)/**(c/158.0)*/);
        a = ((z2-4*LC)*255)/(6*LC);
        r1 = (r2*a+r1*(255-a))/255;
        g1 = (g2*a+g1*(255-a))/255;
        b1 = (b2*a+b1*(255-a))/255;
      // brown rocks
      } else if (z2>=7*LC && z2<=9*LC) {
        Uint32 *pixelHill1 = (Uint32 *) texhill1->pixels + (j+ddy) * texhill1->pitch / 4 + (i+ddx);
        int r2 = (int)(((*pixelHill1 & Rmask) >> Rshift)/**(c/158.0)*/);
        int g2 = (int)(((*pixelHill1 & Gmask) >> Gshift)/**(c/158.0)*/);
        int b2 = (int)(((*pixelHill1 & Bmask) >> Bshift)/**(c/158.0)*/);
        Uint32 *pixelHill2 = (Uint32 *) texhill2->pixels + (j+ddy) * texhill2->pitch / 4 + (i+ddx);
        int r3 = (int)(((*pixelHill2 & Rmask) >> Rshift)/**(c/158.0)*/);
        int g3 = (int)(((*pixelHill2 & Gmask) >> Gshift)/**(c/158.0)*/);
        int b3 = (int)(((*pixelHill2 & Bmask) >> Bshift)/**(c/158.0)*/);
        a = ((z2-7*LC)*255)/(3*LC);
        r1 = (r3*a+r2*(255-a))/255;
        g1 = (g3*a+g2*(255-a))/255;
        b1 = (b3*a+b2*(255-a))/255;
      // gray mountain
      } else if (z2>=10*LC && z2<=12*LC) {
        Uint32 *pixelMountain1 = (Uint32 *) texmountain1->pixels + (j+ddy) * texmountain1->pitch / 4 + (i+ddx);
        r1 = (int)(((*pixelMountain1 & Rmask) >> Rshift)/**(c/158.0)*/);
        g1 = (int)(((*pixelMountain1 & Gmask) >> Gshift)/**(c/158.0)*/);
        b1 = (int)(((*pixelMountain1 & Bmask) >> Bshift)/**(c/158.0)*/);
        Uint32 *pixelHill2 = (Uint32 *) texhill2->pixels + (j+ddy) * texhill2->pitch / 4 + (i+ddx);
        int r3 = (int)(((*pixelHill2 & Rmask) >> Rshift)/**(c/158.0)*/);
        int g3 = (int)(((*pixelHill2 & Gmask) >> Gshift)/**(c/158.0)*/);
        int b3 = (int)(((*pixelHill2 & Bmask) >> Bshift)/**(c/158.0)*/);
        if (r1<0)   r1=0;
        if (r1>255) r1=255;
        if (g1<0)   g1=0;
        if (g1>255) g1=255;
        if (b1<0)   b1=0;
        if (b1>255) b1=255;
        a = ((z2-10*LC)*255)/(3*LC);
        r1 = (r1*a+r3*(255-a))/255;
        g1 = (g1*a+g3*(255-a))/255;
        b1 = (b1*a+b3*(255-a))/255;
      // white mountain
      } else if (z2>=13*LC && z2<=18*LC) {
        Uint32 *pixelMountain2 = (Uint32 *) texmountain2->pixels + (j+ddy) * texmountain2->pitch / 4 + (i+ddx);
        r1 = (int)(((*pixelMountain2 & Rmask) >> Rshift)/**(c/158.0)*/);
        g1 = (int)(((*pixelMountain2 & Gmask) >> Gshift)/**(c/158.0)*/);
        b1 = (int)(((*pixelMountain2 & Bmask) >> Bshift)/**(c/158.0)*/);
        Uint32 *pixelMountain1 = (Uint32 *) texmountain1->pixels + (j+ddy) * texmountain1->pitch / 4 + (i+ddx);
        int r2 = (int)(((*pixelMountain1 & Rmask) >> Rshift)/*(c/158.0)*/);
        int g2 = (int)(((*pixelMountain1 & Gmask) >> Gshift)/*(c/158.0)*/);
        int b2 = (int)(((*pixelMountain1 & Bmask) >> Bshift)/*(c/158.0)*/);
        if (r1<0)   r1=0;
        if (r1>255) r1=255;
        if (g1<0)   g1=0;
        if (g1>255) g1=255;
        if (b1<0)   b1=0;
        if (b1>255) b1=255;
        a = ((z2-13*LC)*255)/(3*LC);
        if (a>255) a = 255;
        r1 = (r1*a+r2*(255-a))/255;
        g1 = (g1*a+g2*(255-a))/255;
        b1 = (b1*a+b2*(255-a))/255;
      }

      if (fieldTile!=NULL){
        Uint32 *pixelField = (Uint32 *) fieldTile->pixels + (j+ddy) * fieldTile->pitch / 4 + (i+ddx);
        int rf = (int)(((*pixelField & Rmask) >> Rshift));
        int gf = (int)(((*pixelField & Gmask) >> Gshift));
        int bf = (int)(((*pixelField & Bmask) >> Bshift));
        int af = (int)(((*pixelField & Amask) >> Ashift));
        if (af==255) {
          r1 = (r1+3*rf)/4;
          g1 = (g1+3*gf)/4;
          b1 = (b1+3*bf)/4;
        }
      }

      if (i!=0 && z>0)
        shadow = 127+(oz-z);
      else
        shadow = 127;
      Uint32 *pixel = ((Uint32 *)(dst->pixels))+(dst->pitch/4)*(j+rect->y+dy)+i+rect->x+dx;

      if (z/LC!=oz/LC && i!=0 && z>0) {
        z2= (z/LC)*LC;
        if (oz>z)
          z2=(oz/LC)*LC;
        c = z2/12;
        if (c>192)
          c=255;
        if (z-oz>0)
          c = 255-c/2;
        else
          c = c/2;
        //pixelRGBA(dst, i+rect->x+dx, j+rect->y+dy        , c, c, c, shadow);
        //*pixel = SDL_MapRGBA(dst->format, c, c, c, 128);
        *pixel = c<<24 | c<<16 | c<<8 | shadow;
      } else
        //*pixel = SDL_MapRGBA(dst->format, r1, g1, b1, 128);
        *pixel = r1<<24 | g1<<16 | b1<<8 | shadow;
        //pixelRGBA(dst, i+rect->x+dx, j+rect->y+dy        , r1, g1, b1, a);
      oz = z;
    }
  }
  int oshadow = 127;
  oz = borders[ax][ay];
  for (i=0; i<ww; i++) {
    for (j=0; j<hh; j++) {
      int ai = ww-i;
      int aj = hh-j;
      //Uint32 *pixel = (Uint32 *) dst->pixels + (j) * dst->pitch / 4 + (i);
      Uint32 *opasspixel = ((Uint32 *)(dst->pixels))+(dst->pitch/4)*(j+rect->y+dy)+i+rect->x+dx;
      r1 = ((*opasspixel) & 0xFF000000) >> 24;
      g1 = ((*opasspixel) & 0x00FF0000) >> 16;
      b1 = ((*opasspixel) & 0x0000FF00) >> 8;
      a1 = ((*opasspixel) & 0x000000FF);

      z =  (w00*borders[ax][ay]*(ai*aj) + w01*borders[ax][by]*(ai*j) +
            w10*borders[bx][ay]*(i*aj)  + w11*borders[bx][by]*(i*j))      / (w00*ai*aj+w01*ai*j+w10*i*aj+w11*i*j);

      if (z<0)
        shadow = 127;
      else if (j==0)
        shadow = oshadow;
      else
        shadow = 255-(127+(oz-z) + a1)/2;
      if (i==1) oshadow = shadow;
      r1 = r1+shadow-127;
      g1 = g1+shadow-127;
      b1 = b1+shadow-127;
      if (r1<0)   r1=0;
      if (r1>255) r1=255;
      if (g1<0)   g1=0;
      if (g1>255) g1=255;
      if (b1<0)   b1=0;
      if (b1>255) b1=255;

      z2= (z/LC)*LC;
      if (oz>z)
        z2=(oz/LC)*LC;
      c = z2/12;
      if (c>192)
        c=255;
      if (z-oz>0)
        c = 255-c/2;
      else
        c = c/2;

      if (z/LC!=oz/LC && j!=0 && z>0)
        //*opasspixel = SDL_MapRGBA(dst->format, c, c, c, 255);
        pixelRGBA(dst, i+rect->x+dx, j+rect->y+dy, c, c, c, 255);
      else
        //*opasspixel = SDL_MapRGBA(dst->format, r1, g1, b1, 255);
        pixelRGBA(dst, i+rect->x+dx, j+rect->y+dy, r1, g1, b1, 255);
      oz = z;
    }
  }
  SDL_UnlockSurface(dst);
}// displaylandscapeQuarter() -------------------------------------------------------------------


/**
 * display the landscape with isoXXX curves.
 */
void landscapeTile(SDL_Surface *dst, SDL_Rect *rect, int borders[3][3], int tt[3][3], int seed, SDL_Surface *fieldTile, const int month, const float lattitude) {
  srand(seed);
  rand();
  int centerWeight = 1+rand()%(CW-2);
  displaylandscapeQuarter(dst, rect, borders, 0, 0, 1, 1, centerWeight, tt, fieldTile, month, lattitude);
  displaylandscapeQuarter(dst, rect, borders, 0, 1, 1, 2, centerWeight, tt, fieldTile, month, lattitude);
  displaylandscapeQuarter(dst, rect, borders, 1, 0, 2, 1, centerWeight, tt, fieldTile, month, lattitude);
  displaylandscapeQuarter(dst, rect, borders, 1, 1, 2, 2, centerWeight, tt, fieldTile, month, lattitude);
}// landscapeTile() --------------------------------------------------------------------



/**
 * display recursively a city portion
 */
void cityTiteQuarter(SDL_Surface *dst, int x, int y, float length, float mainroad, int dx, int dy, int seed) {
  if (mainroad<=0)
    return;
  srand(seed);
  int gl = 16+(rand()%6)*16;
  boxRGBA(dst, x-(int)(length/2), y-(int)(length/2), x+(int)(length/2), y+(int)(length/2),  gl, gl, gl, 255);
  cityTiteQuarter(dst, (int)(x+dx*length/2), (int)(y+dy*length/2), length*2/3,  mainroad-0.5f, dy, dx, rand());
  cityTiteQuarter(dst, (int)(x+dx*length/2), (int)(y+dy*length/2), length*2/3,  mainroad-0.5f, -dy, -dx, rand());
}// cityTiteQuarter() --------------------------------------------------------------------


/**
 * display the streets of a city
 */
void streetTiteQuarter(SDL_Surface *dst, int x, int y, float length, float mainroad, int dx, int dy) {
  if (mainroad<=0)
    return;

  SDL_Surface *texrock   = textures[ROCKDESERT-100];
  SDL_Rect *rect = (SDL_Rect *)malloc(sizeof(SDL_Rect));

  streetTiteQuarter(dst, (int)(x+dx*length/2), (int)(y+dy*length/2), length*2/3, mainroad-0.5f, dy, dx);
  streetTiteQuarter(dst, (int)(x+dx*length/2), (int)(y+dy*length/2), length*2/3, mainroad-0.5f, -dy, -dx);

  rect->x = x-abs(dy)*(int)(mainroad/2);
  rect->y = y-abs(dx)*(int)(mainroad/2);
  int rw = x+dx*((int)(length))+abs(dy)*((int)(mainroad/2))-rect->x;
  int rh = y+dy*(int)(length)+abs(dx)*(int)(mainroad/2)-rect->y;
  if (rw<0) {
    rect->w=-rw;
  } else
    rect->w = rw;
  if (rh<0) {
    rect->h=-rh;
  } else
    rect->h = rh;

  boxRGBA(dst, rect->x, rect->y, rect->x+rect->w, rect->y+rect->h, 0, 0, 0, 255);
  SDL_BlitSurface(texrock, rect, dst, rect);
  free(rect);
}// streetTiteQuarter() --------------------------------------------------------------------


/**
 * draw a textured circle form the forest texture but making the pixels more and more transparent on the border
 */
void progressiveTexturedCircle(SDL_Surface *tc, int x, int y, int r) {
  Uint32 *px1 = (Uint32*)forestImages[FOREST1]->pixels;
  Uint32 *px2 = (Uint32*)tc->pixels;
  int i, j;
  for (i=x-r; i<x+r; i++)
    for (j=y-r; j<y+r; j++) {
      float d= (float)hypot(i-x,j-y);
      if (d<=r && i>=0 && j>=0 && i<TILESIZE && j<TILESIZE) {
        if (d>3*r/4)
          px2[j*forestImages[FOREST1]->pitch/4+i] = (((px1[j*forestImages[FOREST1]->pitch/4+i]&0xf8f8f8f8)>>3)&0xffffffff) +
              (((px1[j*forestImages[FOREST1]->pitch/4+i]&0xf8f8f8f8)>>3)&0xffffffff) +
              (((px1[j*forestImages[FOREST1]->pitch/4+i]&0xf8f8f8f8)>>3)&0xffffffff) +
              (((px2[j*forestImages[FOREST1]->pitch/4+i]&0xf8f8f8f8)>>3)&0xffffffff) +
              (((px2[j*forestImages[FOREST1]->pitch/4+i]&0xf8f8f8f8)>>3)&0xffffffff) +
              (((px2[j*forestImages[FOREST1]->pitch/4+i]&0xf8f8f8f8)>>3)&0xffffffff) +
              (((px2[j*forestImages[FOREST1]->pitch/4+i]&0xf8f8f8f8)>>3)&0xffffffff) +
              (((px2[j*forestImages[FOREST1]->pitch/4+i]&0xf8f8f8f8)>>3)&0xffffffff);
        else if (d>2*r/4)
          px2[j*forestImages[FOREST1]->pitch/4+i] = (((px1[j*forestImages[FOREST1]->pitch/4+i]&0xf8f8f8f8)>>3)&0xffffffff) +
              (((px1[j*forestImages[FOREST1]->pitch/4+i]&0xf8f8f8f8)>>3)&0xffffffff) +
              (((px1[j*forestImages[FOREST1]->pitch/4+i]&0xf8f8f8f8)>>3)&0xffffffff) +
              (((px1[j*forestImages[FOREST1]->pitch/4+i]&0xf8f8f8f8)>>3)&0xffffffff) +
              (((px1[j*forestImages[FOREST1]->pitch/4+i]&0xf8f8f8f8)>>3)&0xffffffff) +
              (((px2[j*forestImages[FOREST1]->pitch/4+i]&0xf8f8f8f8)>>3)&0xffffffff) +
              (((px2[j*forestImages[FOREST1]->pitch/4+i]&0xf8f8f8f8)>>3)&0xffffffff) +
              (((px2[j*forestImages[FOREST1]->pitch/4+i]&0xf8f8f8f8)>>3)&0xffffffff);
        else if (d>r/4)
          px2[j*forestImages[FOREST1]->pitch/4+i] = (((px1[j*forestImages[FOREST1]->pitch/4+i]&0xf8f8f8f8)>>3)&0xffffffff) +
              (((px1[j*forestImages[FOREST1]->pitch/4+i]&0xf8f8f8f8)>>3)&0xffffffff) +
              (((px1[j*forestImages[FOREST1]->pitch/4+i]&0xf8f8f8f8)>>3)&0xffffffff) +
              (((px1[j*forestImages[FOREST1]->pitch/4+i]&0xf8f8f8f8)>>3)&0xffffffff) +
              (((px1[j*forestImages[FOREST1]->pitch/4+i]&0xf8f8f8f8)>>3)&0xffffffff) +
              (((px1[j*forestImages[FOREST1]->pitch/4+i]&0xf8f8f8f8)>>3)&0xffffffff) +
              (((px1[j*forestImages[FOREST1]->pitch/4+i]&0xf8f8f8f8)>>3)&0xffffffff) +
              (((px2[j*forestImages[FOREST1]->pitch/4+i]&0xf8f8f8f8)>>3)&0xffffffff);
        else
          px2[j*forestImages[FOREST1]->pitch/4+i] = px1[j*forestImages[FOREST1]->pitch/4+i];
      }
    }
}// progressiveTexturedCircle() --------------------------------------------------------------------


/**
 * draw a tree as a circle
 */
void treeCircle(SDL_Surface *tc, int x, int y, int r) {
  SDL_Rect *rect = (SDL_Rect *)malloc(sizeof(SDL_Rect));
  rect->x = x-r;
  rect->y = y-r;

  if (r<=8) {
    SDL_BlitSurface(treeImages[TREE5], NULL, tc, rect);
  } else if (r<=16) {
    SDL_BlitSurface(treeImages[TREE6], NULL, tc, rect);
  } else {
    SDL_BlitSurface(treeImages[TREE7], NULL, tc, rect);
  }
  free(rect);
}// treeCircle() --------------------------------------------------------------------


/**
 * draw a tree as a circle but with a texture inside !
 */
void texturedCircle(SDL_Surface *tc, int x, int y, int r) {
  Uint32 *px1 = (Uint32*)forestImages[FOREST1]->pixels;
  Uint32 *px2 = (Uint32*)tc->pixels;
  int i, j;
  for (i=x-r; i<x+r; i++)
    for (j=y-r; j<y+r; j++) {
      float d= (float)hypot(i-x,j-y);
      if (d<=r && i>=0 && j>=0 && i<TILESIZE && j<TILESIZE) {
        px2[j*forestImages[FOREST1]->pitch/4+i] = px1[j*forestImages[FOREST1]->pitch/4+i];
      }
    }
}// texturedCircle() --------------------------------------------------------------------


/**
 * draw a tree's shadow on the ground
 *
void treeShadow(SDL_Surface *tc, int x, int y, int r, unsigned char *zbuf) {
  //if (debug)
  //  printf("treeShadow(%i, %i)  r=%i \n", x, y, r);
  Uint32 *px2 = (Uint32*)tc->pixels;
  int i, j;
  for (i=x-r-2; i<=x+r+2; i++) {
    for (j=y-r-2; j<=y+r+2; j++) {
      if ( i>=0 && j>=0 && i<TILESIZE && j<TILESIZE) {
        unsigned char a = (px2[j*tc->pitch/4+i]&0xFF000000)>>24;
        if (!(tc->flags&SDL_SRCALPHA))
          a = 255;
        float d= (float)hypot(i-x, j-y);
        if (d<=r-1 && a==255){
          // nothing it will be the tree there !
        } else if (d<=r+1 && zbuf[j*TILESIZE+i]==0 && a==255) {
          px2[j*tc->pitch/4+i] = ((px2[j*tc->pitch/4+i] &0x00FCFCFC)>>2) | (px2[j*tc->pitch/4+i] & 0xFF000000);
          zbuf[j*TILESIZE+i]=2;
        } else if (d<=r+1 && zbuf[j*TILESIZE+i]==1 && a==255) {
          px2[j*tc->pitch/4+i] = ((px2[j*tc->pitch/4+i] &0x00FEFEFE)>>1) | (px2[j*tc->pitch/4+i] & 0xFF000000);
          zbuf[j*TILESIZE+i]=2;
        } else if (d<=r+2 && zbuf[j*TILESIZE+i]==0 && a==255) {
          px2[j*tc->pitch/4+i] = ((px2[j*tc->pitch/4+i] &0x00FEFEFE)>>1) | (px2[j*tc->pitch/4+i] & 0xFF000000);
          zbuf[j*TILESIZE+i]=1;
        } else if (a==0 && d<=r){
          px2[j*tc->pitch/4+i] = 0xFF000000;
        } else if (a==0 && d<=r+1){
          px2[j*tc->pitch/4+i] = 0xC0000000;
        } else if (a==0x80 && d<=r+1){
          px2[j*tc->pitch/4+i] = 0xC0000000;
        } else if (a==0 && d<=r+2){
          px2[j*tc->pitch/4+i] = 0x80000000;
        }
      }
    }
  }
}// treeShadow() --------------------------------------------------------------------


/**
 * display almost a circular tree seen from the sky using a z buffer
 */
void tree3d(SDL_Surface *tc, int x, int y, int r, unsigned char *zbuf, int param, unsigned char month) {
  srand (x*y);
  p_init(x*y);
  unsigned char vr, vg, vb, va;
  unsigned char R;// = 64+rand()%192;
  unsigned char G;// = 128+rand()%128;
  unsigned char B;// = 32+rand()%32;
  switch (month){
    case 0 : // january
      int v = rand()%32;rand();rand();
      R = 192+v; vr = 8;
      G = 192+v; vg = 8;
      B = 192+v; vb = 8;
      va= 192;
      break;
    case 1 : // febuary
      v = rand()%16;rand();rand();
      R = 192+v; vr = 8;
      G = 200+v; vg = 8;
      B = 192+v; vb = 8;
      va= 128;
      break;
    case 2 : // march
      v = rand()%16;
      rand();
      R = 192+v;          vr = 16;
      G = 216+rand()%32;  vg = 8;
      B = 192+v;          vb = 16;
      va= 64;
      break;
    case 3 : // april
      R = 64+rand()%32;   vr = 16;
      G = 128+rand()%64;  vg = 16;
      B = 32+rand()%32;   vb = 16;
      va= 32;
      break;
    case 4 : // may= max green
      R = 64+rand()%32;   vr = 16;
      G = 160+rand()%64;  vg = 16;
      B = 32+rand()%32;   vb = 16;
      va=0;
      break;
    case 5 : // june= max yellow
      R = 110+rand()%32;   vr = 16;
      G = 160+rand()%64;   vg = 16;
      B = 32+rand()%32;    vb = 16;
      va=0;
      break;
    case 6 : // july= max yellow
      R = 160+rand()%32;   vr = 16;
      G = 180+rand()%32;   vg = 16;
      B = 32+rand()%32;    vb = 16;
      va=0;
      break;
    case 7 : // august
      R = 160+rand()%32;   vr = 16;
      G = 160+rand()%42;   vg = 16;
      B = 32+rand()%32;    vb = 16;
      va=16;
      break;
    case 8 : // september
      R = 160+rand()%32;   vr = 16;
      G = 140+rand()%52;   vg = 16;
      B = 32+rand()%32;    vb = 16;
      va=32;
      break;

    case 9 : // october= max brown
      R = 160+rand()%32;   vr = 32;
      G = 120+rand()%64;   vg = 32;
      B = 32+rand()%32;    vb = 32;
      va=64;
      break;

    case 10 : // november
      //int v = rand()%32;
      R = 174+rand()%16; vr = 16;
      G = 145+rand()%32; vg = 16;
      B = 82+rand()%16;; vb = 16;
      va=96;
      break;

    case 11 : // december
      v = rand()%16;rand();rand();
      R = 178+v; vr = 8;
      G = 172+v; vg = 8;
      B = 132+v; vb = 8;
      va=128;
      break;

  }

  // if param == 0 we display a circle
  if (param==0) {
    Uint32 *px2 = (Uint32*)tc->pixels;
    int i, j;

    for (i=x-r; i<x+r; i++)
      for (j=y-r; j<y+r; j++) {
        int d= (i-x)*(i-x)+(j-y)*(j-y);
        //float val = noise2(((float)x)/(2*r), ((float)y)/(2*r));
        if (/*val>=0 &&*/ d<=r*r && i>=0 && j>=0 && i<TILESIZE && j<TILESIZE) {
          unsigned char z=((r*r-d));
          if(z>zbuf[j*TILESIZE+i]) {
            zbuf[j*TILESIZE+i]=z;
            //unsigned int pix=0x00aa00;
            int ratio = 4*(i-x+r+j-y+r);
            unsigned char rl = (R+rand()%vr);
            unsigned char gl = (G+rand()%vg);
            unsigned char bl = (B+rand()%vb);
            if (rl>ratio) rl = rl-ratio; else rl = 0;
            if (gl>ratio) gl = gl-ratio; else gl = 0;
            if (bl>ratio) bl = bl-ratio; else bl = 0;
            char hfr = rand()%10;
            switch (hfr){
              case 0 : rl = rl>>1;gl = gl>>1;bl = bl>>1; break;
              case 1 :
              case 2 : rl = (rl>>1)+(rl>>2);gl = (gl>>1)+(gl>>2);bl = (bl>>1)+(bl>>2); break;
              case 3 : 
              case 4 :
              case 5 : rl = (rl>>1)+(rl>>2)+(rl>>3);gl = (gl>>1)+(gl>>2)+(gl>>3);bl = (bl>>1)+(bl>>2)+(bl>>3); break;
              default: break;
            } 
            //unsigned char lInit = (rl+gl+bl)/3;
            //float ratio = ((64-(i-x+r)-(j-y+r)))/(float)lInit;
            //if (i==32 && j==32)
            //printf("ratio=%f    lInit=%i      autre=%i\n",ratio, lInit, ((128-(i-x+r)-(j-y+r))));
            if (rand()%255>va)
              px2[j*tc->pitch/4+i] = ((unsigned char)(rl)<<16)+((unsigned char)(gl)<<8)+((unsigned char)(bl)<<0);
            else {
              Uint32 oldc = (px2[j*tc->pitch/4+i]&0xFEFEFEFE)>>1;
              px2[j*tc->pitch/4+i] = oldc+((unsigned char)(rl/2)<<16)+((unsigned char)(gl/2)<<8)+((unsigned char)(bl/2)<<0);
            }

          }
        }
      }
  } else if (param==1) {
    Uint32 *px1 = (Uint32*)forestImages[FOREST1]->pixels;
    Uint32 *px2 = (Uint32*)tc->pixels;
    int i, j;
    for (i=x-r; i<x+r; i++)
      for (j=y-r; j<y+r; j++) {
        int d= (int)(hypot(i-x,j-y));
        if (d<=r && i>=0 && j>=0 && i<TILESIZE && j<TILESIZE) {
          unsigned char z=(r-d)*10;
          if(z>zbuf[j*TILESIZE+i]) {
            zbuf[j*TILESIZE+i]=z;
            unsigned int pix=px1[j*forestImages[FOREST1]->pitch/4+i];
            unsigned char R = (pix&0x00ff0000)>>16;
            unsigned char G = (pix&0x0000ff00)>>8;
            unsigned char B = (pix&0x000000ff)>>0;
            unsigned char lInit = (R+G+B)/3;
            float ratio = ((64-(i-x+r)-(j-y+r)))/(float)lInit;
            px2[j*tc->pitch/4+i] = ((unsigned char)(R*ratio)<<16)+((unsigned char)(G*ratio)<<8)+((unsigned char)(B*ratio)<<0);
          }
        }
      }
  } else {
    // else we use two different tree images, the one we choose will depend on the size required
    Uint32 *px1;
    if (r<=8) {
      px1 = (Uint32*)treeImages[TREE5]->pixels;
    } else if (r<=16) {
      px1 = (Uint32*)treeImages[TREE6]->pixels;
    } else {
      px1 = (Uint32*)treeImages[TREE7]->pixels;
    }
    Uint32 *px2 = (Uint32*)tc->pixels;
    int i, j;
    unsigned int pix_save=0x0000ff00;
    for (i=x-r; i<x+r; i++)
      for (j=y-r; j<y+r; j++) {
        int d= (i-x)*(i-x)+(j-y)*(j-y);
        if (d<=r*r && i>=0 && j>=0 && i<TILESIZE && j<TILESIZE) {
          unsigned char z=((r*r-d));

          if(z>zbuf[j*TILESIZE+i]) {
            zbuf[j*TILESIZE+i]=z;
            unsigned int pix=0x0000ff00;
            if (r<=8) {
              int tmp_i=(i-(x-r)+1)*8/(2*r+1);
              int tmp_j=(j-(y-r)+1)*8/(2*r+1);
              pix=px1[tmp_j*treeImages[TREE5]->pitch/4+tmp_i];
            } else if (r<=12) {
              int tmp_i=(i-(x-r)+1)*12/(2*r+1);
              int tmp_j=(j-(y-r)+1)*12/(2*r+1);
              pix=px1[tmp_j*treeImages[TREE6]->pitch/4+tmp_i];
            } else {
              int tmp_i=(i-(x-r)+1)*16/(2*r+1);
              int tmp_j=(j-(y-r)+1)*16/(2*r+1);
              pix=px1[tmp_j*treeImages[TREE7]->pitch/4+tmp_i];
            }
            if (pix==0) {
              pix=pix_save;
            } else {
              pix_save=pix;
            }

            unsigned char R = (pix&0x00ff0000)>>16;
            unsigned char G = (pix&0x0000ff00)>>8;
            unsigned char B = (pix&0x000000ff)>>0;
            unsigned char lInit = (R+G+B)/3;
            float ratio = ((64-(i-x+r)-(j-y+r)))/(float)lInit;
            px2[j*tc->pitch/4+i] = ((((unsigned char)(R*ratio))&0xff)<<16)+((((unsigned char)(G*ratio))&0xff)<<8)+((((unsigned char)(B*ratio))&0xff)<<0)+(0xff000000);
          }
        }
      }
  }
}// tree3d() --------------------------------------------------------------------


/**
 * draw a field for agriculture using a texture
 */
void blitField(SDL_Surface *tc, int x, int y, int w, int h, int fieldNumber) {
  SDL_Rect *rect = (SDL_Rect *)malloc(sizeof(SDL_Rect));
  rect->x = x;
  rect->y = y;
  rect->w = w;
  rect->h = h;
  SDL_BlitSurface(fieldImages[fieldNumber], rect, tc, rect);
  free(rect);
}// blitField()


/**
 *  get color from nation and
 *   alpha is an int value between 0 and 100
 *   draw a color with given alpha on a pixel
 */
void drawPixelInfluence(SDL_Surface *tc,int nation,int x, int y,float alpha) {
  Uint32 *pix = (Uint32*)tc->pixels;
  SDL_Color *col = nationColors[nation];
  switch (((int)alpha)/10) {
  case 0:
    pix[y*tc->pitch/4+x] = SDL_MapRGB( tc->format, col->r, col->g, col->b );
    break;
  case 1:
    pix[y*tc->pitch/4+x] += (SDL_MapRGB( tc->format, col->r, col->g, col->b )+0x00000022);
    break;
  case 2:
    pix[y*tc->pitch/4+x] += (SDL_MapRGB( tc->format, col->r, col->g, col->b )+0x00000044);
    break;
  case 3:
    pix[y*tc->pitch/4+x] += (SDL_MapRGB( tc->format, col->r, col->g, col->b )+0x00000066);
    break;
  case 4:
    pix[y*tc->pitch/4+x] += (SDL_MapRGB( tc->format, col->r, col->g, col->b )+0x00000088);
    break;
  case 5:
    pix[y*tc->pitch/4+x] += (SDL_MapRGB( tc->format, col->r, col->g, col->b )+0x000000aa);
    break;
  case 6:
    pix[y*tc->pitch/4+x] += (SDL_MapRGB( tc->format, col->r, col->g, col->b )+0x000000cc);
    break;
  case 7:
    pix[y*tc->pitch/4+x] += (SDL_MapRGB( tc->format, col->r, col->g, col->b )+0x000000dd);
    break;
  case 8:
    pix[y*tc->pitch/4+x] += (SDL_MapRGB( tc->format, col->r, col->g, col->b )+0x000000ee);
    break;
  case 9:
    pix[y*tc->pitch/4+x] += (SDL_MapRGB( tc->format, col->r, col->g, col->b )+0x000000ff);
    break;
  default:
    break;
  }
}//drawPixelInfluence()


/**
 *  bool parameters are set to true if we must draw a border
 */
void drawInfluence(SDL_Surface *tc, int nation, bool topLeft, bool top, bool topRight,
                   bool left, bool right, bool botLeft, bool bot, bool botRight) {
  //Uint32 *pxTc = (Uint32*)tc->pixels;
  // transparency
  float alpha = 0;

  // references used to draw
  int xRef = 0;
  int yRef = 0;
  int xCenter = 0;
  int yCenter = 0;
  int distance = 0;

  // variables used for segmentation
  float smallCoeff = 1/4;
  float bigCoeff   = 3/4;

  if (top) {
    // drawing top border
    for (int y = 0; y<TILESIZE*smallCoeff;y++) {
      for (int x=(int)(TILESIZE*smallCoeff); x<TILESIZE*bigCoeff; x++) {
        alpha = (float)max((100-y*y),0);
        drawPixelInfluence(tc,nation,x,y,alpha);
      }
    }
    // extend in case there is no corner topRight
    if (!topRight) {
      for (int y = 0; y<TILESIZE*smallCoeff;y++) {
        for (int x=(int)(TILESIZE*bigCoeff); x<TILESIZE; x++) {
          alpha = (float)max((100-y*y),0);
          drawPixelInfluence(tc,nation,x,y,alpha);
        }
      }
    }
    // extend in case there is no corner topLeft
    if (!topLeft) {
      for (int y = 0; y<TILESIZE*smallCoeff;y++) {
        for (int x=0; x<TILESIZE*smallCoeff; x++) {
          alpha = (float)max((100-y*y),0);
          drawPixelInfluence(tc,nation,x,y,alpha);
        }
      }
    }
  }

  if (right) {
    // drawing right border
    for (int y = (int)(TILESIZE*smallCoeff); y<TILESIZE*bigCoeff; y++) {
      for (int x=(int)(TILESIZE*bigCoeff); x<TILESIZE; x++) {
        xRef = TILESIZE-x;
        alpha = (float)max((100-xRef*xRef),0);
        drawPixelInfluence(tc,nation,x,y,alpha);
      }
    }
    // extend in case there is no corner topRight
    if (!topRight) {
      for (int y=0; y<TILESIZE*smallCoeff; y++) {
        for (int x=(int)(TILESIZE*bigCoeff); x<TILESIZE; x++) {
          xRef = TILESIZE-x;
          alpha = (float)max((100-xRef*xRef),0);
          drawPixelInfluence(tc,nation,x,y,alpha);
        }
      }
    }
    // extend in case there is no corner botRight
    if (!botRight) {
      for (int y=(int)(TILESIZE*bigCoeff); y<TILESIZE; y++) {
        for (int x=(int)(TILESIZE*bigCoeff); x<TILESIZE; x++) {
          xRef = TILESIZE-x;
          alpha = (float)max((100-xRef*xRef),0);
          drawPixelInfluence(tc,nation,x,y,alpha);
        }
      }
    }
  }

  if (bot) {
    // drawing bot border
    for (int y=(int)(TILESIZE*bigCoeff); y<TILESIZE;y++) {
      for (int x=(int)(TILESIZE*smallCoeff); x<TILESIZE*bigCoeff; x++) {
        yRef=TILESIZE-y;
        alpha = (float)max((100-y*y),0);
        drawPixelInfluence(tc,nation,x,y,alpha);
      }
    }
    // extend in case there is no corner botRight
    if (!botRight) {
      for (int y=(int)(TILESIZE*bigCoeff); y<TILESIZE; y++) {
        for (int x=(int)(TILESIZE*bigCoeff); x<TILESIZE; x++) {
          yRef=TILESIZE-y;
          alpha = (float)max((100-y*y),0);
          drawPixelInfluence(tc,nation,x,y,alpha);
        }
      }
    }
    // extend in case there is no corner botLeft
    if (!botLeft) {
      for (int y=(int)(TILESIZE*bigCoeff); y<TILESIZE;y++) {
        for (int x=0;x<TILESIZE*smallCoeff;x++) {
          yRef=TILESIZE-y;
          alpha = (float)max((100-y*y),0);
          drawPixelInfluence(tc,nation,x,y,alpha);
        }
      }
    }
  }

  if (left) {
    // drawing left corner
    for (int y=(int)(TILESIZE*smallCoeff); y<TILESIZE*bigCoeff;y++) {
      for (int x=0;x<TILESIZE*smallCoeff;x++) {
        alpha = (float)max(0,100-x*x);
        drawPixelInfluence(tc,nation,x,y,alpha);
      }
    }
    // extend in case there is no corner botLeft
    if (!botLeft) {
      for (int y=(int)(TILESIZE*bigCoeff); y<TILESIZE;y++) {
        for (int x=0;x<TILESIZE*smallCoeff;x++) {
          alpha = (float)max(0,100-x*x);
          drawPixelInfluence(tc,nation,x,y,alpha);
        }
      }
    }
    // extend in case there is no corner topLeft
    if (!topLeft) {
      for (int y = 0 ; y < TILESIZE*smallCoeff;y++) {
        for (int x=0;x<TILESIZE*smallCoeff;x++) {
          alpha = (float)max(0,100-x*x);
          drawPixelInfluence(tc,nation,x,y,alpha);
        }
      }
    }
  }

  if (topRight) {
    // drawing topRight corner
    for (int y=0; y<TILESIZE*smallCoeff; y++) {
      for (int x=(int)(TILESIZE*bigCoeff); x<TILESIZE; x++) {
        xCenter=(int)(TILESIZE*bigCoeff);
        yCenter=(int)(TILESIZE*smallCoeff);
        distance=dist(xCenter,yCenter,x,y);
        //test if in semi-circle to draw!
        if (distance<=TILESIZE) {
          alpha = (float)max((100-(TILESIZE-distance)*(TILESIZE-distance)),0);
          drawPixelInfluence(tc,nation,x,y,alpha);
        }
      }
    }
  }

  if (botRight) {
    // drawing botRight corner
    for (int y=(int)(TILESIZE*bigCoeff); y<TILESIZE;y++) {
      for (int x=(int)(TILESIZE*bigCoeff);x<TILESIZE;x++) {
        xCenter=(int)(TILESIZE*bigCoeff);
        yCenter=(int)(TILESIZE*bigCoeff);
        distance=dist(xCenter,yCenter,x,y);
        //test if in semi-circle to draw!
        if (distance<=TILESIZE) {
          alpha = (float)max((100-(TILESIZE-distance)*(TILESIZE-distance)),0);
          drawPixelInfluence(tc,nation,x,y,alpha);
        }
      }
    }
  }

  if (botLeft) {
    // drawing botLeft corner
    for (int y=(int)(TILESIZE*bigCoeff); y<TILESIZE;y++) {
      for (int x=0;x<TILESIZE*smallCoeff;x++) {
        xCenter=(int)(TILESIZE*smallCoeff);
        yCenter=(int)(TILESIZE*bigCoeff);
        distance=dist(xCenter,yCenter,x,y);
        //test if in semi-circle to draw!
        if (distance<=TILESIZE) {
          alpha = (float)max((100-(TILESIZE-distance)*(TILESIZE-distance)),0);
          drawPixelInfluence(tc,nation,x,y,alpha);
        }
      }
    }
  }



  if (topLeft) {
    // drawing topLeft corner
    for (int y = 0; y<TILESIZE*smallCoeff;y++) {
      for (int x=0;x<TILESIZE*smallCoeff;x++) {
        xCenter=(int)(TILESIZE*smallCoeff);
        yCenter=(int)(TILESIZE*smallCoeff);
        distance=dist(xCenter,yCenter,x,y);
        //test if in semi-circle to draw!
        if (distance<=TILESIZE) {
          alpha = (float)max((100-(TILESIZE-distance)*(TILESIZE-distance)),0);
          drawPixelInfluence(tc,nation,x,y,alpha);
        }
      }
    }
  }
}// drawInfluence()


/**
 * return the max of two int
 */
int max(int a, int b) {
  if (a>b)
    return a;
  else
    return b;
}//max()




