/*
 *  extragraphics.h
 *
 *  Created by fredericVernier on 1/25/07.
 *  Copyright 2007 Frederic Vernier. All rights reserved.
 *
 */

#ifndef EXTRAGRAPHICS_H
#define EXTRAGRAPHICS_H

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <kazlib/list.h>
#include <SDL/SDL.h>
#include "newcol.h"
#include <SDL_ttf/SDL_ttf.h>
#pragma warning ( disable : 4996 )

#ifdef __cplusplus
extern "C" {
#endif

  
  
#define FIRST_GROUND     100
#define DESERT           100
#define ROCKDESERT       101
#define PLAIN            102
#define GRASSLAND        103
#define PRAIRIE          104
#define SAVANNAH         105
#define MARSH            106
#define SWAMP            107
#define TUNDRA           108
#define OCEAN            109
#define SEALANE          110
#define ROCK             111
#define ROCK2            112
#define FORESTTREES      113
#define HILLROCK         114
#define ARCTIC           115

#define GRASSLAND1        116
#define GRASSLAND2        117
#define GRASSLAND3        118
#define GRASSLAND4        119
#define GRASSLAND5        120
#define GRASSLAND6        121
#define GRASSLAND7        122
#define GRASSLAND8        123
#define GRASSLAND9        124
#define GRASSLAND10       125
#define GRASSLAND11       126
#define GRASSLAND12       127

#define PLAIN1        128
#define PLAIN2        129
#define PLAIN3        130
#define PLAIN4        131
#define PLAIN5        132
#define PLAIN6        133
#define PLAIN7        134
#define PLAIN8        135
#define PLAIN9        136
#define PLAIN10       137
#define PLAIN11       138
#define PLAIN12       139

#define LAST_GROUND      139
#define GROUND_NUMBER    40
#define LAST_EDIT_GROUND 108

// for shadow xxx filter 
#define SHADOW_SIZE      5  
  
// how important is the center of the tile according to the borders (3 is normal above is more important)
#define CW             4    

// draw a landscape curve every 200m of altitude
#define LC             200  
  
extern SDL_Surface *textures[LAST_GROUND-FIRST_GROUND+1];  
extern SDL_Surface *lastLoadedImage;
extern char        *lastImageNameLoaded;
  
// debug primitives
void SDL_SurfaceInfo(const char * name, const SDL_Surface *thing);

// image loading fcts
SDL_Surface *IMG_LoadAndFormatAlpha(const char* imageName);
SDL_Surface *IMG_LoadAndFormatNoAlpha(const char* imageName);

// special text rendering fcts
SDL_Surface *renderTextShadowed_Blended(TTF_Font *font, const char *text, SDL_Color fg, SDL_Color bg);

// special blit surfaces fcts
void         SDL_BlitSurfaceRGBAMin(SDL_Surface *src, SDL_Rect *rectsrc, SDL_Surface *dst, SDL_Rect *rectdst);
void         SDL_BlitSurfaceRGBAMax(SDL_Surface *src, SDL_Rect *rectsrc, SDL_Surface *dst, SDL_Rect *rectdst);
void         SDL_BlitSurfaceRGBA(SDL_Surface *src, SDL_Rect *rectsrc, SDL_Surface *dst, SDL_Rect *rectdst);
void         SDL_BlitSurfaceRGBAMul(SDL_Surface *src, SDL_Rect *rectsrc, SDL_Surface *dst, SDL_Rect *rectdst);
void         SDL_BlitSurfaceBW    (SDL_Surface *src, SDL_Rect *rectsrc, SDL_Surface *dst, SDL_Rect *rectdst, unsigned char aglob, Uint8 VertMirror);
void         SDL_BlitSurfacePastel(SDL_Surface *src, SDL_Rect *rectsrc, SDL_Surface *dst, SDL_Rect *rectdst, const unsigned char colprop);
void         SDL_BlitSurfaceColW(SDL_Surface *src, SDL_Rect *rectsrc, SDL_Surface *dst, SDL_Rect *rectdst, const unsigned char aglob, Uint8 VertMirror, SDL_Color *col);
void         SDL_BlitSurfaceMirrorV(SDL_Surface *src, SDL_Rect *rectsrc, SDL_Surface *dst, SDL_Rect *rectdst, int ddisp);
void         SDL_BlitSurfaceMirrorH(SDL_Surface *src, SDL_Rect *rectsrc, SDL_Surface *dst, SDL_Rect *rectdst, int ddisp);
void         SDL_BlitSurfaceMirrorHV(SDL_Surface *src, SDL_Rect *rectsrc, SDL_Surface *dst, SDL_Rect *rectdst, int ddisp);
int          isOpaquePixel        (const SDL_Surface *src, const int x, const int y);

// shadows fcts
SDL_Surface *createShadow         (SDL_Surface *src, unsigned char maxTrans, Uint32 color);
SDL_Surface *updateShadow(SDL_Surface *src, SDL_Surface *dst, unsigned char maxTrans, Uint32 color);  

// graphics primitives
void roundedRectangleRGBA(SDL_Surface *dst, Sint16 x, Sint16 y, Sint16 w, Sint16 h, 
                          Uint8 r, Uint8 g, Uint8 b, Uint8 a);
void roundedBoxRGBA(SDL_Surface *dst, Sint16 x, Sint16 y, Sint16 w, Sint16 h, 
                    Uint8 r, Uint8 g, Uint8 b, Uint8 a);
void diamondRGBA(SDL_Surface *dst, Sint16 x, Sint16 y, Sint16 w, Sint16 h, 
                 Uint8 r, Uint8 g, Uint8 b, Uint8 a);
void hgradientboxRGBA(SDL_Surface *dst, Sint16 x, Sint16 y, Sint16 w, Sint16 h, 
                      Uint8 r1, Uint8 g1, Uint8 b1, Uint8 a1, Uint8 r2, Uint8 g2, Uint8 b2, Uint8 a2);
void vgradientboxRGBA(SDL_Surface *dst, Sint16 x, Sint16 y, Sint16 w, Sint16 h, 
                      Uint8 r1, Uint8 g1, Uint8 b1, Uint8 a1, Uint8 r2, Uint8 g2, Uint8 b2, Uint8 a2);
void d1gradientboxRGBA(SDL_Surface *dst, Sint16 x, Sint16 y, Sint16 w, Sint16 h, 
                       Uint8 r1, Uint8 g1, Uint8 b1, Uint8 a1, 
                       Uint8 r2, Uint8 g2, Uint8 b2, Uint8 a2);
void d2gradientboxRGBA(SDL_Surface *dst, Sint16 x, Sint16 y, Sint16 w, Sint16 h,
                       Uint8 r1, Uint8 g1, Uint8 b1, Uint8 a1,
                       Uint8 r2, Uint8 g2, Uint8 b2, Uint8 a2);
void boxRGBAAlpha(SDL_Surface *dst, Sint16 x, Sint16 y, Sint16 w, Sint16 h, 
                  Uint8 r1, Uint8 g1, Uint8 b1, Uint8 a1);
void vgradientIsoTrapezoidRGBA(SDL_Surface *dst, Sint16 x, Sint16 y, Sint16 w1, Sint16 w2, Sint16 h, 
                               Uint8 r1,  Uint8 g1, Uint8 b1, Uint8 a1, Uint8 r2, Uint8 g2, Uint8 b2, Uint8 a2);
void vtextureIsoTrapezoidRGBA(SDL_Surface *dst, Sint16 x, Sint16 y, Sint16 w1, Sint16 w2, Sint16 h, SDL_Surface *texture);

void vgradientboxRGBANoise(SDL_Surface *dst, Sint16 x, Sint16 y, Sint16 w, Sint16 h, Uint8 r1,
                      Uint8 g1, Uint8 b1, Uint8 a1, Uint8 r2, Uint8 g2, Uint8 b2, Uint8 a2, Uint8 noise);
 

// to display path
int     bezier2RGBA(SDL_Surface * dst, const Sint16 * vx, const Sint16 * vy, float coeff1, float coeff2, int n, int s,
                Uint8 r1, Uint8 g1, Uint8 b1, Uint8 a1, Uint8 r2, Uint8 g2, Uint8 b2, Uint8 a2, float percentage, int size);
// to display rivers
int     bezier3Texture(SDL_Surface * dst, const Sint16 * vx, const Sint16 * vy, float coeff1, float coeff2, int n, int s, int w, int dispBorder);

void    landscapeTile(SDL_Surface *dst, SDL_Rect *rect, int borders[3][3], int tt[3][3], int seed, SDL_Surface *fieldTile, const int month, const float lattitude);
void    initTextures();
SDL_Color *getColorforTexture(int tiletype);
void    cityTiteQuarter(SDL_Surface *dst, int x, int y, float length, float mainroad, int dx, int dy, int seed);
void    streetTiteQuarter(SDL_Surface *dst, int x, int y, float length, float mainroad, int dx, int dy);

void    progressiveTexturedCircle(SDL_Surface *tc, int x, int y, int r);
void    texturedCircle(SDL_Surface *tc, int x, int y, int r);
void    treeCircle(SDL_Surface *tc, int x, int y, int r);
void    tree3d(SDL_Surface *tc, int x, int y, int r, unsigned char *zbuf, int param, unsigned char month);
//void    treeShadow(SDL_Surface *tc, int x, int y, int r, unsigned char *zbuf);
void    blitField(SDL_Surface *tc, int xDep, int yDep, int dx, int dy,int fieldNumber);

void    drawPixelInfluence(SDL_Surface *tc,int nation,int x, int y,float alpha);
void    drawInfluence(SDL_Surface *tc,int nation,bool topLeft,bool top,
                      bool topRight,bool left,bool right,bool botLeft,bool bot,bool botRight);

int max(int a, int b);

#ifdef __cplusplus
}
#endif
#endif
