/*
 *  newcol.h
 *  SDLApp
 *
 *  Created by fredericVernier on 1/22/07.
 *  Copyright 2007 Frederic Vernier. All rights reserved.
 *
 */


#ifndef NEWCOL_H
#define NEWCOL_H

#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <math.h>
#include <string.h>
//#include <cstdlib>

#include <SDL/SDL.h>
#include <SDL_mixer/SDL_mixer.h>
#include <SDL_ttf/SDL_ttf.h>
#include <SDL_image/SDL_image.h>
#include <SDL_gfx/SDL_gfxPrimitives.h>
#include <SDL_gfx/SDL_rotozoom.h>
#include "typenewcol.h"
#include <kazlib/list.h>
#include "binarytree.h"
#include "astar.h"
#include "extragraphics.h"
#include "cityview.h"
#include "minimap.h"
#include "unitview.h"
#include "popup.h"
#include "voronoi.h"
#include "initialization.h"
#include "improvedPerlinNoise.h"

#ifdef __cplusplus
extern "C" {
#endif

#define  MAXINITCPT 1693
#define  FINISHLOADINGCPT 897

// global variable of the app
extern SDL_Surface  *screen;
extern SDL_Surface  *helperTile;
extern TTF_Font     *font;
extern long          elapsedTime;
extern long          lastRefreshInfo;
extern int           isMousePressed;
extern int           mousePressedX;
extern int           mousePressedY;
extern int           isDrag;
extern int           uiPaintNb;
extern int           mouseX;
extern int           mouseY;
extern SDL_Surface **tilecache;
extern TTF_Font     *fonts[LAST_FONT];
extern int           gamemode;
extern int           extensionTable[BUILDING_NUMBER];
extern int           minSizeTable[BUILDING_NUMBER];
extern int           hammerTable[BUILDING_NUMBER];
extern int           toolsTable[BUILDING_NUMBER];
extern int           initcpt;
//extern char         *initmsg;
extern good         *dragGood;
extern unit         *dragUnit;
extern int           mousedX;
extern int           mousedY;
extern int           innitialDragPosX;
extern int           innitialDragPosY;
extern int           apptime;
extern int           animID;

// global variables of the game
extern tile         *map;
extern int           MapWidth;
extern int           MapHeight;
extern char          buildingTexts[BUILDING_NUMBER][256];
extern char          statusTexts[STATUS_NUMBER][256];
extern char        **nationTexts;
extern int           debug;
extern list_t       *messageList;

//extern tile *map;
extern list_t       *riverList;
extern list_t       *playerList;
extern player       *nations[NATION_NUMBER];
extern lnode_t      *currentPlayerElement;
extern unit         *unitSample;
extern int          Mapsize, GameDifficulty;

//extern SDL_Rect    *rectSelection, *rectL, *rectC;
extern int          firstVisibleTileX;
extern int          firstVisibleTileY;
extern list_t      *unitSelectionList;
extern SDL_Surface *pathFootImage,  *pathHorseImage, *pathNavalImage, *pathWagonImage;
extern SDL_Surface *coinImage, *crossImage;

//extern SDL_Surface *indianText0,        *indianText,         *indian0Image;

// could be different for other languages
//extern SDL_Surface *WText, *PText, *RText, *DText, *SText, *FText, *GText, *MText;

extern Mix_Chunk *natureSounds[NATURE_SOUND_NUMBER];

extern SDL_Surface *tree1Image, *tree2Image, *tree3Image, *tree4Image;
extern SDL_Surface *fieldImages[FIELD_NUMBER];
extern SDL_Surface *treeImages[TREE_NUMBER];
extern SDL_Surface *forestImages[FOREST_NUMBER];
extern SDL_Surface *bonusImages[BONUS_NUMBER];
extern SDL_Surface *goodsImages[347];//47+50*6
extern SDL_Surface *unitsImages[300]; // 50*6
extern SDL_Surface *buildingImages[BUILDING_NUMBER];
//extern SDL_Surface *miniBuildingImages[BUILDING_NUMBER];
extern SDL_Surface *buildingIconImages[BUILDING_NUMBER];

extern SDL_Color   *nationColors[NATION_NUMBER];
extern SDL_Surface *nationFlags[NATION_NUMBER];
extern SDL_Surface *nationMiniFlags[NATION_NUMBER];


extern SDL_Color   BLACK;
extern SDL_Color   WHITE;
extern SDL_Color   RED;
extern SDL_Color   LIGHTRED;
extern SDL_Color   GREEN;
extern SDL_Color   LIGHTGREEN;
extern SDL_Color   BLUE;
extern SDL_Color   LIGHTBLUE;
extern SDL_Color   YELLOW;
extern SDL_Color   GREY;
extern SDL_Color   LIGHTGREY;
extern SDL_Color   DARKGREY;
extern SDL_Color   ORANGE;

SDL_Surface *getTileCache(int x, int y);
SDL_Surface *getUnitImage(unit*);
SDL_Surface *getUnitGoodImage(unit *u);
SDL_Surface *getBuildingImage(building*);
SDL_Surface *getGoodImage(good* g);

char        *getUnitText(unit*);
char        *getNationText(int nation);
char        *getStatusText(int status);
char        *getGroundText(tile *t);
char        *getGoodText(good* g);
char        *getBonusText(tile *t);

int          getInfluentNation (int i, int j);
void         addInfluence(int xStart, int yStart, int range, int nation);
void  addMsg(const char* msg0);
void  display();
void  displayUnitList(list_t *unitList, int isDrag);
void  displayPath(unit *u,  list_t *Itinerary);
void  displaySnake(tile *t, list_t *tileList, unit*);
void  displayAStar();
void  displayMap();
void  displayBackground(SDL_Surface *dst, int tiletype, SDL_Rect* rect);
void  panMap(int newfirstVisibleTileX, int newfirstVisibleTileY);
void  updateLoading();
void  showPopupBattle();
int   isEnnemyIn(list_t *ul);

void  updatesImageTile(int , int , SDL_Surface*, int dispBonus);
void  updatesCachedTileAt(int , int );
void  updatesRiverInCache(river *r);
void  updatesRiverNodesOnTile(binarytree_t *tree, btnode_t * nodebefore, btnode_t * node,
                             btnode_t * nodeafter, int fx, int fy, SDL_Surface *img);
void  updatesInfluencesForAll();
void  updatesInfluencesOnTile(int i, int j, SDL_Surface *tc);
void  updatesInfluencesInCache();
void  updatesRoadOnTile(int i, int j, SDL_Surface *tc);
void  updatesVegetationOnTile(int i, int j, SDL_Surface *tc);

void  updatesFieldOnTile(int i, int j, SDL_Surface *tc);
void  updatesCityNamesInCache();
void  updatesCulturesInCache();
void  updatesFogOfWarOnTile(int i, int j, SDL_Surface *tc);
//void  drawField(SDL_Surface *tc, int x, int y, int dx, int dy, bool over, int fieldNumber);
//void  drawTree(SDL_Surface *tc, int x, int y, int r, int green);
void  cacheVegetationOnTile(int vegeLevel, int vegeTop, int vegeRight, int vegeBottom, int vegeLeft, SDL_Surface *tc);

void  updatesCache(int x1, int y1, int x2, int y2);
void  updatesBackgroundInCache(int x1, int y1, int x2, int y2);
void  updatesFastVegetationInCache(int x1, int y1, int x2, int y2);
void  updatesCitiesInCache(int x1, int y1, int x2, int y2);
void  updatesBonusesInCache(int x1, int y1, int x2, int y2);
void  updatesRiversInCache(int x1, int y1, int x2, int y2);
void  updatesRoadsInCache(int x1, int y1, int x2, int y2);
void  updatesFogOfWarInCache(int x1, int y1, int x2, int y2);
void  updatesVegetationInCache(int x1, int y1, int x2, int y2);
void  updatesVegetationShadowInCache(int x1, int y1, int x2, int y2);

// unit and game orders
void doPlow(list_t *ul, char *order);
void doRoad(list_t *ul, char *order);
void doMine(list_t *ul, char *order);
void doFortify(list_t *ul, char *order);
void doWait(list_t *ul, char *order);
void doQuit(list_t *ul, char *order);
void doNothing(list_t *ul, char *order);
void doFoundCity(list_t *ul, char *order);
void doReset(list_t *ul, char *order);
void doGoto(list_t *ul, char *order);
void doFight(list_t *ul, char *order);
void doMove(list_t *ul, char *order);
void doInvade(list_t *ul, char *order);
void doDeclareWar(list_t *ul, char *order);
void doGroup(list_t *ul, char *order);
void doUngroup(list_t *ul, char *order);
void doExplore(list_t *ul, char *order);
void doVisitIndianTribe(list_t *ul, char*order);

void doAskCityNameAndBuild(list_t *ul, char *order);
void doAskCityNameAndGoto(list_t *ul, char *order);
void doAskCountry(list_t *ul, char*order);
void doAskDifficulty(list_t *ul, char*order);
void doAskMapSize(list_t *ul, char*order);


void    panToCenter(tile*);
void    updatesLastExplored();
int     updatesVisibleSurroundings(unit *u);
void    updatesInspector(list_t *ul, list_t *oppl);
void    dispatchBonuses(int);
int     main(int argc, char *argv[]);
int     computeDifficulty(tile *, tile *, unit*);
void    destroyUnit(unit *u);
int     addHorses(unit *u);
int     removeHorses(unit *u);
unit   *createUnit (int, player*);
unit   *createUnitAt (int unitType, player* owner, int x, int y);
battle *prepareBattle();
int     getGlobalPrice(int goodType);
int     getBuyPriceFor(int goodType, city *c, player *p0);
int     getSellPriceFor(int goodType, city *c, player *p0);

list_t *copylist(list_t *src);
int   straightCompare(const void *, const void *);
int   getBonusForTile(tile*);
int   getGoodForBonus(int);
int   isNaval(unit*);
int   isArmy(unit*);
int   isIndian(player *p);
int   isPortAt(tile *t);
int   getBattleStrength(const unit *uni, const tile *loc, const int attack);
void  computeIrrigation() ;
int   computeRiverSize(btnode_t*);
int   isColonyOf(int n1, int n2);
city *pickEmigrationPort(player *king);
city *findCity(char *name);
int   addUnitToBuilding(unit *u, city *c, int buildingType);
int   getConnectedToCoast(int x, int y);
int   getConnectedToOcean(int x, int y);
int   isConnectedToWater(int, int);
int   isThereArmy(int x, int y);
unit *isFreeBoarding(tile *t, unit *passenger);

btnode_t *extendRiverNodes(river *river0, btnode_t *node, int x, int y);
int       createRoadBetweenCities(const city *c1, const city *c2);
void      attachRiverToMap_r(btnode_t *node);
int       computeRiverSize_r(btnode_t *node);

float ranf();
float gaussian();
float absolu(float x);
float positiveGaussian();

#ifdef __cplusplus
}
#endif
#endif
