/*
 *  tree.h
 *  VisBoardGame
 *
 *  Created by fredericVernier on 2/4/07.
 *  Copyright 2007 Frederic Vernier. All rights reserved.
 *
 */


#ifndef TREE_H
#define TREE_H

#include <limits.h>
#include <kazlib/list.h>


#ifdef __cplusplus
extern "C" {
#endif

typedef unsigned long treecount_t;
#define TREECOUNT_T_MAX ULONG_MAX

typedef struct tnode_t {
  list_t         *children;
  struct tnode_t *father;
  void           *data;  
} tnode_t;

typedef struct tree_t {
  tnode_t    *root;
  treecount_t nodecount;
  treecount_t maxcount;
} tree_t;

tnode_t    *tnode_create(void *);
tnode_t    *tnode_init(tnode_t *, void *);
void        tnode_destroy(tnode_t *);
void        tnode_put(tnode_t *, void *);
void       *tnode_get(tnode_t *);
int         tnode_is_in_a_tree(tnode_t *);

tree_t     *tree_init(tree_t *, treecount_t);
tree_t     *tree_create(treecount_t);
void        tree_destroy(tree_t *);
void        tree_destroy_nodes(tree_t *);

treecount_t tree_count(tree_t *);
int         tree_isempty(tree_t *);
int         tree_isfull(tree_t *);
int         tree_contains(tree_t *, tnode_t *);

void        tree_append(tree_t *,     tnode_t *);
void        tree_prepend(tree_t *,    tnode_t *);
void        tree_ins_under(tree_t *, tnode_t *, tnode_t *);
void        tree_ins_after(tree_t *,  tnode_t *, tnode_t *);

tnode_t *tree_root(tree_t *);
tnode_t *tree_delete(tree_t *, tnode_t *);

void tree_process(tree_t *, void *, void (*)(tree_t *, tnode_t *, void *));

int tree_verify(tree_t *);

/* destination tree on the left, source on the right */
void     tree_extract(tree_t *, tree_t *, tnode_t *, tnode_t *);
void     tree_transfer(tree_t *, tree_t *, tnode_t *first);
void     tree_merge(tree_t *, tree_t *, int (const void *, const void *));
void     tree_sort(tree_t *, int (const void *, const void *));
tnode_t *tree_find(tree_t *, const void *, int (const void *, const void *));
int      tree_is_sorted(tree_t *, int (const void *, const void *));

#ifdef __cplusplus
}
#endif

#endif
