/*
 *  typenewcol.h
 *  NewCol
 *
 *  Created by frederic Vernier on 2/28/07.
 *  Copyright 2007 Frederic Vernier. All rights reserved.
 *
 */

#ifndef TYPENEWCOL_H
#define TYPENEWCOL_H

#include <kazlib/list.h>
#include "binarytree.h"
#include <SDL/SDL.h>

#define APPNAME "NewCol"
#define VERSION "0.3.2"
#define TITLE_BAR_HEIGHT 20

#ifndef NULL
#define NULL 0
#endif

#define DELETED -2

#ifndef MIN
#define MIN(a, b)  (((a) < (b)) ? (a) : (b))
#endif

#ifndef MAX
#define MAX(a, b)  (((a) > (b)) ? (a) : (b))  
#endif  

#ifndef ABS
#define ABS(a)  (((a) > (0)) ? (a) : (-a))  
#endif  

#define NONE                     0
#define MAX_CITY_RADIUS          3
#define CROSSES_FOR_NEW_COLONIST 100
#define BIG_RIVER_MIN_SIZE       10
#define GRAB_WING                10
#define MARGIN                   2

// dragging stuff
#define UNIT            1
#define MAP             2
#define MINIMAP         3
#define MINIPOS         4
#define DRAGMAP         5
#define CITYVIEW        6
#define UNITVIEW        7
#define INNERCITYVIEW   8
#define INNERUNITVIEW   9
#define GOODS           10

// what the user can add(fields) or remove(trees) to a tile
#define FOREST         -40
#define WOOD           -20
#define WILDGRASS       0
#define PLOWED         20
#define IRRIGATION     40

// other tile adds-on
#define MINE           100
#define PLANTATION     102

// general modes
#define FIRST_MODE     200
#define EDIT           200
#define PLAY           201
#define DEBUG          202
#define LAST_MODE      202

// edition mode
#define E_UNIT         0
#define E_ALTITUDE     1
#define E_CITY         2
#define E_FOREST       3
#define E_RIVER        4
#define E_TRANSPORT    5
#define E_GROUND       6
#define E_BONUS        7
#define E_INFLUENCE    8

// city type
/*#define VILLAGE        300
#define EUROPEAN       301
#define CITY           302
#define INDIAN1        303
#define INDIAN2        304
#define AZTEC          305
#define INCA           306*/

// climates
#define DRY            400
#define TEMPERATE      401
#define HUMID          402
#define COLD           403

#define N              1
#define S              2
#define E              4
#define W              8
#define NW             16
#define SW             32
#define NE             64
#define SE             128

// tile adds-on
#define ROAD           20
#define HIGHWAY        40
#define RAILWAY        60
#define WATER          1024

#define NS             500
#define EW             501
#define NSW            502
#define SWE            503
#define WEN            504
#define ENS            505

// unit types
#define FIRST_UNIT     600
#define COLONIST       600
#define FARMER         601
#define FISHERMAN      602
#define FURTRAPPER     603
#define SILVERMINER    604
#define LUMBERJACK     605
#define OREMINER       606
#define SUGARPLANTER   607
#define COTTONPLANTER  608
#define TOBACCOPLANTER 609
#define PREACHER       610
#define STATEMAN       611
#define CARPENTER      612
#define DISTILLER      613
#define WEAVER         614
#define TOBACCONIST    615
#define FURTRADDER     616
#define BLACKSMITH     617
#define GUNSMITH       618
#define SCOUT          619
#define PIONEER        620
#define SOLDIER        621
#define SERVANT        622
#define CRIMINAL       623
#define CONVERT        624
#define COLONIAL       625

// foreign only 
#define INDIAN         626
#define ROYAL          627

//special units
#define CARAVEL        628
#define FRIGATE        629
#define GALLEON        630
#define MAN_O_WAR      631
#define MERCHANTMAN    632
#define PRIVATEER      633
#define ARTILLERY      634
#define DAM_ARTILLERY  635
#define TREASURE       636
#define WAGON          637
#define MILKMAID       638
#define MORMON         639
#define ROBBER         640
#define PIRATE         641
#define POOR           642
#define MISSIONNARY    643

#define LAST_UNIT      637
#define LAST_SPECIALTY 627
#define FIRST_SPECIAL  628
#define LAST_SPECIAL   643
#define LAST_EUROPEAN_UNIT 625
#define SPECIALTY_NUMBER 28
#define SPECIAL_NUMBER   16
#define UNIT_NUMBER    45

// unit adds-on
#define WITHNONE          0
#define WITHHORSES        1
#define WITHGUNS          2
#define WITHHORSEANDGUNS  3
#define WITHTOOLS         4
#define WITHHORSEANDTOOLS 5
#define POSSIBLE_UNITIMG  6
#define MAX_EXPERIENCE    5

// tile elevations
#define OCEAN1         -2 // -1076m
#define OCEAN0         -1 // -308m
#define SEALEVEL        0 // 768 m
#define HILL            1 // around 1228 metres high
#define HIGHHILL        2 // around 1996 metres high
#define MOUNTAIN        3 // around 2764 metres high
#define HIGHMOUNTAIN    4 // around 3532 metres high
#define FIRST_ELEVATION -2
#define LAST_ELEVATION   4
#define ELEVATION_NUMBER 7

// tile irrigation
#define UNKNOWN        -1
#define NOWATER        0
#define JUST_RAIN      1
#define RIVER          2
#define HUMANWATER     3
#define BIG_RIVER      4
#define LAKE           5
#define MAX_TILE_FOR_LAKE 20

// raw goods
#define FIRST_GOOD     700
#define FIRST_RAW_GOOD 700
#define CORN           700
#define SUGAR          701
#define TOBACCO        702
#define COTTON         703
#define FUR            704
#define LUMBER         705
#define ORE            706
#define COAL           707
#define SILVER         708
#define GOLD           709
#define GRAPE          710
#define SALTPETRE      711
#define RICE           712
#define HEMP           713
#define HERBS          714
#define GEM            715
#define STONE          716
#define WHEAT          717
#define SPICE          718

#define FISH           719
#define WHALE          720
#define CLAMS          721
#define PEARL          722
#define COW            723
#define SHEEP          724
#define OIL            725
#define PIG            726
#define BUFFALO        727
#define LAST_RAW_GOOD  727
#define RAW_GOODS_NUMBER     28

//refined goods
#define FIRST_REFINED_GOOD 728
#define RUM            728
#define CIGAR          729
#define TEXTILE        730
#define LEATHER        731
#define HORSE          732
#define TOOL           733
#define MUSKET         734
#define CLOTH          735
#define COAT           736
#define WINE           737
#define GUNPOWDER      738
//#define TRADEGOOD      739
#define ROPE           739
#define MEDECINE       740
#define JEWEL          741
#define DYES           742
#define BUILDING_STONE 743
#define REFINED_GOODS_NUMBER 16

// virtuals goods
#define FIRST_VIRTUAL_GOOD 744
#define LIBERTYBELL    744
#define CROSS          745
#define HAMMER         746
#define POPULATION     747
#define MONEY          748
#define KNOWLEDGE      749
#define MORALE         750
#define HEALTH         751
#define VIRTUAL_GOODS_NUMBER 8

#define LAST_GOOD      751
#define GOODS_NUMBER   52

// bonuses
#define FIRST_BONUS    700
#define PURESILVER     700
#define PRIMECORN      701
#define PRIMETOBACCO   702
#define PRIMECOTTON    703
#define PRIMESUGAR     704
#define PRIMEFISH      705
#define PUREORE        706
#define PRIMEFUR       707
#define PRIMELUMBER    708
#define PRIMEOIL       709
#define PRIMEGAME      710
#define PRIMECOW       711
#define PRIMEPIG       712
#define PRIMESHEEP     713
#define PRIMEBUFFALO   714
#define PURECOAL       715
#define PUREGOLD       716
#define PRIMEWHALE     717
#define PRIMECLAMS     718
#define PRIMEGEM       719
#define PRIMEHEMP      720
#define PRIMEDYES      721
#define PRIMESALTPETRE 722
#define LAST_BONUS     722
#define BONUS_NUMBER    23

// happy nations
#define FIRST_NATION   800
#define FRANCE         800
#define SPAIN          801
#define ENGLAND        802
#define HOLLAND        803
#define ITALY          804
#define GERMANY        805
#define RUSSIA         806
#define PORTUGAL       807
#define NEW_FRANCE     808
#define NEW_SPAIN      809
#define NEW_ENGLAND    810
#define NEW_HOLLAND    811
#define NEW_ITALY      812
#define NEW_GERMANY    813
#define NEW_RUSSIA     814
#define NEW_PORTUGAL   815
#define AZTEC          816
#define OLMEC          817
#define INCA           818
#define APACHE         819
#define HURON          820
#define CHEROKEE       821
#define IROQUOIS       822
#define SIOUX          823
#define TUPI           824
#define ARAWAK         825
#define CHINOOK        826
#define INUIT          827
#define LAST_NATION    827
#define NATION_NUMBER  28
#define OLD_POWER_NUMBER 8
#define NEW_POWER_NUMBER 8
#define FIRST_OLD_POWER 800
#define LAST_OLD_POWER  807
#define FIRST_NEW_NATION 808
#define LAST_NEW_NATION  815
#define FIRST_INDIAN_NATION 816
#define LAST_INDIAN_NATION  827

#define NATIVE_POWER_NUMBER 12

// status possible for units
#define FIRST_STATUS   900
#define WAITING        900
#define GOINGTO        901
#define PLOWING        902
#define ROADING        903
#define DIGGING        904
#define FORTIFYING     905
#define SENTRY         906
#define MIXEDSTATUS    907
#define ONBOARD        908

#define PRODUCING      909
#define TEACHING       910
#define FIRST_OUTSIDE_STATUS 911
#define CORNING        911
#define FISHING        912
#define TOBACCOTING    913
#define COTTONING      914
#define SUGARING       915
#define FURING         916
#define LUMBERJACKING  917
#define OREING         918
#define COALING        919
#define SILVERING      920
#define GOLDING        921
#define GRAPEING       922
#define HERBING        923
#define SPICING        924
#define SALTPETERING   925
#define GEMING         926
#define OILING         927
#define RICING         928
#define HEMPING        929
#define STONING        930
#define WHEATING       931
#define WHALEING       932
#define CLAMSING       933
#define PEARLING       934
#define COWING         935
#define SHEEPING       936
#define PIGING         937
#define BUFFALOING     938
#define LAST_STATUS    938
#define STATUS_NUMBER  39

// buildings
#define FIRST_BUILDING       1000
#define NEW_WORLD_OFFICE     1000
#define ALCHEMIST_HOUSE      1001
#define ALCHEMIST_SHOP       1002
#define ARMOURY              1003
#define ARSENAL              1004
#define BANK                 1005
#define BARRACKS             1006
#define CARPENTRY_SHOP       1007
#define CHURCH               1008
#define CIGAR_FACTORY        1009
#define TEXTILE_FACTORY      1010
#define FUR_FACTORY          1011
#define COLLEGE              1012
#define COMMAND_POST         1013
#define CONGRESS_HALL        1014
#define CONSTRUCTION_YARD    1015
#define CUSTOM_HOUSE         1016
#define DISTILLER_HOUSE      1017
#define DISTILLER_SHOP       1018
#define DOCK                 1019
#define DRUGSTORE            1020
#define FORGE                1021
#define FUR_WORKER_HOUSE     1022
#define FUR_WORKER_SHOP      1023
#define GRANARY              1024
#define GROCERY              1025
#define GUNPOWDER_FACTORY    1026
#define IRONWORKS            1027
#define JEWELLER_HOUSE       1028
#define JEWELLER_SHOP        1029
#define JEWELWORKS           1030
#define COAT_FACTORY         1031
#define COAT_TRADER_HOUSE    1032
#define COAT_TRADER_SHOP     1033
#define LIGHTHOUSE           1034
#define MAGAZINE             1035
#define MARKETPLACE          1036
#define NEWSPAPER            1037
#define PHARMACIST_HOUSE     1038
#define PHARMACIST_SHOP      1039
#define PRINTING_PRESS       1040
#define ROPE_BINDER_HOUSE    1041
#define ROPE_BINDER_SHOP     1042
#define ROPE_FACTORY         1043
#define RUM_FACTORY          1044
#define SAWMILL              1045
#define SCHOOLHOUSE          1046
#define SMITHY               1047
#define STABLE               1048
#define TAILOR_HOUSE         1049
#define TAILOR_SHOP          1050
#define TAVERN               1051
#define CLOTH_FACTORY        1052
#define TOBACCONIST_HOUSE    1053
#define TOBACCONIST_SHOP     1054
#define TOWN_HALL1           1055
#define TOWN_HALL2           1056
#define TOWN_HALL3           1057
#define TOWN_STATUE          1058
#define TRADE_CONDOMINIUM    1059
#define UNIVERSITY           1060
#define WAR_ACADEMY          1061
#define WAREHOUSE1           1062
#define WAREHOUSE2           1063
#define WATCH_TOWER          1064
#define WATER_STORAGE        1065
#define WATER_TOWER          1066
#define WEAPON_STORAGE       1067
#define WEAVER_HOUSE         1068
#define WEAVER_SHOP          1069
#define WELL                 1070
#define WIND_MILL            1071
#define WINE_FACTORY         1072
#define WINE_HOUSE           1073
#define WINE_SHOP            1074
#define WORKSHOP             1075
#define SHIPYARD             1076
#define DRY_DOCK             1077
#define PORT                 1078
#define HARBOUR              1079
#define JESUIT_CHAPEL        1080
#define CATHEDRAL            1081
#define WAREHOUSE3           1082
#define STOCK_EXCHANGE       1083
#define STOCKADE             1084
#define FORT                 1085
#define FORTRESS             1086
#define LAST_BUILDING        1086
#define BUILDING_NUMBER      87

// sounds
#define FIRST_SOUND           1300
#define OCEAN_SOUND           1300
#define GRASSLANDFOREST_SOUND 1301
#define SAVANNAHFOREST_SOUND  1302
#define LAST_SOUND            1302
#define NATURE_SOUND_NUMBER   3

// religions
#define TRADITIONAL_CATHOLICISM 1100
#define EXTREME_ISLAM           1101
#define FUNDAMENTALIST_ANGLICAN 1102
#define FREE_RELIGION           1103
#define GENERAL_CHRISTIANISM    1104
#define PROTESTANTISM           1105
#define SOFT_CATHOLICISM        1106
#define ORTHODOX                1107
#define ANIMISM                 1108

// diplomatic relations
#define BROTHERHOOD          1200
#define FRIENDLY             1201
#define PEACE                1202
#define UPSET                1203
#define ANGRY                1204
#define COLD_WAR             1205
#define WAR                  1206
#define DEATH_WAR            1207

// message's illustration
#define FIRST_MESSAGE        1300
#define COLONY_BURNT         1300
#define NEW_COLONY           1301
#define BATTLE_SOLDIER_ATT   1302
#define BATTLE_SHIPS1        1303
#define BATTLE_ART_DEF_OUT   1304
#define BATTLE_ART_DEF_IN    1305
#define BATTLE_SOLDIER_DEF   1306
#define BATTLE_ART_ATT       1307
#define BATTLE_CAV_ATT       1308
#define BATTLE_INDIAN        1309
#define LAST_MESSAGE         1309
#define MESSAGE_NUMBER       10

// game's difficulties
#define FIRST_DIFFICULTY     1400
#define DISCOVERER           1400
#define EXPLORER             1401
#define CONQUISTADOR         1402
#define GOVERNOR             1403
#define VICEROY              1404
#define LAST_DIFFICULTY      1404
#define DIFFICULTY_NUMBER    5

// map's size
#define FIRST_MAPSIZE        1500
#define M10800_5400          1500
#define M5400_2700           1501
#define M2700_1350           1502
#define M1350_765            1503
#define M1024_512            1504
#define M675_338             1505
#define LAST_MAPSIZE         1405
#define MAPSIZE_NUMBER       6

// city types
#define FOOD_CITY            1
#define RAISING_CITY         2
#define MINE_CITY            4
#define FOREST_CITY          8
#define FISHING_CITY         16
#define AGRICULTURE_CITY     32
#define INDUSTRIAL_CITY      64
#define COMMERCIAL_CITY      128
#define ACADEMIC_CITY        256
#define LUXURY_CITY          512
#define KNOWN_CITY           1024

// some trees
#define FIRST_TREE           1
#define TREE1                1
#define TREE2                2
#define TREE3                3
#define TREE4                4
#define TREE5                5
#define TREE6                6
#define TREE7                7
#define TREE_NUMBER          7

// some fields
#define FIRST_FIELD          1
#define FIELD1               1
#define FIELD2               2
#define FIELD3               3
#define FIELD4               4
#define FIELD5               5
#define FIELD_NUMBER         5

//some forests
#define FOREST1              0
#define FOREST2              1
#define FOREST_NUMBER        2

#define FIRST_FONT           8
#define LAST_FONT            26
#define FONT_NUMBER          (LAST_FONT-FIRST_FONT+1)

#define TILESIZE             64
#define FALSE                0
#define TRUE                 (!0)

#define POPUP_MENU           3
#define POPUP_MESSAGE        1
#define POPUP_ISVISIBLE      1
#define POPUP_ISMENU         2
#define POPUP_OPTIONS        4
#define POPUP_YESNO          8
#define POPUP_OKCANCEL       12
#define POPUP_TEXT           16

// city and player name
#define MAXNAMELENGTH        64

#define DEFAULT_NEW_FRENCH_PLAYER_NAME  "Jacques Cartier"
#define DEFAULT_NEW_SPANISH_PLAYER_NAME "Christopher Columbus"
#define DEFAULT_NEW_ENGLISH_PLAYER_NAME "Walter Raleigh"
#define DEFAULT_NEW_DUTCH_PLAYER_NAME   "Michiel de Ruyter"
#define DEFAULT_FRENCH_PLAYER_NAME      "Louis XIV"
#define DEFAULT_SPANISH_PLAYER_NAME     "Philip II"
#define DEFAULT_ENGLISH_PLAYER_NAME     "Elisabeth I"
#define DEFAULT_DUTCH_PLAYER_NAME       "William I"
#define DEFAULT_AZTEC_PLAYER_NAME       "King of Aztec"
#define DEFAULT_OLMEC_PLAYER_NAME       "King of Olmec" 
#define DEFAULT_INCA_PLAYER_NAME        "Atahualpa"
#define DEFAULT_APACHE_PLAYER_NAME      "Cochise" 
#define DEFAULT_HURON_PLAYER_NAME       "Big Turtle"
#define DEFAULT_CHEROKEE_PLAYER_NAME    "Dragging Canoe"
#define DEFAULT_IROQUOIS_PLAYER_NAME    "Hiawatha"
#define DEFAULT_SIOUX_PLAYER_NAME       "Sitting Bull"
#define DEFAULT_TUPI_PLAYER_NAME        "Curupeba"
#define DEFAULT_ARAWAK_PLAYER_NAME      "Cacique of Arawak"
#define DEFAULT_CHINOOK_PLAYER_NAME     "Polack"
#define DEFAULT_INUIT_PLAYER_NAME       "Inuit chief"

#define DEFAULT_NEW_ITALIAN_PLAYER_NAME "???"
#define DEFAULT_NEW_GERMAN_PLAYER_NAME  "???"
#define DEFAULT_NEW_RUSSIAN_PLAYER_NAME "???"
#define DEFAULT_NEW_PORTUGUESE_PLAYER_NAME "???"
#define DEFAULT_ITALIAN_PLAYER_NAME    "???"
#define DEFAULT_GERMAN_PLAYER_NAME     "???"
#define DEFAULT_RUSSIAN_PLAYER_NAME    "???"
#define DEFAULT_PORTUGUESE_PLAYER_NAME "???"

typedef char* string;

typedef struct _city city;
typedef struct _tile tile;
typedef struct _unit unit;
typedef struct _river river;
typedef struct _player player;
typedef struct _good good;
typedef struct _building building;
typedef struct _option option;
typedef struct _unitview unitview;
typedef struct _battle battle;

// structures
struct _tile {
  Uint16    posX;
  Uint16    posY;
  Sint8     ground;
  Sint8     vegetation;
  Sint8     elevation;
  Sint8     underground;

  city     *construction;
  //int       climate;
  btnode_t *river_elem;
  list_t   *unitList;
  int      *influences;
  Uint32    seed;
  Sint8     road;
  Sint8     lastExplored;
  Sint8     bonus;
  Sint8     irrigation;
  // influences def
};

struct _unit {
  unsigned int  id;
  int           type;
  int           status;
  unsigned char muskets;
  unsigned char tools;
  int           horses; // could be negative for certain special unit draw A* paths
  //int           grade;
  unsigned char movement;
  unsigned char maxMovement;
  unsigned char maxLoad;
  unsigned char sight;  
  good         *goods;
  tile         *location;
  tile         *destination;
  player       *owner;
  list_t       *itinerary;
  list_t       *group;
  //int       isBlessed;
  building     *lastBuilding;
  unsigned char lastWorkingTile; // id of the tile in the dependenciy list lastcity->dependencies[lastWorkingTile]
  unsigned char maxHealth;
  unsigned char health;
  unsigned char experience;
  city         *lastCity;
};

struct _river {
  char* name;
  // a river doesnt alway finish in the middle of the last tile.
  float finishX;
  float finishY;
  binarytree_t *itinerary;
};


struct _city {
  int          id; // not used until save occurs. id is affected to each city to be refered by later unit saved
  char*        name;
  int          walls;
  int          size;
  tile*        location;
  player*      owner;
  int          seed;
  unit       **dependencies;
  good        *stocks;
  good        *prod;
  list_t      *buildings;
  int          construction;
  int          type;
  SDL_Surface *flagImage;
  list_t      *connectedCities;  
};

struct _player {
  char* name;
  list_t *unitList;
  list_t *cityList;
  int religion;
  int totalCrosses;
  int money;
  int diplomaticRelations[NATION_NUMBER];
  int nation;
};

struct _good {
  int   type;
  int   quantity;
  unit* passenger; // there is someone in the crate !
};

struct _battle {
  player *attacker;
  player *defender;
  int     attackStrength;
  int     defenseStrength;
  float   attackBonusMalus;
  float   defenseBonusMalus;
  tile   *locationAttack;
  tile   *locationDefense;
  int     date;
  char   *text;
  list_t *attackerlist;
  list_t *defenderlist;
};

struct _building {
  int type;
  int constructionDate;
  list_t *workers;
  int typeGoodsIn1;
  int typeGoodsIn2;
  int typeGoodsIn3;
  int typeGoodsOut1;
  city* homecity;
  unsigned int slots;
  int baseProduction1;
  int helper; // building id this building is helping
  int extends; // building id this building is extending
  int minSize;
  int buildingHammers;
  int buildingTools;
};

struct _option {
  char* text;
  void (*act)(list_t *, char *);
  int active;
};

struct _unitview{
  SDL_Rect    *rect;
  SDL_Rect    *shadowrect;
  list_t      *ul;
  list_t      *opponentl;
  list_t      *lastUl;
  tile        *t;
  SDL_Surface *uvbuffer;
  SDL_Surface *uvshadowbuffer;
  int          needUpdate;
};
#endif
