
/*** VORONOI.C ***/

#include "voronoi.h"
#include <stdlib.h>
#include <stdio.h>
#include <math.h>

extern Site * bottomsite ;
extern Halfedge * ELleftend, * ELrightend ;

PolySite *pts = NULL;   
float minx, miny, maxx, maxy;
int sorted, triangulate, plot, /*debug,*/ nsites, siteidx ;
float xmin, xmax, ymin, ymax ;
Freelist sfl ;
Site * sites ;

int scomp(const void * vs1, const void * vs2);
Site *nextone(void);

PolySite *getCVTRandomPoints(int nbPoint, int nbiterations){
  int i;
  sorted = 1; 
  triangulate = 0; 
  plot = 1; 
  debug = 0;
  siteidx = 0 ;
  if (pts!=NULL)
    free(pts);
  pts   = (PolySite *) malloc(nbPoint*sizeof(PolySite));
  sites = (Site *) malloc(nbPoint*sizeof(Site));
  //srand(5);
  for(nsites=0; nsites<nbPoint; nsites++) {
    sites[nsites].coord.y = ((float)1+nsites)/(nbPoint+1);/*((float)(rand()%100000))/99999.0;*/
    sites[nsites].coord.x = ((float)(rand()%100000))/99999.0;
    pts[nsites].coord.y = sites[nsites].coord.y;
    pts[nsites].coord.x = sites[nsites].coord.x;
    //pts[nsites].sumcoord = malloc(sizeof(struct Point2));
    pts[nsites].sumcoord.x = pts[nsites].coord.x;
    pts[nsites].sumcoord.y = pts[nsites].coord.y;
    pts[nsites].min.x = 1;
    pts[nsites].min.y = 1;
    pts[nsites].max.x = 0;
    pts[nsites].max.y = 0;     
    pts[nsites].length = 0;     
    //pts[nsites].weight = 5+rand()%10;   
    pts[nsites].color = rand();     
    pts[nsites].polySegCount = 0;
  	sites[nsites].sitenbr = nsites;
    sites[nsites].refcnt = 0;
  };
  
  //qsort(sites, nsites, sizeof *sites, scomp);
  
  xmin=sites[0].coord.x; 
  xmax=sites[0].coord.x;
  for(i=1; i<nsites; i+=1) {	
    if(sites[i].coord.x < xmin) xmin = sites[i].coord.x;
    if(sites[i].coord.x > xmax) xmax = sites[i].coord.x;
  };
  ymin = sites[0].coord.y;
  ymax = sites[nsites-1].coord.y;

  int k;
  for (k=0;k<nbiterations; k++){
    freeinit(&sfl, sizeof *sites);
    for(nsites=0; nsites<nbPoint; nsites++) {
      //printf("new point %f, %f .....%f, %f --- %f, %f\n", pts[nsites].sumcoord.x, pts[nsites].sumcoord.y, pts[nsites].min.x, pts[nsites].min.y, pts[nsites].max.x, pts[nsites].max.y);
      sites[nsites].coord.x = pts[nsites].sumcoord.x;
      sites[nsites].coord.y = pts[nsites].sumcoord.y;
      pts[nsites].coord.y = sites[nsites].coord.y;
      pts[nsites].coord.x = sites[nsites].coord.x;
      pts[nsites].sumcoord.x = 0;
      pts[nsites].sumcoord.y = 0;
      pts[nsites].min.x = 1;
      pts[nsites].min.y = 1;
      pts[nsites].max.x = 0;
      pts[nsites].max.y = 0;    
      pts[nsites].outpt1.x = 0.5; 
      pts[nsites].outpt1.y = 0.5; 
      pts[nsites].outpt2.x = 0.5; 
      pts[nsites].outpt2.y = 0.5; 
      pts[nsites].length = 0.0f;     
      pts[nsites].polySegCount = 0;
      sites[nsites].sitenbr = nsites;
      sites[nsites].refcnt = 0;              
    }
    qsort(sites, nsites, sizeof *sites, scomp);
    xmin=sites[0].coord.x; 
    xmax=sites[0].coord.x;
    for(i=1; i<nsites; i+=1) {	
      if(sites[i].coord.x < xmin) xmin = sites[i].coord.x;
      if(sites[i].coord.x > xmax) xmax = sites[i].coord.x;
    };
    ymin = sites[0].coord.y;
    ymax = sites[nsites-1].coord.y;
    
    siteidx = 0;
    geominit() ;
    if (plot)
      plotinit() ;

    // setup bounding box
    minx=1;
    maxx=0;
    miny=1;
    maxy=0;
    
    // call voronois algo 
    voronoi(nextone) ;
    
    // add missing points at 4 corners
    for(nsites=0; nsites<nbPoint; nsites++) {   
      if (pts[nsites].min.x<=0 && pts[nsites].min.y<=0) {
        float ax = pts[nsites].outpt1.x;
        float ay = pts[nsites].outpt1.y;
        float bx = pts[nsites].outpt2.x;
        float by = pts[nsites].outpt2.y;
        if ((ax==0.f&&ay==0.5f)||(bx==0.f&&by==0.5f))  
          printf("PROBLEM : only 1 point outside\n");           
        
        float d1 = sqrt((ax-0)*(ax-0)+(ay-0)*(ay-0));
        float d2 = sqrt((0-bx)*(0-bx)+(0-by)*(0-by));
        pts[nsites].sumcoord.x += 0*d1;
        pts[nsites].sumcoord.y += 0*d1;
        pts[nsites].sumcoord.x += ax*d1;
        pts[nsites].sumcoord.y += ay*d1;
        pts[nsites].sumcoord.x += 0*d2;
        pts[nsites].sumcoord.y += 0*d2;              
        pts[nsites].sumcoord.x += bx*d2;
        pts[nsites].sumcoord.y += by*d2;
        pts[nsites].outpt1.x = 0.5f;
        pts[nsites].outpt1.y = 0.5f;
        pts[nsites].outpt2.x = 0.5f;
        pts[nsites].outpt2.y = 0.5f;           
        pts[nsites].polySegCount+=2;
        pts[nsites].length+=(d1+d2);
      }
      else if (pts[nsites].max.x>=1 && pts[nsites].max.y>=1){
        float ax = pts[nsites].outpt1.x;
        float ay = pts[nsites].outpt1.y;
        float bx = pts[nsites].outpt2.x;
        float by = pts[nsites].outpt2.y;  
        if ((ax==0.f&&ay==0.5f)||(bx==0.f&&by==0.5f))  
          printf("PROBLEM : only 1 point outside\n");                          
        float d1 = sqrt((ax-1)*(ax-1)+(ay-1)*(ay-1));
        float d2 = sqrt((1-bx)*(1-bx)+(1-by)*(1-by));
        pts[nsites].sumcoord.x += 1*d1;
        pts[nsites].sumcoord.y += 1*d1;
        pts[nsites].sumcoord.x += ax*d1;
        pts[nsites].sumcoord.y += ay*d1;
        pts[nsites].sumcoord.x += 1*d2;
        pts[nsites].sumcoord.y += 1*d2;              
        pts[nsites].sumcoord.x += bx*d2;
        pts[nsites].sumcoord.y += by*d2;
        pts[nsites].outpt1.x = 0.5f;
        pts[nsites].outpt1.y = 0.5f;
        pts[nsites].outpt2.x = 0.5f;
        pts[nsites].outpt2.y = 0.5f;           
        pts[nsites].polySegCount+=2;
        pts[nsites].length+=d1+d2;
      }
      else if (pts[nsites].min.x<=0 && pts[nsites].max.y>=1){
        float ax = pts[nsites].outpt1.x;
        float ay = pts[nsites].outpt1.y;
        float bx = pts[nsites].outpt2.x;
        float by = pts[nsites].outpt2.y;   
        if ((ax==0.f&&ay==0.5f)||(bx==0.f&&by==0.5f))  
          printf("PROBLEM : only 1 point outside\n");                         
        float d1 = sqrt((ax-0)*(ax-0)+(ay-1)*(ay-1));
        float d2 = sqrt((0-bx)*(0-bx)+(1-by)*(1-by));
        pts[nsites].sumcoord.x += 0*d1;
        pts[nsites].sumcoord.y += 1*d1;
        pts[nsites].sumcoord.x += ax*d1;
        pts[nsites].sumcoord.y += ay*d1;
        pts[nsites].sumcoord.x += 0*d2;
        pts[nsites].sumcoord.y += 1*d2;              
        pts[nsites].sumcoord.x += bx*d2;
        pts[nsites].sumcoord.y += by*d2;
        pts[nsites].outpt1.x = 0.5f;
        pts[nsites].outpt1.y = 0.5f;
        pts[nsites].outpt2.x = 0.5f;
        pts[nsites].outpt2.y = 0.5f;           
        pts[nsites].polySegCount+=2;
        pts[nsites].length+=d1+d2;
      }
      else if (pts[nsites].max.x>=1 && pts[nsites].min.y<=0){
        float ax = pts[nsites].outpt1.x;
        float ay = pts[nsites].outpt1.y;
        float bx = pts[nsites].outpt2.x;
        float by = pts[nsites].outpt2.y;      
        if ((ax==0.f&&ay==0.5f)||(bx==0.f&&by==0.5f))  
          printf("PROBLEM : only 1 point outside\n");                      
        float d1 = sqrt((ax-1)*(ax-1)+(ay-0)*(ay-0));
        float d2 = sqrt((1-bx)*(1-bx)+(0-by)*(0-by));
        pts[nsites].sumcoord.x += 1*d1;
        pts[nsites].sumcoord.y += 0*d1;
        pts[nsites].sumcoord.x += ax*d1;
        pts[nsites].sumcoord.y += ay*d1;
        pts[nsites].sumcoord.x += 1*d2;
        pts[nsites].sumcoord.y += 0*d2;              
        pts[nsites].sumcoord.x += bx*d2;
        pts[nsites].sumcoord.y += by*d2;
        pts[nsites].outpt1.x = 0.5f;
        pts[nsites].outpt1.y = 0.5f;
        pts[nsites].outpt2.x = 0.5f;
        pts[nsites].outpt2.y = 0.5f;           
        pts[nsites].polySegCount+=2;
        pts[nsites].length+=d1+d2;
      }
                      
      else if (pts[nsites].outpt1.x!=0.5 || pts[nsites].outpt1.y!=0.5){
        if (pts[nsites].outpt2.x==0.5 && pts[nsites].outpt2.y==0.5)
          printf("BIG PROBLEM: ONLY ONE OUT POINT for site %d\n", nsites);
        float ax = pts[nsites].outpt1.x;
        float ay = pts[nsites].outpt1.y;
        float bx = pts[nsites].outpt2.x;
        float by = pts[nsites].outpt2.y;
        if ((ax==0.f&&ay==0.5f)||(bx==0.f&&by==0.5f))  
          printf("PROBLEM : only 1 point outside\n");                
        float d = sqrt((ax-bx)*(ax-bx)+(ay-by)*(ay-by));
        pts[nsites].polySegCount+=1;
        pts[nsites].length +=d;
        pts[nsites].sumcoord.x += ax*d;
        pts[nsites].sumcoord.y += ay*d;
        pts[nsites].sumcoord.x += bx*d;
        pts[nsites].sumcoord.y += by*d;
      }
    }
    
    // next turn ...
    for(nsites=0; nsites<nbPoint; nsites++) {            

      //printf("periph = %f\n", pts[nsites].length);
      // compute average
      pts[nsites].sumcoord.x = pts[nsites].sumcoord.x / (pts[nsites].length*2);
      pts[nsites].sumcoord.y = pts[nsites].sumcoord.y / (pts[nsites].length*2);
      
      // normalize result according to bounding box
      pts[nsites].sumcoord.x = (pts[nsites].sumcoord.x-minx)/(maxx-minx);
      pts[nsites].sumcoord.y = (pts[nsites].sumcoord.y-miny)/(maxy-miny);
    }  
  }
  return pts;
}

int scomp(const void * vs1, const void * vs2) {
  Point2 * s1 = (Point2 *)vs1 ;
  Point2 * s2 = (Point2 *)vs2 ;
  
  if (s1->y < s2->y)
    return (-1) ;
  if (s1->y > s2->y)
    return (1) ;
  if (s1->x < s2->x)
    return (-1) ;
  if (s1->x > s2->x)
    return (1) ;
  return (0) ;
}

Site *nextone(void){
  Site * s ;
  if (siteidx < nsites){
    s = &sites[siteidx++];
    return (s) ;
  }
  else {
    return ((Site *)NULL) ;
  }
}

/*** implicit parameters: nsites, sqrt_nsites, xmin, xmax, ymin, ymax,
 : deltax, deltay (can all be estimates).
 : Performance suffers if they are wrong; better to make nsites,
 : deltax, and deltay too big than too small.  (?)
 ***/

  #pragma argsused
  void line(float ax, float ay, float bx, float by, int siteid1, int siteid2){
    minx=MIN(MIN(ax,minx), bx);
    maxx=MAX(MAX(ax,maxx), bx);
    miny=MIN(MIN(ay,miny), by);
    maxy=MAX(MAX(ay,maxy), by);

    if (ax<=0 || ay<=0 || ax>=1 || ay>=1){
      if (pts[siteid1].outpt1.x==0.5 && pts[siteid1].outpt1.y==0.5){
        pts[siteid1].outpt1.x = ax;
        pts[siteid1].outpt1.y = ay;
      } else if (pts[siteid1].outpt2.x==0.5 && pts[siteid1].outpt2.y==0.5){
        pts[siteid1].outpt2.x = ax;
        pts[siteid1].outpt2.y = ay;
      } else
        printf("PROBLEM : 3 points outside %d\n",siteid1);
      if (pts[siteid2].outpt1.x==0.5 && pts[siteid2].outpt1.y==0.5){
        pts[siteid2].outpt1.x = ax;
        pts[siteid2].outpt1.y = ay;
      } else if (pts[siteid2].outpt2.x==0.5 && pts[siteid2].outpt2.y==0.5){
        pts[siteid2].outpt2.x = ax;
        pts[siteid2].outpt2.y = ay;
      } else
        printf("PROBLEM : 3 points outside %d\n",siteid2);
    }

    if (bx<=0 || by<=0 || bx>=1 || by>=1){
      if (pts[siteid1].outpt1.x==0.5 && pts[siteid1].outpt1.y==0.5){
        pts[siteid1].outpt1.x = bx;
        pts[siteid1].outpt1.y = by;
      } else if (pts[siteid1].outpt2.x==0.5 && pts[siteid1].outpt2.y==0.5){
        pts[siteid1].outpt2.x = bx;
        pts[siteid1].outpt2.y = by;
      } else
        printf("PROBLEM : 3 points outside %d\n",siteid1);
      if (pts[siteid2].outpt1.x==0.5 && pts[siteid2].outpt1.y==0.5){
        pts[siteid2].outpt1.x = bx;
        pts[siteid2].outpt1.y = by;
      } else if (pts[siteid2].outpt2.x==0.5 && pts[siteid2].outpt2.y==0.5){
        pts[siteid2].outpt2.x = bx;
        pts[siteid2].outpt2.y = by;
      } else
        printf("PROBLEM : 3 points outside %d\n",siteid2);
    }
    
    float d = sqrt((ax-bx)*(ax-bx)+(ay-by)*(ay-by));
    //printf("%f,%f --- %f,%f => d = %f  +%f\n", ax, ay, bx, by, d, pts[siteid1].length);
    pts[siteid1].length +=d;
    pts[siteid2].length +=d;

    pts[siteid1].polySegCount+=1;
    pts[siteid2].polySegCount+=1;
    pts[siteid1].sumcoord.x += ax*d;
    pts[siteid1].sumcoord.y += ay*d;
    pts[siteid1].sumcoord.x += bx*d;
    pts[siteid1].sumcoord.y += by*d;
    pts[siteid2].sumcoord.x += ax*d;
    pts[siteid2].sumcoord.y += ay*d;
    pts[siteid2].sumcoord.x += bx*d;
    pts[siteid2].sumcoord.y += by*d;
    if (ax<pts[siteid1].min.x) pts[siteid1].min.x = ax;
    if (ax>pts[siteid1].max.x) pts[siteid1].max.x = ax;
    if (ay<pts[siteid1].min.y) pts[siteid1].min.y = ay;
    if (ay>pts[siteid1].max.y) pts[siteid1].max.y = ay;
    if (bx<pts[siteid1].min.x) pts[siteid1].min.x = bx;
    if (bx>pts[siteid1].max.x) pts[siteid1].max.x = bx;
    if (by<pts[siteid1].min.y) pts[siteid1].min.y = by;
    if (by>pts[siteid1].max.y) pts[siteid1].max.y = by;

    if (ax<pts[siteid2].min.x) pts[siteid2].min.x = ax;
    if (ax>pts[siteid2].max.x) pts[siteid2].max.x = ax;
    if (ay<pts[siteid2].min.y) pts[siteid2].min.y = ay;
    if (ay>pts[siteid2].max.y) pts[siteid2].max.y = ay;
    if (bx<pts[siteid2].min.x) pts[siteid2].min.x = bx;
    if (bx>pts[siteid2].max.x) pts[siteid2].max.x = bx;
    if (by<pts[siteid2].min.y) pts[siteid2].min.y = by;
    if (by>pts[siteid2].max.y) pts[siteid2].max.y = by;
  }


void voronoi(Site *(*nextsite)(void))
    {
    Site * newsite, * bot, * top, * temp, * p, * v ;
    Point2 newintstar ;
    int pm ;
    Halfedge * lbnd, * rbnd, * llbnd, * rrbnd, * bisector ;
    Edge * e ;

    PQinitialize() ;
    bottomsite = (*nextsite)() ;
    out_site(bottomsite) ;
    ELinitialize() ;
    newsite = (*nextsite)() ;
    while (1)
        {
        if(!PQempty())
            {
            newintstar = PQ_min() ;
            }
        if (newsite != (Site *)NULL && (PQempty()
            || newsite -> coord.y < newintstar.y
            || (newsite->coord.y == newintstar.y
            && newsite->coord.x < newintstar.x))) {/* new site is smallest */
            {
            out_site(newsite) ;
            }
        lbnd = ELleftbnd(&(newsite->coord)) ;
        rbnd = ELright(lbnd) ;
        bot = rightreg(lbnd) ;
        e = bisect(bot, newsite) ;
        bisector = HEcreate(e, le) ;
        ELinsert(lbnd, bisector) ;
        p = intersect(lbnd, bisector) ;
        if (p != (Site *)NULL)
            {
            PQdelete(lbnd) ;
            PQinsert(lbnd, p, dist(p,newsite)) ;
            }
        lbnd = bisector ;
        bisector = HEcreate(e, re) ;
        ELinsert(lbnd, bisector) ;
        p = intersect(bisector, rbnd) ;
        if (p != (Site *)NULL)
            {
            PQinsert(bisector, p, dist(p,newsite)) ;
            }
        newsite = (*nextsite)() ;
        }
    else if (!PQempty())   /* intersection is smallest */
            {
            lbnd = PQextractmin() ;
            llbnd = ELleft(lbnd) ;
            rbnd = ELright(lbnd) ;
            rrbnd = ELright(rbnd) ;
            bot = leftreg(lbnd) ;
            top = rightreg(rbnd) ;
            out_triple(bot, top, rightreg(lbnd)) ;
            v = lbnd->vertex ;
            makevertex(v) ;
            endpoint(lbnd->ELedge, lbnd->ELpm, v);
            endpoint(rbnd->ELedge, rbnd->ELpm, v) ;
            ELdelete(lbnd) ;
            PQdelete(rbnd) ;
            ELdelete(rbnd) ;
            pm = le ;
            if (bot->coord.y > top->coord.y)
                {
                temp = bot ;
                bot = top ;
                top = temp ;
                pm = re ;
                }
            e = bisect(bot, top) ;
            bisector = HEcreate(e, pm) ;
            ELinsert(llbnd, bisector) ;
            endpoint(e, re-pm, v) ;
            deref(v) ;
            p = intersect(llbnd, bisector) ;
            if (p  != (Site *) NULL)
                {
                PQdelete(llbnd) ;
                PQinsert(llbnd, p, dist(p,bot)) ;
                }
            p = intersect(bisector, rrbnd) ;
            if (p != (Site *) NULL)
                {
                PQinsert(bisector, p, dist(p,bot)) ;
                }
            }
        else
            {
            break ;
            }
        }

    for( lbnd = ELright(ELleftend) ;
         lbnd != ELrightend ;
         lbnd = ELright(lbnd))
        {
        e = lbnd->ELedge ;
        out_ep(e) ;
        }
    }

