/*
 *  initialization.cpp
 *  NewCol
 *
 *  Created by frederic Vernier on 2/10/08.
 *  Copyright 2008 __MyCompanyName__. All rights reserved.
 *
 */

#include "initialization.h"

char isRobinsonTransfo;

/**
 * Dispatch bonuses all other the tiles
 */
void initBonuses(int percentage) {
  addMsg("Initializing bonuses");
  updateLoading();

  srand((unsigned int)time(0));
  int i, j;
  // update bonuses
  for (i=0; i<MapWidth; i++) {
    for (j=0; j<MapHeight; j++) {
      if (map[i+MapWidth*(j)].elevation!=HIGHMOUNTAIN && map[i+MapWidth*(j)].ground!=SWAMP) {
        if (rand()%100<percentage) {
          map[i+MapWidth*(j)].bonus = 1;
          if (rand()%(100)<percentage)
            map[i+MapWidth*(j)].bonus = 2;
        }
      }
    }
  }
}// initBonuses() --------------------------------------------------------------------


/**
 * Dispatch ground type using perlin noise
 */
void initGround() {
  int i, j   = 0;
  srand((unsigned int)time(0));

  addMsg("Initializing ground");
  updateLoading();

  for (j=0; j<MapHeight; j++){
    float la = 0;
    if (j<=MapHeight/2+MapHeight/6 &&
        j>=MapHeight/2-MapHeight/6)
      la = -0.5f*(cos(8*M_PI*(j-MapHeight/2)/(MapHeight)));
    else if (j<=MapHeight/2-MapHeight/7)
      la = ((float)0.25*j/(MapHeight/2-MapHeight/7));
    else if (j>=MapHeight/2+MapHeight/7)
      la = (((float)0.25*(MapHeight-j))/(MapHeight/2-MapHeight/7));
    //printf("%f  %f\n", la, ((float)j)/MapHeight/2-MapHeight/7);

    for (i=0; i<MapWidth; i++){
      /*if (map[j*MapWidth+i].elevation<SEALEVEL && !getConnectedToCoast(i, j))
        map[j*MapWidth+i].ground = DESERT;
      else*/ {
        float x = la+0.5f*noise2(((float)i)/10, ((float)j)/10);

        if (x<-0.45)
          map[j*MapWidth+i].ground = SWAMP;
        else if (x<-0.2)
          map[j*MapWidth+i].ground = SAVANNAH;
        else if (x<0.0)
          map[j*MapWidth+i].ground = PLAIN;
        else if (x<0.2)
          map[j*MapWidth+i].ground = GRASSLAND;
        else if (x<0.45)
          map[j*MapWidth+i].ground = PRAIRIE;
        else
          map[j*MapWidth+i].ground = DESERT;
      }
    }
  }
}// initGround() --------------------------------------------------------------------


/**
 * create the lost city rumors
 */
void initLostCityRumors(int percentage){
  addMsg("Initializing lost city rumors");
  updateLoading();
  //printf("initLostCityRumors(%d) \n", percentage);fflush(stdout);

  srand((unsigned int)time(0));
  int i, j;
  // update lost city rumors
  for (i=0; i<MapWidth; i++) {
    for (j=0; j<MapHeight; j++) {
      if (map[i+MapWidth*(j)].elevation>=SEALEVEL && map[i+MapWidth*(j)].ground!=SWAMP && map[i+MapWidth*(j)].lastExplored<0 && rand()%100<percentage) {
        createLostCityRumor(&map[i+MapWidth*(j)]);
      }
    }
  }
}// initLostCityRumors()


/**
 * set a forest around the given center with the given radius
 */
void plotForest(int x, int y, int r){
  int i, j;
  int R= r*r;
  for (j=-r; j<r; j++) {
    for (i=-r; i<r; i++) {
      if (j*j+i*i<R+R/10*noise2(i, j)) {
        float pot = noise2(((float)i)/10, ((float)j)/10);
        if (j*j+i*i>9*(R+R/10*noise2(i, j))/10)
          pot = 0.5;
        if (map[(x+i)+MapWidth*(y+j)].elevation<SEALEVEL || map[(x+i)+MapWidth*(y+j)].elevation>=MOUNTAIN || pot>0.6)
          map[(x+i)+MapWidth*(y+j)].vegetation = NONE;
        else if (pot<0)
          map[(x+i)+MapWidth*(y+j)].vegetation = FOREST;
        else
          map[(x+i)+MapWidth*(y+j)].vegetation = WOOD;
      }
    }
  }
}// plotForest()


/**
 * set the indian cities around the given center with the given radius
 */
void plotIndianCities(int number, int nation, int x, int y, int r){
  //printf("plotIndianCities around %d, %d  radius=%d\n", x, y, r);
  int i, j, k;
  int R= r*r;


  for (k=0; k<number; k++) {
    initcpt++;
    updateLoading();
    int maxscore = 0;
    int selx = x;
    int sely = y;
    for (i=0; i<100; i++) {
      float d = rand()%r;
      float angle = ((rand()%360)*M_PI)/180.0f;
      int xc = x+d*cos(angle);
      int yc = y+d*sin(angle);
      //
      float mindist = r*2;

      for (j=FIRST_NATION; j<=LAST_NATION; j++) {
        if (nations[j-FIRST_NATION]!=NULL) {
          list_t *cityList = nations[j-FIRST_NATION]->cityList;
          lnode_t *ln = list_first(cityList);
          while (ln !=NULL && ln!=&cityList->list_nilnode){
            city *c = (city*)ln->list_data;
            float dist = ((c->location->posX-xc)*(c->location->posX-xc))+((c->location->posY-yc)*(c->location->posY-yc));
            if (dist<mindist)
              mindist = dist;
            ln = ln->list_next;
          }
        }
      }

      int score = getCityLocationScore(xc, yc);
      list_t *cityList = nations[nation-FIRST_NATION]->cityList;
      // todo : make capital?
      if (list_count(cityList)==0)
        score = score*5;
      else if (mindist<=2)
        score = 0;
      else if (mindist>=6 && mindist<=12 && score>0)
        score = score+100;
      else if (mindist<6 && score>0)
        score += 100-((6-mindist)*10);
      else if (mindist>12 && score>0)
        score += 112-(mindist);

      //printf("    (%f,%f => %d,%d) = %d\n", d, angle, xc, yc, score);fflush(stdout);
      if (score>maxscore){
        maxscore = score;
        selx = xc;
        sely = yc;
      }
    }

    if ((selx != x || sely != y) && maxscore > 10) {
      char *name = (char *)malloc(32*sizeof(char));
      sprintf(name, "%s village #%d", getNationText(nation), k);
      int size = 1+rand()%3;
      //printf("  native village %s created at %d, %d with the score %d\n\n", name, selx, sely, maxscore);fflush(stdout);
      city *c = createCity (size, name, &map[selx+MapWidth*(sely)], nations[nation-FIRST_NATION]);
      c->type = AGRICULTURE_CITY;
      if (c!=NULL)
        for (i=0; i<c->size; i++) {
          unit *u = createUnit (INDIAN, nations[nation-FIRST_NATION]);
          findBestJob(u, c);
        }

      free(name);
    }else {
      printf("\n  ERROR: native village %s#%d couldn't be initialized with score %d \n", getNationText(nation), k, maxscore);
    }
  }
}// plotIndianCities()


/**
 * Dispatch forests around the world
 */
void initForests() {
  addMsg("Initializing forests from forests.txt");
  updateLoading();

  FILE * pFile = fopen("forests.txt", "r");
  //printf("* %p    %c %c\n", pFile, '\n', EOF);fflush(stdout);
  int i, index = 0;
  char c;
  double coords[4];
  char *line = (char *)malloc(1024*sizeof(char));
  do {
    i=0;
    //printf("START\n");
    while(c=='\n' || c==',' || c==' ')
      c = getc(pFile);
    //printf("READING\n");
    do {
      line[i++] = c;
      c = getc(pFile);
    } while(c!=EOF && c!='\n' && i<1024 && c!=','&& c!=' ');
    line[i++] = '\0';
    //printf("%s = %4.10f    at %i\n", line, atof(line), index);
    coords[index++] = atof(line);
    if (index == 4){
      int mapindex0 = mapTransfo(- coords[1], coords[0]);
      int mapindex1 = mapTransfo(- coords[3], coords[2]);
      plotForest((mapindex0%MapWidth+mapindex1%MapWidth)/2, (mapindex0/MapWidth+mapindex1/MapWidth)/2, (mapindex1%MapWidth-mapindex0%MapWidth)/2);
      initcpt++;
      updateLoading();
      //printf("--- %f %f %f %f\n", coords[0], coords[1], coords[2], coords[3]);  fflush(stdout);
      index = 0;
    }
    //printf("END\n");
  } while(c!=EOF);

  free(line);
}// initForests() --------------------------------------------------------------------


/**
 * Dispatch rivers according to the real earth rivers
 */
void initWorldRivers() {
  addMsg("Initializing rivers");
  updateLoading();

  /* rivers */
  btnode_t *btnode0;
  btnode_t *btnode1;

  river* river0 = (river *)malloc(sizeof(river));
  river0->name = "St Laurent";
  river0->itinerary = binarytree_create(LISTCOUNT_T_MAX);

  btnode0 = binarytree_ins_left(river0->itinerary, NULL, btnode_create(&map[676*MapWidth/2160+MapWidth*(244*MapHeight/1080)]));
  //btnode_t *btnode0 = binarytree_ins_left(river0->itinerary, NULL, btnode_create(&map[655*MapWidth/2160+MapWidth*(259*MapHeight/1080)]));

  btnode0 = extendRiverNodes(river0, btnode0, 654*MapWidth/2160, 258*MapHeight/1080);
  btnode0 = extendRiverNodes(river0, btnode0, 653*MapWidth/2160, 258*MapHeight/1080);
  btnode0 = extendRiverNodes(river0, btnode0, 647*MapWidth/2160, 260*MapHeight/1080);
  btnode0 = extendRiverNodes(river0, btnode0, 641*MapWidth/2160, 264*MapHeight/1080);

  btnode1 = extendRiverNodes(river0, btnode0, 632*MapWidth/2160, 268*MapHeight/1080);
  btnode0 = extendRiverNodes(river0, btnode0, 640*MapWidth/2160, 276*MapHeight/1080);

  btnode0 = extendRiverNodes(river0, btnode1, 622*MapWidth/2160, 276*MapHeight/1080);
  btnode0 = extendRiverNodes(river0, btnode1, 615*MapWidth/2160, 263*MapHeight/1080);

  attachRiverToMap_r(binarytree_root(river0->itinerary));
  computeRiverSize_r(binarytree_root(river0->itinerary));
  river0->finishX = 0.2f;
  river0->finishY = 0.5f;
  list_append(riverList, lnode_create(river0));
  //printRiver_r(binarytree_root(river0->itinerary), 0);

  river* river1 = (river *)malloc(sizeof(river));
  river1->name = "Mississippi";
  river1->itinerary = binarytree_create(LISTCOUNT_T_MAX);
  btnode1 = binarytree_ins_left(river1->itinerary, NULL, btnode_create(&map[543*MapWidth/2160+MapWidth*(365*MapHeight/1080)]));
  //btnode1 = binarytree_ins_left(river1->itinerary, NULL, btnode_create(&map[539*MapWidth/2160+MapWidth*(362*MapHeight/1080)]));
  btnode1 = extendRiverNodes(river1, btnode1, 543*MapWidth/2160, 365*MapHeight/1080);
  btnode1 = extendRiverNodes(river1, btnode1, 527*MapWidth/2160, 354*MapHeight/1080);
  btnode_t *btnode2 = extendRiverNodes(river1, btnode1, 531*MapWidth/2160, 347*MapHeight/1080);
  btnode1 = extendRiverNodes(river1, btnode1, 517*MapWidth/2160, 345*MapHeight/1080);
  btnode_t *btnode3 = extendRiverNodes(river1, btnode2, 535*MapWidth/2160, 337*MapHeight/1080);
  btnode1 = extendRiverNodes(river1, btnode2, 522*MapWidth/2160, 336*MapHeight/1080);
  btnode2 = extendRiverNodes(river1, btnode3, 531*MapWidth/2160, 331*MapHeight/1080);
  btnode1 = extendRiverNodes(river1, btnode2, 533*MapWidth/2160, 324*MapHeight/1080);
  btnode2 = extendRiverNodes(river1, btnode2, 520*MapWidth/2160, 328*MapHeight/1080);
  btnode2 = extendRiverNodes(river1, btnode2, 512*MapWidth/2160, 328*MapHeight/1080);
  btnode2 = extendRiverNodes(river1, btnode2, 500*MapWidth/2160, 314*MapHeight/1080);
  btnode1 = extendRiverNodes(river1, btnode1, 543*MapWidth/2160, 317*MapHeight/1080);
  btnode1 = extendRiverNodes(river1, btnode1, 538*MapWidth/2160, 319*MapHeight/1080);
  btnode1 = extendRiverNodes(river1, btnode1, 543*MapWidth/2160, 314*MapHeight/1080);
  btnode1 = extendRiverNodes(river1, btnode1, 536*MapWidth/2160, 310*MapHeight/1080);
  btnode3 = extendRiverNodes(river1, btnode1, 535*MapWidth/2160, 300*MapHeight/1080);
  btnode2 = extendRiverNodes(river1, btnode1, 528*MapWidth/2160, 306*MapHeight/1080);

  btnode2 = extendRiverNodes(river1, btnode2, 517*MapWidth/2160, 310*MapHeight/1080);
  btnode2 = extendRiverNodes(river1, btnode2, 504*MapWidth/2160, 292*MapHeight/1080);

  btnode3 = extendRiverNodes(river1, btnode3, 544*MapWidth/2160, 291*MapHeight/1080);
  btnode3 = extendRiverNodes(river1, btnode3, 532*MapWidth/2160, 278*MapHeight/1080);
  attachRiverToMap_r(binarytree_root(river1->itinerary));
  computeRiverSize_r(binarytree_root(river1->itinerary));
  river1->finishX = 0.5f;
  river1->finishY = 0.5f;
  list_append(riverList, lnode_create(river1));
  //printRiver_r(binarytree_root(river1->itinerary), 0);

  river* river2 = (river *)malloc(sizeof(river));
  river2->name = "Garonne";
  river2->itinerary = binarytree_create(LISTCOUNT_T_MAX);
  btnode0 = binarytree_ins_left(river2->itinerary, NULL, btnode_create(&map[1074*MapWidth/2160+MapWidth*(266*MapHeight/1080)]));
  btnode3 = extendRiverNodes(river2, btnode0, 1076*MapWidth/2160, 269*MapHeight/1080);
  btnode0 = extendRiverNodes(river2, btnode3, 1077*MapWidth/2160, 271*MapHeight/1080);
  btnode0 = extendRiverNodes(river2, btnode0, 1080*MapWidth/2160, 272*MapHeight/1080);
  btnode1 = extendRiverNodes(river2, btnode0, 1083*MapWidth/2160, 274*MapHeight/1080);
  btnode0 = extendRiverNodes(river2, btnode1, 1087*MapWidth/2160, 278*MapHeight/1080);
  btnode1 = extendRiverNodes(river2, btnode1, 1081*MapWidth/2160, 282*MapHeight/1080);

  btnode3 = extendRiverNodes(river2, btnode3, 1083*MapWidth/2160, 269*MapHeight/1080);
  btnode3 = extendRiverNodes(river2, btnode3, 1088*MapWidth/2160, 270*MapHeight/1080);
  btnode3 = extendRiverNodes(river2, btnode3, 1093*MapWidth/2160, 265*MapHeight/1080);

  attachRiverToMap_r(binarytree_root(river2->itinerary));
  computeRiverSize_r(binarytree_root(river2->itinerary));
  river2->finishX = 0.2f;
  river2->finishY = 0.5f;
  list_append(riverList, lnode_create(river2));
  //printRiver_r(binarytree_root(river2->itinerary), 0);

  river* river3 = (river *)malloc(sizeof(river));
  river3->name = "Charles river";
  river3->itinerary = binarytree_create(LISTCOUNT_T_MAX);
  btnode0 = binarytree_ins_left(river3->itinerary, NULL, btnode_create(&map[654*MapWidth/2160+MapWidth*(285*MapHeight/1080)]));
  btnode0 = extendRiverNodes(river3, btnode0, 651*MapWidth/2160, 285*MapHeight/1080);
  btnode0 = extendRiverNodes(river3, btnode0, 652*MapWidth/2160, 286*MapHeight/1080);
  btnode0 = extendRiverNodes(river3, btnode0, 651*MapWidth/2160, 286*MapHeight/1080);
  btnode0 = extendRiverNodes(river3, btnode0, 650*MapWidth/2160, 286*MapHeight/1080);
  btnode0 = extendRiverNodes(river3, btnode0, 650*MapWidth/2160, 287*MapHeight/1080);
  attachRiverToMap_r(binarytree_root(river3->itinerary));
  computeRiverSize_r(binarytree_root(river3->itinerary));
  river3->finishX = 0.5f;
  river3->finishY = 0.5f;
  list_append(riverList, lnode_create(river3));

  river* river4 = (river *)malloc(sizeof(river));
  river4->name = "Rhones";
  river4->itinerary = binarytree_create(LISTCOUNT_T_MAX);
  btnode0 = binarytree_ins_left(river4->itinerary, NULL, btnode_create(&map[1109*MapWidth/2160+MapWidth*(279*MapHeight/1080)]));
  btnode0 = extendRiverNodes(river4, btnode0, 1109*MapWidth/2160, 267*MapHeight/1080);
  btnode1 = extendRiverNodes(river4, btnode0, 1114*MapWidth/2160, 265*MapHeight/1080);
  //btnode1 = extendRiverNodes(river4, btnode1, 1117*MapWidth/2160, 262*MapHeight/1080);
  //btnode0 = extendRiverNodes(river4, btnode0, 650*MapWidth/2160, 286*MapHeight/1080);
  //btnode0 = extendRiverNodes(river4, btnode0, 650*MapWidth/2160, 287*MapHeight/1080);

  attachRiverToMap_r(binarytree_root(river4->itinerary));
  computeRiverSize_r(binarytree_root(river4->itinerary));
  river4->finishX = 0.5f;
  river4->finishY = 0.5f;
  list_append(riverList, lnode_create(river4));
}// initWorldRivers() --------------------------------------------------------------------


/**
 * reveal a given ath found by astar on the map
 **/
void revealPath(){
   lnode_t *ln = list_first(closedList);
   while (ln!=NULL && ln!=&closedList->list_nilnode){
      tile *t = ((etile *)ln->list_data)->location;
      if (t->lastExplored==-1) t->lastExplored = 5;
      ln = ln->list_next;
   }
   ln = list_first(openList);
   while (ln!=NULL && ln!=&openList->list_nilnode){
      tile *t = ((etile *)ln->list_data)->location;
      if (t->lastExplored==-1) t->lastExplored = 5;
      ln = ln->list_next;
   }   
}// revealPath()


/**
 * init cities and units to help dedugging
 */
void initDebugCitiesAndUnits() {
  nations[NEW_FRANCE-FIRST_NATION]->money = 1234;
  nations[NEW_ENGLAND-FIRST_NATION]->money = 3456;

  tile *pos = &(map[mapTransfo(44.711111, -65.61)]);
  unit *uspe = createUnitAt(WAGON, nations[NEW_FRANCE-FIRST_NATION], pos->posX, pos->posY);
  if (uspe!=NULL) {
    uspe->goods[0].type      = ROPE;
    uspe->goods[0].quantity  = 60;
    uspe->goods[1].type      = HEMP;
    uspe->goods[1].quantity  = 80;
  }
  //astarSearch (uspe->location->posX, uspe->location->posY, 689, 271, uspe);

  unit *u0 = createUnitAt(SOLDIER, nations[NEW_FRANCE-FIRST_NATION], 677*MapWidth/2160, 249*MapHeight/1080);
  if (u0!=NULL) {
    u0->experience=0;
    u0 = createUnitAt(SOLDIER, nations[NEW_FRANCE-FIRST_NATION], 677*MapWidth/2160, 249*MapHeight/1080);
    u0->experience=2;
    addHorses(u0);
  }
  u0 = createUnitAt(SOLDIER, nations[NEW_FRANCE-FIRST_NATION], 675*MapWidth/2160, 249*MapHeight/1080);
  if (u0!=NULL) {
    u0->experience=5;
    addHorses(u0);
  }
  u0 = createUnitAt(SOLDIER, nations[NEW_ENGLAND-FIRST_NATION], 676*MapWidth/2160, 247*MapHeight/1080);
  if (u0!=NULL) {
    u0->experience=1;
    u0 = createUnitAt(SOLDIER, nations[NEW_ENGLAND-FIRST_NATION], 676*MapWidth/2160, 247*MapHeight/1080);
    u0->experience=1;
    addHorses(u0);
    u0 = createUnitAt(COLONIST, nations[NEW_ENGLAND-FIRST_NATION], 676*MapWidth/2160, 247*MapHeight/1080);

  }
  u0 = createUnitAt(GALLEON, nations[NEW_FRANCE-FIRST_NATION], map[mapTransfo(44.868611, -2.958333)].posX, map[mapTransfo(44.868611, -2.958333)].posY);
  if (u0!=NULL) {
    u0->goods[0].type      = FUR;
    u0->goods[0].quantity  = 60;
    u0->goods[1].type      = COTTON;
    u0->goods[1].quantity  = 80;
  }

  city *c0 = createCity (6, "Port Royal", &map[mapTransfo(44.711111, -65.61)], nations[NEW_FRANCE-FIRST_NATION]);
  if (c0!=NULL) {
    c0->type    = FISHING_CITY;
    building *b = createBuilding (DOCK, c0);
    if (uspe!=NULL)
      astarSearch (uspe->location->posX, uspe->location->posY, 689*MapWidth/2160, 271*MapHeight/1080, uspe);

    unit *u     = createUnit (CARPENTER, nations[NEW_FRANCE-FIRST_NATION]);
    addUnitToBuilding(u, c0, CARPENTRY_SHOP);
    c0->stocks[CORN-FIRST_GOOD].quantity = 50;
    if (uspe!=NULL)
       astarSearch (uspe->location->posX, uspe->location->posY, 689*MapWidth/2160, 271*MapHeight/1080, uspe);

    u = createUnit (BLACKSMITH, nations[NEW_FRANCE-FIRST_NATION]);
    addUnitToBuilding(u, c0, SMITHY);
    c0->stocks[ORE-FIRST_GOOD].quantity = 60;
    c0->stocks[LUMBER-FIRST_GOOD].quantity = 80;
    if (uspe!=NULL)
      astarSearch (uspe->location->posX, uspe->location->posY, 689*MapWidth/2160, 271*MapHeight/1080, uspe);

    u = createUnit (FARMER, nations[NEW_FRANCE-FIRST_NATION]);
    findBestJob(u, c0);
    u = createUnit (LUMBERJACK, nations[NEW_FRANCE-FIRST_NATION]);
    findBestJob(u, c0);
    u = createUnit (OREMINER, nations[NEW_FRANCE-FIRST_NATION]);
    findBestJob(u, c0);
    u = createUnit (TOBACCOPLANTER, nations[NEW_FRANCE-FIRST_NATION]);
    findBestJob(u, c0);
  }

  city *c1 = createCity (1, "Tadoussac", &map[mapTransfo(48.150005, -69.716672)], nations[NEW_FRANCE-FIRST_NATION]);
  if (c1!=NULL){
    unit *u = createUnit (CARPENTER, nations[NEW_FRANCE-FIRST_NATION]);
    addUnitToBuilding(u, c1, CARPENTRY_SHOP);
    c1->stocks[CORN-FIRST_GOOD].quantity = 50;
  }

  city *c2 = createCity (4, "Montreal", &map[mapTransfo(45.508889, -73.554167)], nations[NEW_FRANCE-FIRST_NATION]);
  if (c2!=NULL) {
    c2->type = FOREST_CITY;
    building *b = createBuilding (ARMOURY, c2);
    unit *u = createUnit (GUNSMITH, nations[NEW_FRANCE-FIRST_NATION]);
    addUnitToBuilding(u, c2, ARMOURY);
    u = createUnit (CARPENTER, nations[NEW_FRANCE-FIRST_NATION]);
    addUnitToBuilding(u, c2, CARPENTRY_SHOP);
    u = createUnit (COLONIST, nations[NEW_FRANCE-FIRST_NATION]);
    addUnitToBuilding(u, c2, CARPENTRY_SHOP);
    c2->stocks[CORN-FIRST_GOOD].quantity = 50;
    u = createUnit (BLACKSMITH, nations[NEW_FRANCE-FIRST_NATION]);
    addUnitToBuilding(u, c2, SMITHY);
    c2->stocks[ORE-FIRST_GOOD].quantity = 60;
    c2->stocks[LUMBER-FIRST_GOOD].quantity = 80;
    u = createUnit (FARMER, nations[NEW_FRANCE-FIRST_NATION]);
    findBestJob(u, c2);
    u = createUnit (LUMBERJACK, nations[NEW_ENGLAND-FIRST_NATION]);
    findBestJob(u, c2);
  }

  city *c3 = createCity (6, "Detroit", &map[mapTransfo(42.3316, -83.0475)], nations[NEW_FRANCE-FIRST_NATION]);
  if (c3!=NULL) {
    c3->type = AGRICULTURE_CITY;
    unit *u = createUnit (CARPENTER, nations[NEW_FRANCE-FIRST_NATION]);
    addUnitToBuilding(u, c3, CARPENTRY_SHOP);
    c3->stocks[CORN-FIRST_GOOD].quantity = 50;
    u = createUnit (BLACKSMITH, nations[NEW_FRANCE-FIRST_NATION]);
    addUnitToBuilding(u, c3, SMITHY);
    c3->stocks[ORE-FIRST_GOOD].quantity = 60;
    c3->stocks[LUMBER-FIRST_GOOD].quantity = 80;
    u = createUnit (FARMER, nations[NEW_FRANCE-FIRST_NATION]);
    findBestJob(u, c3);
    u = createUnit (LUMBERJACK, nations[NEW_FRANCE-FIRST_NATION]);
    findBestJob(u, c3);
    u = createUnit (OREMINER, nations[NEW_FRANCE-FIRST_NATION]);
    findBestJob(u, c3);
    u = createUnit (TOBACCOPLANTER, nations[NEW_FRANCE-FIRST_NATION]);
    findBestJob(u, c3);
  }

  city *c4 = createCity (6, "Quebec", &map[mapTransfo(46.816111, -71.224167)], nations[NEW_FRANCE-FIRST_NATION]);
  if (c4!=NULL) {
    c4->type = MINE_CITY;
    unit *u = createUnit (CARPENTER, nations[NEW_FRANCE-FIRST_NATION]);
    addUnitToBuilding(u, c4, CARPENTRY_SHOP);
    c4->stocks[CORN-FIRST_GOOD].quantity = 50;
    u = createUnit (BLACKSMITH, nations[NEW_FRANCE-FIRST_NATION]);
    addUnitToBuilding(u, c4, SMITHY);
    c4->stocks[ORE-FIRST_GOOD].quantity = 60;
    c4->stocks[LUMBER-FIRST_GOOD].quantity = 80;
    u = createUnit (FARMER, nations[NEW_FRANCE-FIRST_NATION]);
    findBestJob(u, c4);
    u = createUnit (LUMBERJACK, nations[NEW_FRANCE-FIRST_NATION]);
    findBestJob(u, c4);
    u = createUnit (OREMINER, nations[NEW_FRANCE-FIRST_NATION]);
    findBestJob(u, c4);
    u = createUnit (TOBACCOPLANTER, nations[NEW_FRANCE-FIRST_NATION]);
    findBestJob(u, c4);
  }

  city *c1e = createCity (6, "Washington", &map[mapTransfo(38.89, -77.03)], nations[NEW_ENGLAND-FIRST_NATION]);
  if (c1e!=NULL) {
    c1e->type = AGRICULTURE_CITY;
    unit *u = createUnit (CARPENTER, nations[NEW_ENGLAND-FIRST_NATION]);
    addUnitToBuilding(u, c1e, CARPENTRY_SHOP);
    c1e->stocks[CORN-FIRST_GOOD].quantity = 50;
    u = createUnit (BLACKSMITH, nations[NEW_ENGLAND-FIRST_NATION]);
    addUnitToBuilding(u, c1e, SMITHY);
    c1e->stocks[ORE-FIRST_GOOD].quantity = 60;
    c1e->stocks[LUMBER-FIRST_GOOD].quantity = 80;
    u = createUnit (FARMER, nations[NEW_ENGLAND-FIRST_NATION]);
    findBestJob(u, c1e);
    u = createUnit (LUMBERJACK, nations[NEW_ENGLAND-FIRST_NATION]);
    findBestJob(u, c1e);
    u = createUnit (OREMINER, nations[NEW_ENGLAND-FIRST_NATION]);
    findBestJob(u, c1e);
    u = createUnit (TOBACCOPLANTER, nations[NEW_ENGLAND-FIRST_NATION]);
    findBestJob(u, c1e);
  }

  city *c2e = createCity (1, "Boston", &map[mapTransfo(42.357778, -71.061667)], nations[NEW_ENGLAND-FIRST_NATION]);
  if (c2e!=NULL) {
    c2e->type = AGRICULTURE_CITY;
    unit *u = createUnit (CARPENTER, nations[NEW_ENGLAND-FIRST_NATION]);
    if (u!=NULL)
      addUnitToBuilding(u, c2e, CARPENTRY_SHOP);
    u = createUnit (FARMER, nations[NEW_ENGLAND-FIRST_NATION]);
    findBestJob(u, c2e);
  }

  // create initial units
  unit *u = createUnitAt(STATEMAN, nations[NEW_FRANCE-FIRST_NATION], 155*MapWidth/516, 80*MapHeight/320);
  if (u!=NULL)
    u->experience = 3;

  // one of the two following unit shouldn't be created
  unit *u2 = createUnitAt(SCOUT, nations[NEW_FRANCE-FIRST_NATION], 1, 1);
  unit *u1 = createUnitAt(GALLEON, nations[NEW_FRANCE-FIRST_NATION], 1, 1);
  if (u2!=NULL && u1!=NULL){
    printf("2 units test failed\n");fflush(stdout);
    exit(1001);
  }

  u = createUnitAt(TOBACCONIST, nations[NEW_FRANCE-FIRST_NATION], 155*MapWidth/516, 78*MapHeight/320);
  u = createUnitAt(SCOUT, nations[NEW_FRANCE-FIRST_NATION], 156*MapWidth/516, 78*MapHeight/320);
  if (u!=NULL) addHorses(u);
  u = createUnitAt(BLACKSMITH, nations[NEW_FRANCE-FIRST_NATION], 157*MapWidth/516, 78*MapHeight/320);
  if (u!=NULL) u->tools = 60;
  u = createUnitAt(PREACHER, nations[NEW_FRANCE-FIRST_NATION], 158*MapWidth/516, 78*MapHeight/320);
  if (u!=NULL) u->muskets = 50;
  u = createUnitAt(WEAVER, nations[NEW_FRANCE-FIRST_NATION], 159*MapWidth/516, 78*MapHeight/320);
  if (u!=NULL) u->muskets = 50;
  if (u!=NULL) addHorses(u);
  u = createUnitAt(SUGARPLANTER, nations[NEW_FRANCE-FIRST_NATION], 160*MapWidth/516, 78*MapHeight/320);
  if (u!=NULL) u->tools = 40;
  if (u!=NULL) addHorses(u);

  u = createUnitAt(PIONEER, nations[NEW_FRANCE-FIRST_NATION], 158*MapWidth/516, 77*MapHeight/320);
  u = createUnitAt(SOLDIER, nations[NEW_FRANCE-FIRST_NATION], 614*MapWidth/2160, 307*MapHeight/1080);
  if (u!=NULL) u->health = 75;
  u = createUnitAt(MISSIONNARY, nations[NEW_FRANCE-FIRST_NATION], 150*MapWidth/516, 80*MapHeight/320);
  u = createUnitAt(SCOUT, nations[NEW_ENGLAND-FIRST_NATION], 152*MapWidth/516, 87*MapHeight/320);
  u = createUnitAt(ROYAL, nations[NEW_ENGLAND-FIRST_NATION], 650*MapWidth/2160, 285*MapHeight/1080);
  if (u!=NULL) u->experience = 2;
  u = createUnitAt(PRIVATEER, nations[NEW_ENGLAND-FIRST_NATION], 154*MapWidth/516, 93*MapHeight/320);
  u = createUnitAt(SOLDIER, nations[NEW_ENGLAND-FIRST_NATION], 154*MapWidth/516, 86*MapHeight/320);
  if (u!=NULL) {
    addHorses(u);
    u->maxMovement = u->maxMovement*2;
  }

  //printf("u=%p\n", u);
  tile *t = &map[mapTransfo(42.357778, -30.061667)];;
  u = createUnitAt(CARAVEL, nations[NEW_FRANCE-FIRST_NATION], t->posX,  t->posY);
  if (u!=NULL) {
    u->goods[0].type      = LUMBER;
    u->goods[0].quantity  = 100;
    u->goods[1].type      = FURTRAPPER;
    u->goods[1].quantity  = 1;
    u->goods[1].passenger = createUnit(FURTRAPPER, nations[NEW_FRANCE-FIRST_NATION]);
    u->goods[1].passenger->status = ONBOARD;
  }
  u = createUnitAt(SCOUT, nations[NEW_FRANCE-FIRST_NATION], 151*MapWidth/517, 83*MapHeight/320);

  linkCitiesWithRoad("Tadoussac", "Quebec");
  linkCitiesWithRoad("Quebec", "Montreal");

  if (c3!=NULL && c2!=NULL)
    createRoadBetweenCities(c3, c2);
  if (c1!=NULL && c4!=NULL)
    createRoadBetweenCities(c1, c4);
  if (c0!=NULL && c4!=NULL)
    createRoadBetweenCities(c0, c4);
  if (c1e!=NULL && c2e!=NULL)
    createRoadBetweenCities(c1e, c2e);
   
  u = createUnitAt(GALLEON, nations[NEW_FRANCE-FIRST_NATION], map[mapTransfo(42.357778, -68.061667)].posX, map[mapTransfo(42.357778, -68.061667)].posY);
  if (u!=NULL) {
    u->goods[0].type      = GOLD;
    u->goods[0].quantity  = 100;
    u->goods[1].type      = STATEMAN;
    u->goods[1].quantity  = 1;
    u->goods[1].passenger = createUnit(STATEMAN, nations[NEW_FRANCE-FIRST_NATION]);
    u->goods[1].passenger->status = ONBOARD;
    u->goods[2].type      = SILVER;
    u->goods[2].quantity  = 100;
    u->goods[3].type      = PEARL;
    u->goods[3].quantity  = 100;
    u->goods[4].type      = GEM;
    u->goods[4].quantity  = 100;
    u->goods[5].type      = SOLDIER;
    u->goods[5].quantity  = 1;
    u->goods[5].passenger = createUnit(SOLDIER, nations[NEW_FRANCE-FIRST_NATION]);
    u->goods[5].passenger->status = ONBOARD;
  }   

  lnode_t *nodeC1 = list_find(nations[FRANCE-FIRST_NATION]->cityList, "Brest", compareCityName);
  tile *t1 = ((city*)nodeC1->list_data)->location;
  tile *t2 = c0->location;
  tile *tprev = t1;
  int i, step = 16;
  for (i=0; i<=step;i++) {
    int x = (((step-i)*t1->posX+(i)*t2->posX)/step);
    int y = ((step-i)*t1->posY+(i)*t2->posY)/step;
    tile *ti = &map[x+MapWidth*(y)];
    astarSearch(tprev->posX, tprev->posY, ti->posX, ti->posY, u);
    revealPath();
    tprev = ti;
  }

  nodeC1 = list_find(nations[FRANCE-FIRST_NATION]->cityList, "Biarritz", compareCityName);
  t1 = ((city*)nodeC1->list_data)->location;
  t2 = c2e->location;
  tprev = t1;
  for (i=0; i<=step;i++) {
    int x = (((step-i)*t1->posX+(i)*t2->posX)/step);
    int y = ((step-i)*t1->posY+(i)*t2->posY)/step;
    tile *ti = &map[x+MapWidth*(y)];
    astarSearch(tprev->posX, tprev->posY, ti->posX, ti->posY, u);
    revealPath();
    tprev = ti;
  }  
  
  nodeC1 = list_find(nations[PORTUGAL-FIRST_NATION]->cityList, "Porto", compareCityName);
  t1 = ((city*)nodeC1->list_data)->location;
  t2 = c1e->location;
  tprev = t1;
  for (i=0; i<=step;i++) {
    int x = (((step-i)*t1->posX+(i)*t2->posX)/step);
    int y = ((step-i)*t1->posY+(i)*t2->posY)/step;
    tile *ti = &map[x+MapWidth*(y)];
    astarSearch(tprev->posX, tprev->posY, ti->posX, ti->posY, u);
    revealPath();
    tprev = ti;
  }  
}// initDebugCitiesAndUnits() --------------------------------------------------------------------


/**
 * find the 2 cities with the given names and draw a reoad between them
 */
int linkCitiesWithRoad(const char* cname1, const char* cname2){
  lnode_t *ln = list_first(playerList);
  lnode_t *nodeC1=NULL, *nodeC2=NULL;
  while (ln!=NULL && ln!= &playerList->list_nilnode){
    player *p = (player*)ln->list_data;
    if(nodeC1==NULL)
      nodeC1 = list_find(p->cityList, cname1, compareCityName);
    if(nodeC2==NULL)
      nodeC2 = list_find(p->cityList, cname2, compareCityName);
    ln = ln->list_next;
  }

  if (nodeC1!=NULL && nodeC2!=NULL){
    city *city1 = (city*)nodeC1->list_data;
    city *city2 = (city*)nodeC2->list_data;
    if (city1!=NULL && city2!=NULL) {
      //printf("nodes are %p and %p => %s %s\n", city1, city2,  city1->name, city2->name);
      list_append(city1->connectedCities, lnode_create(city2));
      list_append(city2->connectedCities, lnode_create(city1));

      return createRoadBetweenCities(city1, city2);
    }else
      return 0;
  } else {
     printf("ERROR: can't link cities together (%s - %s) (%p, %p)\n", cname1 ,cname2, nodeC1, nodeC2);fflush(stdout);
     return 0;
  }
}// linkCitiesWithRoad()


/**
 * init cities, units, roads of europe
 */
void initEurope() {
  addMsg("Initializing europe");
  updateLoading();
  initFrance();
  initEngland();
  initSpain();
  initPortugal();
  initHolland();
  initItaly();
  initGermany();
  initRussia();

  linkCitiesWithRoad("Nice", "Genoa");
  linkCitiesWithRoad("Grenoble", "Torino");

  linkCitiesWithRoad("Barcelona", "Perpignan");
  linkCitiesWithRoad("Biarritz", "San Sebastian");
  linkCitiesWithRoad("Andorra", "Barcelona");
  linkCitiesWithRoad("Andorra", "Perpignan");

  linkCitiesWithRoad("Lisboa", "Badajoz");
  linkCitiesWithRoad("Loule", "Sevilla");
  linkCitiesWithRoad("Vigo", "Viana de Castelo");
  linkCitiesWithRoad("Chaves", "Ourense");
  linkCitiesWithRoad("Salamanca", "Guarda");
}// initEurope()


/**
 * init cities, units of natives nations
 */
void initNatives() {
  addMsg("Initializing native nations");
  updateLoading();

  initIroquois();
}// initIndians()


/**
 * reset the map
 */
void initTiles(int w, int h) {
  int i, j;
  MapWidth = w;
  MapHeight = h;
  if (map!=NULL)
    free(map);
  map = NULL;
  printf("Allocating memory for the BIG map(%d * %d = %d tiles = %dMB) for tile sized=%iBytes\n", MapWidth, MapHeight, MapWidth*MapHeight, (MapWidth*MapHeight*sizeof(tile))/(1024*1024), sizeof(tile));fflush(stdout);
  map = (tile*)malloc(MapWidth*MapHeight*sizeof(tile));
  printf("Allocation done %p!\n", map);fflush(stdout);
  if (map==NULL){
    exit(444);
  }

  for(i=0; i<MapWidth; i++) {
    for(j=0; j<MapHeight; j++) {
      map[i+MapWidth*j].unitList     = NULL;
      map[i+MapWidth*j].ground       = PLAIN;
      map[i+MapWidth*j].bonus        = 0;
      map[i+MapWidth*j].unitList     = NULL;
      map[i+MapWidth*j].posX         = i;
      map[i+MapWidth*j].posY         = j;
      map[i+MapWidth*j].lastExplored = -1;
      map[i+MapWidth*j].elevation    = OCEAN1;
      map[i+MapWidth*j].vegetation   = WILDGRASS;
      map[i+MapWidth*j].construction = NULL;
      map[i+MapWidth*j].seed         = ((rand()%8)<<28)+((rand()%8)<<24)+((rand()%8)<<20)+((rand()%8)<<16)+
        ((rand()%8)<<12)+((rand()%8)<<8)+((rand()%8)<<4)+((rand()%8)<<0);
      map[i+MapWidth*j].irrigation   = UNKNOWN;
      map[i+MapWidth*j].river_elem   = NULL;
      //map[i+MapWidth*j].influences   = (int*)malloc(NATION_NUMBER*sizeof(int));
    }
  }
  //updateMiniMap();
}// initTiles()


/**
 * Fonction used in the robinson map projection
 * These two polynoms were used by The national geographic society
 * to interpolate the robinson table (I couldn't find it) which give
 * RobinsonLm and RobinsonLp for every 5 degrees
 */
double RobinsonLm(double Lambda_Arg) {
  return  0.1705296626e-1*Lambda_Arg
  -0.8012589080e-4*pow(Lambda_Arg, 2)
  +0.7438423230e-5*pow(Lambda_Arg, 3)
  -0.3702182944e-6*pow(Lambda_Arg, 4)
  +0.1109413766e-7*pow(Lambda_Arg, 5)
  -0.2031450383e-9*pow(Lambda_Arg, 6)
  +0.2180138366e-11*pow(Lambda_Arg, 7)
  -0.1248672025e-13*pow(Lambda_Arg, 8)
  +0.2905471347e-16*pow(Lambda_Arg, 9);
}// RobinsonLm()


/**
 * Second fonction used in the robinson map projection
 */
double RobinsonLp(double Lambda_Arg) {
  return  2.627878724
  +0.1294792647e-2*Lambda_Arg
  -0.4935937156e-3*pow(Lambda_Arg, 2)
  +0.3614303665e-4*pow(Lambda_Arg, 3)
  -0.1581670382e-5*pow(Lambda_Arg, 4)
  +0.3806466552e-7*pow(Lambda_Arg, 5)
  -0.5800621828e-9*pow(Lambda_Arg, 6)
  +0.5852194027e-11*pow(Lambda_Arg, 7)
  -0.3624112212e-13*pow(Lambda_Arg, 8)
  +0.1024113068e-15*pow(Lambda_Arg, 9);
}// RobinsonLp()


/**
 * Robinson inverse dichotomy researsh
 */
void robinsonInverseTransfo(const double x_Arg, const double y_Arg, double *longitude, double *latitude, double precision_Arg) {
  // here his the parameter of the robinson projection
  double R = 1;
  double x = x_Arg*150.57f;
  double y = y_Arg*77.3f;

  double latMin = -90;
  double latMax = +90;

  while (latMax-latMin>precision_Arg) {
    double latAvg              = (latMin+latMax)/2;

    double latAvgSign          = 1;
    if (latAvg<0) latAvgSign  = -1;
    double yForLatAvg          = R*RobinsonLm(fabs(latAvg))*latAvgSign*180.0/M_PI;

    if (yForLatAvg>=y)
      latMax = latAvg;
    else
      latMin = latAvg;
  }

  *latitude = (latMin+latMax)/2;

  double lonMin = -180;
  double lonMax = +180;

  while (lonMax-lonMin>precision_Arg) {
    double lonAvg              = (lonMin+lonMax)/2;
    double lonAvgSign          = 1;
    if (lonAvg<0) lonAvgSign  = -1;
    double xForLonAvg          = R*RobinsonLp(fabs(*latitude))*lonAvg/M_PI;

    if (xForLonAvg>=x)
      lonMax = lonAvg;
    else
      lonMin = lonAvg;
  }

  *longitude = (lonMin+lonMax)/2;
}// robinsonInverseTransfo()


/**
 * Robinson map projection
 **/
void robinsonTransfo(const double longitude_Arg, const double latitude_Arg, double *x, double *y) {
  // here his the parameter of the robinson projection
  double R = 1;

  double latSign = 1;
  if (latitude_Arg<0) latSign = -1;

  *x = (R*RobinsonLp(fabs(latitude_Arg))*longitude_Arg/M_PI)/150.57;
  *y = (R*RobinsonLm(fabs(latitude_Arg))*latSign*180.0/M_PI)/77.3;
}// RobinsonTransfo()


/**
 * Robinson map projection directly into an index in the tile table
 */
int mapTransfo(const double latitude_Arg, const double longitude_Arg) {
  if (isRobinsonTransfo) {
    double x, y;
    robinsonTransfo(longitude_Arg, -latitude_Arg, &x, &y);
    x = lrint((x*MapWidth/2)+MapWidth/2);
    y = lrint((y*MapHeight/2)+MapHeight/2);
    return (int)x + (MapWidth)*(int)y;
  } else {
    int x = (longitude_Arg+180)*MapWidth/360;
    int y = (latitude_Arg+90)*MapHeight/180;
    return x + MapWidth*y;
  }
}// mapTransfo()


/**
 * return the elevation corresponding to the given color in the colored map
 */
int getElevationForColor (const unsigned char r1, const unsigned char g1, const unsigned char b1){
  int mindcol = 1000000;
  int elev    = SEALEVEL;

  int dcol = (r1-0x0)*(r1-0x0)+(g1-119)*(g1-119)+(b1-29)*(b1-29); // DEEP GREEN
  if (dcol<mindcol) {
    mindcol = dcol;
    elev = SEALEVEL;
  }
  dcol = (r1-0x80)*(r1-0x80)+(g1-0x80)*(g1-0x80)+(b1-0x80)*(b1-0x80); // GRAY
  if (dcol<mindcol) {
    mindcol = dcol;
    elev = MOUNTAIN;
  }
  dcol = (r1-0xF4)*(r1-0xF4)+(g1-0xF4)*(g1-0xF4)+(b1-0xF4)*(b1-0xF4); // WHITE
  if (dcol<mindcol) {
    mindcol = dcol;
    elev = HIGHMOUNTAIN;
  }
  dcol = (r1-0xF0)*(r1-0xF0)+(g1-0xF0)*(g1-0xF0)+(b1-0x0)*(b1-0x0); // YELLOW
  if (dcol<mindcol) {
    mindcol = dcol;
    elev = HILL;
  }
  dcol = (r1-0x0)*(r1-0x0)+(g1-0x0)*(g1-0x0)+(b1-126)*(b1-126); // light blue
  if (dcol<mindcol) {
    mindcol = dcol;
    elev = OCEAN1;
  }
  dcol = (r1-0x82)*(r1-0x82)+(g1-0x3C)*(g1-0x3C)+(b1-0x1E)*(b1-0x1E); // BROWN
  if (dcol<mindcol) {
    mindcol = dcol;
    elev = HIGHHILL;
  }
  dcol = (r1-0x0)*(r1-0x0)+(g1-0x0)*(g1-0x0)+(b1-191)*(b1-191); // DARK BLUE
  if (dcol<mindcol) {
    mindcol = dcol;
    elev = OCEAN0;
  }

  return elev;
}// getElevationForColor()


/**
 * try to set the elevation of every tile from a bitmap
 */
void initGroundsFromBitmap(const char* bitmapName, int ratioX, int ratioY, char isRobinsonTransfo_arg) {
  isRobinsonTransfo = isRobinsonTransfo_arg;
  SDL_Surface *img  = IMG_LoadAndFormatAlpha(bitmapName);
  if (img!=NULL)
    printf(" init ground from %s => %i/%i * %i/%i => %i, %i\n", bitmapName, img->w, ratioX, img->h, ratioY, img->w/ratioX, img->h/ratioY);
  else {
    printf(" Sorry, we couln't find the bitmap %s\n", bitmapName);
    exit(103);
  }
  initTiles(img->w/ratioX, img->h/ratioY);

  Uint32 Rshift = img->format->Rshift;
  Uint32 Gshift = img->format->Gshift;
  Uint32 Bshift = img->format->Bshift;
  Uint32 Ashift = img->format->Ashift;
  Uint32 Rmask  = img->format->Rmask;
  Uint32 Gmask  = img->format->Gmask;
  Uint32 Bmask  = img->format->Bmask;
  Uint32 Amask  = img->format->Amask;
  //printf("init ground from %s => %i*%i rmask=%8x rshift=%8x\n", bitmapName, img->w, img->h, Rmask, Rshift);
  int elev;
  int i, j;
  int percentloading=0, lastpercentloading=0;
  Uint32 *pixelbmp;
  for(j=0; j<MapHeight; j++) {
    int percentloading = j*100/MapHeight;
    if (percentloading!=lastpercentloading){
      initcpt++;
      lastpercentloading = percentloading;
      updateLoading();
    }
    
    for(i=0; i<MapWidth; i++) {
      if (isRobinsonTransfo) {
        double lon, lat;

        /*if ((i<50 && j==0) || (i==662 && j<250 ))
          robinsonInverseTransfo2( ((2.0*i/MapWidth)-1.0), ((2.0*j/MapHeight)-1.0), &lon, &lat, 0.01);
        else*/
        robinsonInverseTransfo( ((2.0*i/MapWidth)-1.0), ((2.0*j/MapHeight)-1.0), &lon, &lat, 0.05);
        //if ((i<50 && j==0) || (i==667 && j<250 ))
        //  printf("  %d, %d => %f, %f \n", i, j, lon, lat);

        lon = (lon+180.0)*img->w/360.0;
        lat = (lat+90.0)*img->h/180.0;
        if (lon<1 || lon>MapWidth-1)
          continue;

        float propx = (lon+0.5)-(int)(lon+0.5);
        float propy = (lat+0.5)-(int)(lat+0.5);
        float apropx, apropy;
        int i2 = (int)(lon+0.5);
        int j2 = (int)(lat+0.5);
        int dx, dy;
        //if ((i<50 && j==0) || (i==667 && j<250 ) || j2<0  || i2<0 || j2>=MapHeight || i2>=MapWidth)
        //  printf("  => %f, %f => %d, %d  (i=%d j=%d) \n", lon, lat, i2, j2, i, j);

        pixelbmp = (Uint32 *) img->pixels + (j2) *(img->w) + (i2);
        Uint32 r1 = (*pixelbmp & Rmask) >> Rshift;
        Uint32 g1 = (*pixelbmp & Gmask) >> Gshift;
        Uint32 b1 = (*pixelbmp & Bmask) >> Bshift;
        float elev2 = 0;
        if (propx>0.5) {dx = 1;  apropx = 2.0*(propx-0.5); propx = 2.0*(1.0-propx);}
        else           {dx = -1; apropx = 2.0*(0.5-propx); propx = 2.0*(propx);}
        if (propy>0.5) {dy = 1;  apropy = 2.0*(propy-0.5); propy = 2.0*(1.0-propy);}
        else           {dy = -1; apropy = 2.0*(0.5-propy); propy = 2.0*(propy);}
        elev2 += propx*propy*getElevationForColor(r1, g1, b1);

        pixelbmp = (Uint32 *) img->pixels + (j2) *(img->w) + (i2+dx);
        r1 = (*pixelbmp & Rmask) >> Rshift;
        g1 = (*pixelbmp & Gmask) >> Gshift;
        b1 = (*pixelbmp & Bmask) >> Bshift;
        elev2 += apropx*propy*getElevationForColor(r1, g1, b1);

        pixelbmp = (Uint32 *) img->pixels + (j2+dy) *(img->w) + (i2);
        r1 = (*pixelbmp & Rmask) >> Rshift;
        g1 = (*pixelbmp & Gmask) >> Gshift;
        b1 = (*pixelbmp & Bmask) >> Bshift;
        elev2 += propx*apropy*getElevationForColor(r1, g1, b1);

        pixelbmp = (Uint32 *) img->pixels + (j2+dy) *(img->w) + (i2+dx);
        r1 = (*pixelbmp & Rmask) >> Rshift;
        g1 = (*pixelbmp & Gmask) >> Gshift;
        b1 = (*pixelbmp & Bmask) >> Bshift;
        elev2 += apropx*apropy*getElevationForColor(r1, g1, b1);
        elev = lrint(elev2);
      } else {
        int imgx1 = img->w*i/MapWidth;
        int imgy1 = img->h*j/MapHeight;
        pixelbmp = (Uint32 *) img->pixels + (imgy1) * (img->w) + (imgx1);
        Uint32 r1 = (*pixelbmp & Rmask) >> Rshift;
        Uint32 g1 = (*pixelbmp & Gmask) >> Gshift;
        Uint32 b1 = (*pixelbmp & Bmask) >> Bshift;
        Uint32 a1 = (*pixelbmp & Amask) >> Ashift;
        if (a1<128)
          elev = OCEAN1;
        else
          elev = getElevationForColor(r1, g1, b1);
      }

      /*if (isRobinsonTransfo) {

        if (i==10 && j<50) {
          robinsonTransfo2(((i-MapWidth/2.0)*180.0)/MapWidth, ((j-MapHeight/2.0)*180.0)/MapHeight, &i2, &j2);
          i2 = (i2*MapWidth)+MapWidth/2;
          j2 = (j2*MapHeight/2)+MapHeight/2;
          printf("  %f, %f => %d, %d => %d\n", i2, j2, lrint(i2), lrint(j2), lrint(i2)+MapWidth*lrint(j2));
          fflush(stdout);
        } else {
          robinsonTransfo3(((i-MapWidth/2.0)*180.0)/MapWidth, ((j-MapHeight/2.0)*180.0)/MapHeight, &i2, &j2);
          i2 = (i2*MapWidth)+MapWidth/2;
          j2 = (j2*MapHeight/2)+MapHeight/2;
        }
        //if (i==0 && j<20)
        //  printf("      %f, %f =>%d, %d \n", i2, j2, (int)i2, (int)j2);
      } else {
        i2 = i;
        j2 = j;
      }*/
      assert(i>=0 &&i<MapWidth && j>=0 && j<MapHeight);
      map[i+MapWidth*(j)].elevation    = elev;
      map[i+MapWidth*(j)].seed         = (((rand()%15)+1)<<28)+(((rand()%15)+1)<<24)+(((rand()%15)+1)<<20)+(((rand()%15)+1)<<16)+(((rand()%15)+1)<<12)+(((rand()%15)+1)<<8)+(((rand()%15)+1)<<4)+(((rand()%15)+1)<<0);
      //boxRGBA(minimap, (int)(i*minimapZoom), (int)(j*minimapZoom), (int)((i+1)*minimapZoom), (int)((j+1)*minimapZoom), r1, g1, b1, 255);
    }
  }
  percentloading=0;
  lastpercentloading=0;

  for(j=0; j<MapHeight; j++) {
    int percentloading = j*100/MapHeight;
    if (percentloading!=lastpercentloading){
      initcpt++;
      lastpercentloading = percentloading;
      updateLoading();
    }
    for(i=0; i<MapWidth; i++) {
      if (i==0 || j==0 || i== MapWidth-1 || j==MapHeight-1) {
        continue;
      }

      if ((map[i+MapWidth*(j)].elevation<map[i+1+MapWidth*(j)].elevation && map[i+MapWidth*(j)].elevation<map[i+MapWidth*(j+1)].elevation && map[i+MapWidth*(j)].elevation<map[i+1+MapWidth*(j+1)].elevation) ||
          (map[i+MapWidth*(j)].elevation>map[i+1+MapWidth*(j)].elevation && map[i+MapWidth*(j)].elevation>map[i+MapWidth*(j+1)].elevation && map[i+MapWidth*(j)].elevation>map[i+1+MapWidth*(j+1)].elevation)) {
        //map[i+MapWidth*(j)].seed         = ((((map[i+MapWidth*(j)].seed&0x0000000F)>>0)/2)<<0)+map[i+MapWidth*(j)].seed&0xFFFFFFF0;
        unsigned char inf = (map[i+MapWidth*(j)].seed&0x0000000F)>>0;
        inf = inf/4;
        map[i+MapWidth*(j)].seed =  (inf<<0) | (map[i+MapWidth*(j)].seed&0xFFFFFFF0);
        inf = (map[i+MapWidth*(j)].seed&0x000F0000)>>16;
        inf = inf/2+1;
        map[i+MapWidth*(j)].seed =  (inf<<16) | (map[i+MapWidth*(j)].seed&0xFFF0FFFF);
        inf = (map[i+MapWidth*(j)].seed&0x000000F0)>>4;
        inf = inf/2+1;
        map[i+MapWidth*(j)].seed =  (inf<<4) | (map[i+MapWidth*(j)].seed&0xFFFFFF0F);
      }
      if ((map[i+MapWidth*(j)].elevation>map[i-1+MapWidth*(j-1)].elevation && map[i-1+MapWidth*(j)].elevation>map[i-1+MapWidth*(j-1)].elevation && map[i+MapWidth*(j-1)].elevation>map[i-1+MapWidth*(j-1)].elevation)||
          (map[i+MapWidth*(j)].elevation<map[i-1+MapWidth*(j-1)].elevation && map[i-1+MapWidth*(j)].elevation<map[i-1+MapWidth*(j-1)].elevation && map[i+MapWidth*(j-1)].elevation<map[i-1+MapWidth*(j-1)].elevation)){
        //map[i+MapWidth*(j)].seed         = ((((map[i+MapWidth*(j)].seed&0xF0000000)>>28)+4)<<0)+map[i+MapWidth*(j)].seed&0x0FFFFFFF;
        unsigned char inf = (map[i+MapWidth*(j)].seed&0xF0000000)>>28;
        if (inf<8)
          inf = inf*2;
        else inf = 15;
        map[i+MapWidth*(j)].seed = (inf<<28) | (map[i+MapWidth*(j)].seed&0x0FFFFFFF);
      }
      if ((map[i+MapWidth*(j)].elevation>map[i-1+MapWidth*(j)].elevation && map[i+MapWidth*(j)].elevation>map[i+MapWidth*(j-1)].elevation && map[i+MapWidth*(j)].elevation>map[i-1+MapWidth*(j-1)].elevation)||
          (map[i+MapWidth*(j)].elevation<map[i-1+MapWidth*(j)].elevation && map[i+MapWidth*(j)].elevation<map[i+MapWidth*(j-1)].elevation && map[i+MapWidth*(j)].elevation<map[i-1+MapWidth*(j-1)].elevation)){
        //map[i+MapWidth*(j)].seed         =((((map[i+MapWidth*(j)].seed&0xF0000000)>>28)/2)<<28)+map[i+MapWidth*(j)].seed&0x0FFFFFFF;
        unsigned char inf = (map[i+MapWidth*(j)].seed&0xF0000000)>>28;
        inf = inf/4;
        map[i+MapWidth*(j)].seed = (inf<<28) | (map[i+MapWidth*(j)].seed&0x0FFFFFFF);
        inf = (map[i+MapWidth*(j)].seed&0x0F000000)>>24;
        inf = inf/2+1;
        map[i+MapWidth*(j)].seed = (inf<<24) | (map[i+MapWidth*(j)].seed&0xF0FFFFFF);
        inf = (map[i+MapWidth*(j)].seed&0x0000F000)>>12;
        inf = inf/2+1;
        map[i+MapWidth*(j)].seed = (inf<<12) | (map[i+MapWidth*(j)].seed&0xFFFF0FFF);
      }
      if ((map[i+MapWidth*(j)].elevation<map[i+1+MapWidth*(j+1)].elevation && map[i+1+MapWidth*(j)].elevation<map[i+1+MapWidth*(j+1)].elevation && map[i+MapWidth*(j+1)].elevation<map[i+1+MapWidth*(j+1)].elevation) ||
          (map[i+MapWidth*(j)].elevation>map[i+1+MapWidth*(j+1)].elevation && map[i+1+MapWidth*(j)].elevation>map[i+1+MapWidth*(j+1)].elevation && map[i+MapWidth*(j+1)].elevation>map[i+1+MapWidth*(j+1)].elevation)){
        //map[i+MapWidth*(j)].seed         = ((((map[i+MapWidth*(j)].seed&0xF0000000)>>28)+4)<<0)+map[i+MapWidth*(j)].seed&0x0FFFFFFF;
        unsigned char inf = (map[i+MapWidth*(j)].seed&0x0000000F)>>0;
        if (inf<8)
          inf = inf*2;
        else inf = 15;
        map[i+MapWidth*(j)].seed = (inf<<0) | (map[i+MapWidth*(j)].seed&0xFFFFFFF0);
      }

      if ((map[i+MapWidth*(j)].elevation<map[i-1+MapWidth*(j)].elevation && map[i+MapWidth*(j)].elevation<map[i+MapWidth*(j+1)].elevation && map[i+MapWidth*(j)].elevation<map[i-1+MapWidth*(j+1)].elevation) ||
          (map[i+MapWidth*(j)].elevation>map[i-1+MapWidth*(j)].elevation && map[i+MapWidth*(j)].elevation>map[i+MapWidth*(j+1)].elevation && map[i+MapWidth*(j)].elevation>map[i-1+MapWidth*(j+1)].elevation)) {
        //map[i+MapWidth*(j)].seed         = ((((map[i+MapWidth*(j)].seed&0x0000000F)>>0)/2)<<0)+map[i+MapWidth*(j)].seed&0xFFFFFFF0;
        unsigned char inf = (map[i+MapWidth*(j)].seed&0x00000F00)>>8;
        inf = inf/4;
        map[i+MapWidth*(j)].seed =  (inf<<0)  | (map[i+MapWidth*(j)].seed&0xFFFFF0FF);
        inf = (map[i+MapWidth*(j)].seed&0x0000F000)>>12;
        inf = inf/2+1;
        map[i+MapWidth*(j)].seed =  (inf<<12) | (map[i+MapWidth*(j)].seed&0xFFFF0FFF);
        inf = (map[i+MapWidth*(j)].seed&0x000000F0)>>4;
        inf = inf/2+1;
        map[i+MapWidth*(j)].seed =  (inf<<4)  | (map[i+MapWidth*(j)].seed&0xFFFFFF0F);
      }
      if ((map[i+MapWidth*(j)].elevation>map[i-1+MapWidth*(j+1)].elevation && map[i-1+MapWidth*(j)].elevation>map[i-1+MapWidth*(j+1)].elevation && map[i+MapWidth*(j+1)].elevation>map[i-1+MapWidth*(j+1)].elevation)||
          (map[i+MapWidth*(j)].elevation<map[i-1+MapWidth*(j+1)].elevation && map[i-1+MapWidth*(j)].elevation<map[i-1+MapWidth*(j+1)].elevation && map[i+MapWidth*(j+1)].elevation<map[i-1+MapWidth*(j+1)].elevation)){
        //map[i+MapWidth*(j)].seed         = ((((map[i+MapWidth*(j)].seed&0xF0000000)>>28)+4)<<0)+map[i+MapWidth*(j)].seed&0x0FFFFFFF;
        unsigned char inf = (map[i+MapWidth*(j)].seed&0x00000F00)>>8;
        if (inf<8)
          inf = inf*2;
        else inf = 15;
        map[i+MapWidth*(j)].seed = (inf<<8) | (map[i+MapWidth*(j)].seed&0xFFFFF0FF);
      }
      if ((map[i+MapWidth*(j)].elevation>map[i+1+MapWidth*(j)].elevation && map[i+MapWidth*(j)].elevation>map[i+MapWidth*(j-1)].elevation && map[i+MapWidth*(j)].elevation>map[i+1+MapWidth*(j-1)].elevation)||
          (map[i+MapWidth*(j)].elevation<map[i+1+MapWidth*(j)].elevation && map[i+MapWidth*(j)].elevation<map[i+MapWidth*(j-1)].elevation && map[i+MapWidth*(j)].elevation<map[i+1+MapWidth*(j-1)].elevation)){
        //map[i+MapWidth*(j)].seed         =((((map[i+MapWidth*(j)].seed&0xF0000000)>>28)/2)<<28)+map[i+MapWidth*(j)].seed&0x0FFFFFFF;
        unsigned char inf = (map[i+MapWidth*(j)].seed&0x00F00000)>>20;
        inf = inf/4;
        map[i+MapWidth*(j)].seed = (inf<<20) | (map[i+MapWidth*(j)].seed&0xFF0FFFFF);
        inf = (map[i+MapWidth*(j)].seed&0x0F000000)>>24;
        inf = inf/2+1;
        map[i+MapWidth*(j)].seed = (inf<<24) | (map[i+MapWidth*(j)].seed&0xF0FFFFFF);
        inf = (map[i+MapWidth*(j)].seed&0x000F0000)>>16;
        inf = inf/2+1;
        map[i+MapWidth*(j)].seed = (inf<<16) | (map[i+MapWidth*(j)].seed&0xFFF0FFFF);
      }
      if ((map[i+MapWidth*(j)].elevation>map[i+1+MapWidth*(j-1)].elevation && map[i+1+MapWidth*(j)].elevation>map[i+1+MapWidth*(j-1)].elevation && map[i+MapWidth*(j-1)].elevation>map[i+1+MapWidth*(j-1)].elevation) ||
          (map[i+MapWidth*(j)].elevation<map[i+1+MapWidth*(j-1)].elevation && map[i+1+MapWidth*(j)].elevation<map[i+1+MapWidth*(j-1)].elevation && map[i+MapWidth*(j-1)].elevation<map[i+1+MapWidth*(j-1)].elevation)){
        //map[i+MapWidth*(j)].seed         = ((((map[i+MapWidth*(j)].seed&0xF0000000)>>28)+4)<<0)+map[i+MapWidth*(j)].seed&0x0FFFFFFF;
        unsigned char inf = (map[i+MapWidth*(j)].seed&0x00F00000)>>20;
        if (inf<8)
          inf = inf*2;
        else inf = 15;
        map[i+MapWidth*(j)].seed = (inf<<20) | (map[i+MapWidth*(j)].seed&0xFF0FFFFF);
      }
    }
  }
  SDL_FreeSurface(img);
  lastLoadedImage = NULL;
}// initGroundsFromBitmap() --------------------------------------------------------------------



/**
 * init cities, units, roads of France
 */
void initFrance() {
  char *name  = (char*)malloc(256*sizeof(char));
  float *path;
  strcpy(name, "Le Loir");
  {
    float path0[]={1.053884406998364,48.37575455422252, 1.054573135021329,48.37594203631, 974, 1.05518650192063,48.37605495180299, 1.055496566881104,
       48.37614981073614, 1.056236782658573,48.37628290442246, 1.057037203590141,48.37639718307727, 1.057498808239046,48.37653060632322, 1.058220623202189,
       48.37673986637816, 1.058882768800686,48.37691066526335, 1.059483737424543,48.37702471366804, 1.06008444566524,48.37717688363555, 1.060625797969195,
       48.37733063797277, 1.060911710609574,48.37742673566844, 1.061600400036795,48.37765657298708, 1.062088788322313,48.37778975986659, 1.062634688956479,
       48.37798080671297, 1.063238354507647,48.37822928147514, 1.063727798379318,48.37857383091129, 1.063986803782925,48.37874593212909, 1.064159735971245,
       48.37891822477388, 1.064447490341877,48.37909026999987, 1.064850270490695,48.37928137494816, 1.06530701303366,48.37967924271106, 1.065560239573514,
       48.37988414883661, 1.066348668074199,48.38015761876538, 1.066630537597916,48.38032520054441, 1.066999946143014,48.38062711507067, 1.067312541259199,
       48.38096717871945, 1.067399962493525,48.38115826740406, 1.067578354039043,48.38158178762568, 1.067871561350088,48.38201859993324, 1.068042572974235,
       48.38217041142522, 1.068328127516722,48.38241736775206, 1.068812926181622,48.3824927221883, 1.069298643513662,48.38253006773222, 1.069986820695888,
       48.38258669100102, 1.070877445247518,48.38262552845009, 1.071192802933098,48.38262582326335, 1.071622866731232,48.38266385765257, 1.072425771557221,
       48.38289224472539, 1.072712592184089,48.38296836952149, 1.073142306927792,48.38294930246133, 1.073428683941273,48.38291121134004, 1.073829289977155,
       48.38283507707484, 1.0745183657083,48.38281582941771, 1.075008177115933,48.38285366590683, 1.0760183470681,48.38287223090985, 1.076448677918762,
       48.38283403794321, 1.078019875496523,48.38266267282251, 1.078937766956443,48.38251009715193, 1.07931384654826,48.38250978806873, 1.079921959871385,
       48.3825092846572, 1.080725008375708,48.38243291242161, 1.081098643941772,48.38239466936043, 1.081790247404115,48.38231805957645, 1.082223994325815,
       48.38224143345266, 1.082636601829123,48.38236183083762, 1.082948278110166,48.38242189445695, 1.083460962307115,48.38238730648064, 1.083893134458421,
       48.38233004890262, 1.084643419220825,48.38221456656652, 1.085191267541262,48.38221456179007, 1.085566247244032,48.38223357234394, 1.085912778549229,
       48.38223259361136, 1.086662309287348,48.38229018906513, 1.087037174900544,48.38234850490247, 1.087556384028758,48.38234836962738, 1.08787391991936,
       48.38242549132288, 1.088537196031695,48.38253963427282, 1.088854724032942,48.38261645658479, 1.089172313661535,48.38267413869901, 1.08957636629971,
       48.38273162829715, 1.089865289176956,48.38278939893318, 1.090503299818664,48.38300268557148, 1.090937440207355,48.38309915552464, 1.09116713391559,
       48.38305968732566, 1.091944782236027,48.38300068147933, 1.092559440233501,48.38310110052498, 1.093338735657156,48.38311517478772, 1.093657821344905,
       48.38321105577904, 1.094346510025176,48.38344222120775, 1.09495475070834,48.38365350142962, 1.09521556298742,48.38374950704117, 1.096060062539166,
       48.38407664239688, 1.096905054984987,48.38448656562531, 1.097460167415164,48.38489753794108, 1.097631403809416,48.38568100894433, 1.097953489831482,
       48.38646483623152, 1.098297786518202,48.38706147636444, 1.098362746831739,48.38747558704372, 1.098214248871187,48.38825920940246, 1.098359629192927,
       48.38926697873586, 1.099142649025794,48.38980814129758, 1.100397205614286,48.39030636663206, 1.101282667822602,48.39090811553367, 1.102119691946868,
       48.39104033601129, 1.103027011461506,48.39094183427886, 1.10419235481001,48.39090015840188, 1.105334517909085,48.39115252174889, 1.105955127144128,
       48.39069409659891, 1.106579579652248,48.39037057252546, 1.107550455682741,48.39023007636457, 1.108592075649241,48.38994972474742, 1.109420023152495,48.38981290314108, 1.110039240661777,48.38963193447477, 1.110933427667177,48.38922338072852, 1.111830654169395,48.38881192989379, 1.1129348240258,48.38854075877706, 1.113487227224994,48.38840572920601, 1.11410970570708,48.38808555230756, 1.115634466581696,48.38781175366503, 1.116605822167742,48.38771718622758, 1.117229799555503,48.38753186253165, 1.118755820997436,48.38771529717414, 1.119378542505302,48.38766624631836, 1.120276705742398,48.38743191950005, 1.120897593922123,48.38728069839635, 1.121234282817241,48.3873921898443, 1.122117383078006,48.3876852964364, 1.122556083141831,48.38813276160123, 1.122826760612611,48.3883567150284, 1.12323000117236,48.38853511218998, 1.1238754545831,48.38858122971291, 1.124515504578056,48.38860346843926, 1.124817864246254,48.38873704045426, 1.125356249484895,48.38887059452099, 1.125861470044527,48.38895966244166, 1.126770989944845,48.38909343510975, 1.127647054870351,48.38911516136002, 1.128052319503055,48.38909234432139, 1.129033206702801,48.38918097419793, 1.129473315740157,48.38918058863304, 1.130625012429746,48.38908987601837, 1.131475599399565,48.38899908144243, 1.131850429768718,48.38890871473684, 1.132429027439088,48.38868314734028, 1.132871093811019,48.38857015508571, 1.134120407832,48.38823244224602, 1.134899676390603,48.38809690134301, 1.136264112002181,48.38802711148467, 1.13718676248115,48.38800299234973, 1.138220512212243,48.38806802586315, 1.139054982810919,48.38824612717792, 1.13969208254801,48.3883142168705, 1.141303410699848,48.38847252318204, 1.142213682417861,48.38865158892564, 1.143608384794646,48.38887625513093, 1.145218430311644,48.38916884460949, 1.147014979642535,48.38934745485594, 1.148572186848853,48.3897513200882, 1.149452900251859,48.39006626851912, 1.150165461926662,48.39024611111055, 1.151047843231211,48.39042516233715, 1.152270439172725,48.39069540137733, 1.153324465015317,48.39094327985886, 1.154244282502233,48.39096633459549, 1.155435424894961,48.39078534356479, 1.155979516023223,48.39067205931411, 1.15669504708103,48.3906040303559, 1.157883132450849,48.39058004629488, 1.158700192878613,48.39064720904457, 1.159516228520729,48.39080449551351, 1.16005712798929,48.39091637373914, 1.161179091248371,48.3910282346482, 1.161694908227361,48.39116410705154, 1.16283762751803,48.3913469459505, 1.163467404112836,48.39148475258221, 1.164451997374528,48.39155503947853, 1.165367421454178,48.39151095363643, 1.166115818327427,48.39142067202554, 1.167473412102154,48.39142422172544, 1.168534769538184,48.39160684787491, 1.169323433740047,48.39160667013261, 1.170009873600236,48.39146909926969, 1.170902316813013,48.3914684060865, 1.171554604767735,48.39151380697546, 1.172104234174583,48.3916050624638, 1.172894687928718,48.39167353041779, 1.173375818954341,48.39167319036073, 1.174063282873561,48.39167273674354, 1.174750983842515,48.39174069207037, 1.175266637448857,48.39176303675458, 1.17612735015021,48.39178581417025, 1.176745426650726,48.39178476697676, 1.177538140752276,48.39203575861255, 1.178192642884454,48.39219527508266, 1.17860680532813,48.39233237910965, 1.179014500360125,48.3927225036333, 1.179391755388124,48.39288289763949, 1.179974180097247,48.39302108500115, 1.180694133843915,48.3932055819857, 1.182071186936895,48.39350348766297, 1.182657059340968,48.39375561960829, 1.183519160673825,48.39398455817334, 1.184967401188544,48.39403024427348, 1.185760583338976,48.39410041506156, 1.186451036293136,48.39416984597506, 1.187176474283049,48.3942611608653, 1.187832447643615,48.39423658918395, 1.188730137201361,48.39421259842239, 1.189629656577367,48.39421484413132, 1.190425945241188,48.39426191342986, 1.191152863920564,48.39426226240149, 1.191810172638853,48.39419317791255, 1.192293968307126,48.39414651447483, 1.193122078271145,48.39398380886061, 1.193846131472384,48.39382170008479, 1.194329602682132,48.39384451567724, 1.194882839855012,48.39375169887497, 1.195401385001716,48.39370500434281, 1.196056398626894,48.39363719337381, 1.196642512504994,48.3936389729942, 1.197818284892406,48.39363904390913, 1.198510335376739,48.39366216791524, 1.199445093982874,48.39368520500737, 1.199964559470018,48.39368479455458, 1.200933673290294,48.3935678045937, 1.201383665370898,48.39354409164951, 1.201904691786695,48.39379820819317, 1.202528710746861,48.39386694264992, 1.202978865871671,48.3938432004713, 1.203670987675636,48.39400200682294, 1.203951206128195,48.39441788749617, 1.20457468311524,48.3945086604861, 1.205443995563698,48.39460221648233, 1.205653078251274,48.39464883931061, 1.20666751750727,48.39507108245963, 1.20697895330018,48.39516291144876, 1.207810611536871,48.39523103807976, 1.208262285650719,48.39529938099805, 1.208853503877421,48.39545945514082, 1.209477524506909,48.39559687047746, 1.209960343854803,48.3955969986297, 1.210547156940365,48.39568937082892, 1.211345058232539,48.39575757232492, 1.211863221034903,48.39575789052379, 1.212832932981684,48.39573470324576, 1.213283890205815,48.39587306765058, 1.213735033246593,48.39598839931617, 1.21411666262108,48.39605778039346, 1.214568043852594,48.39617338877995, 1.215123807883174,48.39626602822452, 1.215575722466795,48.39635867155382, 1.216201643002535,48.39640525883797, 1.217141504618866,48.39647506288662, 1.218291507328047,48.39659038031652, 1.219442109514546,48.39668222915126, 1.220104151196765,48.39679641093428, 1.221009037547148,48.3967019075947, 1.221078247333793,48.39644766193697, 1.221496312465291,48.39628549626996, 1.221949473344282,48.39619260728614, 1.222507697921884,48.39630766510124, 1.223031673386885,48.39630755055083, 1.22376393424257,48.39626039652323, 1.224634324736101,48.3961893698762, 1.225052059561563,48.39618857293775, 1.225540896813172,48.39616505238324, 1.226135700321024,48.39604926252716, 1.226626971324009,48.39560977710796, 1.227326955303624,48.39521565417118, 1.227572552073525,48.39498391268605, 1.227855067577451,48.39475227069453, 1.229006032912977,48.39419417752632, 1.229177297582496,48.39391558307531, 1.229349592542391,48.39354441438037, 1.229981442182762,48.3926854928735, 1.23036656053479,48.39236008726532, 1.230718046512948,48.39210413009977, 1.231139910345561,48.39182470027618, 1.231757284710838,48.39132505642755, 1.231826954237491,48.3912089029246, 1.23207122107641,48.39081394447695, 1.232278807074323,48.39062839543283, 1.232660176686892,48.3902104292341, 1.232728619310748,48.38995506185747, 1.232831021676816,48.38958341205591, 1.232829848392484,48.38918870728054, 1.233108298000758,48.38860814462965, 1.233212993886511,48.38851521217683, 1.23345575319081,48.38784205315602, 1.233559948776858,48.38760997420031, 1.233593569125091,48.38719235644461, 1.233801707987965,48.38672808399067, 1.233940444530688,48.38635709041107, 1.233729303014641,48.38589353534512, 1.233553227582718,48.38563764546102, 1.233306324664511,48.38519567974891, 1.233164690863063,48.38484654837198, 1.233057441183888,48.3843808563108, 1.232914605701595,48.38382156556186, 1.232737793243217,48.38335654505662, 1.232526166433345,48.38279964526555, 1.232279944785503,48.38221723551195, 1.23206885544694,48.38168308846754, 1.23189302877465,48.38126519960473, 1.231856891270027,48.38091691258328, 1.231960835938565,48.38056840611374, 1.23213444340071,48.37973248264262, 1.232168014715271,48.37915237150043, 1.232167417453212,48.37892038355246, 1.232060590931084,48.37847938633573, 1.231989736803925,48.37813149373927, 1.231917674629628,48.37773651374816, 1.231740503461168,48.37731820110648, 1.231386718824012,48.37671327031176, 1.231315157119326,48.37631823227229, 1.231173126018192,48.3758532939125, 1.231207655539166,48.37562126158604, 1.231203550026365,48.37501486802556, 1.231237016877927,48.37478150866853, 1.231374443760515,48.37440764674516, 1.231511818830782,48.37403338015005, 1.231549514152675,48.3737367077454, 1.23151303696902,48.37334145580955, 1.231115043286991,48.36953259204491, 1.231220789307201,48.36905500537788, 1.231148863641977,48.36856762630228, 1.230796436771598,48.36829090670242, 1.230548875161446,48.36796711715906, 1.230440379904138,48.36755021855597, 1.230472530470919,48.3671790157731, 1.230858169036594,48.36678197698146, 1.231031859194786,48.36629322863123, 1.230994832354924,48.36582840838806, 1.231168473531426,48.36536300362224, 1.231307169089078,48.3650838375905, 1.231305761754058,48.36478158687702, 1.231269788577791,48.36445603291287, 1.231339469577762,48.36415358625471, 1.231300666479953,48.36359538385995, 1.23094657070817,48.36324650555113, 1.230839349069854,48.36282752807201, 1.230979726041099,48.3623856266291, 1.230979848552337,48.36215328379889, 1.230767965345621,48.36154881997049, 1.230659553114037,48.36110586938953, 1.230587713748051,48.36087262891504, 1.230549931023163,48.36045276769755, 1.230866055013763,48.35991893287569, 1.230794465576736,48.35956964459626, 1.230900437566291,48.35910653898194, 1.23079497452818,48.35878174858144, 1.230407814683318,48.35833974749696, 1.230090825883807,48.3581064141226, 1.229736040876091,48.3577311349449, 1.229558086233041,48.35738048418214, 1.229095912980455,48.35682526415284, 1.228989967922317,48.35645267834896, 1.228988440997938,48.35605654338449, 1.229235384380573,48.35570740902519, 1.229341164813448,48.35531198166364, 1.229483575861082,48.3549871480435, 1.229552507297257,48.35445214966248, 1.229481693497093,48.3540105448944, 1.229300079089654,48.35342625525725, 1.229016841557012,48.35289038770985, 1.228662768074671,48.35242367704177, 1.22851844223986,48.35170043149456, 1.228691637436811,48.35148929442055, 1.22904091230015,48.35139458558159, 1.230089645982659,48.35115644626905, 1.230228996912544,48.35103955990235, 1.230438127611692,48.35061947622628, 1.230613234999751,48.3501299219996, 1.231069002584093,48.34994285994598, 1.231839557357446,48.34991784325552, 1.232714651475916,48.34984531640938, 1.233663096622166,48.34972807411978, 1.234295166041704,48.34958844368905, 1.23446982201169,48.34930893187955, 1.235028764092647,48.34856425166377, 1.234605928143679,48.3480278070526, 1.234288512313441,48.34776920452309, 1.234039512600504,48.34727531474859, 1.234036686286757,48.34680493832896, 1.234210435436026,48.34638242799053, 1.234243974831514,48.34598424002901, 1.234242585023935,48.34563298827272, 1.234276813063261,48.34537566043828, 1.234590840865408,48.34465339817034, 1.234695342393266,48.3443022762104, 1.234835116380069,48.34402145363742, 1.234798524685658,48.34348375579424, 1.234129041349448,48.34320497956781, 1.234163709205916,48.34292442063778, 1.234162089664764,48.34241054852508, 1.23409075440874,48.34210702642761, 1.233878193709742,48.34166348770127, 1.233665591737619,48.34121985389238, 1.233347482548751,48.34084618782326, 1.23278231482648,48.34037889040094, 1.232710550706614,48.33993495558676, 1.232992609144816,48.3396093144442, 1.232638850365902,48.33928151482525, 1.232567789643085,48.33904799963173, 1.232637645330314,48.33867506997846, 1.232565419841777,48.33783560963137, 1.232141394372235,48.33760043500002, 1.232034914413831,48.33729686057389, 1.231645427389523,48.33680511492049, 1.231255491839804,48.33642910798268, 1.231077735283464,48.3361707762048, 1.231076412368523,48.33591287774262, 1.230615142518786,48.33565220789081, 1.230118984228611,48.33532247603151, 1.229799441248545,48.33498704880934, 1.229551094189134,48.33477682257995, 1.229549755102354,48.33437865500499, 1.229549556995139,48.33407413994636, 1.229407574751065,48.33365282634969, 1.229441982739474,48.33337177641833, 1.229546098457107,48.3329501012511, 1.229544768352795,48.3325520108247, 1.229790027616886,48.33213016854741, 1.229683246733686,48.3318024900287, 1.229434461628316,48.33114718396246, 1.229116026786812,48.3307261474847, 1.229184270375669,48.33032749488176, 1.229287852815887,48.32992873671834, 1.229212719383213,48.32950506389612, 1.22875008369877,48.32915204807809, 1.228467108878277,48.32896474594668, 1.227826691940635,48.32823656620919, 1.2275056906077,48.32781329948185, 1.227255689645872,48.32724971345805, 1.227394343976315,48.32669593806672, 1.227710974584198,48.32646082064609, 1.2273918807294,48.32617969939966, 1.226720942808939,48.32594598959664, 1.226578265822738,48.32578191641809, 1.226187360245191,48.32547731094829, 1.225550728657808,48.32526694540474, 1.225477822741723,48.3250556467555, 1.22487383955712,48.32458669225995, 1.224766831562832,48.32442239299921, 1.224726724783395,48.32399914529231, 1.224449552860676,48.32355463014824, 1.22413180002556,48.32280446925572, 1.223810992924897,48.32226465591378, 1.223915526035299,48.32186540249751, 1.223909964685831,48.32134746193086, 1.22433005742603,48.32089969967189, 1.223902935199121,48.32042976396335, 1.224112849262795,48.31995937674962, 1.224570091413582,48.31965274508801, 1.224287251117218,48.31932474181294, 1.224393651225938,48.31897316100062, 1.224343089948206,48.31838485701643, 1.224342031781049,48.31819711986476, 1.224620743913818,48.31789206755875, 1.224866963189015,48.31758629508239, 1.224830545809596,48.31721040675041, 1.225257171303809,48.31678668020594, 1.225363359105785,48.31666912119176, 1.225683259108905,48.31629322858348, 1.226353833821662,48.31596403276348, 1.22593058179244,48.31554282465535, 1.226037381523454,48.31512143501442, 1.226283298496824,48.31491019590727, 1.226670059968612,48.31465192193138, 1.226846684210372,48.3144180510621, 1.227301729774808,48.31392541972575, 1.227505455243573,48.31379065906042, 1.227859691125147,48.31345537928489, 1.228660838105967,48.31288703633906, 1.229278453234286,48.31244843633917, 1.229684528108109,48.3121132166749, 1.230283516003325,48.31181277210973, 1.231465866209689,48.31152814643261, 1.232452867436336,48.31122111857731, 1.233371349371453,48.31096383543347, 1.23426329455117,48.31034112150084, 1.234650557693911,48.30997800360303, 1.23456860831395,48.30922696463548, 1.234491051776743,48.30891610996413, 1.234333376934103,48.30863140826082, 1.234415313952013,48.3083721982336, 1.234179586081822,48.30785460149424, 1.233982578901731,48.30749225869204, 1.234172175600094,48.3070253868962, 1.234050981459504,48.30668834568627, 1.234009147258902,48.30611813903659, 1.23416455696929,48.30567756447402, 1.233854735408585,48.30531599675721, 1.233394548836017,48.30474927302223, 1.233402171949024,48.30438951573044, 1.233678498269541,48.30408035727825, 1.234952863309218,48.30410103851784, 1.235266169721385,48.30384293903178, 1.236123351903572,48.30389390634127, 1.236787250477761,48.30376503238171, 1.237638261317088,48.30391641554719, 1.238022739379716,48.30412017878525, 1.238721307940395,48.30437645658741, 1.23973079470106,48.3045282662145, 1.240586990859669,48.3047331850335, 1.240936792223167,48.30444750578018, 1.2411297897536,48.30410966351755, 1.241126837482852,48.3036677058803, 1.24104572850915,48.30312208465984, 1.241436344808378,48.3025539381264, 1.241357117657458,48.30206209414578, 1.240923090603289,48.30177246759839, 1.24045346523377,48.30161635604934, 1.239946645567684,48.30151404095065, 1.239362953730634,48.30115441685345, 1.2388964498126,48.30071719755355, 1.238857267959972,48.30034091841082, 1.238818328482253,48.30008146385137, 1.239059309392246,48.29937908185489, 1.239373885099141,48.29898946776068, 1.239645699995918,48.29865265147005, 1.239953647940851,48.29831611167168, 1.240262520986406,48.29813463191538, 1.240726315162959,48.2978751738233, 1.241384354896113,48.29730448591069, 1.24161625793522,48.29720040100668, 1.242119935256696,48.29714749517517, 1.242974468176527,48.29711980410814, 1.243791306707422,48.29709215955009, 1.244454554050421,48.29716889494632, 1.244962425088829,48.29719424639679, 1.245391622327404,48.29716780007591, 1.244961788339097,48.29703857145925, 1.24468669368477,48.29662355880517, 1.244607470731154,48.2963639839624, 1.244410451472632,48.29589689311974, 1.244136085409437,48.29545641696527, 1.243743513712019,48.29504084553989, 1.243664280880793,48.29478108900658, 1.243623691841805,48.29421056948682, 1.243777786577752,48.29356190449744, 1.243854630944227,48.29317284938228, 1.244086472521857,48.29270418954221, 1.244474142482218,48.29215691313085, 1.245058479798018,48.29187014703723, 1.245486972148811,48.29153461554764, 1.245837338632612,48.29132668155373, 1.246538202567209,48.290960627499, 1.247318595498015,48.29090581954014, 1.24829486446842,48.29084948266568, 1.248566824766499,48.29064306219445, 1.248876432720195,48.29028284218781, 1.248987696513311,48.289697748382, 1.248985376925418,48.28943776886869, 1.248944313337518,48.28917819831387, 1.248666408619016,48.28863380493277, 1.248352785854806,48.28834954650228, 1.247732887543975,48.28811831744234, 1.246839023713065,48.28799048899469, 1.24625378115978,48.28791358413995, 1.245901927736085,48.28778426711055, 1.245510094824766,48.28742118357404, 1.245235437656021,48.28705790851207, 1.245078075081196,48.28679816047116, 1.24503767999957,48.28635682889595, 1.244997724987144,48.28596796134157, 1.244644838892342,48.2853978807881, 1.244370216968363,48.28513791490197, 1.244563399493188,48.2845407528479, 1.244563421089069,48.28448922676483, 1.24475753207463,48.28410055754425, 1.244990581243116,48.28389188636219, 1.245848299355558,48.28381069258237, 1.246355289958302,48.28370475266601, 1.246900668498213,48.28336555929502, 1.246939191314723,48.28326269617489, 1.247364041945232,48.2822003278572, 1.248105301262314,48.28209384005104, 1.248455859226269,48.28180633732065, 1.248961947808169,48.28141547943273, 1.24923457648967,48.28104951712226, 1.249849894931316,48.28078005853186, 1.250199664247276,48.28052006815479, 1.250627847429897,48.2802336819688, 1.251642552286654,48.28010258553196, 1.252033537769052,48.28002339799708, 1.252697468801543,48.27989194204068, 1.2530874284259,48.27970929299769, 1.253555646463681,48.27955222455228, 1.253554554553048,48.27926626670108, 1.253631360670127,48.27892832054966, 1.253747146995906,48.27856426791736, 1.253863165082493,48.27825210020428, 1.254096228177843,48.27788764487272, 1.25444603718789,48.27739290785092, 1.254873935191725,48.27687197118211, 1.25514588258126,48.27650761353177, 1.2558063961485,48.27570103211456, 1.255958201199675,48.27492288257401, 1.256187912033084,48.27411931243549, 1.257004944700086,48.27359888872314, 1.257902456952962,48.2733362110758, 1.258490693111753,48.27335835712513, 1.259473926362475,48.2734812443717, 1.260061943221857,48.27342678658654, 1.260684480489118,48.27295814649199, 1.260956094856979,48.27269821934653, 1.261507169772614,48.27266846293776, 1.262213436838849,48.27258740187192, 1.262799563704363,48.27232574290638, 1.263111819774338,48.27214290271461, 1.263658323772489,48.27185552131667, 1.264518683346267,48.27148871239157, 1.265065493352282,48.27122723787842, 1.265568542259658,48.27083815373336, 1.266389979447723,48.27031839338239, 1.266486386950263,48.27003408389459, 1.266490210970176,48.26949770644065, 1.266634107935914,48.26911926504209, 1.267108176363905,48.2688349420211, 1.267581783543989,48.26864516542761, 1.268241325205352,48.26845532904569, 1.269041851851682,48.26835966165804, 1.269277267838177,48.26883162740577, 1.269795624688925,48.2687994268378, 1.271070581114145,48.26860834168943, 1.272628770292116,48.2685429239737, 1.27310190453845,48.26869980980905, 1.2739535492438,48.26885624305183, 1.274142747662637,48.26885598463456, 1.274853365685374,48.26885464917955, 1.275421551593993,48.2687589287839, 1.276841557675482,48.26844067499253, 1.277077284244285,48.26818800401313, 1.277691263607835,48.2678085963503, 1.278305633552606,48.26755527510783, 1.27844589049009,48.26711369077432, 1.278113161398323,48.2667989935393, 1.278158068488935,48.26619966873595, 1.278157038747168,48.26594729281695, 1.278106416319716,48.26519055625629, 1.277915539886834,48.26487578035687, 1.27725052718165,48.26443585237739, 1.27682386201802,48.26434207873761, 1.276396784375805,48.26412266441018, 1.276300416833792,48.26371340395941, 1.276250631178343,48.26311513016835, 1.276438724827486,48.26279993682344, 1.276721621594334,48.26251631615464, 1.277431407142132,48.26245047288573, 1.278045910466275,48.26219665558625, 1.278187313769649,48.26203858529081, 1.278850879643073,48.26216317024564, 1.279373011503694,48.26244595331197, 1.279845718067207,48.26219228659745, 1.280604645075983,48.26222060064414, 1.281503952987606,48.26202961562544, 1.282071630483785,48.26183926647398, 1.28263740119904,48.26165111995077, 1.283205371564364,48.26165034860134, 1.283913505199578,48.26136613841161, 1.284435467470317,48.26117415051172, 1.285430292752764,48.26082382180379, 1.28609288773049,48.2606647401878, 1.286755495408386,48.26050565159929, 1.28808015250317,48.26009295352112, 1.288554527160656,48.25999664223459, 1.288648379572976,48.25971230858409, 1.288788621475305,48.25939694240329, 1.289738419342953,48.2589820657934, 1.290259055043421,48.25917134154809, 1.29068593059786,48.25910693744969, 1.291684756471142,48.25916598785322, 1.292305152225993,48.25916231522817, 1.293587531801705,48.25909521872613, 1.294810378198989,48.25893955985075, 1.295398408715911,48.25795919647052, 1.295571629403028,48.25738121968744, 1.295226594424946,48.25674638314217, 1.295223393318736,48.25605480796768, 1.295653074308802,48.25542002331221, 1.296257840577923,48.25495747308922, 1.297115072576602,48.25397670914502, 1.297460721873122,48.25368765804234, 1.29815218723158,48.25299426821069, 1.299018707711468,48.2526464030022, 1.300487845315772,48.25247057280335, 1.302128274705396,48.25258293739753, 1.302472006776597,48.25194800906429, 1.302904552220809,48.25171646424112, 1.304026008358576,48.25119523877186, 1.305236282067227,48.25073144038046, 1.306102261024918,48.25061438796806, 1.307920624534646,48.25061057079674, 1.308959109557478,48.25060824172104, 1.309997620489753,48.25054838935891, 1.310865172132523,48.25094993300792, 1.312425540186169,48.25129227094008, 1.313379250291377,48.25134780716846, 1.314329044082513,48.25094205975383, 1.315797500414092,48.25030459213642, 1.317616327169645,48.24995407616982, 1.318829348913141,48.24995126624399, 1.320474393564595,48.249658993746, 1.321944350590869,48.24902098061948, 1.322376512890003,48.24890469318391, 1.32263279337981,48.2483274087526, 1.323060242094471,48.24757710353731, 1.323663461610238,48.24705676383951, 1.325137370444352,48.24607039673619, 1.326523363398721,48.24606727749683, 1.327647434282255,48.24629650138285, 1.328690174085892,48.24681310885833, 1.329997929244186,48.2474435849354, 1.331129488325615,48.24761324074318, 1.332944818993482,48.24755196910591, 1.334762697100842,48.24650817163169, 1.335195180234749,48.24627619251221, 1.336235537500542,48.24604248379056, 1.338397558380066,48.24632697213402, 1.339780677607735,48.24568880978282, 1.340647527693719,48.24568661443044, 1.34229006553565,48.24562550350186, 1.343599360214469,48.24544725387743, 1.34516083345784,48.24567421923424, 1.346378740695869,48.24515041260958, 1.346723735832356,48.24451417233264, 1.347669798769147,48.24376193945565, 1.34792861572258,48.24329915850184, 1.347233419091793,48.24226073363729, 1.347840848594615,48.24150734299257, 1.348627586468695,48.24098309481331, 1.349143974023534,48.24057794108178, 1.350279642048449,48.2404744520391, 1.350970140451807,48.24051092561037, 1.352062250190343,48.24039327692536, 1.352578079686396,48.24008550922522, 1.353494613189568,48.23947031549153, 1.353493725612069,48.23931710448219, 1.354469619752067,48.23881619673316, 1.354755025714624,48.23839387791813, 1.355786868729808,48.23777819199207, 1.356416237887672,48.23724029403454, 1.356872022165007,48.23643430449894, 1.357500278727613,48.23589717188641, 1.359108820973162,48.2350481751997, 1.360141695183075,48.2347779063961, 1.360711919413603,48.23454866026023, 1.361228351989652,48.23427889126398, 1.362038795719087,48.23408161385994, 1.362840297590282,48.23377409441166, 1.363411668236154,48.2332364910545, 1.364783292362318,48.23258424222922, 1.365012215102567,48.2322767709494, 1.365742985013839,48.23143913144633, 1.366429083637243,48.23105407350179, 1.367401766544816,48.230591415988, 1.368372336546443,48.23020485593078, 1.369346117188314,48.23004823919026, 1.369400652000763,48.22962605509248, 1.369513275668325,48.22908889211014, 1.373802483432286,48.22363656033613, 1.374695884402996,48.22344541420444, 1.375498957690141,48.22325196955984, 1.375554387612567,48.22290678036379, 1.375839294698338,48.22244593730915, 1.375894072277077,48.22198572589617, 1.376521718299429,48.22114060733232, 1.37680676700252,48.22083331267527, 1.377837780139956,48.22075487636229, 1.378810955409037,48.22052307862415, 1.380592468010377,48.22017312402597, 1.381919137843013,48.21966926952452, 1.382329827093116,48.21928222021954, 1.382389036721008,48.21912817596851, 1.382566936655394,48.21858963179945, 1.382212108089973,48.21794069287046, 1.381976183245484,48.21702218943125, 1.381743627196116,48.21618016809527, 1.381687525662841,48.21556768264904, 1.381108668642867,48.21530128411632, 1.380530151766791,48.21465204990951, 1.380240316331921,48.21438479160802, 1.379773074724568,48.21411754800374, 1.379013346119777,48.21369726717155, 1.378546428766427,48.2134296853625, 1.377964373415438,48.21289415270475, 1.377383990027994,48.2121289325482, 1.376860017031575,48.21155476562564, 1.37668720162371,48.21117228492125, 1.376336415288151,48.21044452463381, 1.375468634705797,48.21029260747131, 1.374534713602088,48.20990891799617, 1.373437520511502,48.20929818492342, 1.372804793451977,48.20868730570444, 1.372357996585038,48.20811840943891, 1.37207512257686,48.2076616063727, 1.372592406060662,48.20701006475242, 1.373313296805051,48.20673052144436, 1.373892094394719,48.2062329000626, 1.374066040316621,48.20611823926614, 1.374177518526736,48.20535162267959, 1.373727829072166,48.20486073432928, 1.37419977198573,48.20405779796084, 1.374543201583279,48.20367363691543, 1.375342417721697,48.20317394993199, 1.376315066219604,48.20271186026706, 1.376542355702995,48.20263496351993, 1.377159860832705,48.20236705564917, 1.378932262003431,48.20186467134369, 1.380197437605901,48.20166929530475, 1.38123270393281,48.20135929521958, 1.38146330472955,48.20082121554577, 1.381809805921435,48.20028320117764, 1.382152036197795,48.19978385398785, 1.382435837500381,48.19932292636964, 1.382716624550767,48.19870819181917, 1.382942561486658,48.19820871185901, 1.382940035904852,48.19778682399242, 1.382819068819757,48.19713447250739, 1.382528753110492,48.19656026916552, 1.381316970660024,48.19579715571535, 1.380567016654993,48.19549241389915, 1.379647061875716,48.1948068932568, 1.379019660197741,48.1940850575395, 1.378096346882757,48.19355107105555, 1.377339034804521,48.19316616356333, 1.376184956160472,48.19278522206987, 1.375722240723619,48.19236479000181, 1.37450913508863,48.19221264266493, 1.374450763711664,48.19144790432833, 1.374214050788879,48.19094823814885, 1.374201027122979,48.19010010821106, 1.374200843190451,48.1896413990556, 1.374197226577408,48.18906602936917, 1.374137972526313,48.18856847910057, 1.373620943630428,48.18772423970164, 1.37304364621491,48.1872654385503, 1.372510007544094,48.1861973728338, 1.372506079863954,48.18516081602703, 1.372577871280527,48.18461778134438, 1.373041169154647,48.18438496365448, 1.373440140241286,48.1834091280959, 1.374851764440834,48.18310502699465, 1.376165261427659,48.18275822493175, 1.376856011496415,48.18256401512713, 1.377537441507073,48.18229595917025, 1.37846799697104,48.18175373391048, 1.379606977557899,48.18133149234798, 1.381304759927559,48.1807962907147, 1.382042441435303,48.18052796012921, 1.383753483365183,48.18014245293801, 1.38426523891404,48.1799119558416, 1.385240518345867,48.17952566466177, 1.385873551478781,48.17933179628711, 1.385986622508838,48.17902472394302, 1.385983886446724,48.1787564166577, 1.38649720411442,48.17833310845449, 1.38752960965239,48.17775479656337, 1.387700285629947,48.17729433361306, 1.388159268733517,48.17649019731263, 1.388159160097075,48.17637571016461, 1.387870417128127,48.17591800958791, 1.387524182078366,48.17538442218937, 1.387180382606843,48.17481507150541, 1.386720966838411,48.17424557160508, 1.385744051850665,48.17329609367359, 1.385282891045947,48.17291496284591, 1.384821496469073,48.1724956577832, 1.383605537668217,48.1718039666636, 1.383140360383792,48.17122926803916, 1.382439208963874,48.1708017475648, 1.381737089431098,48.17025939780245, 1.381153659754276,48.16994878464263, 1.380282447539799,48.16952457651409, 1.379413102682761,48.16929308418992, 1.377792394522444,48.1684503019663, 1.377099295941648,48.16715131853392, 1.377099614262093,48.16646427880498, 1.377411180523907,48.1655564576644, 1.377876021065845,48.16520805061894, 1.378616760273289,48.16493921462939, 1.379579510936011,48.16486420399417, 1.380324009090771,48.16467091954168, 1.381022899274115,48.16416670055845, 1.381619036823167,48.16343063310958, 1.382022582930105,48.16308474500136, 1.38225115107231,48.16289313548755, 1.383275784033856,48.16247107912081, 1.384012111055142,48.1622029472768, 1.384576689242749,48.16204983361122, 1.385478334373886,48.16205038145012, 1.386396809990402,48.16200955741813, 1.387487102003661,48.16173807498027, 1.388631786546441,48.16127512926509, 1.389031998130477,48.16100559416019, 1.390120756653573,48.16050372982891, 1.390865696133218,48.16007943588546, 1.391783675997573,48.15969285800561, 1.392529736323724,48.15946078848684, 1.393277098122738,48.15922751647785, 1.393794469846583,48.15903358190465, 1.395002581776851,48.15891482360187, 1.395347649184757,48.15887544261153, 1.396322843995677,48.15856597338087, 1.396722791226054,48.15825808031593, 1.397236305223586,48.15771959538469, 1.397462852396686,48.15683476763962, 1.397462714932061,48.15679629877156, 1.397633707022095,48.1564112585429, 1.397747151039141,48.1559877061509, 1.397628610632381,48.15548886463367, 1.397626383957899,48.15510453972562, 1.397507333121027,48.15445156103606, 1.396871414256822,48.15410814805653, 1.395080118114876,48.15373684639969, 1.395129060652621,48.15301516385974, 1.395576686869759,48.15195051057658, 1.396596975108182,48.15100182710284, 1.397221140503191,48.15058534494232, 1.398570721450938,48.15025805802235, 1.399251375987902,48.15010216234429, 1.400461524964086,48.14956407619125, 1.401505162345075,48.14914283203389, 1.403166094744222,48.14879415220858, 1.405111742007648,48.14859687542304, 1.407003114663638,48.14855318855164, 1.408258414460036,48.14858565825717, 1.409277718446362,48.14880677867462, 1.411098448315883,48.149253055683, 1.412480403934438,48.14986177840787, 1.413172389443048,48.15008992727774, 1.414383672501984,48.15050843193735, 1.415192864697701,48.15096685946052, 1.416232392711081,48.15138625310887, 1.416980478058648,48.15138371120311, 1.417613357186752,48.15130486222625, 1.418417841488065,48.15103361430968, 1.41899204480977,48.1508015036987, 1.420369612046983,48.15029913885784, 1.420654738101775,48.14991405058561, 1.421050758454399,48.14914748831535, 1.421160605064543,48.14849607011693, 1.420983008529163,48.14765044692287, 1.420459091362617,48.14684925955546, 1.41930083413061,48.14563167756455, 1.418435124957412,48.14480613304119, 1.41676465307502,48.14384979681388, 1.414862511564764,48.14289275360386, 1.412087754965697,48.14218996060526, 1.411336099061828,48.14157854499341, 1.40908255951129,48.14168831149814, 1.407469010934932,48.1418063941732, 1.406203637601323,48.14196428787285, 1.404822405427652,48.14223666838048, 1.40355819573044,48.1428161558982, 1.402412185568167,48.14332054236895, 1.401027352088804,48.14389788877308, 1.400228389628879,48.14444008580687, 1.398917490176373,48.14498581384024, 1.397826939738096,48.14529673487332, 1.396551792748539,48.14537438740037, 1.395966358899102,48.14514299601047, 1.394297539470301,48.14545533185483, 1.392635024440423,48.14592272790158, 1.390975520315934,48.14639044087243, 1.389860834846573,48.14638931344148, 1.388641747110953,48.14643824923122, 1.387714018345619,48.14616660429027, 1.386847893205413,48.14601297302277, 1.386272648206457,48.146014714593, 1.385644552745532,48.14636667122905, 1.385014942391436,48.14667958039926, 1.383749214745677,48.14691355900091, 1.383691526822915,48.14691337256467, 1.382770428176841,48.1470289297375, 1.382080283973778,48.14714702163148, 1.380985464214604,48.14707640539131, 1.380178008433646,48.14696473257915, 1.379081474843417,48.14662081368357, 1.37850179094302,48.14600686981692, 1.377982967705506,48.14554263358295, 1.377516949530985,48.14481551881025, 1.377226077494413,48.144316750515, 1.377049330599618,48.14381878659695, 1.376870030586233,48.14316824105916, 1.376636049515938,48.14274667472376, 1.37605801647716,48.14209334808561, 1.375943175441293,48.14170800311608, 1.375827906810798,48.14109231779007, 1.375717239055163,48.1402075741867, 1.375835795704944,48.13951662058243, 1.376120355880389,48.13913307393015, 1.376518588084011,48.13878745163703, 1.376864385677692,48.13817490203897, 1.377433830474601,48.13763789673994, 1.378004886249215,48.13729222056878, 1.379030656389404,48.13709492805572, 1.380059439012214,48.13697244383928, 1.380805216179908,48.13677591744897, 1.381435392277325,48.13623653877331, 1.382298657812143,48.13611778062066, 1.382986727676709,48.13565512023056, 1.384020312836015,48.13523000815143, 1.384652165662559,48.13472633710906, 1.385744471152137,48.13410613151491, 1.386891454504138,48.13310194704643, 1.387235234781173,48.13267704984131, 1.387290247686394,48.13217600920709, 1.387345680720287,48.13175194175348, 1.387458221758976,48.13125075124544, 1.387744425176763,48.13090339179215, 1.3881428228012,48.13036576253062, 1.388359219318502,48.12953088705053, 1.388347546386456,48.12912132902489, 1.388157240208813,48.12841394072638, 1.387520529496072,48.12779192397237, 1.386195028257802,48.12699127649668, 1.385157009778354,48.12638117003975, 1.383950166255346,48.12588721916293, 1.382801254879283,48.12562238661233, 1.381080754923261,48.12535967309594, 1.380163396441063,48.12547720939325, 1.378899334705565,48.12578791429735, 1.378208268209363,48.12590537132083, 1.376998684033708,48.12594750766071, 1.376020535060249,48.12587340030319, 1.373773741080555,48.12591832262201, 1.372679391103844,48.12588296085667, 1.371416818031847,48.12588619166198, 1.370035056505301,48.12604370189635, 1.369172980540782,48.12608439580751, 1.367667804365368,48.12608901377976, 1.366975346909129,48.12601406791505, 1.365756086439518,48.12621019270934, 1.36511363280016,48.12628939878887, 1.363899739643699,48.12606181732944, 1.362917914693469,48.12587200614667, 1.362629108932467,48.12583428904178, 1.361702029714152,48.12549036898137, 1.361123689127993,48.12537641964735, 1.36037052878379,48.1252629169875, 1.359272825359,48.12499629188923, 1.358868952221044,48.12461240202947, 1.358060727548988,48.12399870777246, 1.357196597727707,48.12342388928411, 1.357081676080128,48.12330881726965, 1.35714148368325,48.12269351591829, 1.357431006187142,48.12230844846839, 1.357436011094379,48.12154051540521, 1.357031167261368,48.12054256778595, 1.356797326985301,48.12004323327505, 1.356038462055125,48.11912128084501, 1.35555941611743,48.11823447907814, 1.355203657043788,48.11750267411978, 1.354385976945224,48.11650262899597, 1.353344510123804,48.11631203047542, 1.352708086760444,48.11619770469459, 1.352358120809583,48.11577463228485, 1.351561764536555,48.11509818764383, 1.349778568130958,48.11371943866465, 1.348885717539801,48.11273356988788, 1.349323261851026,48.11164602708391, 1.349313623783303,48.1098681061006, 1.348567826284324,48.10848766544513, 1.347540714969228,48.10750736324535, 1.345762760502495,48.10898845616902, 1.343848121104031,48.1090921847606, 1.341940433235422,48.10899880104819, 1.339120392786313,48.10979446147096, 1.336455171068963,48.11049220987119, 1.334831556742174,48.1101997961112, 1.332610230858577,48.11010654765646, 1.331132957552427,48.11070248495215, 1.327449751055582,48.11130283612073, 1.325368114525958,48.11160482735868, 1.32284309561534,48.11180907565412, 1.321355450933837,48.11161543416755, 1.319126959599586,48.11083033982845, 1.318232169092795,48.10944901311395, 1.317196939714194,48.10866097947533, 1.316176960709043,48.10668925524519, 1.316348903505244,48.10432437363266, 1.317109721508572,48.10275013103551, 1.317728962704393,48.10157385709383, 1.318604790380192,48.10058636220383, 1.321096538694846,48.09870809077972, 1.322711880271982,48.09752235156563, 1.324165166013202,48.09662530568092, 1.325464872167771,48.0961173550802, 1.327049747481652,48.09501016549204, 1.327635820990793,48.09431647599888, 1.328368426326961,48.0931292766383, 1.329244047392109,48.09124943985517, 1.329393044263341,48.09046310743898, 1.329964267604396,48.08867479250134, 1.329526814559216,48.08710878944557, 1.329674945225023,48.08563563720351, 1.331716631403168,48.08442277100867, 1.33392985671955,48.0839131312183, 1.334955607030267,48.08242824221071, 1.33583861632178,48.08094500557297, 1.335988740771683,48.0802551904914, 1.335089104122602,48.07857691776747, 1.334797493238298,48.07749580467771, 1.334065840345716,48.07641829158139, 1.333039897995263,48.0751462324061, 1.332157388898153,48.07436361380353, 1.330979046822961,48.07358132722833, 1.329044513383224,48.07288849620894, 1.32549909430297,48.07191134699391, 1.3237074553778,48.07180675827537, 1.321453386676879,48.07169535372262, 1.318567653894903,48.07094491630404, 1.317697444603269,48.0707566865352, 1.314414055239226,48.07094491449439, 1.31022953349526,48.07054129334598, 1.308738444118916,48.07073696300671, 1.306527062861534,48.07084229513291, 1.30518352244178,48.07143268947377, 1.303410547844919,48.07163414578169, 1.3019262644831,48.07193163504863, 1.298806359927127,48.07173691417544, 1.295989168401817,48.0716422787021, 1.295097007101902,48.07095186120369, 1.293750759577804,48.07035828472665, 1.291678656969097,48.06898000761116, 1.291380419126341,48.06907867818574, 1.290772105189603,48.06851946306365, 1.290440371971997,48.06740403082121, 1.290682904008691,48.06668068894939, 1.290847185645134,48.06584464390097, 1.291596687273952,48.06489593269842, 1.291765316827579,48.06411586014846, 1.291767183688322,48.06344807736723, 1.292350875737995,48.06283519913639, 1.293499466062499,48.06166319079146, 1.294486340387039,48.06026744395072, 1.29497808087838,48.05903970650364, 1.295140267181097,48.05842551240371, 1.294800619799404,48.05697767502476, 1.294798542639057,48.05653186983376, 1.294460153201541,48.05541826938355, 1.293456241154672,48.05475129805319, 1.291375697614043,48.05381430438237, 1.290161559373161,48.05334312079622, 1.288770546745056,48.05347073643764, 1.287768153572102,48.05380067488398, 1.285612973964419,48.0548586529998, 1.282684211767631,48.05590911635605, 1.281341483059919,48.05635171733671, 1.277666433682413,48.056910573638, 1.273700412947861,48.05701784401695, 1.271950124349189,48.0562974164008, 1.270779356819716,48.0552409421299, 1.269533772042664,48.05390867512548, 1.267675393647707,48.05284757833879, 1.265835262580723,48.05179144023273, 1.264468327418527,48.05097488773838, 1.26449808660675,48.05026208320636, 1.264611236237229,48.04866252589565, 1.264815849663577,48.04745728937304, 1.265489452456876,48.04670355799757, 1.265852241791336,48.04528133569288, 1.266481251460334,48.04434218023892, 1.266967683463832,48.04323054839025, 1.267495072590052,48.04194548637229, 1.268370346368684,48.04117040772299, 1.27013748568109,48.04039495601058, 1.271549308336105,48.03927786980344, 1.273050391521707,48.0368231053957, 1.27356834621051,48.03587573898141, 1.275005062213552,48.03470587918259, 1.276651105878173,48.03403398105813, 1.278617723200881,48.03374861467545, 1.280444735961453,48.03341023699366, 1.281850482561353,48.03257042392986, 1.284091055947698,48.03133867309064, 1.284082562634353,48.03055667901263, 1.282909873057761,48.03011184996915, 1.283318474017875,48.02827051242709, 1.282897047541837,48.02654394311823, 1.282811167615392,48.02515201422094, 1.282723912798915,48.02437137262977, 1.283519309636816,48.02356478665137, 1.282287230934628,48.02291135816153, 1.280908455223184,48.02215941312576, 1.276833658775999,48.02205121196278, 1.273785195456526,48.02195695617517, 1.269409504704356,48.02193793468136, 1.266802547326179,48.02187464414769, 1.264296770651995,48.02139267892591, 1.263554490934027,48.02117477429075, 1.26165480259813,48.02118617889287, 1.26090359620197,48.02079750917162, 1.260066602961288,48.02018514001838, 1.258895779945163,48.01951754227049, 1.256309696257716,48.01935419764676, 1.255057700871589,48.01846529776906, 1.253553867175725,48.01763141875989, 1.251214866775214,48.01668672824903, 1.2482915128804,48.01535247265814, 1.24745819836248,48.01529817962623, 1.243876770152681,48.01502476065059, 1.24171089748711,48.01441550950218, 1.239544147096339,48.01380642218722, 1.237623219410452,48.01291810829866, 1.236943235433614,48.01197011484502, 1.235186372796747,48.01091266657862, 1.233683125127485,48.00952135862423, 1.232762819610827,48.00824055631134, 1.232594112333644,48.00757178752077, 1.233508368459852,48.0072908112795, 1.232750435619877,48.00589613261086, 1.231080795339196,48.00478396935499, 1.230755059095301,48.00384302971857, 1.230334147260805,48.00250606546152, 1.230573060799102,48.00132990477899, 1.230813912434201,47.99993175821953, 1.230639155736554,47.99797850939958, 1.231971696716215,47.99680826845736, 1.232054771640236,47.9968078410791, 1.233546666396322,47.99541662870699, 1.234204152408523,47.99418847222079, 1.235620506255117,47.99351662576265, 1.236197798853041,47.99256672862142, 1.237280832824752,47.99105910529387, 1.237363707244001,47.99100309694532, 1.239692651751568,47.98977129212011, 1.241024329119881,47.98865440010643, 1.241937504195789,47.987815207889, 1.243435199488488,47.98736410647977, 1.244433780524673,47.98680389589417, 1.245351645654878,47.98691417937041, 1.246435862531314,47.98691253131669, 1.246849517307965,47.98601792265875, 1.248346775236882,47.98501295920793, 1.25001047824243,47.98400615450404, 1.24975400045399,47.98272976095285, 1.24908206860367,47.98105150334597, 1.248075934381731,47.97972754594759, 1.247905423912826,47.97903012031724, 1.249810847536011,47.97747284705979, 1.251059508379058,47.97734862182149, 1.252225600349064,47.97722766214009, 1.253670723768806,47.97661645303507, 1.255485946129596,47.97578804705581, 1.258054985260571,47.97439682913714, 1.258556667799212,47.97355449902901, 1.25963786554216,47.9722668304932, 1.260381691674724,47.97103876571213, 1.260885080904245,47.96986089980405, 1.260562299298172,47.96840527162525, 1.259041332849336,47.96696427748788, 1.257949149321334,47.96512585960197, 1.257618643882935,47.96384463814569, 1.256199183019447,47.96228686896266, 1.255373397332591,47.96112156014429, 1.254870960099993,47.96062071915441, 1.253769976133583,47.95966952563169, 1.252181301983313,47.95911288921872, 1.251878085815207,47.95834668195754, 1.250291447660104,47.95773492249276, 1.249053670336253,47.95701842544801, 1.246932008473053,47.95565081174816, 1.242460506053774,47.95432998888861, 1.238668551809468,47.95345744138951, 1.234805974453082,47.95239044690482, 1.232297142801755,47.95249376930535, 1.226516454393916,47.95081942677614, 1.225411583443972,47.95074021430539, 1.224579263335201,47.95090749965613, 1.223579765141921,47.95090743696958, 1.221662032208187,47.95085386831926, 1.220495703112476,47.95091048884128, 1.21883752233768,47.95084864271886, 1.217168797022538,47.95029290916117, 1.215830712991775,47.94929102209959, 1.214660393971315,47.94800842903945, 1.214062431163796,47.94667477317319, 1.213808318734664,47.94583805849211, 1.214384072539147,47.94511371658463, 1.21537766926042,47.94405253363172, 1.21562720836846,47.94237439568816, 1.215790130981702,47.94108926904577, 1.215541086425461,47.9401389719045, 1.215037488920672,47.93885494319191, 1.214954464786516,47.93779342149227, 1.215043092094874,47.9363407902911, 1.215043436287672,47.93539181579477, 1.214206532346822,47.93410933982219, 1.213609127470434,47.93316051834768, 1.213112851404717,47.93260336150517, 1.212533507296158,47.93187919740608, 1.210180502577089,47.93115421974012, 1.209493352708805,47.92981172970656, 1.208819139367478,47.92551164507018, 1.208401319150326,47.9247863098085, 1.208645923545717,47.92400260686853, 1.209900365258577,47.92299814803012, 1.210226322892297,47.92249220907416, 1.211724371978548,47.92181936704898, 1.213314721700272,47.92098419956649, 1.214472407360233,47.92002901617879, 1.214472609944792,47.91908140564839, 1.213126602005715,47.91846159628099, 1.212370020625443,47.91772885583445, 1.212287349378542,47.91700141303419, 1.212371706458356,47.91632976544387, 1.212290479773472,47.91509907397838, 1.212205257261871,47.91442873997744, 1.212309864477192,47.91358628773072, 1.212666745132054,47.91296778700396, 1.213548085078358,47.91206792187286, 1.214963268389838,47.91128645992425, 1.21611418615602,47.91056243864998, 1.220853609938217,47.90748922618924, 1.221355677858,47.9066512988233, 1.221278268406451,47.90564814920901, 1.224915931428171,47.90183964430257, 1.225402224927819,47.8986511104391, 1.226291382372082,47.89814330712083, 1.22668946421885,47.89765449521068, 1.227817040040222,47.89676593204296, 1.227883570369329,47.89605663266561, 1.227828076106704,47.89521924058001, 1.227436156780129,47.89455837510409, 1.22663909622959,47.89393846851418, 1.226058846961308,47.89328351247398, 1.225151244994379,47.89241068496343, 1.223720053657843,47.89136502534964, 1.222464896262508,47.89061535709021, 1.220596164540666,47.88958349414241, 1.218243461260258,47.8890297707092, 1.216677054005388,47.88837778845101, 1.215048692206793,47.88809087225835, 1.213405598707439,47.88752156622182, 1.212729644875197,47.88738192560444, 1.211326585580963,47.88715443662927, 1.210571439388502,47.88696532706801, 1.208696429669857,47.8868239286656, 1.207202136314891,47.88680944033551, 1.206480444353076,47.88639698268616, 1.20480274124742,47.88638852028029, 1.203805882525742,47.8863876292644, 1.200771316777917,47.88579546337856, 1.199380856295974,47.88574929234942, 1.197798890918733,47.88566200855296, 1.196945145280466,47.88566447731684, 1.194513030569482,47.88558321130833, 1.193202464710808,47.88563710425396, 1.19050267329753,47.88556397196413, 1.190107345147581,47.88574403023319, 1.189382205495406,47.88614914616569, 1.188323730666497,47.88628453390509, 1.186337917267139,47.88642204169764, 1.184946132205084,47.88656117438613, 1.183820967272126,47.88700668753221, 1.183424161306832,47.88736287073072, 1.181964946620056,47.88736670119118, 1.181301078983253,47.88723509216776, 1.180305930961509,47.88701432767341, 1.178448367240364,47.88634956730611, 1.177917564753329,47.88612788235908, 1.175663808092963,47.88604129247293, 1.175266445117856,47.88586351816095, 1.174670754714398,47.88541880683204, 1.175064642627731,47.88453025908836, 1.174996319045917,47.88404174932858, 1.174003795877241,47.88381980833376, 1.172875287251728,47.88355467218646, 1.172277806141935,47.88319974903122, 1.171878278956257,47.88284480322649, 1.170751198684599,47.88177883053033, 1.170014679098841,47.88115794156936, 1.169356327786531,47.88049131832604, 1.168627510814915,47.8796919769038, 1.167562575339815,47.87884853782736, 1.166897123005683,47.87809348720579, 1.166366807991068,47.87747179877964, 1.16603586801466,47.87676118184015, 1.165705598010533,47.87622847944186, 1.165178351131132,47.87569635036845, 1.164908972423731,47.87520723041056, 1.164575059496414,47.87476281550676, 1.164044803554418,47.8741413704281, 1.163717118200623,47.87352071603662, 1.16271272392936,47.87298598265896, 1.161851136061174,47.87254249555443, 1.160985165602469,47.87236439596831, 1.159649199869335,47.8717407110198, 1.158792045349244,47.87134278045436, 1.157931915894608,47.87103292222287, 1.157001270432531,47.87085522785312, 1.156004131097077,47.87041090534036, 1.155473839806462,47.86987809465882, 1.154546533012052,47.86903481046332, 1.153752118910958,47.8684581580381, 1.152493777007777,47.8677690879774, 1.151894897099012,47.86763470616536, 1.150633023205756,47.86727842019781, 1.150304470335982,47.86692543213707, 1.149445315840637,47.86643924031654, 1.148714775201256,47.86630541222776, 1.147323787127704,47.86608471519332, 1.146529112720259,47.86604119491617, 1.145468566800304,47.86564212922619, 1.145003709332315,47.86533085791052, 1.144142605990908,47.86493225185405, 1.14301221391365,47.86417467175373, 1.142415148856188,47.86368616075881, 1.141685907302673,47.86324232199606, 1.141088127850054,47.86270861884236, 1.140424658610002,47.86173213352851, 1.139298340290665,47.86049041808354, 1.139165558190527,47.86040145274794, 1.137773456682311,47.85969094191126, 1.136780245360726,47.85938180887229, 1.135852271036896,47.8590272250802, 1.134726722770282,47.85867175186486, 1.133599492161082,47.85858417136155, 1.131876872910212,47.85809639935124, 1.130749973254567,47.85756383614768, 1.129489919052517,47.85738681755698, 1.128363236478037,47.85729861397986, 1.128229128898411,47.85676516340866, 1.126899634858475,47.85596516367195, 1.125839376625166,47.85556570552239, 1.12497948950201,47.85525538439304, 1.124186143682195,47.85498956318253, 1.122723552678991,47.85405585388594, 1.122922811983202,47.85343361201954, 1.122922555016459,47.85298909632635, 1.122258460735793,47.85214455062362, 1.121660848851505,47.85156665391841, 1.121197005596898,47.85112250968775, 1.120599113413235,47.85050000546331, 1.119800559783286,47.8502767191859, 1.11887210803387,47.85036586555436, 1.117616076813718,47.85063447428157, 1.116625901306506,47.85019195941306, 1.115559150556692,47.8494788437342, 1.114569979323903,47.84845905473164, 1.114042520937147,47.8478383155295, 1.11344505181151,47.84721599075854, 1.113042804082255,47.84654743932839, 1.11283629998303,47.84574375079436, 1.112837304203907,47.84481089229491, 1.11283785114712,47.84405568810093, 1.112572765291418,47.84374470595895, 1.112417982338783,47.84280262465325, 1.112088404262067,47.84240324279119, 1.111297177690702,47.84187125057309, 1.110438240060745,47.84169452269394, 1.109374895690607,47.84173860666633, 1.108240084912509,47.84204749703666, 1.107972888206135,47.8422689709835, 1.106590594400572,47.84222845421193, 1.105793665340201,47.84205024531856, 1.104462643094337,47.84164828166556, 1.103337180710113,47.84133787151969, 1.102544219347266,47.84111680057107, 1.101287945036148,47.84071835552854, 1.099432943729767,47.84054131225596, 1.097776811222715,47.84071984053504, 1.097511671595044,47.84094203552797, 1.095857305735663,47.84121150300724, 1.094729471868504,47.84094353743847, 1.093668684191712,47.84040959671017, 1.093006069877248,47.83992141857797, 1.092276570494775,47.83907623360078, 1.091481042849495,47.8380978045118, 1.09108283650788,47.83711999840007, 1.090617672491192,47.83596382789614, 1.090219734323753,47.8353856800497, 1.089159106164786,47.83449648445877, 1.08809837869552,47.83365165975368, 1.087764471560121,47.83240454957844, 1.087432848246962,47.83178185358422, 1.086969768371148,47.83049355508794, 1.086969402430128,47.82951524640254, 1.086637753190729,47.82875928438762, 1.086372490263926,47.8280923780239, 1.085775766541346,47.8269808490154, 1.085444606976466,47.82613675783672, 1.08537768691042,47.82529144623768, 1.08550978758916,47.82378020304491, 1.085708343932041,47.82329123919877, 1.08610574670827,47.82266939675443, 1.086371199823873,47.82173240201835, 1.087693749391875,47.82053292147023, 1.087892234432772,47.82053313507167, 1.08888435094967,47.820268012331, 1.089215366593521,47.81973392879046, 1.089545696022171,47.81902280136904, 1.090075433202671,47.81853331774131, 1.091267655985888,47.81746464383925, 1.091598763277787,47.81715292007471, 1.092194734387221,47.81688481275266, 1.095304438818872,47.81670482333161, 1.095437397177236,47.81603858992538, 1.095173281333743,47.81568327259324, 1.094975350702555,47.81501640485685, 1.094908956215193,47.81448263879219, 1.094908818335213,47.81359272323569, 1.094842000013657,47.81283641152726, 1.095040495044529,47.81207969603805, 1.09490764291688,47.81154572755681, 1.094906742689716,47.81065615838773, 1.094773606007664,47.80998906762541, 1.094111272891382,47.80878504524798, 1.093514237905666,47.80820717461016, 1.092452915480394,47.80691850538197, 1.091856467633022,47.80665318392016, 1.089406714559331,47.80625851699934, 1.088412598014628,47.80621179462189, 1.0874857105198,47.8055906567738, 1.085431626399966,47.80438917425075, 1.084635145360977,47.80331991872352, 1.083506336999278,47.80216010150561, 1.083572992807476,47.80109379188278, 1.083506700049867,47.80009887241172, 1.083307734725409,47.79925354176186, 1.083175433073379,47.79845257428445, 1.082776480291201,47.79742976216294, 1.082445521351986,47.79649544614419, 1.081854806551974,47.79511653458655, 1.081588486432821,47.79400494775252, 1.081193602012675,47.79320514713487, 1.080603645402377,47.7922732974971, 1.080136952908831,47.79178363256286, 1.07934477867633,47.79116220855459, 1.077907571498102,47.7903245633354, 1.077178371760795,47.79005756001983, 1.076182608053784,47.78992299894746, 1.075024731763847,47.78957859762915, 1.073745262190926,47.78941424151537, 1.0726084438096,47.78929644008188, 1.071367497485156,47.78934258128377, 1.07028968215505,47.78946765645267, 1.068765663786294,47.78964144254387, 1.066876259026429,47.78982210704144, 1.065783229533878,47.79012309811348, 1.06451987145806,47.79041892525575, 1.063871690445694,47.79076511906879, 1.063205955421395,47.79125327610979, 1.062351123339099,47.79183158024335, 1.061378171225428,47.79254417148777, 1.060636492307708,47.79290323924324, 1.059652410143457,47.79352419131154, 1.05819368142781,47.79414450272845, 1.056192195342035,47.79504299792858, 1.054344313229208,47.7956152242856, 1.053420858463696,47.79578832233695, 1.051701057205705,47.79605291368305, 1.048591072645493,47.79684150950288, 1.047398035652464,47.79715402904345, 1.04627124946775,47.7973312167254, 1.045277763644222,47.79746724690312, 1.044681871295438,47.79755748711851, 1.043754511044952,47.79764669896573, 1.042227396176671,47.7976412578615, 1.041829817113509,47.79777506046816, 1.040902756230707,47.79808785023929, 1.039905969093113,47.79817414361025, 1.039241690890279,47.79812830599519, 1.038510418837845,47.79812659925904, 1.037649823267273,47.79830594511851, 1.035732174591999,47.79870989940202, 1.034205850542762,47.79897587121074, 1.033076107804362,47.7992415944157, 1.031954737426204,47.79955554164379, 1.030824230568001,47.79955387042844, 1.030096777957397,47.79946528719147, 1.029102716550085,47.79919793950362, 1.028237702589124,47.79852844714094, 1.027772070375079,47.79799318429929, 1.027112922521212,47.79759365996819, 1.026391453746086,47.79728483750379, 1.025400665895177,47.79675159643245, 1.024731324757198,47.79652603938848, 1.023936008443457,47.7961250176235, 1.022684365129109,47.79559340669278, 1.021873236459584,47.7947848234231, 1.021407160807286,47.79451651073023, 1.020484463268062,47.79420674157917, 1.019954690742225,47.79371680448062, 1.01942650374292,47.79340575124724, 1.018370012259169,47.7924623043375, 1.018120588687381,47.79212659161423, 1.017955014209321,47.79156723931874, 1.017375027204906,47.79039301145311, 1.017041261511821,47.78983247705538, 1.016287919552127,47.78859823685952, 1.01554113973134,47.78787157360289, 1.014624338428149,47.78686317830804, 1.013208260679974,47.78596690373447, 1.010794092749535,47.78512673389574, 1.008129532335758,47.78417422453718, 1.00688017768465,47.78361380990286, 1.005713225669737,47.78378128062876, 1.004713606333809,47.783388863599, 1.003298301601475,47.78271647801248, 1.002299055608987,47.78193212966927, 1.000468810367082,47.78176411170374, 0.9994682291392252,47.78092336667987, 0.9986361221224627,47.78013927961808, 0.9973870380481742,47.7794107726624, 0.9958151484877078,47.77863081854549, 0.994510381084595,47.77813857737425, 0.9926223385914739,47.77836927669939, 0.9908732034442667,47.77925815095296, 0.98863388940606,47.77992621381794, 0.9876333841675518,47.78037082368716, 0.9866500966240039,47.78148698851444, 0.9856248942595442,47.78254339424637, 0.9848725270354919,47.78360301637506, 0.984611306448386,47.7845518952562, 0.9843516973185018,47.78589246424956, 0.9843454640202651,47.78712187373054, 0.9844255703056178,47.78851873914918, 0.9844183352179243,47.78986095888067, 0.9840845867393144,47.79164841176014, 0.9835089741954165,47.79315326718471, 0.9827145653280951,47.79497786516333, 0.982198432144756,47.79632402208099, 0.9815169251802018,47.79767097365004, 0.9815459725051455,47.7989861813163, 0.9809707228778848,47.79976364805946, 0.9797316585864735,47.80104181222511, 0.9805027862621927,47.80238719820029, 0.9804178232728306,47.80322693797258, 0.9795003274402347,47.80429082672679, 0.978747038026569,47.80591395742493, 0.97824685739117,47.8066426333679, 0.9776629295405712,47.80759564039851, 0.9771602618668993,47.80882706313037, 0.9768237680786173,47.8104504407452, 0.9756546378650768,47.81173864316585, 0.974153936564889,47.81213311917163, 0.9714840885000484,47.81218614536055, 0.9698990380262984,47.81217934855594, 0.9680649363124161,47.81167809762527, 0.96681482608453,47.81156139257207, 0.9642290690784848,47.81144974382923, 0.9625643609797835,47.81094026180774, 0.960477723994913,47.8100475629033, 0.959394015466799,47.80931925827348, 0.9586435193596402,47.80836765545505, 0.957563007838985,47.80730085854767, 0.9563141801059487,47.80645914223238, 0.9563178833242461,47.80561785809278, 0.9571524369961538,47.80399536344905, 0.9569062199216916,47.80298617817928, 0.9564914359459068,47.80203360153822, 0.9562385364743309,47.80136205339905, 0.956241187859419,47.80057758092775, 0.9561593652195342,47.79968121553116, 0.9559105015944456,47.79878468476149, 0.9554136341258186,47.7977199678487, 0.9542546269399281,47.79671157429358, 0.953836286905414,47.79659896418766, 0.9535048355015903,47.79581477510564, 0.9532717264874294,47.79509018447648, 0.9527201573872476,47.79459385019495, 0.9528420122541377,47.79293113494801, 0.9534362829290615,47.7918778288892, 0.9547393671554403,47.79052811485497, 0.9566863087473652,47.78786496246624, 0.9570205826592968,47.78562161534843, 0.9574413651064334,47.78338089737342, 0.9572265133777195,47.78270568542486, 0.9569746144358504,47.78208390838216, 0.9565989051152286,47.78143548763287, 0.9558916492668743,47.78076151949992, 0.9551776270100305,47.78047834522702, 0.9545056710491906,47.78005383483044, 0.9542962609534413,47.7797148839151, 0.9541718836976544,47.77912226581294, 0.9541290641865033,47.77835899971558, 0.9542532204574314,47.77754021340701, 0.9544245207647891,47.77714593492859, 0.9545558582637456,47.77655340629603, 0.9546833541998865,47.77593223182794, 0.9548943246592785,47.77542414127836, 0.9550597310803967,47.77455096053508, 0.9549297153942575,47.77368017831869, 0.9542592289768096,47.77286371842087, 0.953384285648909,47.77209664451133, 0.9522960826019045,47.77144274997243, 0.9513317913257425,47.7709023464714, 0.9495677964621481,47.77044785665815, 0.94835011475584,47.76985296708439, 0.9477635221504386,47.76940292659844, 0.9473441559954524,47.76909223448786, 0.9460001509377493,47.76863808827352, 0.9442402352759842,47.76824655169749, 0.9428510497534717,47.76843930980633, 0.9415867666665748,47.76857571717526, 0.9400739378107943,47.76882892595961, 0.938767561587838,47.76905145344707, 0.9377998293439447,47.76933281245922, 0.9371705978438741,47.76992686072804, 0.9366247818343538,47.77066171744398, 0.9350270252737403,47.7708858355575, 0.9330576250895035,47.77091393966795, 0.9316693183422445,47.77133567236491, 0.9314544921866316,47.77181449545378, 0.9311585037369384,47.77311350515344, 0.9318578708845564,47.77441175073409, 0.9321920967515781,47.77542939045396, 0.9297045687110028,47.77799810033625, 0.9290842048607857,47.77865976555235, 0.9282003508929365,47.77885684147, 0.9275707809967315,47.7788555995644, 0.9267302454457962,47.77924943511341, 0.9266873559857913,47.779560242702, 0.9263455497171975,47.78063466554152, 0.9261342847371454,47.78114274963765, 0.9253348493272284,47.78170656517067, 0.9240716746445088,47.78195975640686, 0.9227690803519693,47.78209928545957, 0.9221811194162799,47.7818727757206, 0.9211321097137812,47.78144701708529, 0.9207120188025613,47.78133302290108, 0.9194914282683726,47.78155730669382, 0.918523201849908,47.7818956232718, 0.916924574714031,47.78243097343177, 0.9161250064677802,47.78279734372314, 0.9147350033145187,47.78299341328981, 0.9140618792502267,47.78282299102892, 0.9123446637348281,47.78236911467555, 0.9114254705052505,47.78183186924395, 0.9106711152697218,47.78112513577356, 0.9102119579515886,47.7808427119447, 0.909798620935547,47.78073066478904, 0.9094970428603234,47.78101108627146, 0.9087900921392986,47.78072917226496, 0.908482110150942,47.78011451019108, 0.9081892774119638,47.77986071358595, 0.9068652040847384,47.77959904717239, 0.9060936717590664,47.77987674759739, 0.9054096997471248,47.77992995285288, 0.9039357660825971,47.77981449731135, 0.9030873297524949,47.78006660265617, 0.9018636993071709,47.78065844613944, 0.900965199467862,47.78091018821193, 0.89971328159632,47.78136248660942, 0.8985315252921566,47.78164412878451, 0.8978143325830672,47.78192598348323, 0.8973513032348849,47.78206663384986, 0.8967614985034942,47.78220719482187, 0.8958789591559793,47.78203648644324, 0.8955446172031714,47.78152734902437, 0.8957971236850995,47.78110434357695, 0.8957991408379229,47.78053924771134, 0.8950443447611462,47.780085815818, 0.8943710893757526,47.77991644104263, 0.8933647894663892,47.77946142491089, 0.8929045943866215,47.7786979597665, 0.8927379112515866,47.77824532709457, 0.8926558649869207,47.77767893185996, 0.8926164645348678,47.77697021852974, 0.8921967460154435,47.77652038891302, 0.8919034643138559,47.7760991935312, 0.8910030234419821,47.77494505669169, 0.8908851060119929,47.77450572707713, 0.8901740578228328,47.77329915714601, 0.8901147095079694,47.77325836733241, 0.8898199592539129,47.77265197961349, 0.8895384227418647,47.76856603219596, 0.889969833208312,47.76220165634317, 0.8900399111007999,47.75887388788083, 0.8914829157857868,47.75514686844634, 0.89136574700197,47.75448082319782, 0.8923843791598216,47.75303917735691, 0.8928685706734429,47.75067391223115, 0.8920415576474454,47.74814756477719, 0.890318057860597,47.74638017209161, 0.8872301009114665,47.74389832071568, 0.8851497303700313,47.74321703278999, 0.8833092706389466,47.74305914795779, 0.8827122734893322,47.74313663697255, 0.881521407677468,47.74305409586968, 0.8808699992237765,47.74269466785515, 0.880276563859212,47.74253460270839, 0.879899904655403,47.74352292412955, 0.8789491597578803,47.74348304767803, 0.8782298797892026,47.74331859288689, 0.8778763984466735,47.74291903451792, 0.8771824173378066,47.74261003660429, 0.876181146959496,47.74229032875112, 0.8757692878350839,47.74204820260109, 0.8750557324622863,47.74188013615314, 0.8742657162892442,47.74210255508904, 0.8724106852845779,47.74251062439187, 0.8715760442928513,47.74278146125486, 0.870641102294698,47.74309956810073, 0.8697874007157084,47.74356213636772, 0.868630464978793,47.74434109283116, 0.8682105434856169,47.74501955880683, 0.8678986811437426,47.74555857085019, 0.8675617245409057,47.74614941731133, 0.8658290907521859,47.7463704133942, 0.8655292563303618,47.74630158663364, 0.8654240337552042,47.74695954453522, 0.8657091445504648,47.74730786019044, 0.8648720739745102,47.74790730671428, 0.8645739286059638,47.74814727295841, 0.8637724025849991,47.74886464915394, 0.8629954211057651,47.7492643057434, 0.8612726217288015,47.75038458747817, 0.8564516174688066,47.75358018169843, 0.8553737571460546,47.75393675214817, 0.8549532232724175,47.75465814777143, 0.8544744047557061,47.75517929425513, 0.8540552269536903,47.7557004252123, 0.853215909060566,47.75621942838593, 0.852616452028471,47.7569004544945, 0.8523739148308067,47.75746112196326, 0.8521957053790175,47.75810393298348, 0.8519561965328535,47.75862534953523, 0.851301121052316,47.75890552318458, 0.8507019177258145,47.75922502816555, 0.8501615980414101,47.75958472720561, 0.8495610215559283,47.75990422235219, 0.8481827480590596,47.7605434502695, 0.8470495842667988,47.76106322101076, 0.8459062694684435,47.76102060636418, 0.8451318766965557,47.7608586843298, 0.8435218590417719,47.76101639908512, 0.8425684566968243,47.76129565123014, 0.8420922403716986,47.7614553133948, 0.8402383766300972,47.76137153891199, 0.8382712961444696,47.76132775657165, 0.8357660949499177,47.76124277197805, 0.8332564382117341,47.76135829259739, 0.831140294236377,47.76151484702161, 0.8300643914607237,47.76191381347985, 0.8294070347909469,47.76211309985337, 0.8272541831976241,47.76291140720871, 0.8263567428215879,47.7633510284363, 0.8254594791104495,47.76395082981165, 0.8248004854635838,47.76443107478234, 0.8240770100015881,47.76523374463222, 0.8238973981839877,47.76559407802429, 0.8233570128089838,47.76619518367853, 0.8226967603145283,47.76667618314043, 0.8222159582804808,47.76719745938052, 0.8220333684721909,47.76787978097458, 0.8210713153433602,47.76988308562978, 0.8196370763408396,47.77056095284265, 0.8185610655172439,47.77087964705824, 0.8175426452603176,47.77127989897814, 0.8163424519068203,47.77144124668112, 0.8158627233223683,47.77144139562991, 0.814550110113018,47.77127750726907, 0.8141922309580139,47.77127645469531, 0.8131782958744652,47.77139364318403, 0.8123387203408043,47.7715537832767, 0.8117410359352563,47.7715927297387, 0.8111504577281751,47.77174836628393, 0.8101461388499729,47.77178118212493, 0.8090690681338515,47.77174006768547, 0.8079235032960241,47.7717424360001, 0.8068436281179174,47.77214274240931, 0.8062472827602534,47.77238142145735, 0.8057645232025976,47.77274301473486, 0.804407228230005,47.77301362364361, 0.8032603883577566,47.77317640263063, 0.8025293349110996,47.77350109901541, 0.801623162972632,47.77367318323731, 0.8003664823890091,47.77355061808477, 0.799887658410981,47.77354981449492, 0.7991679700837437,47.77378946928188, 0.7988684554183436,47.77459082074015, 0.7979719670961742,47.77474934082303, 0.7974941422473593,47.77462797043212, 0.7968969198553424,47.77450635582706, 0.7961214879746734,47.77446431264606, 0.7954614930069539,47.7751450793357, 0.7954017229920539,47.77514494914644, 0.7947453190604962,47.77514336018227, 0.793970092232172,47.77510121998009, 0.793133413992976,47.77505928808273, 0.7919981710458615,47.77505678664998, 0.7910427718240953,47.77493436871054, 0.7901480150997457,47.7747315405237, 0.7888951210664459,47.77444731193148, 0.788000195256899,47.77416376841737, 0.7870458576592655,47.77392101558636, 0.7862708690039426,47.77359798046048, 0.7848978984906838,47.77319258538216, 0.7843620053003378,47.77291047773336, 0.7838264081262309,47.77270910282294, 0.7828097621153205,47.77282689028622, 0.7822714898887284,47.77258377982142, 0.7819145659176031,47.7722616022818, 0.7813786522893906,47.77210020205613, 0.7805442883049665,47.77205892479342, 0.7796486242136638,47.77209716657019, 0.7787507888802878,47.77209431357077, 0.7779120186638266,47.77209146127721, 0.7769537486175723,47.77208843958002, 0.7756975785628129,47.77208506884738, 0.7741401782675014,47.77220108955491, 0.7725313080793654,47.77235919957174, 0.7714558868054837,47.77243701619269, 0.7707941750436333,47.77239430582495, 0.769482549148961,47.77223084430727, 0.7689472274596342,47.7720691713189, 0.7683507616585957,47.77130387173846, 0.7679274464150581,47.77110056310935, 0.7667363102517442,47.77085723025463, 0.7663230946632495,47.77085736638002, 0.7655566683586983,47.77073712903668, 0.7654895715359943,47.77001138009625, 0.7653705525469264,47.76932728315416, 0.7650752926680298,47.76852277387916, 0.7648969767883075,47.76828115758632, 0.7641929517966304,47.76759954656093, 0.7637827193286171,47.7671586592868, 0.7633651059691147,47.7668359309768, 0.7627665391343614,47.76655234331907, 0.7616886195351213,47.76602555526085, 0.7612147644436789,47.76554289106142, 0.7605625578670612,47.76526137407936, 0.7591293104221678,47.76477498859365, 0.7588302620760885,47.76473385917278, 0.7572092040955312,47.76405868642598, 0.7565525887902709,47.76389625635952, 0.7551194769608287,47.76345014887274, 0.7544661338279043,47.76300703037826, 0.7537510683689845,47.76260329883152, 0.7524384497718302,47.76215779983883, 0.7512434155987506,47.76195348638469, 0.7505871748708445,47.76179114557027, 0.7493929018066012,47.76146626527262, 0.7485594353370573,47.76082099693772, 0.7480843931235057,47.76037792177051, 0.7477294221498172,47.75989502642374, 0.7470147018876219,47.75941083750571, 0.7464787509962036,47.75896726355826, 0.7460029603126372,47.75848375122497, 0.7458269525990602,47.75796099683674, 0.7453546535203658,47.75727668062915, 0.7447576905256312,47.75691348292556, 0.7436801026676961,47.75662983055409, 0.742846789650244,47.75594544589882, 0.7426098097002978,47.75554368803429, 0.7415980415537007,47.75521907728057, 0.7405240279422003,47.75505568389222, 0.738851876928905,47.75509095721035, 0.7382538933506169,47.75520945450468, 0.7375365717269915,47.75528760955501, 0.7365808446493247,47.75528496332186, 0.7355658250406947,47.75520178755112, 0.7344895834930014,47.75527883913853, 0.7329942965795057,47.75543506423994, 0.7312613085387909,47.75543012700682, 0.7300052914919632,47.75550676903117, 0.7286315461706465,47.75550299117736, 0.7274961739183824,47.75529865431216, 0.7269626684143446,47.7548957330533, 0.726187934717629,47.75461237182217, 0.7251731406526716,47.7548107229521, 0.7231406346311644,47.75524686100486, 0.721578941666138,47.75592471245859, 0.7200824805400289,47.75648277453358, 0.7188237360909824,47.75712178277896, 0.7167297583068972,47.75771812571097, 0.7149884855459875,47.75847631290771, 0.7137298813053548,47.75871373400851, 0.711996541230306,47.75898969577796, 0.7100136557563701,47.7595065798497, 0.7092879461110755,47.75966573066135, 0.7083374446375644,47.75986319229376, 0.7044518959154544,47.76072784711749, 0.702894622289449,47.76072268202379, 0.7017563928089317,47.76043746067118, 0.7009836173952522,47.76019435565315, 0.7004509478007748,47.75979124831718, 0.6997397775826926,47.75914636341464, 0.6995643799104556,47.75850303585886, 0.698430855417385,47.75801755951085, 0.6974066396757337,47.75785473803965, 0.6966880281989631,47.7578527702109, 0.6954381112867338,47.75792862694079, 0.693590091440484,47.75824337263402, 0.6928127179085237,47.75848186691593, 0.691853755056016,47.75888065415779, 0.6915503937380265,47.75952261696236, 0.6901730755383113,47.75987989316584, 0.6884983244119099,47.76003535691922, 0.6883755046942923,47.76047649523971, 0.6869386125080155,47.76087328963268, 0.6861012631771501,47.76091142931373, 0.6848441756010857,47.76110866803563, 0.6843640650833862,47.76146826733417, 0.6833429702441691,47.76218807158534, 0.6829771242728242,47.76303161564315, 0.6832693390405211,47.76391705601786, 0.6832027647431005,47.76476174954536, 0.6828949954290096,47.76596681842737, 0.6821086194926664,47.76721038094757, 0.681683722210181,47.76801367609211, 0.6805988719594609,47.76905641191522, 0.6796982504140061,47.76945613933529, 0.6776652961902272,47.76960811677511, 0.6759956467189454,47.76947836841428, 0.6746817883072539,47.76911329665487, 0.6734249204816223,47.76875011111942, 0.6721166430283887,47.76842104462349, 0.6711679823812631,47.76789324108966, 0.6707509386890251,47.7674508714374, 0.6701538472834159,47.76684787141805, 0.6695630824242045,47.76644226274302, 0.6686136580474908,47.76567517230055, 0.6678549348459057,47.76478520884695, 0.666835179973873,47.76406141301353, 0.6648612979986979,47.7628113252639, 0.6634216484709157,47.76168262662673, 0.6614574530654522,47.76071060363629, 0.6598601644450071,47.75982009680282, 0.6580636321821913,47.75917091189815, 0.6574697848132303,47.7590080163959, 0.655713958858442,47.75885489229979, 0.6546991698039335,47.75861397633919, 0.6547016846525935,47.75829126867242, 0.6535664018524301,47.75812752845878, 0.6530903236805301,47.75788158067856, 0.652794592479907,47.75747791345773, 0.6525608804153933,47.75679311444462, 0.6516736243468273,47.75550472710773, 0.6504223322118605,47.75497719753247, 0.6492299091126057,47.7544512692093, 0.6485169291992179,47.75392563210219, 0.6475649471874902,47.75355895998577, 0.6469100840306652,47.7532749723695, 0.6459539738879698,47.7531914386872, 0.6443994336099195,47.75306603289297, 0.6432651807191689,47.75302116708061, 0.6422482603353501,47.7526572796059, 0.6411125743178303,47.75221206719487, 0.6398616526528969,47.75176482333608, 0.6387268127534043,47.75147949752309, 0.6375368350451538,47.75147376919987, 0.6364577594099075,47.75155075967706, 0.6347585405775504,47.75158988840364, 0.6319739152890946,47.75181523425358, 0.6305977318249916,47.75197091591881, 0.6288575127483114,47.75204591480345, 0.6274916065945462,47.75183856088869, 0.6258675832198719,47.75151276169867, 0.6250317276676185,47.75114794320735, 0.6245569321137623,47.75070382751164, 0.6236831126362148,47.74988996564043, 0.6236296003777113,47.7491663386683, 0.6235713636736904,47.74896499227737, 0.6229782139906632,47.74848099683488, 0.6219032783199907,47.74843791386114, 0.6211247353244431,47.74847436724886, 0.6204735441055931,47.74783010003961, 0.6203578649577253,47.74730627494119, 0.6186541290447707,47.74359952253575, 0.6184328957588576,47.74138707649215, 0.6189136991052493,47.74114748713037, 0.6196929622453524,47.74090920159241, 0.6199958197483539,47.74038783281334, 0.6201794466631441,47.73990592717765, 0.6202401915495841,47.73974555277346, 0.6200087325229759,47.73893981588527, 0.6195377892580609,47.73809314347043, 0.6191229452576295,47.73768883923626, 0.6185884132381148,47.73716522922178, 0.6178739297710365,47.73676153110985, 0.6175175007867892,47.73647927131705, 0.6168636441813932,47.73603531292092, 0.6163902730529429,47.73547100165732, 0.6155608579527012,47.73462540296849, 0.614846892013631,47.73426057401439, 0.6142516961076439,47.73397659625226, 0.6136575859187445,47.73357243208066, 0.6128816241276382,47.73340714476306, 0.6106758893271547,47.73263300821652, 0.607758058681469,47.73141643371078, 0.6040083579914608,47.73068705554009, 0.6000123516582079,47.72973510498952, 0.5992397169969479,47.72965435088467, 0.5979316829621695,47.72980532949462, 0.5970985700278106,47.72988096632654, 0.5954330911020378,47.72999535855441, 0.5944156022125355,47.73022502575194, 0.5932844794558889,47.73018014301707, 0.5917901903879681,47.73008919560061, 0.5896894972739801,47.73003241027714, 0.5886710635975447,47.72986522324133, 0.5871192121567077,47.72985960684387, 0.5861048842618573,47.72973497974149, 0.5853859806043573,47.72965060107127, 0.5844914420465273,47.72948627431336, 0.5838968189676166,47.72932359909951, 0.5830042653651618,47.72899868706648, 0.5827578964718612,47.72931762593019, 0.5822149010565313,47.72951532108142, 0.5809553633201858,47.72979108785503, 0.5795151485148864,47.73014650964194, 0.5794502765956793,47.73058839333222, 0.5792052108874657,47.73119057334833, 0.5777688024611649,47.73118438770003, 0.5769800067895972,47.73178368912776, 0.5762072659197628,47.73190185305698, 0.5742339780207146,47.73189361787649, 0.5731550953541102,47.7318888400881, 0.5712449442662404,47.73188114607653, 0.5693932238258698,47.73183325180175, 0.5688111162437366,47.73187239503194, 0.5674939723863588,47.73263079376932, 0.5658192540100515,47.73318833855766, 0.565562887737542,47.7332284337156, 0.5645972897950031,47.73350533279719, 0.5640679009913896,47.7339033757396, 0.5632704424140513,47.73426054794663, 0.5622238828931643,47.73413590529957, 0.5609586560260027,47.73397045443259, 0.559564586769568,47.73468675235469, 0.5587869884247993,47.7346431673423, 0.5581249980522216,47.73447943670202, 0.5576304959214724,47.73411616092631, 0.5570685690894663,47.7338344993358, 0.5559198001181755,47.73355174463897, 0.5547476184249405,47.73358518889541, 0.5530484912768401,47.73369503306597, 0.5520391202201296,47.73377025079885, 0.5510815302314427,47.73368626856566, 0.5502441363638582,47.73344332353373, 0.5492910450175117,47.73316084331133, 0.5482210974950874,47.73331387982463, 0.5473230808207461,47.73366855357043, 0.5465489053652703,47.73346474745872, 0.5459662438220505,47.73282656321265, 0.5455571180742709,47.73258853996075, 0.5447204602218676,47.73278192339728, 0.5432098887306722,47.73316935977304, 0.5432568860694715,47.73360997447895, 0.5432507078618016,47.73401205759393, 0.5434230635145951,47.73461657194839, 0.5437155371053564,47.73518173320172, 0.5429311337148732,47.73586255449119, 0.5421550599768921,47.73577819389464, 0.5416219999084433,47.73517210032316, 0.5407861842315924,47.73484643066064, 0.5394729695342591,47.73447819532585, 0.5382729599354263,47.73419097002937, 0.5371400112063185,47.73382335556951, 0.5364851404841033,47.73349821139102, 0.5354721282906496,47.73309095230843, 0.5348765308934418,47.73272573867081, 0.5333889845120297,47.73171199163487, 0.5319639019689221,47.73073936764879, 0.5304987887140555,47.72982732131916, 0.529312077809081,47.72931658619327, 0.526288859311336,47.72887419444452, 0.5241000590009293,47.72853264990547, 0.5234999572563055,47.72828736897375, 0.5217577052114137,47.72775298761118, 0.5208583033249852,47.72750621836504, 0.5197188323455322,47.7271774098702, 0.5189381493512002,47.72709245987625, 0.5181045238502406,47.72672625393157, 0.5170338929110585,47.7262788053147, 0.5170343049419728,47.72623852353142, 0.5149629713295099,47.7260096423803, 0.5143697731706567,47.72544235522737, 0.5138329053432216,47.72523823441112, 0.5129376726250712,47.7249518113169, 0.5122208906380195,47.72478723811522, 0.5113247530372007,47.72458249609316, 0.5104887537145686,47.72433722421578, 0.5094151583989941,47.723929020798, 0.5085229309108712,47.72344053100344, 0.5079292127597768,47.7229544670364, 0.5074569879139261,47.72230780828644, 0.5070501373563587,47.72121823749504, 0.5065208724392676,47.72028946537808, 0.5058747527534138,47.71923947918689, 0.5058211220304685,47.71859481472963, 0.5060105117349565,47.71762909826563, 0.5061968826588554,47.71694513546969, 0.5064507580486719,47.71533316220144, 0.5062768496570982,47.71489008685852, 0.505213144155519,47.71375895023967, 0.5043847237884478,47.71295044865674, 0.5033767899133181,47.71226302138568, 0.5022416188105154,47.7121370289524, 0.5006837268050504,47.71237119333299, 0.4994830287730566,47.71260597103382, 0.4985797360338609,47.71332731243324, 0.4983351047470706,47.71401137517234, 0.4981956372686796,47.71497310843564, 0.4977708808960699,47.71561586030924, 0.4971117656904767,47.71609718439245, 0.4961507543499266,47.71641457712321, 0.4948319561251681,47.71612504366075, 0.4937560264319246,47.71599901445484, 0.4930937467508727,47.71639861825956, 0.4929075486683985,47.71692109319974, 0.4924676082218298,47.71804567348814, 0.4912060207698517,47.71803907468649, 0.4897152951579216,47.71819371543479, 0.4886388382395808,47.7185110478531, 0.4872502254075073,47.71898720278141, 0.4863574516866009,47.71882183282224, 0.4846260661925755,47.71845076773941, 0.4827824284298004,47.71775722897645, 0.4819495995778312,47.71751165566511, 0.4815923261081708,47.71714722443236, 0.4804569684396967,47.71642146800347, 0.4802780191444925,47.71637903831593, 0.4801076318825183,47.71557168468703, 0.4802921294195386,47.71512880694097, 0.480477582828371,47.71456800329619, 0.4806618157415983,47.71412778384446, 0.4805470590822125,47.71368381184833, 0.4803720573566892,47.71327986617563, 0.4804371349164854,47.71279752309953, 0.4805607877885478,47.71243595138658, 0.4805665687683388,47.71191244940744, 0.4802736352999233,47.71134701744164, 0.4799201860975444,47.71082191828462, 0.4789138609332468,47.70981000931972, 0.4783198230676646,47.70940427974143, 0.4771951630516044,47.70843273009218, 0.4767827349424607,47.707948023424, 0.4769112900351474,47.70685963857406, 0.476856263914982,47.70629420165047, 0.4771648437601499,47.70553102643526, 0.4774111044520737,47.70492814904066, 0.4774795477897841,47.70412222981569, 0.4771286530052237,47.70335462801528, 0.476954821714628,47.70287096138597, 0.4773803023384744,47.70226830160203, 0.4775656651983553,47.70170403209262, 0.4776923757965363,47.70105946217264, 0.4777593615038712,47.70033169021531, 0.4780695371731129,47.69936586142606, 0.4783833572131472,47.69820486900088, 0.4785677627933467,47.69776231282273, 0.4783371399965404,47.69691487048659, 0.4781646971275393,47.6961908442053, 0.4786474920902749,47.69587037743181, 0.4820554148335675,47.69088867687707, 0.4807038427982349,47.68862583553189, 0.4794750278456093,47.68604042458672, 0.4772230752289204,47.68393291079563, 0.4715082301944733,47.68144888326537, 0.467142868907402,47.68065773424825, 0.4656484447460372,47.68065112003281, 0.4643877019903613,47.68120992664355, 0.4631744043477113,47.6824519680641, 0.462391474946557,47.68276993871317, 0.4610586311598723,47.68324386753778, 0.4600993151547215,47.68335992704974, 0.4587980694596042,47.68303372024402, 0.4580800107150226,47.68294921293217, 0.4571026018356577,47.68366719158151, 0.4564955307697147,47.68402614413156, 0.4548748752756354,47.68454228478638, 0.4537362404372375,47.6846169941513, 0.4520657508654011,47.68473021315733, 0.4506198083721813,47.68488300029889, 0.4492730808671866,47.68475931340589, 0.4480810624611422,47.68439070240648, 0.4465896082603109,47.68393948176386, 0.4456348991288906,47.68365223583747, 0.443307409865156,47.68295423619748, 0.4416955988820348,47.68258281414411, 0.4401413293113592,47.68237322272056, 0.438467962762573,47.68208212550189, 0.437272836654202,47.68191444411781, 0.4360214544572188,47.68150467819995, 0.4347676620994605,47.68101400002041, 0.4336995351930568,47.68036352338854, 0.4328049776006537,47.67991490838641, 0.4322101285217315,47.67954874707947, 0.4309608057861283,47.6790991705493, 0.430307508260043,47.67873308182735, 0.4294748968755897,47.67836610083725, 0.4282150958493248,47.67847977546668, 0.4271919016936181,47.67899827715577, 0.4256968143788811,47.67858636358761, 0.4255184028282883,47.67842398545946, 0.4253426523273589,47.67797923187369, 0.424987264390154,47.67765473703746, 0.4235671126580058,47.6761952992754, 0.4213677505044871,47.67428566056171, 0.4203004481920374,47.67355451996085, 0.4204900080908658,47.67270848665088, 0.4198445668844785,47.67214310521493, 0.4190697328063351,47.67209976182715, 0.4178708739864732,47.67209246866442, 0.4176364548254696,47.67180961138539, 0.4175204392717973,47.67128374401181, 0.4190199915802566,47.67076623100749, 0.4199274105440766,47.67028912908006, 0.4199897337610851,47.67000672303049, 0.419757882982239,47.66931961095642, 0.419639028512819,47.66907585525841, 0.4204536264555827,47.66826838645446, 0.42039712809283,47.66774290319675, 0.4197992464289655,47.66729494844775, 0.4194391331426033,47.66729276705295, 0.4180698801543486,47.66668040880407, 0.4180145506943235,47.66615556341781, 0.4176568914489707,47.66574989109481, 0.4165827060064515,47.66542133351627, 0.4159258021890082,47.66533711859412, 0.4151480672814359,47.66533283021136, 0.4137730935535729,47.6652446307187, 0.4132372550297845,47.66504012472788, 0.4122257126481372,47.66463143294584, 0.411629213261648,47.66454757443978, 0.4092317889771381,47.66453373381778, 0.4078581589784563,47.66444570762671, 0.4067844403166845,47.66399591901127, 0.4057040758917908,47.66411040681589, 0.4037868156648585,47.66442251157503, 0.4031228338916577,47.66486152451089, 0.4023963628774112,47.66558125235644, 0.4014904138004312,47.66634043693554, 0.4014237257332667,47.66686403968071, 0.4007637673057333,47.66706132790449, 0.3999842643112146,47.66717865299844, 0.3993782317313191,47.66770035209924, 0.398832060081702,47.66826200462714, 0.3980539825492286,47.66837726864622, 0.3965498105540064,47.66897385551354, 0.3963611367602482,47.66953866166568, 0.3955112514579042,47.67030162330744, 0.3951406325221712,47.6712264415819, 0.3939183445430247,47.67311516440489, 0.3931813232839545,47.67442770377986, 0.3923985555631901,47.67466528447041, 0.3916742094129262,47.67518523043257, 0.3905904029987824,47.67554211356531, 0.3889657675091769,47.6759364367284, 0.3877059001209915,47.67609044320103, 0.3863887642702508,47.67624364356201, 0.3856036988823918,47.67644140376175, 0.3847587282066627,47.67675958463971, 0.3837379904929605,47.67703575210532, 0.3817620036526108,47.67726545679687, 0.3809221343493618,47.67742171734906, 0.3798971908360472,47.67745713259695, 0.3789385572689811,47.67725021868782, 0.3777593922513773,47.67699884309512, 0.3710957585464741,47.67631551208569, 0.3687651382422104,47.6757776155781, 0.3664949913892741,47.67527985773317, 0.3655397003169298,47.67511254925775, 0.3650656799803756,47.67490763229, 0.363632423055023,47.67453610338108, 0.3624963464073119,47.67416645819693, 0.3610684936865027,47.67355268238926, 0.3605932285248629,47.67326745333347, 0.3598785383004923,47.67310179762691, 0.3585081261348791,47.67281124508786, 0.3576103724180086,47.67244270695008, 0.3570721278463367,47.67199545262373, 0.3562479972106729,47.67158772455317, 0.3554743436404095,47.67122007281846, 0.3551151091282065,47.67089482026661, 0.3543441029533058,47.67020422253965, 0.3540480886333627,47.66992000873068, 0.3537524024824563,47.66951456519748, 0.3533412675965906,47.66906869245162, 0.3528183757770195,47.66842159712235, 0.352285443692594,47.66805564209641, 0.3522236724758301,47.6676503785968, 0.352053626073328,47.66663991202553, 0.3516499923567267,47.66583201781569, 0.3506488602281053,47.66482744970916, 0.3498719235163605,47.66482284209082, 0.3486141789451125,47.66493632049288, 0.3480133854360474,47.66509403637161, 0.3465757321696685,47.66512561662073, 0.3463455900359094,47.66447881621588, 0.3462334581498823,47.66391322935824, 0.3466560966957388,47.66351254304703, 0.3464802350756315,47.66318894908787, 0.3460656699115816,47.66282363437728, 0.3452913409639251,47.66253665853989, 0.344335982840633,47.66240923389166, 0.3430219441164014,47.66215921238925, 0.3420095797235661,47.66174955836436, 0.3412989720607698,47.66118086652676, 0.3399251142666202,47.66097105009703, 0.3382524827413819,47.66063732919524, 0.3369382859186629,47.66042760939015, 0.3354450480875215,47.66033878263723, 0.3339487948132529,47.66012663353437, 0.3320934889119656,47.65999360547188, 0.3313153256600572,47.65986719922936, 0.3307185770521386,47.65958042519656, 0.3299397610369255,47.65937273571322, 0.3286827799940397,47.65940564930255, 0.3267610233677516,47.65951363163066, 0.3255620104767303,47.65982973394696, 0.3244224970563607,47.66022678079804, 0.3232122638143409,47.6605404545213, 0.3228495327689985,47.66090159584216, 0.3222441653369685,47.66150323735835, 0.3216984121067554,47.6621050953325, 0.3209739152011793,47.66262500652672, 0.3203085305215296,47.66290303311356, 0.3200059096453742,47.66338528753624, 0.3195836778168086,47.66370528864584, 0.3194000733453067,47.66418807506544, 0.3191549190922416,47.664831623048, 0.3188983131964704,47.66579806074815, 0.3184632593816303,47.66652186475821, 0.3173172922062975,47.66687792045619, 0.3166567073739799,47.66687387761134, 0.3153355744534565,47.66694631173264, 0.3141979439954952,47.6666971780408, 0.3126437911618325,47.66616285979843, 0.3118754332751337,47.66559270587662, 0.3111724777381966,47.66509227652906, 0.3104626492875298,47.66448267073182, 0.3100429109789786,47.66419495816566, 0.3093865612598563,47.6640701024154, 0.3080135141735241,47.66386051000335, 0.3065190514130762,47.6636899288021, 0.3058625123218235,47.66340198449525, 0.3050873407266437,47.66315485318964, 0.3043127888420543,47.66286740363953, 0.3031810315751216,47.66249779724833, 0.3021115210761849,47.66196736999045, 0.301159840481489,47.66147676678508, 0.3004489821100833,47.66082505689892, 0.3000364121975707,47.66025598141727, 0.2992735341640582,47.65916177119338, 0.2986834247462298,47.65855278728089, 0.2982083104835454,47.65818528221875, 0.2972565006276966,47.65773407308685, 0.2961234999542057,47.65756328654402, 0.2943921646022858,47.65783009296292, 0.2927866271247126,47.65805643542142, 0.2913579932046702,47.65828643807902, 0.2904634284109449,47.65852051291729, 0.2887879594022139,47.65887210588135, 0.2871679086796775,47.65910541072209, 0.2860252769413308,47.65934100818994, 0.2847245261566229,47.65956874272107, 0.2832866953472575,47.66022037645162, 0.2823094550059011,47.66034059794328, 0.2808085738591046,47.66021106272945, 0.2794267741373346,47.66020276987996, 0.2777494677031598,47.66051300832708, 0.2765611113324695,47.66030171927672, 0.2756661004910228,47.66005372495982, 0.2748254735302282,47.65988755627776, 0.2735688597182673,47.65967760078586, 0.2717411348522063,47.65893659594579, 0.2701383107185649,47.65868270873875, 0.2687764045713267,47.65851103879945, 0.2677007369891129,47.65842313537647, 0.2671543030724592,47.6585005973592, 0.2653211392250796,47.65913582305231, 0.2642803760751775,47.6598261295798, 0.2636142024696114,47.66058717233668, 0.2636077686097593,47.6610309643448, 0.2638970136028434,47.66159817501065, 0.2644931478180723,47.66196511906276, 0.2650854467705273,47.66261465476397, 0.2649530370357525,47.66313871074478, 0.2641748651683855,47.66369806542404, 0.2631564155623636,47.66401400852892, 0.2621332381688291,47.66420888182348, 0.2614067445466475,47.66436532468497, 0.2605639100590196,47.66456134367846, 0.2596624997831327,47.66463580247124, 0.2590651827005388,47.66455095989439, 0.258890904432235,47.66410567248297, 0.2590763874532485,47.66358228898861, 0.2589619427893268,47.66301675188095, 0.258607172582245,47.66253047335311, 0.2579582660474244,47.66216218720842, 0.2569437531148537,47.66227517897129, 0.2561634063208373,47.6622699551266, 0.2558107010599855,47.66178315311635, 0.2558201073920259,47.66125798241831, 0.255346382824186,47.66085105787913, 0.2546821579632308,47.66100885442844, 0.2540145789026127,47.66172985765506, 0.2532954720799836,47.66176491750916, 0.25294507688254,47.66107655423757, 0.2522976234962755,47.66006526697473, 0.2509257709895962,47.65969228517876, 0.2496746644862924,47.65927902520588, 0.2484788392716695,47.65914790750728, 0.2468619179137477,47.65905334501809, 0.2462036887506542,47.65896913341377, 0.2456189301673533,47.65803787816034, 0.2458093666579089,47.65735276792697, 0.2459997621784993,47.6566671899023, 0.2460122241850087,47.65585890100653, 0.2448820903019282,47.65528546963149, 0.2442255724992606,47.65507787511665, 0.2430277882214824,47.65494656167351, 0.2420080705956309,47.65498018496545, 0.2414040559404214,47.65521716947492, 0.2407438710744145,47.65529428172062, 0.2400273441861743,47.65512821350812, 0.2393107311529832,47.6549207797859, 0.2386612468902882,47.65439322694876, 0.2380626984879655,47.65438993421037, 0.2379297239303924,47.65503201732876, 0.2377441978199642,47.65547623914384, 0.2370819505180748,47.655673757686, 0.2354699551674884,47.65542204586119, 0.2344388606955761,47.65593695230415, 0.2334113153467854,47.65604695847338, 0.2326961065054718,47.65628883078094, 0.2315999071157933,47.65605885169802, 0.2312649940291891,47.65550064766832, 0.2308768822352221,47.65462195235746, 0.2306511407010561,47.65389767108167, 0.2305872703058611,47.65344781864113, 0.2305186539737747,47.6519758917202, 0.2302615035164451,47.65110668860775, 0.2302697685602198,47.65058325013792, 0.2302287419990421,47.64989150858406, 0.2305766096392574,47.64850595519398, 0.2313044861764837,47.64747052796673, 0.2328262590944235,47.64682600138375, 0.233435055598477,47.64618482742414, 0.2338085049316163,47.64513916163302, 0.233462646151537,47.64424807464788, 0.232820961908601,47.64315276971048, 0.2317607633298593,47.64197386096377, 0.2314692616904495,47.6414061800512, 0.2312435201281318,47.64051589733888, 0.2309602274847009,47.63954487717917, 0.230249846132864,47.63909629464168, 0.2292440590780842,47.63848486599599, 0.2285335314148682,47.63799564926433, 0.2274016672328827,47.6374617505497, 0.2204967564042022,47.63430065325548, 0.2196568300118571,47.63429408917367, 0.2186899517290613,47.63440565498316, 0.2179138859938803,47.63407598643966, 0.2172141416557878,47.63318441376444, 0.2158361817673154,47.63301188737386, 0.2139161820944224,47.63287509090381, 0.2128995340090967,47.63270596735979, 0.2114580586453627,47.63273502651899, 0.2094763329100573,47.63320677656614, 0.2079758191660801,47.63347929312718, 0.2007158976379202,47.63811253798061, 0.1996863272640326,47.63879153783063, 0.1996093282809142,47.63984095897914, 0.1959828719690506,47.64150979567287, 0.1938852640867737,47.64516939943865, 0.1913788141245192,47.64785757350971, 0.1854549901435547,47.64979635503868, 0.1833393517259481,47.64998218356486, 0.1816054060140573,47.64984494369065, 0.1808301312067044,47.6495964930809, 0.1799340338851181,47.64934769978573, 0.1788041263797879,47.6487334028531, 0.1782693580094019,47.64840629668677, 0.1773236000684904,47.64779157359449, 0.1769161992736201,47.64706130817847, 0.1768658487611958,47.64633452095273, 0.1769324071932346,47.64585056753144, 0.1773066766082821,47.64504510301008, 0.1776780300494494,47.64436104170004, 0.178228378061263,47.6436784410109, 0.1783591336005657,47.64311363930543, 0.1781908172642659,47.64242547744789, 0.1777175977387202,47.64181565741704, 0.1771236077010201,47.64144735761011, 0.1755698737400451,47.64155708108297, 0.1738361775993411,47.64194798221788, 0.1718511311432707,47.64221531438733, 0.1695199271480874,47.64239918381897, 0.1671466616263757,47.64262259683991, 0.1655542656102966,47.64297226990764, 0.1640747242618801,47.64344301972963, 0.1627548712854718,47.64411641857898, 0.1621435611704811,47.64475561402261, 0.1614525056380671,47.645765219211, 0.1606240603553317,47.64721502105591, 0.1606059913569027,47.64786208439035, 0.1601450369387436,47.64895113849206, 0.1600046732131517,47.64995694800756, 0.1594365381800031,47.65132281832941, 0.1595468390439265,47.65197044046559, 0.1599457775644595,47.65306342612903, 0.1606372048539839,47.65440118918571, 0.1617545440211359,47.65562245170651, 0.1626421690285582,47.65639734028304, 0.1633444964149788,47.65753397643349, 0.1638035902903568,47.65878985850951, 0.1641996378508566,47.66016706386748, 0.1646712510540453,47.6606556240866, 0.1650832694437554,47.66110375452865, 0.1653064483202797,47.66203658862827, 0.1657127971320307,47.66284798524924, 0.165917504459955,47.6649498156129, 0.1658907092753452,47.66652540152032, 0.165571227419278,47.66761705213078, 0.1637651349530955,47.67155688446525, 0.16188859555443,47.67239157522938, 0.1598410350172657,47.6727396428406, 0.1571275964500387,47.67364703852896, 0.154768851447102,47.67423722720012, 0.1536803308628691,47.6745528391949, 0.1524882574046984,47.67446049816879, 0.1509968484310065,47.6738025158259, 0.1495534996183494,47.6741944248102, 0.1484379400074971,47.6756032066161, 0.1478799872587757,47.67640862587666, 0.1469175312304429,47.67664284334548, 0.1455493240340928,47.67658801977268, 0.143906217688496,47.67722503937413, 0.1426766522033205,47.67859163123714, 0.142310547461932,47.67915345013736, 0.1416444224148847,47.67963233035307, 0.1393024600473609,47.67945297060166, 0.1380948619542579,47.67932474673441, 0.1368150433050914,47.67961903771506, 0.1356163615992188,47.67948751139811, 0.1345334676684757,47.6795605105479, 0.1329717019927728,47.67954810425697, 0.1315918389451412,47.67941591114063, 0.1293159434258655,47.67907376376776, 0.1270961412321725,47.67889415222059, 0.1256648594694809,47.67839708030404, 0.1250739056061434,47.67774533474024, 0.1244339582946736,47.67652668635047, 0.1235601145135092,47.67534749162731, 0.1224277862787204,47.67469082508805, 0.1215383642186225,47.67415828486806, 0.1208924711072541,47.67322248989085, 0.1188700451093894,47.67252038957656, 0.1173160326923011,47.67218453333686, 0.1147852567081316,47.67212094470871, 0.1134627344813168,47.67243457572235, 0.1124967155201375,47.67315578842053, 0.1119505069382143,47.67404188299548, 0.1113166097129295,47.67540980843486, 0.1093937509691472,47.67587935930655, 0.1080663533897592,47.67651508770582, 0.1068974492459063,47.67747524813851, 0.106271020107954,47.67872324237174, 0.1060763840810211,47.68013535901891, 0.1065066882907075,47.68171803164622, 0.1065429289920843,47.68252868760948, 0.1063931139364485,47.68358136184435, 0.1059014593297718,47.68446622690793, 0.1057121770948015,47.68507098656673, 0.1031254347582689,47.68863150664414, 0.1006930987860955,47.69031178688153, 0.099717243184827,47.69442951388115, 0.1002210692214476,47.69657614941072, 0.0990845899065107,47.69928000559358, 0.09715238049547281,47.70014874092584, 0.08823199985175734,47.70420644982431, 0.08685094322751752,47.70435409562866, 0.08583145166915074,47.70418412441595, 0.08517330586204253,47.70401677514082, 0.08305295276750257,47.70424742032167, 0.08161137784323103,47.70419437991038, 0.08012756115954289,47.70345100895326, 0.07899801279161522,47.70271344396945, 0.07816417547518029,47.70206035698753, 0.0775113709792663,47.70128761569208, 0.07549121850384211,47.70104132704613, 0.07356854282893721,47.70090237580263, 0.07273645426407427,47.70036855402042, 0.07112419931428349,47.6997881430126, 0.06993066244218937,47.69933184178177, 0.06843609169517606,47.69891473940899, 0.06783936746492714,47.69870703468016, 0.06640985551101153,47.69808763855394, 0.0648575188786964,47.69762923670267, 0.06312438905089834,47.69729045554353, 0.06162944072191579,47.6969542653416, 0.05965025776417576,47.69669485196602, 0.0578492994136468,47.6964368782855, 0.05490523925431035,47.69608790068838, 0.05292003792398248,47.69586834240239, 0.05190266430814461,47.69553569175506, 0.05155040572051918,47.69516841331171, 0.05035516799034667,47.69483426045331, 0.0486194295014144,47.69477892916775, 0.04707341139810239,47.69488748738355, 0.04470197767687978,47.69498709779487, 0.04378833456622999,47.69546462640772, 0.04281959078062414,47.69590125973778, 0.04172680557246622,47.69657949590305, 0.04069687874752503,47.69697505413454, 0.03961034547265285,47.697248661818, 0.03210462169315065,47.69932621859152, 0.02988353475382155,47.69938921507287, 0.02891960766523028,47.69937975021648, 0.02675207694963433,47.69948121686149, 0.02446673242417319,47.69945941433882, 0.02399491515224384,47.69901036347171, 0.02334369357055266,47.69851934708542, 0.02220540263917315,47.69834815584304, 0.02028325520431906,47.69828834076969, 0.01866250734877895,47.69827257019108, 0.01536132160794068,47.69832227991144, 0.01380388724409443,47.69838720675764, 0.01152369347141757,47.69864764566815, 0.01007343668533167,47.69891876583648, 0.008076755753421594,47.69906599318982, 0.0057870460929621,47.6992884159441, 0.004518584141791848,47.69960043766666, 0.003196434420360106,47.69991035207916, 0.002459846765759721,47.70035007035602, 0.001236210256507775,47.70062624833128, 0.0003281876799690053,47.70078056386821, -0.001006698008765739,47.7016167787518, -0.001910042541851453,47.70172964725752, -0.003421361483020747,47.70167770651926, -0.004267708330033316,47.7017515834092, -0.006269039134764398,47.70197887232698, -0.008307210752676042,47.70228069630109, -0.009756140186979678,47.70234915770814, -0.01874968268846544,47.70191744496588, -0.02036952967259254,47.70165858206116, -0.02205652398217239,47.70172173020628, -0.02356693325948811,47.70202991014247, -0.02464091319303918,47.7016587881785, -0.02613444881288274,47.70115861426744, -0.02780413882543087,47.70041655436489, -0.0290516585876759,47.69963531589403, -0.02981979344755814,47.69897953402596, -0.03077146278353382,47.69844419253789, -0.03250170291212839,47.69785985302133, -0.0340539584660462,47.69723832823181, -0.03536340284904327,47.69633583964082, -0.03624769141818263,47.69555767260058, -0.0376180560698316,47.69501816368183, -0.03808527839703402,47.6943658640995, -0.03909184293191004,47.69354634715112, -0.03931667818721962,47.69277474834902, -0.03947632639413712,47.69184211523854, -0.03981556618548154,47.69107107881274, -0.04039727429044149,47.6904197738963, -0.04128724289122943,47.68996691806067, -0.0426662533160993,47.68983252825333, -0.04380345089633408,47.68986362346409, -0.04596347042850368,47.68992487421044, -0.0476507894491931,47.68998865428266, -0.04975589924316637,47.69021155945987, -0.05119249611394139,47.69027982756026, -0.05223435572976598,47.69067206733714, -0.05305086059712494,47.69126787639456, -0.05319115954031873,47.69203525240353, -0.05296533802552972,47.69256342737432, -0.0522681043180171,47.69362278810314, -0.05204148221420724,47.69411078140849, -0.05287878053191264,47.69454839388196, -0.05365217950033242,47.69478385627114, -0.05533515302128933,47.69484854442916, -0.05658384004973982,47.69455336295946, -0.05771255330108717,47.69462338623822, -0.05934000143386697,47.69448596704467, -0.0604186668029314,47.69411087897969, -0.06094883610241198,47.69361987123066, -0.06190507800734224,47.69340835892777, -0.064249956098962,47.69350733416104, -0.06563288021229141,47.69357496487534, -0.06750023818634077,47.69375915852017, -0.06894683889218811,47.69398791912079, -0.07063953187974903,47.69437603045118, -0.07226948381908471,47.69484568344222, -0.07431750819606009,47.6953918370483, -0.07577945877321213,47.69586332762601, -0.07713539658914215,47.69682219372687, -0.07882548186742611,47.69741211266218, -0.08076710941530507,47.69743504255964, -0.08219321661798062,47.69632787111268, -0.08391137331257426,47.69586479632372, -0.08420859276199621,47.6956998937713, -0.08540399203485549,47.69544525304737, -0.08612896662238256,47.69539730699479, -0.08703347534471106,47.69563148021212, -0.08775568160871217,47.69574602359301, -0.08925711477192523,47.69544725026926, -0.08972908933311886,47.69511880447575, -0.09080070124052497,47.69474377616789, -0.09211132626954566,47.69416355756368, -0.09288306257705871,47.69350826170612, -0.09321918173327504,47.69229105667233, -0.09362048397535716,47.69147662708436, -0.09420386356741517,47.69070075760399, -0.09449047151093738,47.6900501414151, -0.0950106815324574,47.68907360429631, -0.0953502860301967,47.68813776835477, -0.09563354297318938,47.68736442728702, -0.09567977581065179,47.68675519777036, -0.09595854707984586,47.68577847516243, -0.09630194842911521,47.68496560780968, -0.09648000084341392,47.68484321763949, -0.09716539815637759,47.68366437086941, -0.09768516581983988,47.6831989518283, -0.09857717826032797,47.6824254052481, -0.1001377985819261,47.68108721291137, -0.1019377945858224,47.68024980988018, -0.1038980845708579,47.6798214746984, -0.1072280681838294,47.67973848592008, -0.1097468127851187,47.68063151746153, -0.1112749723261004,47.68133150220163, -0.1112266673604836,47.68265889637123, -0.1113966974453291,47.68342095966447, -0.1117286290955377,47.68464038068854, -0.1117574440879423,47.68606869667722, -0.1113474411269522,47.68795855357138, -0.1115325250190438,47.68958824140488, -0.1130048457413722,47.691000613104, -0.1155250841107716,47.6918927161155, -0.1192480535275706,47.69226299249687, -0.1211480894988171,47.69214245099549, -0.1236378501733384,47.69170879808081, -0.125047114997372,47.69047011844464, -0.1262999504674127,47.68923338000406, -0.1274897123192034,47.68809915155943, -0.1281972446516591,47.68574557363633, -0.1286768797597091,47.68375269350183, -0.130632589558104,47.68286422368424, -0.132519966248239,47.68243602964722, -0.135092073309167,47.68271751798712, -0.1367558013749459,47.68280312833076, -0.1384098586034384,47.68293922494735, -0.1411382846021922,47.6831118868536, -0.1425951931866791,47.68381343302788, -0.1440431676433178,47.68420623010553, -0.1454900147130616,47.68454913616208, -0.1483690411550072,47.68462090552323, -0.1501731331523644,47.68398856968619, -0.1521869369425275,47.68264173139959, -0.1531382706706239,47.68115162158681, -0.1540182603775264,47.67986649836823, -0.1555683003393256,47.67848259290276, -0.1586741865832408,47.67855090279294, -0.1599741917612363,47.67889004606798, -0.1621181347582166,47.6796799889086, -0.1647273045699933,47.68123445208617, -0.1653077565806456,47.6829073094481, -0.1656275400297886,47.68397768746865, -0.1660324156261908,47.68540284558817, -0.167206902082782,47.68656137946257, -0.1682105016738262,47.68716234264375, -0.1695784298611898,47.68760747992591, -0.1718628472688551,47.68768448888317, -0.1732047623003723,47.6869052001404, -0.1746191937405968,47.68586990830177, -0.1766338763654716,47.68472671204348, -0.1774409217828179,47.68333967107403, -0.1781460643587849,47.68113853978348, -0.1774441737681514,47.6799190388096, -0.1764979051512939,47.67834710550802, -0.1763135205939222,47.67738340541704, -0.1762724154184383,47.67559787791859, -0.1756437518047562,47.67427383742271, -0.1765624227297309,47.66870503298652, -0.1765261752705811,47.6671746624079, -0.1764337504358338,47.6664619751977, -0.1768640544465825,47.66543702024789, -0.177901078887416,47.66445657225856, -0.1787142863918945,47.66363179283938, -0.1801303997390988,47.66264734675315, -0.1816937447045461,47.66155995109448, -0.1836485951697426,47.66092644867749, -0.1850877904179669,47.66085925197553, -0.1881164555902188,47.66082663403216, -0.1898568262245846,47.66075679947979, -0.192900397742747,47.66102857505625, -0.1941926778447112,47.66126997045853, -0.1956468017893371,47.66201998696661, -0.2007554872835584,47.66686155148474, -0.2060042192558172,47.67058043570363, -0.2092935414034116,47.67156642407502, -0.2115844022860784,47.67184972132623, -0.2130283906809467,47.67198701610987, -0.2146126210502218,47.67196801768176, -0.2182618300439856,47.67248552449993, -0.220769401199334,47.67245444887104, -0.2224452510760102,47.67253440761498, -0.2237163172773365,47.67232134367949, -0.2263456888798743,47.67194152834651, -0.2281632081058547,47.6718185062765, -0.2307298378513281,47.67138141107042, -0.2323916333149327,47.67105485780134, -0.2342086179904186,47.67087909182285, -0.235562135747216,47.67056020032521, -0.2366204227237887,47.67044608698949, -0.2391196424659939,47.67036607213665, -0.2406346251981546,47.67034923403856, -0.2427430906331125,47.66981563718739, -0.244991900580331,47.6688210571792, -0.2460403316315478,47.66829942262662, -0.2478514924701614,47.66797291490828, -0.2508615838547144,47.66717242649929, -0.2530412737586119,47.66653471614423, -0.2554533814119395,47.66589563196425, -0.2568775192334272,47.6654195352894, -0.2587502754270048,47.66468358751982, -0.2596534723936372,47.66426537055818, -0.2598684022959361,47.66370161526397, -0.2639064052643789,47.66120480124696, -0.2737261273577616,47.65445844245274, -0.279976542722693,47.64984250875004, -0.2870543744283935,47.64566690055071, -0.2884661474953277,47.6446814703565, -0.289981091472626,47.64461258561176, -0.2915499810835769,47.64382916248811, -0.2936423712504706,47.64273329372882, -0.2976974803427623,47.64146145994547, -0.3019773513237709,47.64023868280059, -0.306820236999485,47.63971980704988, -0.3105281710548695,47.63921448543666, -0.3136369680490533,47.63871499583065, -0.3183018233678041,47.63774180195453, -0.3199569827051789,47.63695428818064, -0.3216871664771901,47.6367820835023, -0.3243757934076967,47.63552714811022, -0.3251994688498314,47.63510846756372, -0.3260975229560614,47.63463785458562, -0.3268505266822075,47.63442434233149, -0.3279820608873649,47.63420629895261, -0.3290355497904045,47.63404114836779, -0.3304011703231567,47.6339212247604, -0.3324429118010528,47.63384592934959, -0.33332120934287,47.63286666486755, -0.3341310557701434,47.63204019097493, -0.335698882670684,47.63120428190155, -0.3375841964615348,47.63077259012537, -0.3389299449550265,47.63014354456053, -0.3398975812474276,47.62946782772142, -0.3409304376433084,47.62838308798231, -0.3422052494754695,47.62790764989786, -0.343627072469971,47.62722698940157, -0.3454053059931539,47.62572313534826, -0.3493830416652202,47.62434885518471, -0.352249920938732,47.62390516233364, -0.3557968513175213,47.62340211515464, -0.3601128285864945,47.62329765400384, -0.3613265758284571,47.6235398103583, -0.3639054104278756,47.62360949527086, -0.366521332826764,47.62384326494053, -0.3680959504088293,47.62449281038822, -0.369115679074028,47.6254450119639, -0.3704060605144406,47.62695236198952, -0.3722539109061245,47.62794904769247, -0.3735189781187103,47.62900395905655, -0.3751463863026743,47.62985169519209, -0.3775874716593393,47.63074023011052, -0.3812673816264862,47.63166549842263, -0.3835162040190312,47.63138105048347, -0.38488637939372,47.63192538708067, -0.3867960322744324,47.6325145726707, -0.3893643190813805,47.63222714308241, -0.3915051455761898,47.63291556266547, -0.3947826070838971,47.63358904772621, -0.396526723405387,47.63361833514574, -0.4005242298890437,47.63300497653106, -0.402703739827615,47.63231403503647, -0.4045647014034303,47.63091244443819, -0.4064306799245685,47.6299689478907, -0.4076808748104099,47.62877831271646, -0.4086529291801214,47.62825522338404, -0.4097568822325286,47.62701527934567, -0.410405346131583,47.62593498656518, -0.4103736711204675,47.62455492641507, -0.4099248732984649,47.62210902244586, -0.4093543745222804,47.62053070943775, -0.4091065749176153,47.61987062974257, -0.4083727191430171,47.61683203676905, -0.4052111832813381,47.61151631207935, -0.4015802711134171,47.60886034330256, -0.4002541278526195,47.6075557595566, -0.3997814081488258,47.60695301167107, -0.3970007704454446,47.60495909311374, -0.3942352797837856,47.60337669013185, -0.3929356621094371,47.60278566721324, -0.3904352636915007,47.60179706762747, -0.3794038543141198,47.59863909930113, -0.3769667249813865,47.59801266069998, -0.3758862767379867,47.59726253955665, -0.3736770652607505,47.59683449194618, -0.3722343106621225,47.59669880634414, -0.3690987150336015,47.59561414768993, -0.3668796671882859,47.59482464496367, -0.3650995651723009,47.59341610078876, -0.3623424333157033,47.59232636900924, -0.361135147828847,47.59218721353265, -0.3599250495588678,47.5917929164088, -0.3603945000791242,47.58704044877377, -0.36060046234212,47.58580251375935, -0.361386402027847,47.58462667825821, -0.3624031138485293,47.58339311902138, -0.3628973248519145,47.58211028499905, -0.3639775662290816,47.58000190633975, -0.3642494375976925,47.57846476155501, -0.3651781456524503,47.57687086590203, -0.3660534731002372,47.57578659554554, -0.3672821938872914,47.57337171516403, -0.3684331407228996,47.57106102103932, -0.3709002368180592,47.57000714943437, -0.3741480859992947,47.56980707117107, -0.3778683649791788,47.57012255468635, -0.3813495511614491,47.56997429953542, -0.3860443756826442,47.56991559027222, -0.3895857453586444,47.56936197349729, -0.3915951526216048,47.56805886127517, -0.3926198382299475,47.56661247620967, -0.3937204031138749,47.56526869009646, -0.3942137874970229,47.56398548187562, -0.394271386665622,47.56337223865216, -0.3951399971345211,47.56276729532013, -0.3948932836126177,47.56205565300552, -0.3942691248072379,47.56139870476225, -0.3941707198066383,47.56058204979177, -0.3950526330486639,47.55965183322793, -0.397303611395543,47.55895952908063, -0.39970780434723,47.55831613785529, -0.4010652903587584,47.55809428916352, -0.4040907360722343,47.55815863434504, -0.4065291427002364,47.55858667619908, -0.407452191510302,47.55903453749874, -0.4104212861137455,47.55950752972837, -0.4136842235997591,47.55992563095501, -0.4167333422940943,47.56090790607344, -0.4200200521362945,47.56188757136641, -0.4227507557620527,47.56282054334521, -0.4241506864259606,47.56382482883605, -0.4257122260294182,47.56498178711745, -0.4278398031296717,47.56526079494682, -0.4308725921263061,47.56522245083889, -0.4328230927840053,47.56534707593533, -0.4349224131525063,47.5657110564942, -0.4359369622704321,47.56682105052602, -0.438237527496824,47.56766221527828, -0.4394673165848388,47.56825989127332, -0.4412490342826203,47.56961608595623, -0.4427093196777605,47.57066634886564, -0.4461422583337651,47.57154113960289, -0.4519265850480423,47.5725383974444, -0.4554334467070902,47.57325908777613, -0.4574458365951132,47.57476556570639, -0.4617651065360491,47.57470921406144, -0.4640436753520589,47.57503613029338, -0.4662205961735566,47.57429212420144, -0.4692946325464931,47.5732291436352, -0.4716271781989434,47.5726871085471, -0.4738029772094166,47.57224913199085, -0.4759518206678593,47.57089235597265, -0.4769473528041033,47.57077650304557, -0.4783922980615786,47.57060355974748, -0.479594646956457,47.57038314175021, -0.4820039046000631,47.570504813305, -0.4839113885598607,47.57063226143493, -0.4856843947995486,47.57076126433518, -0.4872884045391289,47.57111784430972, -0.4882830787255159,47.57135532235203, -0.4899482151299425,47.57128004520597, -0.4902316761260721,47.57061126095729, -0.4913637292307557,47.57043973992884, -0.4931025184265536,47.5703691028694, -0.4946621500784119,47.56932769015764, -0.4956920147534163,47.56829427253754, -0.4965856997177128,47.56777046645662, -0.4973964895431493,47.56703762479547, -0.4975309670534216,47.5664726880698, -0.4973480401793893,47.56540380976096, -0.4962561105711815,47.56434127071328, -0.4963924379083125,47.56383034953535, -0.497881613763127,47.56294644619411, -0.49869669418527,47.56237107069865, -0.4990547468757996,47.56170099899109, -0.499113443396098,47.56108877587236, -0.4995486004693391,47.56041921663532, -0.5009505325354695,47.55917345020799, -0.5006157310327666,47.55815523877985, -0.5012700610566725,47.55712485192198, -0.5028598334036758,47.55475284378143, -0.5039515187501991,47.55310112746609, -0.5048174981475739,47.55181362669933, -0.5054015687125997,47.55098648277817, -0.5065146435464243,47.55036025854727, -0.507862503651296,47.54967459798116, -0.5095993992467243,47.5493936445958, -0.5116601604416928,47.54956572114761, -0.5130055459052159,47.5493993538637, -0.5151810514467314,47.54911851805809, -0.5171829718448606,47.54995705226627, -0.5189314528575721,47.55008561833647, -0.5210587580156982,47.55015754861041, -0.5218198181348015,47.55055789617851, -0.5237610561237744,47.5518072741434, -0.5242999767881429,47.55287621238982, -0.5243531168913336,47.55420278392343, -0.5249226561876129,47.55526798210232, -0.5259325440830196,47.55591859743531, -0.5264059566601276,47.55667894580284, -0.5266651917603866,47.5580550885481, -0.5277450948812117,47.55839860590051, };
    path = &path0[0];
  }
  city *c = createCity (8, "Bordeaux", &map[mapTransfo(44.868611, -0.578333)], nations[FRANCE-FIRST_NATION]);
  if (c!=NULL) {
    c->type = FISHING_CITY;
    unit *u = createUnit (CARPENTER, nations[FRANCE-FIRST_NATION]);
    addUnitToBuilding(u, c, CARPENTRY_SHOP);
    createBuilding(SHIPYARD, c);
    createBuilding(CHURCH, c);
    u = createUnit (PREACHER, nations[FRANCE-FIRST_NATION]);
    addUnitToBuilding(u, c, CHURCH);
    createBuilding(PORT, c);
    createBuilding(NEW_WORLD_OFFICE, c);
    u = createUnit (COLONIST, nations[FRANCE-FIRST_NATION]);
    addUnitToBuilding(u, c, CARPENTRY_SHOP);
    c->stocks[WHEAT-FIRST_GOOD].quantity = 50;
    u = createUnit (BLACKSMITH, nations[FRANCE-FIRST_NATION]);
    addUnitToBuilding(u, c, SMITHY);
    c->stocks[ORE-FIRST_GOOD].quantity = 60;
    c->stocks[LUMBER-FIRST_GOOD].quantity = 80;
    u = createUnit (FARMER, nations[FRANCE-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (LUMBERJACK, nations[FRANCE-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (FISHERMAN, nations[FRANCE-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (COTTONPLANTER, nations[FRANCE-FIRST_NATION]);
    findBestJob(u, c);
  }

  c = createCity (7, "Nantes", &map[mapTransfo(47.218056, -1.552778)], nations[FRANCE-FIRST_NATION]);
  if (c!=NULL) {
    c->type = MINE_CITY;
    unit *u = createUnit (CARPENTER, nations[FRANCE-FIRST_NATION]);
    addUnitToBuilding(u, c, CARPENTRY_SHOP);
    createBuilding(CHURCH, c);
    u = createUnit (PREACHER, nations[FRANCE-FIRST_NATION]);
    addUnitToBuilding(u, c, CHURCH);
    u = createUnit (BLACKSMITH, nations[FRANCE-FIRST_NATION]);
    addUnitToBuilding(u, c, SMITHY);

    createBuilding(SHIPYARD, c);
    createBuilding(PORT, c);
    createBuilding(NEW_WORLD_OFFICE, c);
    c->stocks[MONEY-FIRST_GOOD].quantity  = 1250;
    c->stocks[WHEAT-FIRST_GOOD].quantity  = 50;
    c->stocks[ORE-FIRST_GOOD].quantity    = 60;
    c->stocks[LUMBER-FIRST_GOOD].quantity = 80;
    c->stocks[STONE-FIRST_GOOD].quantity  = 20;
    c->stocks[MUSKET-FIRST_GOOD].quantity = 40;
    c->stocks[TOOL-FIRST_GOOD].quantity   = 60;
    u = createUnit (FARMER, nations[FRANCE-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (LUMBERJACK, nations[FRANCE-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (FISHERMAN, nations[FRANCE-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (TOBACCOPLANTER, nations[FRANCE-FIRST_NATION]);
    findBestJob(u, c);
  }

  c = createCity (5, "Brest", &map[mapTransfo(48.390833, -4.485556)], nations[FRANCE-FIRST_NATION]);
  if (c!=NULL) {
    c->type = FISHING_CITY;
    unit *u = createUnit (CARPENTER, nations[FRANCE-FIRST_NATION]);
    addUnitToBuilding(u, c, CARPENTRY_SHOP);
    createBuilding(CHURCH, c);
    u = createUnit (PREACHER, nations[FRANCE-FIRST_NATION]);
    addUnitToBuilding(u, c, CHURCH);
    createBuilding(SHIPYARD, c);
    createBuilding(PORT, c);
    createBuilding(NEW_WORLD_OFFICE, c);
    c->stocks[WHEAT-FIRST_GOOD].quantity = 50;
    c->stocks[ORE-FIRST_GOOD].quantity = 60;
    c->stocks[LUMBER-FIRST_GOOD].quantity = 80;
    u = createUnit (FARMER, nations[FRANCE-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (FISHERMAN, nations[FRANCE-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (TOBACCOPLANTER, nations[FRANCE-FIRST_NATION]);
    findBestJob(u, c);
  }

  c = createCity (3, "St Malo", &map[mapTransfo(48.648056, -2.0075)], nations[FRANCE-FIRST_NATION]);
  if (c!=NULL) {
    c->type = FISHING_CITY;
    createBuilding(SHIPYARD, c);
    createBuilding(PORT, c);
    createBuilding(NEW_WORLD_OFFICE, c);
    c->stocks[WHEAT-FIRST_GOOD].quantity = 50;
    unit *u = createUnit (FISHERMAN, nations[FRANCE-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (FISHERMAN, nations[FRANCE-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (FISHERMAN, nations[FRANCE-FIRST_NATION]);
    findBestJob(u, c);
  }

  c = createCity (3, "Le Havre", &map[mapTransfo(49.5, 0.12)], nations[FRANCE-FIRST_NATION]);
  if (c!=NULL) {
    c->type = FISHING_CITY;
    createBuilding(SHIPYARD, c);
    createBuilding(PORT, c);
    createBuilding(NEW_WORLD_OFFICE, c);
    c->stocks[WHEAT-FIRST_GOOD].quantity = 50;
    c->stocks[LUMBER-FIRST_GOOD].quantity = 80;
    unit *u = createUnit (FISHERMAN, nations[FRANCE-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (FISHERMAN, nations[FRANCE-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (FISHERMAN, nations[FRANCE-FIRST_NATION]);
    findBestJob(u, c);
  }

  c = createCity (3, "Calais", &map[mapTransfo(50.948056, 1.856389)], nations[FRANCE-FIRST_NATION]);
  if (c!=NULL) {
    c->type = FISHING_CITY;
    createBuilding(SHIPYARD, c);
    createBuilding(PORT, c);
    createBuilding(NEW_WORLD_OFFICE, c);
    c->stocks[WHEAT-FIRST_GOOD].quantity = 50;
    c->stocks[ORE-FIRST_GOOD].quantity = 60;
    c->stocks[LUMBER-FIRST_GOOD].quantity = 80;
    unit *u = createUnit (FISHERMAN, nations[FRANCE-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (FISHERMAN, nations[FRANCE-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (OREMINER, nations[FRANCE-FIRST_NATION]);
    findBestJob(u, c);
  }

  c = createCity (10, "Paris", &map[mapTransfo(48.866667, 2.333056)], nations[FRANCE-FIRST_NATION]);
  if (c!=NULL) {
    c->type = AGRICULTURE_CITY;
    createBuilding(IRONWORKS,   c);
    createBuilding(JEWELWORKS,  c);
    createBuilding(CATHEDRAL,   c);
    createBuilding(GRANARY,     c);
    createBuilding(TOWN_STATUE, c);
    createBuilding(CONSTRUCTION_YARD, c);
    createBuilding(SAWMILL,     c);
    createBuilding(WORKSHOP,    c);
    createBuilding(UNIVERSITY,  c);

    unit *u = createUnit (PREACHER, nations[FRANCE-FIRST_NATION]);
    addUnitToBuilding(u, c, CATHEDRAL);
    u = createUnit (BLACKSMITH, nations[FRANCE-FIRST_NATION]);
    addUnitToBuilding(u, c, IRONWORKS);
    u = createUnit (CARPENTER, nations[FRANCE-FIRST_NATION]);
    addUnitToBuilding(u, c, CONSTRUCTION_YARD);
    u = createUnit (COLONIST, nations[FRANCE-FIRST_NATION]);
    addUnitToBuilding(u, c, JEWELWORKS);
    u = createUnit (STATEMAN, nations[FRANCE-FIRST_NATION]);
    addUnitToBuilding(u, c, UNIVERSITY);

    c->stocks[WHEAT-FIRST_GOOD].quantity = 50;
    c->stocks[ORE-FIRST_GOOD].quantity = 60;
    c->stocks[LUMBER-FIRST_GOOD].quantity = 80;

    u = createUnit (LUMBERJACK, nations[FRANCE-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (FARMER, nations[FRANCE-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (FARMER, nations[FRANCE-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (FARMER, nations[FRANCE-FIRST_NATION]);
    findBestJob(u, c);
  }

  c = createCity (8, "Lyon", &map[mapTransfo(45.766944, 4.834167)], nations[FRANCE-FIRST_NATION]);
  if (c!=NULL) {
    c->type = AGRICULTURE_CITY;
    createBuilding(IRONWORKS,   c);
    createBuilding(JEWELWORKS,  c);
    createBuilding(CATHEDRAL,   c);
    createBuilding(GRANARY,     c);
    createBuilding(TOWN_STATUE, c);
    createBuilding(CONSTRUCTION_YARD, c);
    createBuilding(SAWMILL,     c);
    createBuilding(WORKSHOP,    c);

    unit *u = createUnit (PREACHER, nations[FRANCE-FIRST_NATION]);
    addUnitToBuilding(u, c, CATHEDRAL);
    u = createUnit (BLACKSMITH, nations[FRANCE-FIRST_NATION]);
    addUnitToBuilding(u, c, IRONWORKS);
    u = createUnit (CARPENTER, nations[FRANCE-FIRST_NATION]);
    addUnitToBuilding(u, c, CONSTRUCTION_YARD);
    u = createUnit (COLONIST, nations[FRANCE-FIRST_NATION]);
    addUnitToBuilding(u, c, JEWELWORKS);

    c->stocks[WHEAT-FIRST_GOOD].quantity = 50;
    c->stocks[ORE-FIRST_GOOD].quantity = 60;
    c->stocks[LUMBER-FIRST_GOOD].quantity = 80;

    u = createUnit (LUMBERJACK, nations[FRANCE-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (FARMER, nations[FRANCE-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (FARMER, nations[FRANCE-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (FARMER, nations[FRANCE-FIRST_NATION]);
    findBestJob(u, c);
  }

  c = createCity (8, "Marseille", &map[mapTransfo(43.2975, 5.377222)], nations[FRANCE-FIRST_NATION]);
  if (c!=NULL) {
    c->type = FISHING_CITY;
    createBuilding(IRONWORKS,   c);
    createBuilding(NEW_WORLD_OFFICE,  c);
    createBuilding(CATHEDRAL,   c);
    createBuilding(GRANARY,     c);
    createBuilding(TOWN_STATUE, c);
    createBuilding(CONSTRUCTION_YARD, c);
    createBuilding(SAWMILL,     c);
    createBuilding(WORKSHOP,    c);

    unit *u = createUnit (PREACHER, nations[FRANCE-FIRST_NATION]);
    addUnitToBuilding(u, c, CATHEDRAL);
    u = createUnit (BLACKSMITH, nations[FRANCE-FIRST_NATION]);
    addUnitToBuilding(u, c, IRONWORKS);
    u = createUnit (CARPENTER, nations[FRANCE-FIRST_NATION]);
    addUnitToBuilding(u, c, CONSTRUCTION_YARD);

    c->stocks[WHEAT-FIRST_GOOD].quantity = 50;
    c->stocks[ORE-FIRST_GOOD].quantity = 60;
    c->stocks[LUMBER-FIRST_GOOD].quantity = 80;

    u = createUnit (LUMBERJACK, nations[FRANCE-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (FISHERMAN, nations[FRANCE-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (FISHERMAN, nations[FRANCE-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (FISHERMAN, nations[FRANCE-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (FISHERMAN, nations[FRANCE-FIRST_NATION]);
    findBestJob(u, c);
  }

  c = createCity (8, "Toulouse", &map[mapTransfo(43.605278, 1.442778)], nations[FRANCE-FIRST_NATION]);
  if (c!=NULL) {
    c->type = MINE_CITY;

    createBuilding(IRONWORKS,   c);
    createBuilding(JEWELWORKS,  c);
    createBuilding(CATHEDRAL,   c);
    createBuilding(GRANARY,     c);
    createBuilding(TOWN_STATUE, c);
    createBuilding(CONSTRUCTION_YARD, c);
    createBuilding(SAWMILL,     c);
    createBuilding(WORKSHOP,    c);

    unit *u = createUnit (PREACHER, nations[FRANCE-FIRST_NATION]);
    addUnitToBuilding(u, c, CATHEDRAL);
    u = createUnit (BLACKSMITH, nations[FRANCE-FIRST_NATION]);
    addUnitToBuilding(u, c, IRONWORKS);
    u = createUnit (CARPENTER, nations[FRANCE-FIRST_NATION]);
    addUnitToBuilding(u, c, CONSTRUCTION_YARD);
    u = createUnit (COLONIST, nations[FRANCE-FIRST_NATION]);
    addUnitToBuilding(u, c, JEWELWORKS);

    c->stocks[WHEAT-FIRST_GOOD].quantity = 50;
    c->stocks[ORE-FIRST_GOOD].quantity = 60;
    c->stocks[LUMBER-FIRST_GOOD].quantity = 80;

    u = createUnit (LUMBERJACK, nations[FRANCE-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (FARMER, nations[FRANCE-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (FARMER, nations[FRANCE-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (FARMER, nations[FRANCE-FIRST_NATION]);
    findBestJob(u, c);
  }

  c = createCity (3, "Caen", &map[mapTransfo(49.183, 0.368)], nations[FRANCE-FIRST_NATION]);
  if (c!=NULL) {
    c->type = AGRICULTURE_CITY;
    createBuilding(CHURCH, c);
    createBuilding(SMITHY, c);
    c->stocks[WHEAT-FIRST_GOOD].quantity = 50;
    c->stocks[ORE-FIRST_GOOD].quantity = 60;
    c->stocks[LUMBER-FIRST_GOOD].quantity = 80;
    unit *u = createUnit (FARMER, nations[FRANCE-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (FARMER, nations[FRANCE-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (FARMER, nations[FRANCE-FIRST_NATION]);
    findBestJob(u, c);
  }

  c = createCity (3, "Nice", &map[mapTransfo(43.702778, 7.269167)], nations[FRANCE-FIRST_NATION]);
  if (c!=NULL) {
    c->type = FISHING_CITY;
    createBuilding(SHIPYARD, c);
    createBuilding(PORT, c);
    createBuilding(NEW_WORLD_OFFICE, c);
    c->stocks[WHEAT-FIRST_GOOD].quantity = 50;
    c->stocks[LUMBER-FIRST_GOOD].quantity = 80;
    unit *u = createUnit (FISHERMAN, nations[FRANCE-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (FISHERMAN, nations[FRANCE-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (FISHERMAN, nations[FRANCE-FIRST_NATION]);
    findBestJob(u, c);
  }

  c = createCity (3, "Strasbourg", &map[mapTransfo(48.584444, 7.748611)], nations[FRANCE-FIRST_NATION]);
  if (c!=NULL) {
    c->type = AGRICULTURE_CITY;

    createBuilding(CHURCH, c);
    createBuilding(SMITHY, c);
    c->stocks[WHEAT-FIRST_GOOD].quantity = 50;
    c->stocks[ORE-FIRST_GOOD].quantity = 60;
    c->stocks[LUMBER-FIRST_GOOD].quantity = 80;
    unit *u = createUnit (FARMER, nations[FRANCE-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (FARMER, nations[FRANCE-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (FARMER, nations[FRANCE-FIRST_NATION]);
    findBestJob(u, c);
  }

  c = createCity (3, "Reims", &map[mapTransfo(49.3, 4.033333)], nations[FRANCE-FIRST_NATION]);
  if (c!=NULL) {
    c->type = AGRICULTURE_CITY;

    createBuilding(CHURCH, c);
    createBuilding(SMITHY, c);
    c->stocks[WHEAT-FIRST_GOOD].quantity = 50;
    c->stocks[ORE-FIRST_GOOD].quantity = 60;
    c->stocks[LUMBER-FIRST_GOOD].quantity = 80;
    unit *u = createUnit (FARMER, nations[FRANCE-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (FARMER, nations[FRANCE-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (FARMER, nations[FRANCE-FIRST_NATION]);
    findBestJob(u, c);
  }

  c = createCity (3, "Metz", &map[mapTransfo(49.120278, 6.177778)], nations[FRANCE-FIRST_NATION]);
  if (c!=NULL) {
    c->type = AGRICULTURE_CITY;

    createBuilding(CHURCH, c);
    createBuilding(SMITHY, c);
    c->stocks[WHEAT-FIRST_GOOD].quantity = 50;
    c->stocks[ORE-FIRST_GOOD].quantity = 60;
    c->stocks[LUMBER-FIRST_GOOD].quantity = 80;
    unit *u = createUnit (FARMER, nations[FRANCE-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (FARMER, nations[FRANCE-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (FARMER, nations[FRANCE-FIRST_NATION]);
    findBestJob(u, c);
  }

  c = createCity (3, "Dijon", &map[mapTransfo(47.323889, 5.042778)], nations[FRANCE-FIRST_NATION]);
  if (c!=NULL) {
    c->type = AGRICULTURE_CITY;

    createBuilding(CHURCH, c);
    createBuilding(SMITHY, c);
    c->stocks[WHEAT-FIRST_GOOD].quantity = 50;
    c->stocks[ORE-FIRST_GOOD].quantity = 60;
    c->stocks[LUMBER-FIRST_GOOD].quantity = 80;
    unit *u = createUnit (FARMER, nations[FRANCE-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (FARMER, nations[FRANCE-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (FARMER, nations[FRANCE-FIRST_NATION]);
    findBestJob(u, c);
  }

  c = createCity (3, "Limoges", &map[mapTransfo(45.819167, 1.251389)], nations[FRANCE-FIRST_NATION]);
  if (c!=NULL) {
    c->type = AGRICULTURE_CITY;

    createBuilding(CHURCH, c);
    createBuilding(SMITHY, c);
    c->stocks[WHEAT-FIRST_GOOD].quantity = 50;
    c->stocks[ORE-FIRST_GOOD].quantity = 60;
    c->stocks[LUMBER-FIRST_GOOD].quantity = 80;
    unit *u = createUnit (FARMER, nations[FRANCE-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (FARMER, nations[FRANCE-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (FARMER, nations[FRANCE-FIRST_NATION]);
    findBestJob(u, c);
  }

  c = createCity (3, "Le Mans", &map[mapTransfo(48.005, 0.197778)], nations[FRANCE-FIRST_NATION]);
  if (c!=NULL) {
    c->type = AGRICULTURE_CITY;
    createBuilding(CHURCH, c);
    createBuilding(SMITHY, c);
    c->stocks[WHEAT-FIRST_GOOD].quantity = 50;
    c->stocks[ORE-FIRST_GOOD].quantity = 60;
    c->stocks[LUMBER-FIRST_GOOD].quantity = 80;
    unit *u = createUnit (FARMER, nations[FRANCE-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (FARMER, nations[FRANCE-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (FARMER, nations[FRANCE-FIRST_NATION]);
    findBestJob(u, c);
  }

  c = createCity (3, "Clermont-Ferrand", &map[mapTransfo(45.780556, 3.0875)], nations[FRANCE-FIRST_NATION]);
  if (c!=NULL) {
    c->type = MINE_CITY;
    createBuilding(CHURCH, c);
    createBuilding(SMITHY, c);
    c->stocks[WHEAT-FIRST_GOOD].quantity = 50;
    c->stocks[ORE-FIRST_GOOD].quantity = 60;
    c->stocks[LUMBER-FIRST_GOOD].quantity = 80;
    unit *u = createUnit (FARMER, nations[FRANCE-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (FARMER, nations[FRANCE-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (OREMINER, nations[FRANCE-FIRST_NATION]);
    findBestJob(u, c);
  }

  c = createCity (5, "Montpellier", &map[mapTransfo(43.611944, 3.877222)], nations[FRANCE-FIRST_NATION]);
  if (c!=NULL) {
    c->type = FISHING_CITY;
    createBuilding(SHIPYARD, c);
    createBuilding(PORT, c);
    createBuilding(NEW_WORLD_OFFICE, c);
    c->stocks[WHEAT-FIRST_GOOD].quantity = 50;
    c->stocks[LUMBER-FIRST_GOOD].quantity = 80;
    unit *u = createUnit (FISHERMAN, nations[FRANCE-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (LUMBERJACK, nations[FRANCE-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (CARPENTER, nations[FRANCE-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (FISHERMAN, nations[FRANCE-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (FISHERMAN, nations[FRANCE-FIRST_NATION]);
    findBestJob(u, c);
  }


  c = createCity (1, "Biarritz", &map[mapTransfo(43.481667, -1.556111)], nations[FRANCE-FIRST_NATION]);
  if (c!=NULL) {
    c->type = FISHING_CITY;
    createBuilding(SHIPYARD, c);
    createBuilding(PORT, c);
    createBuilding(NEW_WORLD_OFFICE, c);
    c->stocks[WHEAT-FIRST_GOOD].quantity = 50;
    c->stocks[LUMBER-FIRST_GOOD].quantity = 80;
    unit *u = createUnit (FISHERMAN, nations[FRANCE-FIRST_NATION]);
    findBestJob(u, c);
  }

  c = createCity (1, "Perpignan", &map[mapTransfo(42.698611, 2.89555)], nations[FRANCE-FIRST_NATION]);
  if (c!=NULL) {
    c->type = FISHING_CITY;
    createBuilding(SHIPYARD, c);
    createBuilding(PORT, c);
    createBuilding(NEW_WORLD_OFFICE, c);
    c->stocks[WHEAT-FIRST_GOOD].quantity = 50;
    c->stocks[LUMBER-FIRST_GOOD].quantity = 80;
    unit *u = createUnit (FISHERMAN, nations[FRANCE-FIRST_NATION]);
    findBestJob(u, c);
  }


  c = createCity (1, "Cherbourg", &map[mapTransfo(49.633, -1.6167)], nations[FRANCE-FIRST_NATION]);
  if (c!=NULL) {
    c->type = FISHING_CITY;
    createBuilding(SHIPYARD, c);
    createBuilding(PORT, c);
    createBuilding(NEW_WORLD_OFFICE, c);
    c->stocks[WHEAT-FIRST_GOOD].quantity = 50;
    c->stocks[LUMBER-FIRST_GOOD].quantity = 80;
    unit *u = createUnit (FISHERMAN, nations[FRANCE-FIRST_NATION]);
    findBestJob(u, c);
  }

  c = createCity (1, "Lorient", &map[mapTransfo(47.75, -3.36)], nations[FRANCE-FIRST_NATION]);
  if (c!=NULL) {
    c->type = FISHING_CITY;
    createBuilding(SHIPYARD, c);
    createBuilding(PORT, c);
    createBuilding(NEW_WORLD_OFFICE, c);
    c->stocks[WHEAT-FIRST_GOOD].quantity = 50;
    c->stocks[LUMBER-FIRST_GOOD].quantity = 80;
    unit *u = createUnit (FISHERMAN, nations[FRANCE-FIRST_NATION]);
    findBestJob(u, c);
  }

  c = createCity (1, "Poitiers", &map[mapTransfo(46.581945, 0.336112)], nations[FRANCE-FIRST_NATION]);
  if (c!=NULL) {
    c->type = AGRICULTURE_CITY;
    createBuilding(GRANARY, c);
    c->stocks[WHEAT-FIRST_GOOD].quantity = 50;
    c->stocks[LUMBER-FIRST_GOOD].quantity = 80;
    unit *u = createUnit (FISHERMAN, nations[FRANCE-FIRST_NATION]);
    findBestJob(u, c);
  }

  c = createCity (1, "Lille", &map[mapTransfo(50.637222, 3.063333)], nations[FRANCE-FIRST_NATION]);
  if (c!=NULL) {
    c->type = AGRICULTURE_CITY;
    createBuilding(GRANARY, c);
    c->stocks[WHEAT-FIRST_GOOD].quantity = 50;
    c->stocks[LUMBER-FIRST_GOOD].quantity = 80;
    unit *u = createUnit (FARMER, nations[FRANCE-FIRST_NATION]);
    findBestJob(u, c);
  }

  c = createCity (1, "Mulhouse", &map[mapTransfo(47.749481, 7.33994)], nations[FRANCE-FIRST_NATION]);
  if (c!=NULL) {
    c->type = AGRICULTURE_CITY;
    createBuilding(GRANARY, c);
    c->stocks[WHEAT-FIRST_GOOD].quantity = 50;
    c->stocks[LUMBER-FIRST_GOOD].quantity = 80;
    unit *u = createUnit (FISHERMAN, nations[FRANCE-FIRST_NATION]);
    findBestJob(u, c);
  }

  c = createCity (1, "Besancon", &map[mapTransfo(47.24306, 6.02194)], nations[FRANCE-FIRST_NATION]);
  if (c!=NULL) {
    c->type = AGRICULTURE_CITY;
    createBuilding(GRANARY, c);
    c->stocks[WHEAT-FIRST_GOOD].quantity = 50;
    c->stocks[LUMBER-FIRST_GOOD].quantity = 80;
    unit *u = createUnit (FISHERMAN, nations[FRANCE-FIRST_NATION]);
    findBestJob(u, c);
  }

  c = createCity (3, "Auxerre", &map[mapTransfo(47.4754, 3.3402)], nations[FRANCE-FIRST_NATION]);
  if (c!=NULL) {
    c->type = AGRICULTURE_CITY;

    createBuilding(CHURCH, c);
    createBuilding(SMITHY, c);
    c->stocks[WHEAT-FIRST_GOOD].quantity  = 50;
    c->stocks[ORE-FIRST_GOOD].quantity    = 60;
    c->stocks[LUMBER-FIRST_GOOD].quantity = 80;
    unit *u = createUnit (FARMER, nations[FRANCE-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (FARMER, nations[FRANCE-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (FARMER, nations[FRANCE-FIRST_NATION]);
    findBestJob(u, c);
  }

  c = createCity (3, "Grenoble", &map[mapTransfo(45.187778, 5.726945)], nations[FRANCE-FIRST_NATION]);
  if (c!=NULL) {
    c->type = MINE_CITY;

    createBuilding(CHURCH, c);
    createBuilding(SMITHY, c);
    c->stocks[WHEAT-FIRST_GOOD].quantity = 50;
    c->stocks[ORE-FIRST_GOOD].quantity = 60;
    c->stocks[LUMBER-FIRST_GOOD].quantity = 80;
    unit *u = createUnit (OREMINER, nations[FRANCE-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (OREMINER, nations[FRANCE-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (FARMER, nations[FRANCE-FIRST_NATION]);
    findBestJob(u, c);
  }

  c = createCity (3, "Pau", &map[mapTransfo(43.301667, -0.368611)], nations[FRANCE-FIRST_NATION]);
  if (c!=NULL) {
    c->type = AGRICULTURE_CITY;

    createBuilding(CHURCH, c);
    createBuilding(SMITHY, c);
    c->stocks[WHEAT-FIRST_GOOD].quantity  = 50;
    c->stocks[ORE-FIRST_GOOD].quantity    = 60;
    c->stocks[LUMBER-FIRST_GOOD].quantity = 80;
    unit *u = createUnit (FARMER, nations[FRANCE-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (FARMER, nations[FRANCE-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (FARMER, nations[FRANCE-FIRST_NATION]);
    findBestJob(u, c);
  }

  c = createCity (1, "Die", &map[mapTransfo(44.753611, 5.370278)], nations[FRANCE-FIRST_NATION]);
  if (c!=NULL) {
    c->type = AGRICULTURE_CITY;
    createBuilding(GRANARY, c);
    c->stocks[WHEAT-FIRST_GOOD].quantity  = 50;
    c->stocks[ORE-FIRST_GOOD].quantity    = 60;
    unit *u = createUnit (FARMER, nations[FRANCE-FIRST_NATION]);
    findBestJob(u, c);
  }

  c = createCity (1, "Rodez", &map[mapTransfo(44.350556, 2.575)], nations[FRANCE-FIRST_NATION]);
  if (c!=NULL) {
    c->type = AGRICULTURE_CITY;
    createBuilding(GRANARY, c);
    c->stocks[WHEAT-FIRST_GOOD].quantity  = 50;
    c->stocks[ORE-FIRST_GOOD].quantity    = 60;
    unit *u = createUnit (FARMER, nations[FRANCE-FIRST_NATION]);
    findBestJob(u, c);
  }

  c = createCity (4, "Orleans", &map[mapTransfo(47.903056, 1.905)], nations[FRANCE-FIRST_NATION]);
  if (c!=NULL) {
    c->type = AGRICULTURE_CITY;
    createBuilding(GRANARY, c);
    c->stocks[WHEAT-FIRST_GOOD].quantity  = 50;
    c->stocks[ORE-FIRST_GOOD].quantity    = 60;
    unit *u = createUnit (FARMER, nations[FRANCE-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (FARMER, nations[FRANCE-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (FARMER, nations[FRANCE-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (FARMER, nations[FRANCE-FIRST_NATION]);
    findBestJob(u, c);
  }

  c = createCity (1, "Avignon", &map[mapTransfo(43.95, 4.817)], nations[FRANCE-FIRST_NATION]);
  if (c!=NULL) {
    c->type = AGRICULTURE_CITY;
    createBuilding(GRANARY, c);
    c->stocks[WHEAT-FIRST_GOOD].quantity  = 50;
    c->stocks[ORE-FIRST_GOOD].quantity    = 60;
    unit *u = createUnit (FARMER, nations[FRANCE-FIRST_NATION]);
    findBestJob(u, c);
  }

  c = createCity (1, "Ajaccio", &map[mapTransfo(41.92667, 8.73694)], nations[FRANCE-FIRST_NATION]);
  if (c!=NULL) {
    c->type = FISHING_CITY;
    createBuilding(PORT, c);
    c->stocks[WHEAT-FIRST_GOOD].quantity  = 50;
    c->stocks[ORE-FIRST_GOOD].quantity    = 60;
    unit *u = createUnit (FISHERMAN, nations[FRANCE-FIRST_NATION]);
    findBestJob(u, c);
  }

  c = createCity (1, "Bastia", &map[mapTransfo(42.70083, 9.45028)], nations[FRANCE-FIRST_NATION]);
  if (c!=NULL) {
    c->type = FISHING_CITY;
    createBuilding(PORT, c);
    c->stocks[WHEAT-FIRST_GOOD].quantity  = 50;
    c->stocks[ORE-FIRST_GOOD].quantity    = 60;
    unit *u = createUnit (FISHERMAN, nations[FRANCE-FIRST_NATION]);
    findBestJob(u, c);
  }

  c = createCity (1, "Bonifacio", &map[mapTransfo(41.39, 9.15)], nations[FRANCE-FIRST_NATION]);
  if (c!=NULL) {
    c->type = FISHING_CITY;
    createBuilding(PORT, c);
    createBuilding(DOCK, c);
    c->stocks[WHEAT-FIRST_GOOD].quantity  = 50;
    c->stocks[ORE-FIRST_GOOD].quantity    = 60;
    unit *u = createUnit (FISHERMAN, nations[FRANCE-FIRST_NATION]);
    findBestJob(u, c);
  }

  c = createCity (1, "La Rochelle", &map[mapTransfo(46.059444, -1.151389)], nations[FRANCE-FIRST_NATION]);
  if (c!=NULL) {
    c->type = FISHING_CITY;
    createBuilding(PORT, c);
    createBuilding(DOCK, c);
    createBuilding(NEW_WORLD_OFFICE, c);
    c->stocks[WHEAT-FIRST_GOOD].quantity  = 50;
    c->stocks[ORE-FIRST_GOOD].quantity    = 60;
    unit *u = createUnit (FISHERMAN, nations[FRANCE-FIRST_NATION]);
    findBestJob(u, c);
  }

  c = createCity (1, "Valence", &map[mapTransfo(44.93333, 4.891667)], nations[FRANCE-FIRST_NATION]);
  if (c!=NULL) {
    c->type = AGRICULTURE_CITY;
    createBuilding(PORT, c);
    createBuilding(DOCK, c);
    createBuilding(NEW_WORLD_OFFICE, c);
    c->stocks[WHEAT-FIRST_GOOD].quantity  = 50;
    c->stocks[ORE-FIRST_GOOD].quantity    = 60;
    unit *u = createUnit (FARMER, nations[FRANCE-FIRST_NATION]);
    findBestJob(u, c);
  }

  c = createCity (2, "Toulon", &map[mapTransfo(43.073, 5.983)], nations[FRANCE-FIRST_NATION]);
  if (c!=NULL) {
    c->type = FISHING_CITY;
    createBuilding(PORT, c);
    createBuilding(DOCK, c);
    createBuilding(NEW_WORLD_OFFICE, c);
    c->stocks[WHEAT-FIRST_GOOD].quantity  = 50;
    c->stocks[ORE-FIRST_GOOD].quantity    = 60;
    unit *u = createUnit (FISHERMAN, nations[FRANCE-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (LUMBERJACK, nations[FRANCE-FIRST_NATION]);
    findBestJob(u, c);
  }

  c = createCity (2, "Rennes", &map[mapTransfo(48.114722, -1.679444)], nations[FRANCE-FIRST_NATION]);
  if (c!=NULL) {
    c->type = AGRICULTURE_CITY;
    createBuilding(PORT, c);
    createBuilding(DOCK, c);
    createBuilding(NEW_WORLD_OFFICE, c);
    c->stocks[WHEAT-FIRST_GOOD].quantity  = 50;
    c->stocks[ORE-FIRST_GOOD].quantity    = 60;
    unit *u = createUnit (FARMER, nations[FRANCE-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (FARMER, nations[FRANCE-FIRST_NATION]);
    findBestJob(u, c);
  }

  c = createCity (1, "Bourges", &map[mapTransfo(47.084444, 2.396389)], nations[FRANCE-FIRST_NATION]);
  if (c!=NULL) {
    c->type = AGRICULTURE_CITY;
    createBuilding(PORT, c);
    createBuilding(DOCK, c);
    createBuilding(NEW_WORLD_OFFICE, c);
    c->stocks[WHEAT-FIRST_GOOD].quantity  = 50;
    c->stocks[ORE-FIRST_GOOD].quantity    = 60;
    unit *u = createUnit (FARMER, nations[FRANCE-FIRST_NATION]);
    findBestJob(u, c);
  }

  // Corsica
  linkCitiesWithRoad("Bonifacio",      "Bastia");
  linkCitiesWithRoad("Ajaccio",        "Bastia");

  // France
  linkCitiesWithRoad("Perpignan",    "Toulouse");
  linkCitiesWithRoad("Perpignan",    "Montpellier");
  linkCitiesWithRoad("Biarritz",     "Pau");
  linkCitiesWithRoad("Biarritz",     "Bordeaux");
  linkCitiesWithRoad("Toulouse",     "Bordeaux");
  linkCitiesWithRoad("Toulouse",     "Montpellier");
  linkCitiesWithRoad("Toulouse",     "Rodez");
  linkCitiesWithRoad("Toulouse",     "Pau");
  linkCitiesWithRoad("Rodez",        "Montpellier");
  linkCitiesWithRoad("Avignon",      "Montpellier");
  linkCitiesWithRoad("Avignon",      "Marseille");
  linkCitiesWithRoad("Toulon",       "Marseille");
  linkCitiesWithRoad("Toulon",       "Nice");
  linkCitiesWithRoad("La Rochelle",  "Bordeaux");
  linkCitiesWithRoad("Avignon",      "Valence");
  linkCitiesWithRoad("Valence",      "Die");
  linkCitiesWithRoad("Valence",      "Lyon");
  linkCitiesWithRoad("Valence",      "Grenoble");
  linkCitiesWithRoad("Nice",         "Grenoble");
  linkCitiesWithRoad("Lyon",         "Grenoble");
  linkCitiesWithRoad("La Rochelle",  "Poitiers");
  linkCitiesWithRoad("La Rochelle",  "Nantes");
  linkCitiesWithRoad("Limoges",      "Poitiers");
  linkCitiesWithRoad("Limoges",      "Clermont-Ferrand");
  linkCitiesWithRoad("Lyon",         "Clermont-Ferrand");
  linkCitiesWithRoad("Rodez",        "Clermont-Ferrand");
  linkCitiesWithRoad("Lyon",         "Dijon");
  linkCitiesWithRoad("Auxerre",      "Dijon");
  linkCitiesWithRoad("Besancon",     "Dijon");
  linkCitiesWithRoad("Besancon",     "Mulhouse");
  linkCitiesWithRoad("Strasbourg",   "Mulhouse");
  linkCitiesWithRoad("Auxerre",      "Clermont-Ferrand");
  linkCitiesWithRoad("Auxerre",      "Orleans");
  linkCitiesWithRoad("Auxerre",      "Paris");
  linkCitiesWithRoad("Orleans",      "Poitiers");
  linkCitiesWithRoad("Nantes",       "Poitiers");
  linkCitiesWithRoad("Nantes",       "Lorient");
  linkCitiesWithRoad("Limoges",      "Le Mans");
  linkCitiesWithRoad("Orleans",      "Le Mans");
  linkCitiesWithRoad("Nantes",       "Le Mans");
  linkCitiesWithRoad("Nantes",       "Rennes");
  linkCitiesWithRoad("St Malo",      "Rennes");
  linkCitiesWithRoad("Lorient",      "Rennes");
  linkCitiesWithRoad("St Malo",      "Brest");
  linkCitiesWithRoad("Lorient",      "Brest");
  linkCitiesWithRoad("St Malo",      "Cherbourg");
  linkCitiesWithRoad("Caen",         "Cherbourg");
  linkCitiesWithRoad("Caen",         "Le Mans");
  linkCitiesWithRoad("Caen",         "Le Havre");
  linkCitiesWithRoad("Bourges",      "Limoges");
  linkCitiesWithRoad("Bourges",      "Auxerre");
  linkCitiesWithRoad("Bourges",      "Orleans");
  linkCitiesWithRoad("Paris",        "Orleans");
  linkCitiesWithRoad("Paris",        "Lille");
  linkCitiesWithRoad("Paris",        "Reims");
  linkCitiesWithRoad("Paris",        "Caen");
  linkCitiesWithRoad("Calais",       "Le Havre");
  linkCitiesWithRoad("Calais",       "Lille");
  linkCitiesWithRoad("Reims",        "Lille");
  linkCitiesWithRoad("Reims",        "Metz");
  linkCitiesWithRoad("Strasbourg",   "Metz");
  linkCitiesWithRoad("Dijon",        "Metz");
} //initFrance() ---------------------------------------------------------


/**
 *
 */
void initEngland(){
  city *c = createCity (10, "London", &map[mapTransfo(51.507778, -0.128056)], nations[ENGLAND-FIRST_NATION]);
  if (c!=NULL) {
    createBuilding(GRANARY, c);
    createBuilding(BARRACKS, c);
    createBuilding(CATHEDRAL, c);
    c->type = AGRICULTURE_CITY;
    unit *u = createUnit (FARMER, nations[ENGLAND-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (FARMER, nations[ENGLAND-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (FARMER, nations[ENGLAND-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (FARMER, nations[ENGLAND-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (LUMBERJACK, nations[ENGLAND-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (OREMINER, nations[ENGLAND-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (PREACHER, nations[ENGLAND-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (STATEMAN, nations[ENGLAND-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (BLACKSMITH, nations[ENGLAND-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (CARPENTER, nations[ENGLAND-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (CARPENTER, nations[ENGLAND-FIRST_NATION]);
    findBestJob(u, c);
  }

  c = createCity (8, "Birmingham", &map[mapTransfo(52.483611, -1.906389)], nations[ENGLAND-FIRST_NATION]);
  if (c!=NULL) {
    createBuilding(GRANARY, c);
    createBuilding(BARRACKS, c);
    createBuilding(CATHEDRAL, c);
    c->type = AGRICULTURE_CITY;
    unit *u = createUnit (FARMER, nations[ENGLAND-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (FARMER, nations[ENGLAND-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (FARMER, nations[ENGLAND-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (LUMBERJACK, nations[ENGLAND-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (OREMINER, nations[ENGLAND-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (STATEMAN, nations[ENGLAND-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (BLACKSMITH, nations[ENGLAND-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (CARPENTER, nations[ENGLAND-FIRST_NATION]);
    findBestJob(u, c);
  }

  c = createCity (7, "Glasgow", &map[mapTransfo(55.858, -4.259)], nations[ENGLAND-FIRST_NATION]);
  if (c!=NULL) {
    createBuilding(GRANARY, c);
    createBuilding(BARRACKS, c);
    createBuilding(NEW_WORLD_OFFICE, c);
    c->type = AGRICULTURE_CITY;
    unit *u = createUnit (FARMER, nations[ENGLAND-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (FARMER, nations[ENGLAND-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (FISHERMAN, nations[ENGLAND-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (LUMBERJACK, nations[ENGLAND-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (OREMINER, nations[ENGLAND-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (BLACKSMITH, nations[ENGLAND-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (CARPENTER, nations[ENGLAND-FIRST_NATION]);
    findBestJob(u, c);
  }

   c = createCity (5, "Liverpool", &map[mapTransfo(53.4, -2.983333)], nations[ENGLAND-FIRST_NATION]);
  if (c!=NULL) {
    createBuilding(GRANARY, c);
    createBuilding(BARRACKS, c);
    createBuilding(JESUIT_CHAPEL, c);
    c->type = AGRICULTURE_CITY;
    unit *u = createUnit (FARMER, nations[ENGLAND-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (FARMER, nations[ENGLAND-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (LUMBERJACK, nations[ENGLAND-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (CARPENTER, nations[ENGLAND-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (PREACHER, nations[ENGLAND-FIRST_NATION]);
    findBestJob(u, c);
  }

   c = createCity (4, "Leeds", &map[mapTransfo(53.7998, -1.5482)], nations[ENGLAND-FIRST_NATION]);
  if (c!=NULL) {
    createBuilding(GRANARY, c);
    createBuilding(BARRACKS, c);
    createBuilding(JESUIT_CHAPEL, c);
    c->type = AGRICULTURE_CITY;
    unit *u = createUnit (FARMER, nations[ENGLAND-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (FARMER, nations[ENGLAND-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (LUMBERJACK, nations[ENGLAND-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (CARPENTER, nations[ENGLAND-FIRST_NATION]);
    findBestJob(u, c);
  }


   c = createCity (4, "Sheffield", &map[mapTransfo(53.385833, -1.469444)], nations[ENGLAND-FIRST_NATION]);
  if (c!=NULL) {
    createBuilding(GRANARY, c);
    createBuilding(BARRACKS, c);
    c->type = AGRICULTURE_CITY;
    unit *u = createUnit (FARMER, nations[ENGLAND-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (FARMER, nations[ENGLAND-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (LUMBERJACK, nations[ENGLAND-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (CARPENTER, nations[ENGLAND-FIRST_NATION]);
    findBestJob(u, c);
  }

   c = createCity (4, "Edinburgh", &map[mapTransfo(55.949556, -3.160288)], nations[ENGLAND-FIRST_NATION]);
  if (c!=NULL) {
    createBuilding(GRANARY, c);
    createBuilding(BARRACKS, c);
    createBuilding(NEW_WORLD_OFFICE, c);
    createBuilding(PORT, c);
    createBuilding(DOCK, c);
    c->type = FISHING_CITY;
    unit *u = createUnit (FISHERMAN, nations[ENGLAND-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (FISHERMAN, nations[ENGLAND-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (LUMBERJACK, nations[ENGLAND-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (CARPENTER, nations[ENGLAND-FIRST_NATION]);
    findBestJob(u, c);
  }

   c = createCity (4, "Manchester", &map[mapTransfo(53.466667, -2.233333)], nations[ENGLAND-FIRST_NATION]);
  if (c!=NULL) {
    createBuilding(GRANARY, c);
    createBuilding(BARRACKS, c);
    c->type = AGRICULTURE_CITY;
    unit *u = createUnit (FARMER, nations[ENGLAND-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (FARMER, nations[ENGLAND-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (LUMBERJACK, nations[ENGLAND-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (CARPENTER, nations[ENGLAND-FIRST_NATION]);
    findBestJob(u, c);
  }

   c = createCity (4, "Leicester", &map[mapTransfo(52.635, -1.135)], nations[ENGLAND-FIRST_NATION]);
  if (c!=NULL) {
    createBuilding(GRANARY, c);
    createBuilding(BARRACKS, c);
    c->type = AGRICULTURE_CITY;
    unit *u = createUnit (FARMER, nations[ENGLAND-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (LUMBERJACK, nations[ENGLAND-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (CARPENTER, nations[ENGLAND-FIRST_NATION]);
    findBestJob(u, c);
  }

  c = createCity (4, "Coventry", &map[mapTransfo(52.408056, -1.510556)], nations[ENGLAND-FIRST_NATION]);
  if (c!=NULL) {
    createBuilding(GRANARY, c);
    createBuilding(BARRACKS, c);
    c->type = AGRICULTURE_CITY;
    unit *u = createUnit (FARMER, nations[ENGLAND-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (LUMBERJACK, nations[ENGLAND-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (CARPENTER, nations[ENGLAND-FIRST_NATION]);
    findBestJob(u, c);
  }


  c = createCity (3, "Cardiff", &map[mapTransfo(51.485278, -3.186667)], nations[ENGLAND-FIRST_NATION]);
  if (c!=NULL) {
    createBuilding(GRANARY, c);
    createBuilding(BARRACKS, c);
    createBuilding(NEW_WORLD_OFFICE, c);
    createBuilding(PORT, c);
    createBuilding(DOCK, c);
    c->type = FISHING_CITY;
    unit *u = createUnit (FISHERMAN, nations[ENGLAND-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (LUMBERJACK, nations[ENGLAND-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (CARPENTER, nations[ENGLAND-FIRST_NATION]);
    findBestJob(u, c);
  }

  c = createCity (3, "Belfast", &map[mapTransfo(54.5973, -5.9301)], nations[ENGLAND-FIRST_NATION]);
  if (c!=NULL) {
    createBuilding(GRANARY, c);
    createBuilding(BARRACKS, c);
    createBuilding(NEW_WORLD_OFFICE, c);
    createBuilding(PORT, c);
    createBuilding(DOCK, c);
    c->type = FISHING_CITY;
    unit *u = createUnit (FISHERMAN, nations[ENGLAND-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (LUMBERJACK, nations[ENGLAND-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (CARPENTER, nations[ENGLAND-FIRST_NATION]);
    findBestJob(u, c);
  }

  c = createCity (3, "Nottingham", &map[mapTransfo(52.95, -1.133333)], nations[ENGLAND-FIRST_NATION]);
  if (c!=NULL) {
    createBuilding(GRANARY, c);
    createBuilding(BARRACKS, c);
    c->type = AGRICULTURE_CITY;
    unit *u = createUnit (FARMER, nations[ENGLAND-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (LUMBERJACK, nations[ENGLAND-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (CARPENTER, nations[ENGLAND-FIRST_NATION]);
    findBestJob(u, c);
  }

  c = createCity (1, "Dover", &map[mapTransfo(51.1295, 1.3089)], nations[ENGLAND-FIRST_NATION]);
  if (c!=NULL) {
    createBuilding(GRANARY, c);
    createBuilding(PORT, c);
    createBuilding(DOCK, c);
    c->type = FISHING_CITY;
    unit *u = createUnit (FISHERMAN, nations[ENGLAND-FIRST_NATION]);
    findBestJob(u, c);
  }

  c = createCity (1, "Portsmouth", &map[mapTransfo(50.816667, -1.083333)], nations[ENGLAND-FIRST_NATION]);
  if (c!=NULL) {
    createBuilding(GRANARY, c);
    createBuilding(PORT, c);
    createBuilding(DOCK, c);
    c->type = FISHING_CITY;
    unit *u = createUnit (FISHERMAN, nations[ENGLAND-FIRST_NATION]);
    findBestJob(u, c);
  }

  c = createCity (3, "Plymouth", &map[mapTransfo(50.37, -4.14)], nations[ENGLAND-FIRST_NATION]);
  if (c!=NULL) {
    createBuilding(GRANARY, c);
    createBuilding(PORT, c);
    createBuilding(DOCK, c);
    createBuilding(BARRACKS, c);
    createBuilding(NEW_WORLD_OFFICE, c);
    createBuilding(CATHEDRAL, c);
    c->type = FISHING_CITY;
    unit *u = createUnit (FISHERMAN, nations[ENGLAND-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (FISHERMAN, nations[ENGLAND-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (LUMBERJACK, nations[ENGLAND-FIRST_NATION]);
    findBestJob(u, c);
  }

  c = createCity (1, "Exeter", &map[mapTransfo(50.92154, -3.31311)], nations[ENGLAND-FIRST_NATION]);
  if (c!=NULL) {
    createBuilding(GRANARY, c);
    createBuilding(PORT, c);
    createBuilding(DOCK, c);
    c->type = FISHING_CITY;
    unit *u = createUnit (FISHERMAN, nations[ENGLAND-FIRST_NATION]);
    findBestJob(u, c);
  }
  
  c = createCity (3, "Southampton", &map[mapTransfo(50.89696, -1.40416)], nations[ENGLAND-FIRST_NATION]);
  if (c!=NULL) {
    createBuilding(GRANARY, c);
    createBuilding(PORT, c);
    createBuilding(DOCK, c);
    createBuilding(BARRACKS, c);
    createBuilding(NEW_WORLD_OFFICE, c);
    createBuilding(CATHEDRAL, c);
    c->type = FISHING_CITY;
    unit *u = createUnit (FISHERMAN, nations[ENGLAND-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (FISHERMAN, nations[ENGLAND-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (LUMBERJACK, nations[ENGLAND-FIRST_NATION]);
    findBestJob(u, c);
  }



  c = createCity (4, "Bristol", &map[mapTransfo(51.453889, -2.596667)], nations[ENGLAND-FIRST_NATION]);
  if (c!=NULL) {
    createBuilding(GRANARY, c);
    createBuilding(PORT, c);
    createBuilding(DOCK, c);
    createBuilding(BARRACKS, c);
    createBuilding(NEW_WORLD_OFFICE, c);
    createBuilding(CATHEDRAL, c);
    c->type = FISHING_CITY;
    unit *u = createUnit (FISHERMAN, nations[ENGLAND-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (FISHERMAN, nations[ENGLAND-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (LUMBERJACK, nations[ENGLAND-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (CARPENTER, nations[ENGLAND-FIRST_NATION]);
    findBestJob(u, c);
  }

  linkCitiesWithRoad("Exeter",       "Bristol");
  linkCitiesWithRoad("Exeter",       "Plymouth");
  linkCitiesWithRoad("Dover",        "London");
} //initEngland() ---------------------------------------------------------


/**
 *
 */
void initSpain(){
  city *c = createCity (8, "Barcelona", &map[mapTransfo(41.583333, 2.266667)], nations[SPAIN-FIRST_NATION]);
  if (c!=NULL) {
    createBuilding(GRANARY, c);
    createBuilding(DOCK, c);
    createBuilding(CONSTRUCTION_YARD, c);
    createBuilding(BARRACKS, c);
    createBuilding(NEW_WORLD_OFFICE, c);
    createBuilding(CATHEDRAL, c);
    c->type = FISHING_CITY;
    unit *u = createUnit (FISHERMAN, nations[SPAIN-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (FISHERMAN, nations[SPAIN-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (FISHERMAN, nations[SPAIN-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (FISHERMAN, nations[SPAIN-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (LUMBERJACK, nations[SPAIN-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (OREMINER, nations[SPAIN-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (PREACHER, nations[SPAIN-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (STATEMAN, nations[SPAIN-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (BLACKSMITH, nations[SPAIN-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (CARPENTER, nations[SPAIN-FIRST_NATION]);
    findBestJob(u, c);
  }

  c = createCity (6, "Cadiz", &map[mapTransfo(36.533333, -6.283333)], nations[SPAIN-FIRST_NATION]);
  if (c!=NULL) {
    createBuilding(GRANARY, c);
    createBuilding(PORT, c);
    createBuilding(DOCK, c);
    createBuilding(BARRACKS, c);
    createBuilding(NEW_WORLD_OFFICE, c);
    createBuilding(JESUIT_CHAPEL, c);
    c->type = FISHING_CITY;
    unit *u = createUnit (FARMER, nations[SPAIN-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (FISHERMAN, nations[SPAIN-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (FISHERMAN, nations[SPAIN-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (LUMBERJACK, nations[SPAIN-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (PREACHER, nations[SPAIN-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (CARPENTER, nations[SPAIN-FIRST_NATION]);
    findBestJob(u, c);
  }

  c = createCity (5, "Bilbao", &map[mapTransfo(43.266667, -2.933333)], nations[SPAIN-FIRST_NATION]);
  if (c!=NULL) {
    createBuilding(GRANARY, c);
    createBuilding(DOCK, c);
    createBuilding(PORT, c);
    createBuilding(BARRACKS, c);
    createBuilding(NEW_WORLD_OFFICE, c);
    createBuilding(JESUIT_CHAPEL, c);
    c->type = FISHING_CITY;
    unit *u = createUnit (FARMER, nations[SPAIN-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (FISHERMAN, nations[SPAIN-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (FISHERMAN, nations[SPAIN-FIRST_NATION]);
    findBestJob(u, c);

    u = createUnit (LUMBERJACK, nations[SPAIN-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (CARPENTER, nations[SPAIN-FIRST_NATION]);
    findBestJob(u, c);
  }

  c = createCity (4, "Gijon", &map[mapTransfo(43.533333, -5.7)], nations[SPAIN-FIRST_NATION]);
  if (c!=NULL) {
    createBuilding(GRANARY, c);
    createBuilding(PORT, c);
    createBuilding(DOCK, c);
    createBuilding(BARRACKS, c);
    createBuilding(NEW_WORLD_OFFICE, c);
    createBuilding(JESUIT_CHAPEL, c);
    c->type = FISHING_CITY;
    unit *u = createUnit (FARMER, nations[SPAIN-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (FISHERMAN, nations[SPAIN-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (LUMBERJACK, nations[SPAIN-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (CARPENTER, nations[SPAIN-FIRST_NATION]);
    findBestJob(u, c);
  }

  c = createCity (4, "La Coruna", &map[mapTransfo(43.366667, -8.383333)], nations[SPAIN-FIRST_NATION]);
  if (c!=NULL) {
    createBuilding(GRANARY, c);
    createBuilding(PORT, c);
    createBuilding(DOCK, c);
    createBuilding(BARRACKS, c);
    createBuilding(NEW_WORLD_OFFICE, c);
    createBuilding(JESUIT_CHAPEL, c);
    c->type = FISHING_CITY;
    unit *u = createUnit (FARMER, nations[SPAIN-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (FISHERMAN, nations[SPAIN-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (LUMBERJACK, nations[SPAIN-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (CARPENTER, nations[SPAIN-FIRST_NATION]);
    findBestJob(u, c);
  }

  c = createCity (5, "Malaga", &map[mapTransfo(36.666667, -4.483333)], nations[SPAIN-FIRST_NATION]);
  if (c!=NULL) {
    createBuilding(GRANARY, c);
    createBuilding(PORT, c);
    createBuilding(DOCK, c);
    createBuilding(BARRACKS, c);
    createBuilding(NEW_WORLD_OFFICE, c);
    createBuilding(JESUIT_CHAPEL, c);
    c->type = FISHING_CITY;
    unit *u = createUnit (FARMER, nations[SPAIN-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (FISHERMAN, nations[SPAIN-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (FISHERMAN, nations[SPAIN-FIRST_NATION]);
    findBestJob(u, c);

    u = createUnit (LUMBERJACK, nations[SPAIN-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (CARPENTER, nations[SPAIN-FIRST_NATION]);
    findBestJob(u, c);
  }

  c = createCity (4, "Almeria", &map[mapTransfo(36.833333, -2.45)], nations[SPAIN-FIRST_NATION]);
  if (c!=NULL) {
    createBuilding(GRANARY, c);
    createBuilding(DOCK, c);
    createBuilding(BARRACKS, c);
    createBuilding(NEW_WORLD_OFFICE, c);
    createBuilding(JESUIT_CHAPEL, c);
    c->type = FISHING_CITY;
    unit *u = createUnit (FARMER, nations[SPAIN-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (FISHERMAN, nations[SPAIN-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (LUMBERJACK, nations[SPAIN-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (CARPENTER, nations[SPAIN-FIRST_NATION]);
    findBestJob(u, c);
  }

  c = createCity (4, "Santander", &map[mapTransfo(43.46, -3.8)], nations[SPAIN-FIRST_NATION]);
  if (c!=NULL) {
    createBuilding(GRANARY, c);
    createBuilding(PORT, c);
    createBuilding(DOCK, c);
    createBuilding(BARRACKS, c);
    createBuilding(NEW_WORLD_OFFICE, c);
    createBuilding(JESUIT_CHAPEL, c);
    c->type = FISHING_CITY;
    unit *u = createUnit (FARMER, nations[SPAIN-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (FISHERMAN, nations[SPAIN-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (LUMBERJACK, nations[SPAIN-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (CARPENTER, nations[SPAIN-FIRST_NATION]);
    findBestJob(u, c);
  }

  c = createCity (9, "Madrid", &map[mapTransfo(40.433333, -3.683333)], nations[SPAIN-FIRST_NATION]);
  if (c!=NULL) {
    createBuilding(GRANARY, c);
    createBuilding(BARRACKS, c);
    createBuilding(CATHEDRAL, c);
    c->type = AGRICULTURE_CITY;
    unit *u = createUnit (FARMER, nations[SPAIN-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (FARMER, nations[SPAIN-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (FARMER, nations[SPAIN-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (FARMER, nations[SPAIN-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (LUMBERJACK, nations[SPAIN-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (CARPENTER, nations[SPAIN-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (BLACKSMITH, nations[SPAIN-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (OREMINER, nations[SPAIN-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (SILVERMINER, nations[SPAIN-FIRST_NATION]);
    findBestJob(u, c);
  }

  c = createCity (4, "Murcia", &map[mapTransfo(37.983333, -1.133333)], nations[SPAIN-FIRST_NATION]);
  if (c!=NULL) {
    createBuilding(GRANARY, c);
    createBuilding(BARRACKS, c);
    createBuilding(JESUIT_CHAPEL, c);
    c->type = AGRICULTURE_CITY;
    unit *u = createUnit (FARMER, nations[SPAIN-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (FARMER, nations[SPAIN-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (LUMBERJACK, nations[SPAIN-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (CARPENTER, nations[SPAIN-FIRST_NATION]);
    findBestJob(u, c);
  }

  c = createCity (3, "Granada", &map[mapTransfo(37.178056, -3.600833)], nations[SPAIN-FIRST_NATION]);
  if (c!=NULL) {
    createBuilding(GRANARY, c);
    createBuilding(BARRACKS, c);
    createBuilding(JESUIT_CHAPEL, c);
    c->type = AGRICULTURE_CITY;
    unit *u = createUnit (FARMER, nations[SPAIN-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (FARMER, nations[SPAIN-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (LUMBERJACK, nations[SPAIN-FIRST_NATION]);
    findBestJob(u, c);
  }

  c = createCity (3, "Alicante", &map[mapTransfo(38.345278, -0.483056)], nations[SPAIN-FIRST_NATION]);
  if (c!=NULL) {
    createBuilding(GRANARY, c);
    createBuilding(PORT, c);
    createBuilding(DOCK, c);
    createBuilding(BARRACKS, c);
    createBuilding(NEW_WORLD_OFFICE, c);
    createBuilding(JESUIT_CHAPEL, c);
    c->type = FISHING_CITY;
    unit *u = createUnit (FARMER, nations[SPAIN-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (FISHERMAN, nations[SPAIN-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (LUMBERJACK, nations[SPAIN-FIRST_NATION]);
    findBestJob(u, c);
  }

  c = createCity (3, "Vigo", &map[mapTransfo(42.233333, -8.66666)], nations[SPAIN-FIRST_NATION]);
  if (c!=NULL) {
    createBuilding(GRANARY, c);
    createBuilding(PORT, c);
    createBuilding(DOCK, c);
    createBuilding(BARRACKS, c);
    createBuilding(NEW_WORLD_OFFICE, c);
    createBuilding(JESUIT_CHAPEL, c);
    c->type = FISHING_CITY;
    unit *u = createUnit (FARMER, nations[SPAIN-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (FISHERMAN, nations[SPAIN-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (LUMBERJACK, nations[SPAIN-FIRST_NATION]);
    findBestJob(u, c);
  }

  c = createCity (7, "Valencia", &map[mapTransfo(39.47, -0.376667)], nations[SPAIN-FIRST_NATION]);
  if (c!=NULL) {
    createBuilding(GRANARY, c);
    createBuilding(PORT, c);
    createBuilding(DOCK, c);
    createBuilding(BARRACKS, c);
    createBuilding(NEW_WORLD_OFFICE, c);
    createBuilding(JESUIT_CHAPEL, c);
    c->type = FISHING_CITY;
    unit *u = createUnit (FARMER, nations[SPAIN-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (FARMER, nations[SPAIN-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (FISHERMAN, nations[SPAIN-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (FISHERMAN, nations[SPAIN-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (LUMBERJACK, nations[SPAIN-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (BLACKSMITH, nations[SPAIN-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (STATEMAN, nations[SPAIN-FIRST_NATION]);
    findBestJob(u, c);
  }

  c = createCity (3, "Cordoba", &map[mapTransfo(37.883333, -4.766667)], nations[SPAIN-FIRST_NATION]);
  if (c!=NULL) {
    createBuilding(GRANARY, c);
    createBuilding(BARRACKS, c);
    createBuilding(JESUIT_CHAPEL, c);
    c->type = AGRICULTURE_CITY;
    unit *u = createUnit (FARMER, nations[SPAIN-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (FARMER, nations[SPAIN-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (LUMBERJACK, nations[SPAIN-FIRST_NATION]);
    findBestJob(u, c);
  }

  c = createCity (3, "Pamplona", &map[mapTransfo(42.818333, -1.644167)], nations[SPAIN-FIRST_NATION]);
  if (c!=NULL) {
    createBuilding(GRANARY, c);
    createBuilding(BARRACKS, c);
    createBuilding(JESUIT_CHAPEL, c);
    c->type = AGRICULTURE_CITY;
    unit *u = createUnit (FARMER, nations[SPAIN-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (FARMER, nations[SPAIN-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (LUMBERJACK, nations[SPAIN-FIRST_NATION]);
    findBestJob(u, c);
  }

  c = createCity (6, "Zaragosa", &map[mapTransfo(41.65, -0.883333)], nations[SPAIN-FIRST_NATION]);
  if (c!=NULL) {
    createBuilding(GRANARY, c);
    createBuilding(BARRACKS, c);
    createBuilding(JESUIT_CHAPEL, c);
    c->type = MINE_CITY;
    unit *u = createUnit (FARMER, nations[SPAIN-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (FARMER, nations[SPAIN-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (LUMBERJACK, nations[SPAIN-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (OREMINER, nations[SPAIN-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (OREMINER, nations[SPAIN-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (BLACKSMITH, nations[SPAIN-FIRST_NATION]);
    findBestJob(u, c);
  }

  c = createCity (1, "Caceres", &map[mapTransfo(39.473056, -6.371111)], nations[SPAIN-FIRST_NATION]);
  if (c!=NULL) {
    createBuilding(GRANARY, c);
    c->type = AGRICULTURE_CITY;
    unit *u = createUnit (FARMER, nations[SPAIN-FIRST_NATION]);
    findBestJob(u, c);
  }

  c = createCity (1, "Toledo", &map[mapTransfo(39.861667, -4.025)], nations[SPAIN-FIRST_NATION]);
  if (c!=NULL) {
    createBuilding(GRANARY, c);
    c->type = AGRICULTURE_CITY;
    unit *u = createUnit (FARMER, nations[SPAIN-FIRST_NATION]);
    findBestJob(u, c);
  }

  c = createCity (2, "Salamanca", &map[mapTransfo(40.966667, -5.666667)], nations[SPAIN-FIRST_NATION]);
  if (c!=NULL) {
    createBuilding(GRANARY, c);
    c->type = AGRICULTURE_CITY;
    unit *u = createUnit (FARMER, nations[SPAIN-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (FARMER, nations[SPAIN-FIRST_NATION]);
    findBestJob(u, c);
  }

  c = createCity (1, "Albacete", &map[mapTransfo(38.995556, -1.855833)], nations[SPAIN-FIRST_NATION]);
  if (c!=NULL) {
    createBuilding(GRANARY, c);
    c->type = AGRICULTURE_CITY;
    unit *u = createUnit (FARMER, nations[SPAIN-FIRST_NATION]);
    findBestJob(u, c);
  }

  c = createCity (1, "Ciudad Real", &map[mapTransfo(38.98333, -3.916667)], nations[SPAIN-FIRST_NATION]);
  if (c!=NULL) {
    createBuilding(GRANARY, c);
    c->type = AGRICULTURE_CITY;
    unit *u = createUnit (FARMER, nations[SPAIN-FIRST_NATION]);
    findBestJob(u, c);
  }

  c = createCity (1, "Soria", &map[mapTransfo(41.766667, -2.466667)], nations[SPAIN-FIRST_NATION]);
  if (c!=NULL) {
    createBuilding(GRANARY, c);
    c->type = AGRICULTURE_CITY;
    unit *u = createUnit (FARMER, nations[SPAIN-FIRST_NATION]);
    findBestJob(u, c);
  }

  c = createCity (2, "Logrono", &map[mapTransfo(42.466667, -2.45)], nations[SPAIN-FIRST_NATION]);
  if (c!=NULL) {
    createBuilding(GRANARY, c);
    c->type = AGRICULTURE_CITY;
    unit *u = createUnit (FARMER, nations[SPAIN-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (OREMINER, nations[SPAIN-FIRST_NATION]);
    findBestJob(u, c);
  }

  c = createCity (4, "Palma De Mallorca", &map[mapTransfo(39.566667, 2.65)], nations[SPAIN-FIRST_NATION]);
  if (c!=NULL) {
    createBuilding(GRANARY, c);
    createBuilding(PORT, c);
    createBuilding(DOCK, c);
    createBuilding(BARRACKS, c);
    createBuilding(NEW_WORLD_OFFICE, c);
    createBuilding(JESUIT_CHAPEL, c);
    c->type = FISHING_CITY;
    unit *u = createUnit (FARMER, nations[SPAIN-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (FISHERMAN, nations[SPAIN-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (LUMBERJACK, nations[SPAIN-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (CARPENTER, nations[SPAIN-FIRST_NATION]);
    findBestJob(u, c);
  }

  c = createCity (7, "Sevilla", &map[mapTransfo(37.366667, -5.983333)], nations[SPAIN-FIRST_NATION]);
  if (c!=NULL) {
    createBuilding(GRANARY, c);
    createBuilding(PORT, c);
    createBuilding(DOCK, c);
    createBuilding(BARRACKS, c);
    createBuilding(NEW_WORLD_OFFICE, c);
    createBuilding(JESUIT_CHAPEL, c);
    c->type = FISHING_CITY;
    unit *u = createUnit (FARMER, nations[SPAIN-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (FARMER, nations[SPAIN-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (FISHERMAN, nations[SPAIN-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (FISHERMAN, nations[SPAIN-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (LUMBERJACK, nations[SPAIN-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (BLACKSMITH, nations[SPAIN-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (STATEMAN, nations[SPAIN-FIRST_NATION]);
    findBestJob(u, c);
  }

  c = createCity (2, "Burgos", &map[mapTransfo(42.35, -3.7)], nations[SPAIN-FIRST_NATION]);
  if (c!=NULL) {
    createBuilding(GRANARY, c);
    c->type = AGRICULTURE_CITY;
    unit *u = createUnit (FARMER, nations[SPAIN-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (OREMINER, nations[SPAIN-FIRST_NATION]);
    findBestJob(u, c);
  }

  c = createCity (2, "Ourense", &map[mapTransfo(42.336389, -7.863333)], nations[SPAIN-FIRST_NATION]);
  if (c!=NULL) {
    createBuilding(GRANARY, c);
    c->type = AGRICULTURE_CITY;
    unit *u = createUnit (FARMER, nations[SPAIN-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (OREMINER, nations[SPAIN-FIRST_NATION]);
    findBestJob(u, c);
  }

  c = createCity (2, "Leon", &map[mapTransfo(42.6, -5.583333)], nations[SPAIN-FIRST_NATION]);
  if (c!=NULL) {
    createBuilding(GRANARY, c);
    c->type = AGRICULTURE_CITY;
    unit *u = createUnit (FARMER, nations[SPAIN-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (LUMBERJACK, nations[SPAIN-FIRST_NATION]);
    findBestJob(u, c);
  }

  c = createCity (1, "Lugo", &map[mapTransfo(43.016667, -7.55)], nations[SPAIN-FIRST_NATION]);
  if (c!=NULL) {
    createBuilding(GRANARY, c);
    c->type = AGRICULTURE_CITY;
    unit *u = createUnit (FARMER, nations[SPAIN-FIRST_NATION]);
    findBestJob(u, c);
  }

  c = createCity (3, "Valladolid", &map[mapTransfo(41.633333, -4.716667)], nations[SPAIN-FIRST_NATION]);
  if (c!=NULL) {
    createBuilding(GRANARY, c);
    c->type = AGRICULTURE_CITY;
    unit *u = createUnit (FARMER, nations[SPAIN-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (LUMBERJACK, nations[SPAIN-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (PREACHER, nations[SPAIN-FIRST_NATION]);
    findBestJob(u, c);
  }

  c = createCity (1, "Lorca", &map[mapTransfo(37.683333, -1.7)], nations[SPAIN-FIRST_NATION]);
  if (c!=NULL) {
    createBuilding(GRANARY, c);
    c->type = AGRICULTURE_CITY;
    unit *u = createUnit (FARMER, nations[SPAIN-FIRST_NATION]);
    findBestJob(u, c);
  }

  c = createCity (1, "Tortosa", &map[mapTransfo(40.8125, 0.521111)], nations[SPAIN-FIRST_NATION]);
  if (c!=NULL) {
    createBuilding(GRANARY, c);
    c->type = AGRICULTURE_CITY;
    unit *u = createUnit (FARMER, nations[SPAIN-FIRST_NATION]);
    findBestJob(u, c);
  }

  c = createCity (1, "Badajoz", &map[mapTransfo(38.883056, -6.966666)], nations[SPAIN-FIRST_NATION]);
  if (c!=NULL) {
    createBuilding(GRANARY, c);
    c->type = AGRICULTURE_CITY;
    unit *u = createUnit (FARMER, nations[SPAIN-FIRST_NATION]);
    findBestJob(u, c);
  }

  c = createCity (1, "Teruel", &map[mapTransfo(40.343611, -1.1072222)], nations[SPAIN-FIRST_NATION]);
  if (c!=NULL) {
    createBuilding(GRANARY, c);
    c->type = AGRICULTURE_CITY;
    unit *u = createUnit (FARMER, nations[SPAIN-FIRST_NATION]);
    findBestJob(u, c);
  }

  c = createCity (2, "San Sebastian", &map[mapTransfo(43.321389, -1.985555)], nations[SPAIN-FIRST_NATION]);
  if (c!=NULL) {
    createBuilding(GRANARY, c);
    c->type = MINE_CITY;
    unit *u = createUnit (FARMER, nations[SPAIN-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (OREMINER, nations[SPAIN-FIRST_NATION]);
    findBestJob(u, c);
  }

  c = createCity (1, "Andorra", &map[mapTransfo(42.5, 1.516667)], nations[SPAIN-FIRST_NATION]);
  if (c!=NULL) {
    createBuilding(GRANARY, c);
    c->type = AGRICULTURE_CITY;
    unit *u = createUnit (FARMER, nations[SPAIN-FIRST_NATION]);
    findBestJob(u, c);
  }

  c = createCity (1, "Gibraltar", &map[mapTransfo(36.133333, -5.35)], nations[SPAIN-FIRST_NATION]);
  if (c!=NULL) {
    createBuilding(GRANARY, c);
    c->type = FISHING_CITY;
    unit *u = createUnit (FISHERMAN, nations[SPAIN-FIRST_NATION]);
    findBestJob(u, c);
  }

  c = createCity (1, "Palos de la Frontera", &map[mapTransfo(37.233333,-6.9)], nations[SPAIN-FIRST_NATION]);
  if (c!=NULL) {
    createBuilding(GRANARY, c);
    createBuilding(JESUIT_CHAPEL, c);
    createBuilding(CHURCH, c);
    c->type = FISHING_CITY;
    unit *u = createUnit (PREACHER, nations[SPAIN-FIRST_NATION]);
    findBestJob(u, c);
  }
  
  linkCitiesWithRoad("San Sebastian", "Bilbao");
  linkCitiesWithRoad("Barcelona", "Tortosa");
  linkCitiesWithRoad("Zaragosa", "Tortosa");
  linkCitiesWithRoad("Zaragosa", "Pamplona");
  linkCitiesWithRoad("Zaragosa", "Logrono");
  linkCitiesWithRoad("Bilbao", "Logrono");
  linkCitiesWithRoad("Soria", "Logrono");
  linkCitiesWithRoad("Pamplona", "Logrono");
  linkCitiesWithRoad("Burgos", "Logrono");
  linkCitiesWithRoad("Burgos", "Madrid");
  linkCitiesWithRoad("Albacete", "Madrid");
  linkCitiesWithRoad("Zaragosa", "Madrid");
  linkCitiesWithRoad("Soria", "Madrid");
  linkCitiesWithRoad("Albacete", "Alicante");
  linkCitiesWithRoad("Albacete", "Murcia");
  linkCitiesWithRoad("Alicante", "Murcia");
  linkCitiesWithRoad("Alicante", "Valencia");
  linkCitiesWithRoad("Tortosa", "Valencia");
  linkCitiesWithRoad("Zaragosa", "Valencia");
  linkCitiesWithRoad("Madrid", "Valencia");
  linkCitiesWithRoad("San Sebastian", "Pamplona");
  linkCitiesWithRoad("Burgos", "Valladolid");
  linkCitiesWithRoad("Leon", "Valladolid");
  linkCitiesWithRoad("Burgos", "Leon");
  linkCitiesWithRoad("Gijon", "Leon");
  linkCitiesWithRoad("Gijon", "Santander");
  linkCitiesWithRoad("Bilbao", "Santander");
  linkCitiesWithRoad("Valladolid", "Santander");
  linkCitiesWithRoad("Lorca", "Murcia");
  linkCitiesWithRoad("Malaga", "Almeria");
  linkCitiesWithRoad("Lorca", "Granada");
  linkCitiesWithRoad("Malaga", "Granada");
  linkCitiesWithRoad("Lorca", "Almeria");
  linkCitiesWithRoad("Madrid", "Granada");
  linkCitiesWithRoad("Sevilla", "Granada");
  linkCitiesWithRoad("Sevilla", "Cadiz");
  linkCitiesWithRoad("Sevilla", "Cordoba");
  linkCitiesWithRoad("Cordoba", "Madrid");
  linkCitiesWithRoad("Gibraltar", "Cadiz");
  linkCitiesWithRoad("Gibraltar", "Malaga");
  linkCitiesWithRoad("Sevilla", "Badajoz");
  linkCitiesWithRoad("Sevilla", "Caceres");
  linkCitiesWithRoad("Madrid", "Caceres");
  linkCitiesWithRoad("Salamanca", "Caceres");
  linkCitiesWithRoad("Salamanca", "Madrid");
  linkCitiesWithRoad("Salamanca", "Valladolid");
  linkCitiesWithRoad("Lugo", "Leon");
  linkCitiesWithRoad("Lugo", "La Coruna");
  linkCitiesWithRoad("Ourense", "La Coruna");
  linkCitiesWithRoad("Ourense", "Valladolid");
  linkCitiesWithRoad("Vigo", "La Coruna");
  linkCitiesWithRoad("Ourense", "Vigo");
  linkCitiesWithRoad("Teruel", "Zaragosa");
  linkCitiesWithRoad("Teruel", "Valencia");
  linkCitiesWithRoad("Albacete", "Valencia");

  linkCitiesWithRoad("Gibraltar", "Malaga");
  linkCitiesWithRoad("Gibraltar", "Cadiz");
} //initSpain() ---------------------------------------------------------


/**
 *
 */
void initHolland(){

} //initHolland() ---------------------------------------------------------


/**
 * from nubers given by Mariano La Via (mlavia@hotmail)
 */
void initItaly(){
  city *c = createCity (1, "Ancona", &map[mapTransfo(43.616667, 13.516667)], nations[ITALY-FIRST_NATION]);
  if (c!=NULL) {
    createBuilding(GRANARY, c);
    createBuilding(PORT, c);
    createBuilding(BARRACKS, c);
    createBuilding(JESUIT_CHAPEL, c);
    c->type = FISHING_CITY;
    unit *u = createUnit (FISHERMAN, nations[ITALY-FIRST_NATION]);
    findBestJob(u, c);
  }

  c = createCity (2, "Bari", &map[mapTransfo(41.133333, 16.866667)], nations[ITALY-FIRST_NATION]);
  if (c!=NULL) {
    c->type = FISHING_CITY;
    createBuilding(PORT, c);
    createBuilding(GRANARY, c);
    createBuilding(BARRACKS, c);
    createBuilding(JESUIT_CHAPEL, c);
    unit *u = createUnit (FISHERMAN, nations[ITALY-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (FISHERMAN, nations[ITALY-FIRST_NATION]);
    findBestJob(u, c);
  }

  c = createCity (6, "Bologna", &map[mapTransfo(44.5, 11.35)], nations[ITALY-FIRST_NATION]);
  if (c!=NULL) {
    c->type = AGRICULTURE_CITY;
    createBuilding(GRANARY, c);
    createBuilding(BARRACKS, c);
    createBuilding(JESUIT_CHAPEL, c);
    createBuilding(UNIVERSITY, c);

    unit *u = createUnit (FARMER, nations[ITALY-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (FARMER, nations[ITALY-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (FARMER, nations[ITALY-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (LUMBERJACK, nations[ITALY-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (CARPENTER, nations[ITALY-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (STATEMAN, nations[ITALY-FIRST_NATION]);
    findBestJob(u, c);
  }

  c = createCity (2, "Cagliari", &map[mapTransfo(39.246389, 9.0575)], nations[ITALY-FIRST_NATION]);
  if (c!=NULL) {
    c->type = FISHING_CITY;
    createBuilding(PORT, c);
    createBuilding(GRANARY, c);
    createBuilding(BARRACKS, c);
    createBuilding(JESUIT_CHAPEL, c);
    unit *u = createUnit (FISHERMAN, nations[ITALY-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (FISHERMAN, nations[ITALY-FIRST_NATION]);
    findBestJob(u, c);
  }

  c = createCity (2, "Catania", &map[mapTransfo(37.516667, 15.066667)], nations[ITALY-FIRST_NATION]);
  if (c!=NULL) {
    c->type = FISHING_CITY;
    createBuilding(PORT, c);
    createBuilding(NEW_WORLD_OFFICE, c);
    createBuilding(GRANARY, c);
    createBuilding(BARRACKS, c);
    createBuilding(JESUIT_CHAPEL, c);
    unit *u = createUnit (FISHERMAN, nations[ITALY-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (FISHERMAN, nations[ITALY-FIRST_NATION]);
    findBestJob(u, c);
  }

  c = createCity (8, "Firenze", &map[mapTransfo(43.771667, 11.253611)], nations[ITALY-FIRST_NATION]);
  if (c!=NULL) {
    c->type = AGRICULTURE_CITY;
    createBuilding(GRANARY, c);
    createBuilding(BARRACKS, c);
    createBuilding(JESUIT_CHAPEL, c);
    createBuilding(UNIVERSITY, c);
    createBuilding(BANK, c);
    createBuilding(STOCK_EXCHANGE, c);
    createBuilding(JEWELWORKS, c);
    createBuilding(CLOTH_FACTORY, c);
    createBuilding(TEXTILE_FACTORY, c);
    createBuilding(COLLEGE, c);

    unit *u = createUnit (FARMER, nations[ITALY-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (FARMER, nations[ITALY-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (FARMER, nations[ITALY-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (LUMBERJACK, nations[ITALY-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (CARPENTER, nations[ITALY-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (STATEMAN, nations[ITALY-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (PREACHER, nations[ITALY-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (BLACKSMITH, nations[ITALY-FIRST_NATION]);
    findBestJob(u, c);
  }

  c = createCity (7, "Genoa", &map[mapTransfo(44.4, 8.916667)], nations[ITALY-FIRST_NATION]);
  if (c!=NULL) {
    c->type = AGRICULTURE_CITY;
    createBuilding(PORT, c);
    createBuilding(NEW_WORLD_OFFICE, c);
    createBuilding(GRANARY, c);
    createBuilding(BARRACKS, c);
    createBuilding(JESUIT_CHAPEL, c);
    createBuilding(UNIVERSITY, c);
    createBuilding(BANK, c);
    createBuilding(STOCK_EXCHANGE, c);
    createBuilding(JEWELWORKS, c);
    createBuilding(CLOTH_FACTORY, c);
    createBuilding(TEXTILE_FACTORY, c);
    createBuilding(COLLEGE, c);

    unit *u = createUnit (FARMER, nations[ITALY-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (FARMER, nations[ITALY-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (FARMER, nations[ITALY-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (LUMBERJACK, nations[ITALY-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (CARPENTER, nations[ITALY-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (STATEMAN, nations[ITALY-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (PREACHER, nations[ITALY-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (BLACKSMITH, nations[ITALY-FIRST_NATION]);
    findBestJob(u, c);
  }

  c = createCity (2, "Messina", &map[mapTransfo(38.183333, 15.35)], nations[ITALY-FIRST_NATION]);
  if (c!=NULL) {
    c->type = FISHING_CITY;
    createBuilding(PORT, c);
    createBuilding(NEW_WORLD_OFFICE, c);
    createBuilding(GRANARY, c);
    createBuilding(BARRACKS, c);
    createBuilding(JESUIT_CHAPEL, c);
    unit *u = createUnit (FISHERMAN, nations[ITALY-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (FISHERMAN, nations[ITALY-FIRST_NATION]);
    findBestJob(u, c);
  }


  c = createCity (7, "Milano", &map[mapTransfo(45.466667, 9.166667)], nations[ITALY-FIRST_NATION]);
  if (c!=NULL) {
    c->type = AGRICULTURE_CITY;
    createBuilding(GRANARY, c);
    createBuilding(BARRACKS, c);
    createBuilding(CATHEDRAL, c);
    createBuilding(UNIVERSITY, c);
    createBuilding(BANK, c);
    createBuilding(STOCK_EXCHANGE, c);
    createBuilding(JEWELWORKS, c);
    createBuilding(CLOTH_FACTORY, c);
    createBuilding(TEXTILE_FACTORY, c);
    createBuilding(COLLEGE, c);

    unit *u = createUnit (FARMER, nations[ITALY-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (FARMER, nations[ITALY-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (FARMER, nations[ITALY-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (LUMBERJACK, nations[ITALY-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (CARPENTER, nations[ITALY-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (STATEMAN, nations[ITALY-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (PREACHER, nations[ITALY-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (BLACKSMITH, nations[ITALY-FIRST_NATION]);
    findBestJob(u, c);
  }

  c = createCity (10, "Napoli", &map[mapTransfo(40.833333, 14.25)], nations[ITALY-FIRST_NATION]);
  if (c!=NULL) {
    c->type = FISHING_CITY;
    createBuilding(PORT, c);
    createBuilding(NEW_WORLD_OFFICE, c);
    createBuilding(GRANARY, c);
    createBuilding(BARRACKS, c);
    createBuilding(CATHEDRAL, c);
    createBuilding(UNIVERSITY, c);
    createBuilding(BANK, c);
    createBuilding(STOCK_EXCHANGE, c);
    createBuilding(JEWELWORKS, c);
    createBuilding(CLOTH_FACTORY, c);
    createBuilding(TEXTILE_FACTORY, c);
    createBuilding(COLLEGE, c);

    unit *u = createUnit (FISHERMAN, nations[ITALY-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (FISHERMAN, nations[ITALY-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (FISHERMAN, nations[ITALY-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (FISHERMAN, nations[ITALY-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (LUMBERJACK, nations[ITALY-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (OREMINER, nations[ITALY-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (CARPENTER, nations[ITALY-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (STATEMAN, nations[ITALY-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (PREACHER, nations[ITALY-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (BLACKSMITH, nations[ITALY-FIRST_NATION]);
    findBestJob(u, c);
  }


  c = createCity (9, "Palermo", &map[mapTransfo(38.115556, 13.361944)], nations[ITALY-FIRST_NATION]);
  if (c!=NULL) {
    c->type = AGRICULTURE_CITY;
    createBuilding(PORT, c);
    createBuilding(NEW_WORLD_OFFICE, c);
    createBuilding(GRANARY, c);
    createBuilding(BARRACKS, c);
    createBuilding(JESUIT_CHAPEL, c);
    createBuilding(UNIVERSITY, c);
    createBuilding(BANK, c);
    createBuilding(STOCK_EXCHANGE, c);
    createBuilding(JEWELWORKS, c);
    createBuilding(CLOTH_FACTORY, c);
    createBuilding(TEXTILE_FACTORY, c);
    createBuilding(COLLEGE, c);

    unit *u = createUnit (FARMER, nations[ITALY-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (FARMER, nations[ITALY-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (FARMER, nations[ITALY-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (LUMBERJACK, nations[ITALY-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (CARPENTER, nations[ITALY-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (STATEMAN, nations[ITALY-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (PREACHER, nations[ITALY-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (BLACKSMITH, nations[ITALY-FIRST_NATION]);
    findBestJob(u, c);
  }

  c = createCity (2, "Pisa", &map[mapTransfo(43.716667, 10.4)], nations[ITALY-FIRST_NATION]);
  if (c!=NULL) {
    c->type = FISHING_CITY;
    createBuilding(PORT, c);
    createBuilding(NEW_WORLD_OFFICE, c);
    createBuilding(GRANARY, c);
    createBuilding(WATCH_TOWER, c);
    createBuilding(JESUIT_CHAPEL, c);
    unit *u = createUnit (FISHERMAN, nations[ITALY-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (FISHERMAN, nations[ITALY-FIRST_NATION]);
    findBestJob(u, c);
  }

  c = createCity (6, "Roma", &map[mapTransfo(41.893056, 12.483333)], nations[ITALY-FIRST_NATION]);
  if (c!=NULL) {
    c->type = FISHING_CITY;
    createBuilding(PORT, c);
    createBuilding(NEW_WORLD_OFFICE, c);
    createBuilding(GRANARY, c);
    createBuilding(BARRACKS, c);
    createBuilding(CATHEDRAL, c);
    createBuilding(UNIVERSITY, c);
    createBuilding(BANK, c);
    createBuilding(STOCK_EXCHANGE, c);
    createBuilding(JEWELWORKS, c);
    createBuilding(CLOTH_FACTORY, c);
    createBuilding(TEXTILE_FACTORY, c);
    createBuilding(COLLEGE, c);

    unit *u = createUnit (FISHERMAN, nations[ITALY-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (FARMER, nations[ITALY-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (LUMBERJACK, nations[ITALY-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (CARPENTER, nations[ITALY-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (STATEMAN, nations[ITALY-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (PREACHER, nations[ITALY-FIRST_NATION]);
    findBestJob(u, c);
  }

  c = createCity (2, "Siracusa", &map[mapTransfo(37.083333, 15.283333)], nations[ITALY-FIRST_NATION]);
  if (c!=NULL) {
    c->type = FISHING_CITY;
    createBuilding(PORT, c);
    createBuilding(NEW_WORLD_OFFICE, c);
    createBuilding(GRANARY, c);
    createBuilding(WATCH_TOWER, c);
    createBuilding(JESUIT_CHAPEL, c);
    unit *u = createUnit (FISHERMAN, nations[ITALY-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (FISHERMAN, nations[ITALY-FIRST_NATION]);
    findBestJob(u, c);
  }

  c = createCity (4, "Torino", &map[mapTransfo(45.066667, 7.7)], nations[ITALY-FIRST_NATION]);
  if (c!=NULL) {
    c->type = MINE_CITY;

    createBuilding(GRANARY, c);
    createBuilding(IRONWORKS, c);
    createBuilding(COLLEGE, c);

    unit *u = createUnit (FARMER, nations[ITALY-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (FARMER, nations[ITALY-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (OREMINER, nations[ITALY-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (BLACKSMITH, nations[ITALY-FIRST_NATION]);
    findBestJob(u, c);
  }

  c = createCity (3, "Verona", &map[mapTransfo(45.438158, 10.993742)], nations[ITALY-FIRST_NATION]);
  if (c!=NULL) {
    c->type = MINE_CITY;

    createBuilding(GRANARY, c);
    createBuilding(IRONWORKS, c);
    createBuilding(COLLEGE, c);

    unit *u = createUnit (FARMER, nations[ITALY-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (FARMER, nations[ITALY-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (OREMINER, nations[ITALY-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (BLACKSMITH, nations[ITALY-FIRST_NATION]);
    findBestJob(u, c);
  }


  c = createCity (9, "Venezia", &map[mapTransfo(45.4375, 12.335833)], nations[ITALY-FIRST_NATION]);
  if (c!=NULL) {
    c->type = FISHING_CITY;
    createBuilding(PORT, c);
    createBuilding(NEW_WORLD_OFFICE, c);
    createBuilding(GRANARY, c);
    createBuilding(BARRACKS, c);
    createBuilding(JESUIT_CHAPEL, c);
    createBuilding(UNIVERSITY, c);
    createBuilding(BANK, c);
    createBuilding(STOCK_EXCHANGE, c);
    createBuilding(JEWELWORKS, c);
    createBuilding(CLOTH_FACTORY, c);
    createBuilding(TEXTILE_FACTORY, c);
    createBuilding(COLLEGE, c);

    unit *u = createUnit (FISHERMAN, nations[ITALY-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (FISHERMAN, nations[ITALY-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (FISHERMAN, nations[ITALY-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (FISHERMAN, nations[ITALY-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (LUMBERJACK, nations[ITALY-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (CARPENTER, nations[ITALY-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (STATEMAN, nations[ITALY-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (PREACHER, nations[ITALY-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (BLACKSMITH, nations[ITALY-FIRST_NATION]);
    findBestJob(u, c);
  }

  c = createCity (7, "Parma", &map[mapTransfo(44.8, 10.333333)], nations[ITALY-FIRST_NATION]);
  if (c!=NULL) {
    c->type = AGRICULTURE_CITY;
    createBuilding(GRANARY, c);
    createBuilding(BARRACKS, c);
    createBuilding(JESUIT_CHAPEL, c);
    createBuilding(UNIVERSITY, c);
    createBuilding(BANK, c);
    createBuilding(STOCK_EXCHANGE, c);
    createBuilding(JEWELWORKS, c);
    createBuilding(CLOTH_FACTORY, c);
    createBuilding(TEXTILE_FACTORY, c);
    createBuilding(COLLEGE, c);

    unit *u = createUnit (FARMER, nations[ITALY-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (FARMER, nations[ITALY-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (FARMER, nations[ITALY-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (FARMER, nations[ITALY-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (LUMBERJACK, nations[ITALY-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (CARPENTER, nations[ITALY-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (STATEMAN, nations[ITALY-FIRST_NATION]);
    findBestJob(u, c);
  }


  c = createCity (1, "Reggio Di Calabria", &map[mapTransfo(38.111389, 15.661944)], nations[ITALY-FIRST_NATION]);
  if (c!=NULL) {
    c->type = FISHING_CITY;
    createBuilding(CHURCH, c);
    createBuilding(PORT, c);
    unit *u = createUnit (FISHERMAN, nations[ITALY-FIRST_NATION]);
    findBestJob(u, c);
  }


  c = createCity (2, "Trento", &map[mapTransfo(46.066667, 11.116667)], nations[ITALY-FIRST_NATION]);
  if (c!=NULL) {
    c->type = MINE_CITY;
    createBuilding(CHURCH, c);
    unit *u = createUnit (FARMER, nations[ITALY-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (OREMINER, nations[ITALY-FIRST_NATION]);
    findBestJob(u, c);
  }

  c = createCity (2, "L'Aquila", &map[mapTransfo(42.366111, 13.39444)], nations[ITALY-FIRST_NATION]);
  if (c!=NULL) {
    c->type = MINE_CITY;
    createBuilding(CHURCH, c);
    unit *u = createUnit (FARMER, nations[ITALY-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (OREMINER, nations[ITALY-FIRST_NATION]);
    findBestJob(u, c);
  }

  c = createCity (1, "Taranto", &map[mapTransfo(40.466667, 17.23333)], nations[ITALY-FIRST_NATION]);
  if (c!=NULL) {
    c->type = FISHING_CITY;
    createBuilding(CHURCH, c);
    createBuilding(PORT, c);
    unit *u = createUnit (FISHERMAN, nations[ITALY-FIRST_NATION]);
    findBestJob(u, c);
  }

  c = createCity (1, "Catanzaro", &map[mapTransfo(38.9, 16.6)], nations[ITALY-FIRST_NATION]);
  if (c!=NULL) {
    c->type = AGRICULTURE_CITY;
    createBuilding(CHURCH, c);
    unit *u = createUnit (FARMER, nations[ITALY-FIRST_NATION]);
    findBestJob(u, c);
  }

  c = createCity (1, "Foggia", &map[mapTransfo(41.466667, 15.56666)], nations[ITALY-FIRST_NATION]);
  if (c!=NULL) {
    c->type = AGRICULTURE_CITY;
    createBuilding(CHURCH, c);
    unit *u = createUnit (FARMER, nations[ITALY-FIRST_NATION]);
    findBestJob(u, c);
  }

  c = createCity (1, "Lecce", &map[mapTransfo(40.352011, 18.169139)], nations[ITALY-FIRST_NATION]);
  if (c!=NULL) {
    c->type = AGRICULTURE_CITY;
    createBuilding(CHURCH, c);
    unit *u = createUnit (FARMER, nations[ITALY-FIRST_NATION]);
    findBestJob(u, c);
  }

  c = createCity (1, "Trieste", &map[mapTransfo(45.636111, 13.804167)], nations[ITALY-FIRST_NATION]);
  if (c!=NULL) {
    c->type = FISHING_CITY;
    createBuilding(CHURCH, c);
    createBuilding(PORT, c);
    unit *u = createUnit (FISHERMAN, nations[ITALY-FIRST_NATION]);
    findBestJob(u, c);
  }

  c = createCity (1, "Grosseto", &map[mapTransfo(42.772222, 11.108889)], nations[ITALY-FIRST_NATION]);
  if (c!=NULL) {
    c->type = AGRICULTURE_CITY;
    createBuilding(CHURCH, c);
    unit *u = createUnit (FARMER, nations[ITALY-FIRST_NATION]);
    findBestJob(u, c);
  }

  c = createCity (1, "Sassari", &map[mapTransfo(40.733333, 8.55)], nations[ITALY-FIRST_NATION]);
  if (c!=NULL) {
    c->type = FISHING_CITY;
    createBuilding(CHURCH, c);
    createBuilding(PORT, c);
    unit *u = createUnit (FISHERMAN, nations[ITALY-FIRST_NATION]);
    findBestJob(u, c);
  }

  c = createCity (1, "Olbia", &map[mapTransfo(40.996667, 9.5)], nations[ITALY-FIRST_NATION]);
  if (c!=NULL) {
    c->type = FISHING_CITY;
    createBuilding(CHURCH, c);
    createBuilding(PORT, c);
    unit *u = createUnit (FISHERMAN, nations[ITALY-FIRST_NATION]);
    findBestJob(u, c);
  }

  c = createCity (1, "Trapani", &map[mapTransfo(38.016667, 12.516667)], nations[ITALY-FIRST_NATION]);
  if (c!=NULL) {
    c->type = FISHING_CITY;
    createBuilding(CHURCH, c);
    createBuilding(PORT, c);
    unit *u = createUnit (FISHERMAN, nations[ITALY-FIRST_NATION]);
    findBestJob(u, c);
  }

  c = createCity (1, "Salerno", &map[mapTransfo(40.683333, 14.766667)], nations[ITALY-FIRST_NATION]);
  if (c!=NULL) {
    c->type = FISHING_CITY;
    createBuilding(CHURCH, c);
    createBuilding(PORT, c);
    unit *u = createUnit (FISHERMAN, nations[ITALY-FIRST_NATION]);
    findBestJob(u, c);
  }

  c = createCity (1, "Pescara", &map[mapTransfo(42.463889, 14.214167)], nations[ITALY-FIRST_NATION]);
  if (c!=NULL) {
    c->type = FISHING_CITY;
    createBuilding(CHURCH, c);
    createBuilding(PORT, c);
    unit *u = createUnit (FISHERMAN, nations[ITALY-FIRST_NATION]);
    findBestJob(u, c);
  }

  // Sardegna
  linkCitiesWithRoad("Olbia", "Sassari");
  linkCitiesWithRoad("Cagliari", "Sassari");
  linkCitiesWithRoad("Olbia", "Cagliari");

  // mainland
  linkCitiesWithRoad("Reggio Di Calabria", "Salerno");
  linkCitiesWithRoad("Catanzaro", "Salerno");
  linkCitiesWithRoad("Napoli", "Salerno");
  linkCitiesWithRoad("Napoli", "Foggia");
  linkCitiesWithRoad("Foggia", "Bari");
  linkCitiesWithRoad("Taranto", "Bari");
  linkCitiesWithRoad("Taranto", "Lecce");
  linkCitiesWithRoad("Pescara", "Foggia");
  linkCitiesWithRoad("L'Aquila", "Foggia");
  linkCitiesWithRoad("L'Aquila", "Roma");
  linkCitiesWithRoad("Napoli", "Roma");
  linkCitiesWithRoad("Grosseto", "Roma");
  linkCitiesWithRoad("Grosseto", "Pisa");
  linkCitiesWithRoad("Firenze", "Pisa");
  linkCitiesWithRoad("Firenze", "Bologna");
  linkCitiesWithRoad("Firenze", "Roma");
  linkCitiesWithRoad("Roma", "Ancona");
  linkCitiesWithRoad("Ancona", "Bologna");
  linkCitiesWithRoad("Ancona", "Pescara");
  linkCitiesWithRoad("Venezia", "Bologna");
  linkCitiesWithRoad("Venezia", "Trieste");
  linkCitiesWithRoad("Venezia", "Verona");
  linkCitiesWithRoad("Trento", "Verona");
  linkCitiesWithRoad("Milano", "Verona");
  linkCitiesWithRoad("Milano", "Genoa");
  linkCitiesWithRoad("Milano", "Parma");
  linkCitiesWithRoad("Bologna", "Parma");
  linkCitiesWithRoad("Torino", "Parma");
  linkCitiesWithRoad("Milano", "Torino");

  // sicilia
  linkCitiesWithRoad("Messina", "Palermo");
  linkCitiesWithRoad("Siracusa", "Catania");
  linkCitiesWithRoad("Messina", "Catania");
  linkCitiesWithRoad("Siracusa", "Palermo");
  linkCitiesWithRoad("Trapani", "Palermo");
} //initItaly() ---------------------------------------------------------


/**
 *
 */
void initPortugal(){
  city *c = createCity (10, "Porto", &map[mapTransfo(41.133334, -8.666667)], nations[PORTUGAL-FIRST_NATION]);
  if (c!=NULL) {
    createBuilding(GRANARY, c);
    createBuilding(DOCK, c);
    createBuilding(CONSTRUCTION_YARD, c);
    createBuilding(BARRACKS, c);
    createBuilding(NEW_WORLD_OFFICE, c);
    createBuilding(CATHEDRAL, c);
    c->type = FISHING_CITY;
    unit *u = createUnit (FISHERMAN, nations[PORTUGAL-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (FISHERMAN, nations[PORTUGAL-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (FISHERMAN, nations[PORTUGAL-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (FISHERMAN, nations[PORTUGAL-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (LUMBERJACK, nations[PORTUGAL-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (OREMINER, nations[PORTUGAL-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (PREACHER, nations[PORTUGAL-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (STATEMAN, nations[PORTUGAL-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (BLACKSMITH, nations[PORTUGAL-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (CARPENTER, nations[PORTUGAL-FIRST_NATION]);
    findBestJob(u, c);
  }

  c = createCity (10, "Lisboa", &map[mapTransfo(38.7, -9.166667)], nations[PORTUGAL-FIRST_NATION]);
  if (c!=NULL) {
    createBuilding(GRANARY, c);
    createBuilding(DOCK, c);
    createBuilding(CONSTRUCTION_YARD, c);
    createBuilding(BARRACKS, c);
    createBuilding(NEW_WORLD_OFFICE, c);
    createBuilding(CATHEDRAL, c);
    c->type = FISHING_CITY;
    unit *u = createUnit (FISHERMAN, nations[PORTUGAL-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (FISHERMAN, nations[PORTUGAL-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (FISHERMAN, nations[PORTUGAL-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (FISHERMAN, nations[PORTUGAL-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (LUMBERJACK, nations[PORTUGAL-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (OREMINER, nations[PORTUGAL-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (PREACHER, nations[PORTUGAL-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (STATEMAN, nations[PORTUGAL-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (BLACKSMITH, nations[PORTUGAL-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (CARPENTER, nations[PORTUGAL-FIRST_NATION]);
    findBestJob(u, c);
  }

  c = createCity (8, "Guimaraes", &map[mapTransfo(41.46666, -8.4)], nations[PORTUGAL-FIRST_NATION]);
  if (c!=NULL) {
    createBuilding(GRANARY, c);
    createBuilding(BARRACKS, c);
    createBuilding(NEW_WORLD_OFFICE, c);
    createBuilding(CATHEDRAL, c);
    c->type = AGRICULTURE_CITY;
    unit *u = createUnit (FARMER, nations[PORTUGAL-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (FARMER, nations[PORTUGAL-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (FARMER, nations[PORTUGAL-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (LUMBERJACK, nations[PORTUGAL-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (OREMINER, nations[PORTUGAL-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (STATEMAN, nations[PORTUGAL-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (BLACKSMITH, nations[PORTUGAL-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (CARPENTER, nations[PORTUGAL-FIRST_NATION]);
    findBestJob(u, c);
  }


  c = createCity (1, "Viana de Castelo", &map[mapTransfo(41.7, -8.833333)], nations[PORTUGAL-FIRST_NATION]);
  if (c!=NULL) {
    createBuilding(GRANARY, c);
    c->type = FISHING_CITY;
    unit *u = createUnit (FISHERMAN, nations[PORTUGAL-FIRST_NATION]);
    findBestJob(u, c);
  }

  c = createCity (1, "Chaves", &map[mapTransfo(41.75, -7.533333)], nations[PORTUGAL-FIRST_NATION]);
  if (c!=NULL) {
    createBuilding(GRANARY, c);
    c->type = AGRICULTURE_CITY;
    unit *u = createUnit (FARMER, nations[PORTUGAL-FIRST_NATION]);
    findBestJob(u, c);
  }

  c = createCity (1, "Viseu", &map[mapTransfo(40.666666, -7.916667)], nations[PORTUGAL-FIRST_NATION]);
  if (c!=NULL) {
    createBuilding(GRANARY, c);
    c->type = AGRICULTURE_CITY;
    unit *u = createUnit (FARMER, nations[PORTUGAL-FIRST_NATION]);
    findBestJob(u, c);
  }

  c = createCity (1, "Aveiro", &map[mapTransfo(40.616667, -8.63333333)], nations[PORTUGAL-FIRST_NATION]);
  if (c!=NULL) {
    createBuilding(GRANARY, c);
    c->type = FISHING_CITY;
    unit *u = createUnit (FISHERMAN, nations[PORTUGAL-FIRST_NATION]);
    findBestJob(u, c);
  }

  c = createCity (1, "Guarda", &map[mapTransfo(40.533333, -7.333333)], nations[PORTUGAL-FIRST_NATION]);
  if (c!=NULL) {
    createBuilding(GRANARY, c);
    c->type = AGRICULTURE_CITY;
    unit *u = createUnit (FARMER, nations[PORTUGAL-FIRST_NATION]);
    findBestJob(u, c);
  }

  c = createCity (1, "Figueira Da Foz", &map[mapTransfo(40.116667, -8.9)], nations[PORTUGAL-FIRST_NATION]);
  if (c!=NULL) {
    createBuilding(GRANARY, c);
    c->type = FISHING_CITY;
    unit *u = createUnit (FISHERMAN, nations[PORTUGAL-FIRST_NATION]);
    findBestJob(u, c);
  }

  c = createCity (2, "Coimbra", &map[mapTransfo(40.25, -8.45)], nations[PORTUGAL-FIRST_NATION]);
  if (c!=NULL) {
    createBuilding(GRANARY, c);
    c->type = AGRICULTURE_CITY;
    unit *u = createUnit (FARMER, nations[PORTUGAL-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (LUMBERJACK, nations[PORTUGAL-FIRST_NATION]);
    findBestJob(u, c);
  }

  c = createCity (2, "Leiria", &map[mapTransfo(39.766667, -8.88333333)], nations[PORTUGAL-FIRST_NATION]);
  if (c!=NULL) {
    createBuilding(GRANARY, c);
    c->type = FISHING_CITY;
    unit *u = createUnit (FISHERMAN, nations[PORTUGAL-FIRST_NATION]);
    findBestJob(u, c);
    u = createUnit (CARPENTER, nations[PORTUGAL-FIRST_NATION]);
    findBestJob(u, c);
  }

  c = createCity (1, "Castelo Branco", &map[mapTransfo(39.83333, -7.46666)], nations[PORTUGAL-FIRST_NATION]);
  if (c!=NULL) {
    createBuilding(GRANARY, c);
    c->type = AGRICULTURE_CITY;
    unit *u = createUnit (FARMER, nations[PORTUGAL-FIRST_NATION]);
    findBestJob(u, c);
  }

  c = createCity (1, "Santarem", &map[mapTransfo(39.2, -8.7)], nations[PORTUGAL-FIRST_NATION]);
  if (c!=NULL) {
    createBuilding(GRANARY, c);
    c->type = AGRICULTURE_CITY;
    unit *u = createUnit (FARMER, nations[PORTUGAL-FIRST_NATION]);
    findBestJob(u, c);
  }

  c = createCity (1, "Ourem", &map[mapTransfo(39.65, -8.583333)], nations[PORTUGAL-FIRST_NATION]);
  if (c!=NULL) {
    createBuilding(GRANARY, c);
    c->type = AGRICULTURE_CITY;
    unit *u = createUnit (FARMER, nations[PORTUGAL-FIRST_NATION]);
    findBestJob(u, c);
  }

  c = createCity (1, "Loule", &map[mapTransfo(37.15, -8)], nations[PORTUGAL-FIRST_NATION]);
  if (c!=NULL) {
    createBuilding(GRANARY, c);
    c->type = FISHING_CITY;
    unit *u = createUnit (FISHERMAN, nations[PORTUGAL-FIRST_NATION]);
    findBestJob(u, c);
  }

  linkCitiesWithRoad("Porto", "Viana de Castelo");
  linkCitiesWithRoad("Porto", "Guimaraes");
  linkCitiesWithRoad("Porto", "Aveiro");
  linkCitiesWithRoad("Chaves", "Guimaraes");
  linkCitiesWithRoad("Chaves", "Viseu");
  linkCitiesWithRoad("Guimaraes", "Viana de Castelo");
  linkCitiesWithRoad("Viseu", "Aveiro");
  linkCitiesWithRoad("Viseu", "Guarda");
  linkCitiesWithRoad("Figueira Da Foz", "Aveiro");
  linkCitiesWithRoad("Coimbra", "Aveiro");
  linkCitiesWithRoad("Figueira Da Foz", "Leiria");
  linkCitiesWithRoad("Coimbra", "Leiria");
  linkCitiesWithRoad("Castelo Branco", "Guarda");
  linkCitiesWithRoad("Castelo Branco", "Santarem");
  linkCitiesWithRoad("Ourem", "Leiria");
  linkCitiesWithRoad("Ourem", "Santarem");
  linkCitiesWithRoad("Ourem", "Lisboa");
  linkCitiesWithRoad("Loule", "Lisboa");
} //initPortugal() ---------------------------------------------------------


/**
 *
 */
void initGermany(){

} //initGermany() ---------------------------------------------------------


/**
 *
 */
void initRussia(){

} //initRussia() ---------------------------------------------------------


/**
*
 */
void initIroquois(){
  int x = (-78.1787109375 + -63.6767578125)/2;
  int y = (46.2744140625 + 37.2216796875)/2;
  tile *t = &map[mapTransfo(y, x)];
  tile *t0 = &map[mapTransfo(46.2744140625, -78.1787109375)];
  tile *t1 = &map[mapTransfo(37.2216796875, -63.6767578125)];
  plotIndianCities(20, IROQUOIS, t->posX, t->posY, abs(t0->posX-t1->posX)/2);
} //initIroquois() ---------------------------------------------------------


